@@ -164,6 +164,7 @@ typedef enum
     CODE_PATCH_NO_PullSnapped                = ( 0x1 << 0x1 ),
     CODE_PATCH_NO_PullCentroid               = ( 0x1 << 0x2 ),
     CODE_PATCH_NO_ZWDelta                    = ( 0x1 << 0x3 ),
+    CODE_PATCH_NO_UNSTABLE_PLATFORM          = ( 0x1 << 0x4 ),
 } CODE_PATCH_FLAG_t;
 
 // Fastest compilation experimental flags
@@ -178,14 +179,14 @@ typedef enum
     FCEXP_LOCAL_SCHEDULING              = ( 0x1 << 0x5 ),
     FCEXP_PRERA_SCHEDULING              = ( 0x1 << 0x6 ),
     FCEXP_NO_REMAT                      = ( 0x1 << 0x7 ),
-    FCEXP_SPILL_COMPRESSION             = ( 0x1 << 0x8 ),
+    FCEXP_NO_SPILL_COMPRESSION          = ( 0x1 << 0x8 ),
     FCEXP_LOCAL_DECL_SPLIT_GLOBAL_RA    = ( 0x1 << 0x9 ),
     FCEXP_QUICKTOKEN_ALLOC              = ( 0x1 << 0xa ),
     FCEXP_DISABLE_UNROLL                = ( 0x1 << 0xb ),
     FCEXP_TOBE_DESIGNED                 = ( 0x1 << 0xc ),
 
     // Current default stage 1 options. *Must* be updated whenever the default changes.
-    FCEXP_DEFAULT                       = ( FCEXP_DISABLE_LVN | FCEXP_LINEARSCAN | FCEXP_DISABLE_GOPT | FCEXP_LOCAL_SCHEDULING | FCEXP_PRERA_SCHEDULING | FCEXP_SPILL_COMPRESSION | FCEXP_QUICKTOKEN_ALLOC ),
+    FCEXP_DEFAULT                       = ( FCEXP_DISABLE_LVN | FCEXP_LINEARSCAN | FCEXP_DISABLE_GOPT | FCEXP_LOCAL_SCHEDULING | FCEXP_PRERA_SCHEDULING | FCEXP_NO_SPILL_COMPRESSION | FCEXP_QUICKTOKEN_ALLOC ),
 
     // Alias for UMD to indicate staged compilation must be disabled
     FCEXP_DISABLED                      = FCEXP_TOBE_DESIGNED
