@@ -85,7 +85,7 @@ bool AddImplicitArgs::runOnModule(Module &M)
         if (m_pMdUtils->findFunctionsInfoItem(&func) == m_pMdUtils->end_FunctionsInfo()) continue;
         // Skip functions called from functions marked with stackcall attribute
         // Also skip the dummy kernel if one was created
-        if (hasStackCallInCG(&func, *ctx) || IGC::isIntelSymbolTableVoidProgram(&func))
+        if (hasStackCallInCG(&func) || IGC::isIntelSymbolTableVoidProgram(&func))
         {
             FunctionInfoMetaDataHandle funcInfo = m_pMdUtils->getFunctionsInfoItem(&func);
             funcInfo->clearImplicitArgInfoList();
@@ -176,26 +176,17 @@ bool AddImplicitArgs::runOnModule(Module &M)
     return true;
 }
 
-bool AddImplicitArgs::hasStackCallInCG(
-    const Function* F, const CodeGenContext& Ctx)
+bool AddImplicitArgs::hasStackCallInCG(const Function* F)
 {
-    if (F->hasFnAttribute("visaStackCall"))
+    if (F->hasFnAttribute("visaStackCall") || F->hasFnAttribute("referenced-indirectly"))
         return true;
 
-    if (F->hasFnAttribute("referenced-indirectly"))
-    {
-        auto* MD = Ctx.getModuleMetaData();
-        auto I = MD->FuncMD.find(const_cast<Function*>(F));
-        if (I == MD->FuncMD.end() || !isContinuation(I->second))
-            return true;
-    }
-
     for (auto u = F->user_begin(), e = F->user_end(); u != e; u++)
     {
         if (const CallInst* call = dyn_cast<CallInst>(*u))
         {
             const Function* parent = call->getParent()->getParent();
-            if (hasStackCallInCG(parent, Ctx))
+            if (parent != F && hasStackCallInCG(parent))
                 return true;
         }
     }
@@ -424,9 +415,7 @@ void AddImplicitArgs::replaceAllUsesWithNewOCLBuiltinFunction(llvm::Function* ol
 
                 // tracing it on parent function argument list
                 Value* callArg = ValueTracker::track(cInst, argImpToExpNum[&(*new_arg_iter)]);
-                Argument* arg = dyn_cast<Argument>(callArg);
-
-                IGC_ASSERT_MESSAGE(arg, "Not supported");
+                Argument* arg = cast<Argument>(callArg);
 
                 // build info
 
@@ -554,8 +543,8 @@ bool BuiltinCallGraphAnalysis::pruneCallGraphForStackCalls(CallGraph& CG)
         }
     }
 
-    if (IGC_IS_FLAG_ENABLED(PrintStackCallDebugInfo))
-        std::cout << std::endl << "Force Inlined Functions via ForceInlineStackCallWithImplArg:" << std::endl;
+    if (IGC_IS_FLAG_ENABLED(PrintStackCallDebugInfo) && !PrunedFuncs.empty())
+        dbgs() << "\nForce Inlined Functions via ForceInlineStackCallWithImplArg:\n";
 
     for (auto pF : PrunedFuncs)
     {
@@ -584,8 +573,7 @@ bool BuiltinCallGraphAnalysis::pruneCallGraphForStackCalls(CallGraph& CG)
         pF->addFnAttr(llvm::Attribute::AlwaysInline);
 
         if (IGC_IS_FLAG_ENABLED(PrintStackCallDebugInfo))
-            std::cout << pF->getName().str() << std::endl;
-
+            dbgs() << pF->getName().str() << "\n";
         changed = true;
     }
     return changed;
