@@ -29,6 +29,7 @@ SPDX-License-Identifier: MIT
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvm/IR/Module.h"
 #include "llvm/IR/InstIterator.h"
+#include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Analysis/PostDominators.h"
 #include "llvm/IR/Dominators.h"
 #include "llvm/IR/Verifier.h"
@@ -96,10 +97,11 @@ bool DivergentBarrierPass::hasDivergentBarrier(
 
     PostDominatorTree PDT(*F);
     DominatorTree DT(*F);
+    LoopInfo LI(DT);
 
     TranslationTable TT;
     TT.run(*F);
-    WIAnalysisRunner WI(F, &DT, &PDT, m_MDUtils, m_CGCtx, ModMD, &TT);
+    WIAnalysisRunner WI(F, &LI, &DT, &PDT, m_MDUtils, m_CGCtx, ModMD, &TT);
     WI.run();
 
     return llvm::any_of(Barriers, [&](Instruction* I) {
@@ -491,18 +493,22 @@ bool DivergentBarrierPass::processShader(Function* F)
     // so no need to add the divergent barrier WA.
     bool skipTGbarriers = false;
     unsigned forcedSimdSize = 0;
-    if (IGC_IS_FLAG_ENABLED(ForceCSSIMD32) ||
-        m_CGCtx->getModuleMetaData()->csInfo.waveSize == 32 ||
-        m_CGCtx->getModuleMetaData()->csInfo.forcedSIMDSize == 32)
+    if (IGC_IS_FLAG_ENABLED(ForceCSSIMD32))
     {
         forcedSimdSize = 32;
     }
-    else if (IGC_IS_FLAG_ENABLED(ForceCSSIMD16) ||
-        m_CGCtx->getModuleMetaData()->csInfo.waveSize == 16 ||
-        m_CGCtx->getModuleMetaData()->csInfo.forcedSIMDSize == 16)
+    else if (IGC_IS_FLAG_ENABLED(ForceCSSIMD16))
     {
         forcedSimdSize = 16;
     }
+    else if (m_CGCtx->getModuleMetaData()->csInfo.forcedSIMDSize >= 16)
+    {
+        forcedSimdSize = m_CGCtx->getModuleMetaData()->csInfo.forcedSIMDSize;
+    }
+    else if (m_CGCtx->getModuleMetaData()->csInfo.waveSize >= 16)
+    {
+        forcedSimdSize = m_CGCtx->getModuleMetaData()->csInfo.waveSize;
+    }
     ConstantInt* groupSize = dyn_cast<ConstantInt>(getGroupSize(*F));
     if (groupSize &&
         forcedSimdSize >= groupSize->getValue().getZExtValue())
