@@ -50,6 +50,12 @@ public:
     // Set the GfxCoreFamily as the specified value.
     void setGfxCoreFamily(GFXCORE_FAMILY value);
 
+    // Set VISA ABI version used in generated code.
+    void setVISAABIVersion(unsigned int ver);
+
+    // Set the GmdId as the specified value.
+    void setGmdID(GFX_GMD_ID value);
+
     // Pair of name for the section (1st elem) and VISA asm text (2nd elem).
     using NamedVISAAsm = std::pair<std::string, std::string>;
 
@@ -83,6 +89,9 @@ public:
 
     void printBinaryObject(const std::string& filename);
 
+    // print .ze_info into a file with given filename
+    void printZEInfo(const std::string& filename);
+
 private:
     /// ------------ program scope helper functions ------------
     /// add program scope information. This function will be called in the ctor.
