@@ -48,7 +48,7 @@ THE SOFTWARE.
 #include "common/LLVMWarningsPush.hpp"
 
 #include "llvmWrapper/IR/DerivedTypes.h"
-#include "llvmWrapper/Bitcode/BitcodeWriter.h"
+#include "llvm/Bitcode/BitcodeWriter.h"
 #include "llvm/IR/Attributes.h"
 #include "llvm/Support/ToolOutputFile.h"
 
@@ -72,7 +72,7 @@ saveLLVMModule(Module *M, const std::string &OutputFile) {
   std::error_code EC;
   llvm::ToolOutputFile Out(OutputFile.c_str(), EC, sys::fs::OF_None);
   IGC_ASSERT_EXIT_MESSAGE((!EC), "Failed to open file");
-  IGCLLVM::WriteBitcodeToFile(M, Out.os());
+  llvm::WriteBitcodeToFile(*M, Out.os());
   Out.keep();
 }
 
@@ -152,7 +152,7 @@ std::string recursive_mangle(const Type* pType)
         case Type::PointerTyID:
         {
             unsigned int AS = pType->getPointerAddressSpace();
-            Type* pPointeeType = pType->getPointerElementType();
+            Type* pPointeeType = IGCLLVM::getNonOpaquePtrEltTy(pType);
 
             StructType* ST = dyn_cast<StructType>(pPointeeType);
             if (ST && ST->isOpaque())
@@ -205,6 +205,15 @@ std::string recursive_mangle(const Type* pType)
                 if (isPipe_wo) return "Pipe_wo";
                 bool isReserveId = structName.startswith(std::string(kSPIRVTypeName::PrefixAndDelim) + std::string(kSPIRVTypeName::ReserveId));
                 if (isReserveId) return "ReserveId";
+                bool isJointMatrix = structName.startswith(std::string(kJointMatrixName::TypePrefix));
+                if (isJointMatrix)
+                {
+                    bool found = structName.consume_front(kJointMatrixName::TypePrefix);
+                    IGC_ASSERT_MESSAGE(found, "Inconsistent Joint Matrix type!");
+                    found = structName.consume_back(kJointMatrixName::TypeSuffix);
+                    IGC_ASSERT_MESSAGE(found, "Inconsistent Joint Matrix type!");
+                    return structName.str();
+                }
                 return "i64";
             }
 
@@ -431,7 +440,7 @@ std::string getSPIRVImageSampledTypeName(SPIRVType* Ty) {
 
 bool isSPIRVSamplerType(llvm::Type* Ty) {
   if (auto PT = dyn_cast<PointerType>(Ty))
-    if (auto ST = dyn_cast<StructType>(PT->getPointerElementType()))
+    if (auto ST = dyn_cast<StructType>(IGCLLVM::getNonOpaquePtrEltTy(PT)))
       if (ST->isOpaque()) {
         auto Name = ST->getName();
         if (Name.startswith(std::string(kSPIRVTypeName::PrefixAndDelim) + kSPIRVTypeName::Sampler)) {
