@@ -64,6 +64,8 @@ SPIRVEntry::create(Op OpCode) {
 #define _SPIRV_OP(x,...) case Op##x: return igc_spv::create<SPIRV##x>();
 #include "SPIRVOpCodeEnum.h"
 #undef _SPIRV_OP
+  case OpTypeJointMatrixINTEL_OLD: // Remove once OpTypeJointMatrixINTEL_OLD is removed
+    return new SPIRVTypeJointMatrixINTEL(OpTypeJointMatrixINTEL_OLD);
   default:
     break;
   }
@@ -336,6 +338,14 @@ SPIRVEntry::getDecorations(Decoration Kind) const {
   return Decors;
 }
 
+std::vector<SPIRVDecorate const*> SPIRVEntry::getDecorations() const {
+    std::vector<SPIRVDecorate const*> Decors;
+    Decors.reserve(Decorates.size());
+    for (auto& DecoPair : Decorates)
+        Decors.push_back(DecoPair.second);
+    return Decors;
+}
+
 std::vector<SPIRVId>
 SPIRVEntry::getDecorationIdLiterals(Decoration Kind) const {
     auto Loc = DecorateIds.find(Kind);
@@ -427,16 +437,19 @@ operator>>(std::istream &I, SPIRVEntry &E) {
 
 SPIRVEntryPoint::SPIRVEntryPoint(SPIRVModule *TheModule,
   SPIRVExecutionModelKind TheExecModel, SPIRVId TheId,
-  const std::string &TheName)
+  const std::string &TheName, std::vector<SPIRVId> Variables)
   :SPIRVAnnotation(TheModule->get<SPIRVFunction>(TheId),
-   getSizeInWords(TheName) + 3), ExecModel(TheExecModel), Name(TheName){
+    getSizeInWords(TheName) + Variables.size() + 3), ExecModel(TheExecModel),
+    Name(TheName), Variables(Variables) {
 }
 
 void
 SPIRVEntryPoint::decode(std::istream &I) {
   getDecoder(I) >> ExecModel >> Target >> Name;
+  Variables.resize(WordCount - FixedWC - getSizeInWords(Name) + 1);
+  getDecoder(I) >> Variables;
   Module->setName(getOrCreateTarget(), Name);
-  Module->addEntryPoint(ExecModel, Target);
+  Module->addEntryPoint(ExecModel, Target, Name, Variables);
 }
 
 void
