@@ -1057,6 +1057,7 @@ _SPIRV_OP(Any)
 _SPIRV_OP(All)
 _SPIRV_OP(ConvertFToBF16INTEL)
 _SPIRV_OP(ConvertBF16ToFINTEL)
+_SPIRV_OP(RoundFToTF32INTEL)
 _SPIRV_OP(ArithmeticFenceINTEL)
 _SPIRV_OP(BitReverse)
 #undef _SPIRV_OP
@@ -2342,6 +2343,7 @@ _SPIRV_OP(JointMatrixSUMad, true, 6)
 _SPIRV_OP(JointMatrixUSMad, true, 6)
 _SPIRV_OP(JointMatrixUUMad, true, 6)
 _SPIRV_OP(JointMatrixWorkItemLength, true, 4)
+_SPIRV_OP(JointMatrixGetElementCoord, true, 5)
 #undef _SPIRV_OP
 }
 #endif // SPIRVINSTRUCTION_HPP_
