@@ -20,6 +20,13 @@ SPDX-License-Identifier: MIT
 #include <string>
 #include <queue>
 
+#define VISIT_FUNCTION_IMPL(InstructionType) \
+    void visit##InstructionType(llvm::InstructionType& instruction) \
+    { \
+        changed |= !wasPromoted(instruction.getFunction()) \
+                   && getOrCreatePromotedValue(&instruction) != &instruction; \
+    }
+
 namespace IGC
 {
     class PromoteBools : public llvm::ModulePass, public llvm::InstVisitor<PromoteBools>
@@ -37,15 +44,19 @@ namespace IGC
 
         virtual bool runOnModule(llvm::Module& module) override;
 
-        void visitAllocaInst(llvm::AllocaInst& alloca);
-        void visitCallInst(llvm::CallInst& call);
-        void visitLoadInst(llvm::LoadInst& load);
-        void visitStoreInst(llvm::StoreInst& store);
+        VISIT_FUNCTION_IMPL(AllocaInst)
+        VISIT_FUNCTION_IMPL(CallInst)
+        VISIT_FUNCTION_IMPL(GetElementPtrInst)
+        VISIT_FUNCTION_IMPL(LoadInst)
+        VISIT_FUNCTION_IMPL(StoreInst)
+        VISIT_FUNCTION_IMPL(ICmpInst)
 
     private:
         bool changed;
 
-        llvm::Value* createZextIfNeeded(llvm::Value* argument, llvm::Instruction* insertBefore);
+        llvm::Value* convertI1ToI8(llvm::Value* argument, llvm::Instruction* insertBefore);
+        llvm::Value* convertI8ToI1(llvm::Value* argument, llvm::Instruction* insertBefore);
+        llvm::Value* castTo(llvm::Value* value, llvm::Type* desiredType, llvm::Instruction* insertBefore);
         void cleanUp(llvm::Module& module);
 
         // Checking if type needs promotion
@@ -57,10 +68,18 @@ namespace IGC
 
         // Promoting values
         llvm::DenseMap<llvm::Value*, llvm::Value*> promotedValuesCache;
-        llvm::DenseSet<llvm::Value*> doNotRemove;
         std::queue<llvm::Value*> promotionQueue;
-        llvm::Value* getOrCreatePromotedValue(llvm::Value* value);
         bool wasPromoted(llvm::Value* value);
+
+        template <typename Range>
+        bool wasPromotedAnyOf(const Range& range)
+        {
+            return std::any_of(std::begin(range), std::end(range), [this](const auto& item) {
+                return wasPromoted(item);
+            });
+        }
+
+        llvm::Value* getOrCreatePromotedValue(llvm::Value* value);
         llvm::Function* promoteFunction(llvm::Function* function);
         llvm::GlobalVariable* promoteGlobalVariable(llvm::GlobalVariable* globalVariable);
         llvm::Constant* promoteConstant(llvm::Constant* constant);
@@ -68,9 +87,18 @@ namespace IGC
         llvm::AddrSpaceCastInst* promoteAddrSpaceCast(llvm::AddrSpaceCastInst* addrSpaceCast);
         llvm::Value* promoteBitCast(llvm::BitCastInst* bitcast);
         llvm::CallInst* promoteCall(llvm::CallInst* call);
+        llvm::CallInst* promoteIndirectCallOrInlineAsm(llvm::CallInst* call);
         llvm::ExtractValueInst* promoteExtractValue(llvm::ExtractValueInst* extractValue);
+        llvm::GetElementPtrInst* promoteGetElementPtr(llvm::GetElementPtrInst* getElementPtr);
+        llvm::ICmpInst* promoteICmp(llvm::ICmpInst* icmp);
+        llvm::InlineAsm* promoteInlineAsm(llvm::InlineAsm* inlineAsm);
         llvm::InsertValueInst* promoteInsertValue(llvm::InsertValueInst* insertValue);
         llvm::LoadInst* promoteLoad(llvm::LoadInst* load);
+        llvm::PHINode* promotePHI(llvm::PHINode* phi);
         llvm::StoreInst* promoteStore(llvm::StoreInst* store);
+        llvm::IntToPtrInst* promoteIntToPtr(llvm::IntToPtrInst* inttoptr);
+
+        // Promoting values - helping vars
+        llvm::DenseSet<llvm::PHINode*> visitedPHINodes;
     };
 }
