@@ -130,8 +130,7 @@ bool BlendToDiscard::runOnFunction(Function& F)
     {
         for (auto& II : BI)
         {
-            GenIntrinsicInst* out = dyn_cast<GenIntrinsicInst>(
-                &II, GenISAIntrinsic::GenISA_OUTPUT);
+            GenIntrinsicInst* out = dyn_cast<GenIntrinsicInst>(&II, GetOutputPSIntrinsic(m_cgCtx->platform));
 
             if (out)
             {
@@ -338,7 +337,6 @@ bool BlendToDiscard::blendToDiscardMRT(
             cond = irb.CreateFCmpOEQ(alpha, f1);
             break;
 
-        case USC::BLEND_OPTIMIZATION_SRC_COLOR_ZERO_ALPHA_IGNORE:
         case USC::BLEND_OPTIMIZATION_SRC_COLOR_ZERO:
             // discard: src.rgb == 0
             cond = irb.CreateAllValuesAreZeroF(colors, nColors);
@@ -373,6 +371,17 @@ bool BlendToDiscard::blendToDiscardMRT(
                 irb.CreateAllValuesAreZeroF(colors, nColors));
             break;
 
+        case USC::BLEND_OPTIMIZATION_SRC_COLOR_ZERO_ALPHA_IGNORE:
+        {
+            // Discard: src.rgb == 0 and don't compute src.a
+            cond = irb.CreateAllValuesAreZeroF(colors, nColors);
+            Value* nAlpha = m_cgCtx->getModuleMetaData()->compOpt.EnableUndefAlphaOutputAsRed ?
+                outInst[i]->getOperand(0) : f0;
+
+            outInst[i]->setOperand(3, nAlpha);
+            break;
+        }
+
         default:
             IGC_ASSERT_MESSAGE(0, "Need to handle more cases");
             break;
