@@ -123,6 +123,7 @@ namespace IGC
         VISA_Align GetVISAAlign(CVariable* var);
 
         void SetAbortOnSpillThreshold(bool canAbortOnSpill, bool AllowSpill);
+        void SetMaxRegForThreadDispatch();
         void SetDispatchSimdSize();
         void SetSpillMemOffset();
         void SetStackFunctionArgSize(uint size);  // size in GRFs
@@ -260,15 +261,18 @@ namespace IGC
                             LSC_DATA_ELEMS numElems, unsigned blockOffset,
                             ResourceDescriptor *resource,
                             LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order,
-                            int immOffset, LSC_CACHE_OPTS cacheOpts);
+                            int immOffset, int immScale, LSC_CACHE_OPTS cacheOpts,
+                            LSC_DOC_ADDR_SPACE addrSpace);
         void LSC_StoreScatter(LSC_OP subOp,
                               CVariable *src, CVariable *offset,
                               LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
                               unsigned blockOffset,
                               ResourceDescriptor *resource,
                               LSC_ADDR_SIZE addr_size,
-                              LSC_DATA_ORDER data_order, int immOffset,
-                              LSC_CACHE_OPTS cacheOpts);
+                              LSC_DATA_ORDER data_order,
+                              int immOffset, int immScale,
+                              LSC_CACHE_OPTS cacheOpts,
+                              LSC_DOC_ADDR_SPACE addrSpace);
         void LSC_LoadBlock1D(
             CVariable* dst, CVariable* offset,
             LSC_DATA_SIZE elemSize, LSC_DATA_ELEMS numElems,
