@@ -14,6 +14,7 @@ SPDX-License-Identifier: MIT
 #include "DebugInfo/DwarfDebug.hpp"
 #include "DebugInfo/VISADebugInfo.hpp"
 #include "Compiler/CISACodeGen/DebugInfo.hpp"
+#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "llvm/IR/IntrinsicInst.h"
 
 using namespace llvm;
@@ -26,6 +27,27 @@ using namespace CLElfLib;
 char DebugInfoPass::ID = 0;
 char CatchAllLineNumber::ID = 0;
 
+
+
+// Register pass to igc-opt
+#define PASS_FLAG1 "igc-debug-finalize"
+#define PASS_DESCRIPTION1 "DebugInfo pass, llvmIR part(WAs)"
+#define PASS_CFG_ONLY1 false
+#define PASS_ANALYSIS1 false
+
+IGC_INITIALIZE_PASS_BEGIN(DebugInfoPass, PASS_FLAG1, PASS_DESCRIPTION1, PASS_CFG_ONLY1, PASS_ANALYSIS1)
+IGC_INITIALIZE_PASS_END(DebugInfoPass, PASS_FLAG1, PASS_DESCRIPTION1, PASS_CFG_ONLY1, PASS_ANALYSIS1)
+
+// Used for opt testing, could be removed if KernelShaderMap is moved to ctx.
+static CShaderProgram::KernelShaderMap KernelShaderMap;
+
+// Default ctor used for igc-opt testing
+DebugInfoPass::DebugInfoPass() :
+    ModulePass(ID),
+    kernels(KernelShaderMap)
+{ }
+
+
 DebugInfoPass::DebugInfoPass(CShaderProgram::KernelShaderMap& k) :
     ModulePass(ID),
     kernels(k)
@@ -49,6 +71,25 @@ bool DebugInfoPass::doFinalization(llvm::Module& M)
 
 bool DebugInfoPass::runOnModule(llvm::Module& M)
 {
+    // This loop is just a workaround till we add support for DIArgList metadata.
+    // If we implement DIArgList support, it should be deleted.
+#if LLVM_VERSION_MAJOR > 12
+    for (auto &F : M) {
+      for (auto &BB : F) {
+        for (auto &I : BB) {
+          if (auto *dbgInst = dyn_cast<DbgVariableIntrinsic>(&I)) {
+            if (dbgInst->getNumVariableLocationOps() > 1) {
+              dbgInst->setUndef();
+            }
+          }
+        }
+      }
+    }
+#endif
+    // Early out
+    if (kernels.empty())
+       return false;
+
     std::vector<CShader*> units;
 
     auto isCandidate = [](CShaderProgram* shaderProgram, SIMDMode m, ShaderDispatchMode mode = ShaderDispatchMode::NOT_APPLICABLE)
@@ -257,7 +298,14 @@ static void debugDump(const CShader* Shader, llvm::StringRef Ext,
         return;
 
     auto ExtStr = Ext.str();
-    std::string DumpName = IGC::Debug::GetDumpName(Shader, ExtStr.c_str());
+    IGC::Debug::DumpName DumpNameObj = IGC::Debug::GetDumpNameObj(Shader, ExtStr.c_str());
+    std::string DumpName = DumpNameObj.str();
+    if (IGC_IS_FLAG_ENABLED(DebugDumpNamePrefix))
+    {
+        auto hash = ShaderHash();
+        DumpNameObj = DumpNameObj.Hash(hash);
+        DumpName = DumpNameObj.AbsolutePath(IGC_GET_REGKEYSTRING(DebugDumpNamePrefix));
+    }
     FILE* const DumpFile = fopen(DumpName.c_str(), "wb+");
     if (nullptr == DumpFile)
         return;
@@ -618,7 +666,8 @@ CatchAllLineNumber::~CatchAllLineNumber()
 bool CatchAllLineNumber::runOnFunction(llvm::Function& F)
 {
     // Insert placeholder intrinsic instruction in each kernel/stack call function.
-    if (!F.getSubprogram() || F.isDeclaration())
+    if (!F.getSubprogram() || F.isDeclaration() ||
+      IGC_IS_FLAG_ENABLED(NoCatchAllDebugLine))
         return false;
 
     if (F.getCallingConv() != llvm::CallingConv::SPIR_KERNEL &&
