@@ -363,6 +363,16 @@ namespace IGC
 
         // Specifies alignment of indirect data
         virtual unsigned getCrossThreadDataAlignment() const { return 32; }
+
+        // Informs if LSC immediate global offset for A64 is supported
+        virtual bool supportsLSCImmediateGlobalBaseOffsetForA64() const { return true; }
+
+        // Informs if LSC immediate global offset for A32 is supported
+        virtual bool supportsLSCImmediateGlobalBaseOffsetForA32() const { return true; }
+
+        // If enabled IGC must not hoist convergent instructions.
+        virtual bool DisableConvergentInstructionsHoisting() const { return false; }
+
 protected:
     bool autoGRFSelection = false;
     };
