@@ -99,6 +99,7 @@ public:
     void Select(const SSource sources[3], const DstModifier& modifier);
     void PredAdd(const SSource& pred, bool invert, const SSource sources[2], const DstModifier& modifier);
     void Mul(const SSource[2], const DstModifier& modifier);
+    void Powi(const SSource[2], const DstModifier& modifier);
     void Mov(const SSource& source, const DstModifier& modifier);
     void Unary(e_opcode opCode, const SSource sources[1], const DstModifier& modifier);
     void Binary(e_opcode opCode, const SSource sources[2], const DstModifier& modifier);
@@ -173,12 +174,12 @@ public:
 
     // TODO: unify the functions below and clean up
     void emitStore(llvm::StoreInst *inst, llvm::Value *varOffset,
-                   llvm::ConstantInt *immOffset
+                   llvm::ConstantInt *immOffset, ConstantInt *immScale = nullptr
     );
     void emitStore3DInner(llvm::Value* pllValToStore, llvm::Value* pllDstPtr, llvm::Value* pllElmIdx);
 
     void emitLoad(llvm::LoadInst *inst, llvm::Value *varOffset,
-                  llvm::ConstantInt *immOffset
+                  llvm::ConstantInt *immOffset, ConstantInt *immScale = nullptr
     ); // single load, no pattern
     void emitLoad3DInner(llvm::LdRawIntrinsic* inst, ResourceDescriptor& resource, llvm::Value* elemIdxV);
 
@@ -427,12 +428,17 @@ public:
     void emitLSCVectorLoad(llvm::Instruction *Inst,
                            llvm::Value *Ptr,
                            llvm::Value *offset, llvm::ConstantInt *immOffset,
-                           LSC_CACHE_OPTS cacheOpts);
+                           ConstantInt *immScale,
+                           LSC_CACHE_OPTS cacheOpts,
+                           LSC_DOC_ADDR_SPACE addrSpace
+    );
     void emitLSCVectorStore(llvm::Value *Ptr,
                             llvm::Value *offset, llvm::ConstantInt *immOffset,
-                            llvm::Value *storedVal, llvm::BasicBlock* BB,
-                            LSC_CACHE_OPTS cacheOpts,
-                            alignment_t align, bool dontForceDMask);
+                            llvm::ConstantInt *immScale, llvm::Value *storedVal,
+                            llvm::BasicBlock *BB, LSC_CACHE_OPTS cacheOpts,
+                            alignment_t align, bool dontForceDMask,
+                            LSC_DOC_ADDR_SPACE addrSpace
+      );
     void emitUniformVectorCopy(CVariable* Dst, CVariable* Src, uint32_t nElts,
         uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0);
     void emitVectorCopy(CVariable* Dst, CVariable* Src, uint32_t nElts,
@@ -542,7 +548,8 @@ public:
         ResourceDescriptor* resource,
         LSC_ADDR_SIZE addr_size,
         LSC_DATA_ORDER data_order,
-        int immOffset);
+        int immOffset,
+        int immScale);
     void emitLSCLoad(
         LSC_CACHE_OPTS cacheOpts,
         CVariable* dst,
@@ -553,7 +560,9 @@ public:
         ResourceDescriptor* resource,
         LSC_ADDR_SIZE addr_size,
         LSC_DATA_ORDER data_order,
-        int immOffset);
+        int immOffset,
+        int immScale,
+        LSC_DOC_ADDR_SPACE addrSpace);
     void emitLSCStore(
         llvm::Instruction* inst,
         CVariable* src,
@@ -564,7 +573,8 @@ public:
         ResourceDescriptor* resource,
         LSC_ADDR_SIZE addr_size,
         LSC_DATA_ORDER data_order,
-        int immOffset);
+        int immOffset,
+        int immScale);
     void emitLSCStore(
         LSC_CACHE_OPTS cacheOpts,
         CVariable* src,
@@ -575,7 +585,9 @@ public:
         ResourceDescriptor* resource,
         LSC_ADDR_SIZE addr_size,
         LSC_DATA_ORDER data_order,
-        int immOffset);
+        int immOffset,
+        int immScale,
+        LSC_DOC_ADDR_SPACE addrSpace);
     ////////////////////////////////////////////////////////////////////
     // NOTE: for vector load/stores instructions pass the
     // optional instruction argument checks additional constraints
@@ -874,7 +886,12 @@ private:
     // If constant vector is stored and there is already var instance for it
     // try reusing it (if it was defined in the same basic block)
     // or create a new var instance and make it available for reusing in further stores
-    CVariable* tryReusingConstVectorStoreData(llvm::Value* storedVal, llvm::BasicBlock* BB, bool isBroadcast);
+    CVariable* tryReusingConstVectorStoreData(llvm::Value* storedVal,
+        llvm::BasicBlock* BB, bool isBroadcast);
+
+    CVariable* tryReusingXYZWPayload(llvm::Value* storedVal,
+        llvm::BasicBlock* BB, unsigned numElems, VISA_Type type, CVariable* pSrc_X,
+        CVariable* pSrc_Y, CVariable* pSrc_Z, CVariable* pSrc_W, const unsigned int numEltGRF);
 
     // Emit code in slice starting from (reverse) iterator I. Return the
     // iterator to the next pattern to emit.
@@ -934,26 +951,41 @@ private:
     CVariable *prepareDataForUniform(CVariable *DataVar, uint32_t ExecSz,
                                      e_alignment Align);
     // sub-function of vector load/store
-    void emitLSCVectorLoad_subDW(LSC_CACHE_OPTS cacheOpts, bool UseA32,
+    void emitLSCVectorLoad_subDW(LSC_CACHE_OPTS CacheOpts, bool UseA32,
                                  ResourceDescriptor &Resource, CVariable *Dest,
                                  CVariable *Offset, int ImmOffset,
-                                 uint32_t NumElts, uint32_t EltBytes);
-    void emitLSCVectorLoad_uniform(LSC_CACHE_OPTS cacheOpts, bool UseA32,
+                                 int ImmScale, uint32_t NumElts,
+                                 uint32_t EltBytes,
+                                 LSC_DOC_ADDR_SPACE AddrSpace,
+                                 LSC_ADDR_SIZE AddrSize
+    );
+    void emitLSCVectorLoad_uniform(LSC_CACHE_OPTS CacheOpts, bool UseA32,
                                    ResourceDescriptor &Resource,
-                                   CVariable *Dest, CVariable *Offset,
-                                   int ImmOffset, uint32_t NumElts,
+                                   CVariable *Dest,
+                                   CVariable *Offset, int ImmOffset,
+                                   int ImmScale, uint32_t NumElts,
                                    uint32_t EltBytes, uint64_t Align,
-                                   uint32_t Addrspace);
-    void emitLSCVectorStore_subDW(LSC_CACHE_OPTS cacheOpts, bool UseA32,
+                                   uint32_t Addrspace,
+                                   LSC_DOC_ADDR_SPACE UserAddrSpace,
+                                   LSC_ADDR_SIZE AddrSize
+      );
+    void emitLSCVectorStore_subDW(LSC_CACHE_OPTS CacheOpts, bool UseA32,
                                   ResourceDescriptor &Resource,
                                   CVariable *StoreVar, CVariable *Offset,
-                                  int ImmOffset, uint32_t NumElts,
-                                  uint32_t EltBytes, alignment_t Align);
-    void emitLSCVectorStore_uniform(LSC_CACHE_OPTS cacheOpts, bool UseA32,
+                                  int ImmOffset, int ImmScale, uint32_t NumElts,
+                                  uint32_t EltBytes, alignment_t Align,
+                                  LSC_DOC_ADDR_SPACE AddrSpace,
+                                  LSC_ADDR_SIZE AddrSize
+      );
+    void emitLSCVectorStore_uniform(LSC_CACHE_OPTS CacheOpts, bool UseA32,
                                     ResourceDescriptor &Resource,
                                     CVariable *StoreVar, CVariable *Offset,
-                                    int ImmOffset, uint32_t NumElts,
-                                    uint32_t EltBytes, alignment_t Align);
+                                    int ImmOffset, int ImmScale,
+                                    uint32_t NumElts, uint32_t EltBytes,
+                                    alignment_t Align,
+                                    LSC_DOC_ADDR_SPACE AddrSpace,
+                                    LSC_ADDR_SIZE AddrSize
+      );
     LSC_FENCE_OP getLSCMemoryFenceOp(bool IsGlobalMemFence, bool InvalidateL1,
                                      bool EvictL1) const;
 
