@@ -1924,8 +1924,21 @@ bool InstExpander::visitCall(CallInst& Call) {
     IGC_ASSERT(nullptr != Emu);
 
     const Function* F = Call.getCalledFunction();
+    bool doInt64BitCall = Emu->isInt64(&Call);
+    if (!doInt64BitCall) {
+        for (auto& Op : Call.operands()) {
+            if (Emu->isInt64(Op.get())) {
+                doInt64BitCall = true;
+                break;
+            }
+        }
+    }
+    if (!doInt64BitCall) {
+        return false;
+    }
     if (F && F->isDeclaration()) {
-        switch (F->getIntrinsicID()) {
+        Intrinsic::ID IntrID = F->getIntrinsicID();
+        switch (IntrID) {
         default:
             break;
             // Ignore the following intrinsics in CG.
@@ -1945,9 +1958,6 @@ bool InstExpander::visitCall(CallInst& Call) {
         // emulate @llvm.abs.i64
         case Intrinsic::abs:
         {
-            if (!Emu->isInt64(&Call))
-                return false;
-
             Value* OldVal = Call.getArgOperand(0);
             Value* Lo = nullptr, * Hi = nullptr;
             std::tie(Lo, Hi) = Emu->getExpandedValues(OldVal);
@@ -1967,23 +1977,61 @@ bool InstExpander::visitCall(CallInst& Call) {
             Emu->setExpandedValues(&Call, SelectLo, SelectHo);
             return true;
         }
+        // emulate LLVM min/max intrinsics
+        case Intrinsic::smax:
+        case Intrinsic::smin:
+        case Intrinsic::umax:
+        case Intrinsic::umin:
+        {
+            // The least significant halves' comparison is dependent on that
+            // for the most significant halves, so we gain nothing by lowering
+            // this into i32 min/max calls. Basic cmp/sel sequence should
+            // suffice
+            const DenseMap<Intrinsic::ID, CmpInst::Predicate> CmpPredMap {
+                {Intrinsic::smax, CmpInst::Predicate::ICMP_SGT},
+                {Intrinsic::smin, CmpInst::Predicate::ICMP_SLT},
+                {Intrinsic::umax, CmpInst::Predicate::ICMP_UGT},
+                {Intrinsic::umin, CmpInst::Predicate::ICMP_ULT}
+            };
+            Value* LHS = Call.getArgOperand(0), * RHS = Call.getArgOperand(1);
+            // FIXME: Note that we aren't producing expanded/emulated values
+            // here, but rather replacing the call uses with the result of a
+            // newly generated i64 instruction. To make that work, 2 criteria
+            // should be satisfied from the perspective of Emu64Ops::expandInsts
+            // algorithm:
+            // 1. Inst-over-BB iterators cannot be invalidated
+            // 2. Due to averse inst-over-BB iteration order, the cmp/sel
+            //    sequence must be inserted after the current min/max call,
+            //    before its first use - regardless of the fact that the call
+            //    itself will be unlinked from those uses and marked for
+            //    deletion.
+            // For 1, we're entirely relying on IRBuilder's internal validation
+            // of instruction numbering within the BB. For 2, we're basically
+            // exploiting the knowledge that the inst-over-BB iteration in the
+            // parent method strictly heeds the averse order.
+            // TODO: Instead of hacking the iteration logic from within the
+            // helper InstExpander method, we should encapsulate this use-case
+            // (inserting new i64 insts into the emulation queue) at the
+            // Emu64Ops class level. One of the options is implementing a util
+            // akin to LLVM's InstructionWorklist, which would support averse
+            // iteration order and handle the deferred instructions upon their
+            // creation. Such a worklist class might have its use in a broader
+            // set of IGC passes, hence implementing this a "global" IGC util
+            // could be an idea.
+            IRB->SetInsertPoint(&*std::next(BasicBlock::iterator(Call)));
+            auto* Cmp = cast<Instruction>(
+                IRB->CreateICmp(CmpPredMap.lookup(IntrID), LHS, RHS));
+            Call.replaceAllUsesWith(IRB->CreateSelect(Cmp, LHS, RHS));
+            return true;
+        }
 #endif
         }
     }
-    bool doInt64BitCall = Emu->isInt64(&Call);
-    if (!doInt64BitCall) {
-        for (auto& Op : Call.operands()) {
-            if (Emu->isInt64(Op.get())) {
-                doInt64BitCall = true;
-                break;
-            }
-        }
-    }
-    if (!doInt64BitCall) {
-        return false;
-    }
 
     // Recreate Call with its operands/result emulated
+    // TODO: Investigate whether we should replace the call with two
+    // i32-operating calls for Lo and Hi instead (at least for certain
+    // intrinsics)
     auto* CallCopy = Call.clone();
     IGC_ASSERT(nullptr != CallCopy);
     CallCopy->insertBefore(&Call);
