@@ -19,9 +19,9 @@ SPDX-License-Identifier: MIT
 #include "llvm/Config/llvm-config.h"
 #include "llvm/IR/Argument.h"
 #include "llvmWrapper/IR/Attributes.h"
-#include "llvmWrapper/Analysis/InlineCost.h"
 #include "llvm/ADT/SetVector.h"
 #include "llvm/ADT/SCCIterator.h"
+#include "llvm/Analysis/InlineCost.h"
 #include "llvm/Analysis/ValueTracking.h"
 #include "llvmWrapper/IR/CallSite.h"
 #include "llvm/IR/Module.h"
@@ -37,6 +37,7 @@ SPDX-License-Identifier: MIT
 #include "DebugInfo/VISADebugEmitter.hpp"
 #include <numeric>
 #include <utility>
+#include <iostream>
 #include "Probe/Assertion.h"
 
 using namespace llvm;
@@ -108,15 +109,16 @@ static inline void CloneFuncMetadata(IGCMD::MetaDataUtils* pM,
     pM->setFunctionsInfoItem(ClonedF, Info);
 }
 
-Function* GenXCodeGenModule::cloneFunc(Function* F)
+static Function* cloneFunc(IGCMD::MetaDataUtils* pM, Function* F, string prefix = "", string postfix = "_GenXClone")
 {
     ValueToValueMapTy VMap;
 
     Function* ClonedFunc = CloneFunction(F, VMap);
+    ClonedFunc->setName(prefix + F->getName().str() + postfix);
     //if the function is not added as part of clone, add it
     if (!F->getParent()->getFunction(ClonedFunc->getName()))
         F->getParent()->getFunctionList().push_back(ClonedFunc);
-    CloneFuncMetadata(pMdUtils, ClonedFunc, F);
+    CloneFuncMetadata(pM, ClonedFunc, F);
 
     return ClonedFunc;
 }
@@ -138,9 +140,26 @@ void GenXCodeGenModule::processFunction(Function& F)
 {
     // See what FunctionGroups this Function is called from.
     SetVector<std::pair<FunctionGroup*, Function*>> CallerFGs;
-    for (auto U : F.users())
+
+    std::vector<llvm::Function*> Callers;
+    if (IGC_IS_FLAG_ENABLED(StackOverflowDetection)) {
+        if (F.getName().equals("__stackoverflow_detection")) {
+          // Mark all stack calls as users of this detection function.
+          // It will be used as a subroutine, so it needs to be cloned for
+          // each of stack call functions.
+          for (auto& Func : F.getParent()->getFunctionList()) {
+            if (Func.hasFnAttribute("visaStackCall")) {
+              Callers.push_back(&Func);
+            }
+          }
+        }
+    }
+    for (auto U : F.users()) {
+        Callers.push_back(getCallerFunc(U));
+    }
+
+    for (auto Caller : Callers)
     {
-        Function* Caller = getCallerFunc(U);
         FunctionGroup* FG = FGA->getGroup(Caller);
         Function* SubGrpH = FGA->useStackCall(&F) ? (&F) : FGA->getSubGroupMap(Caller);
         if (FG == nullptr || SubGrpH == nullptr)
@@ -150,15 +169,84 @@ void GenXCodeGenModule::processFunction(Function& F)
 
     IGC_ASSERT(CallerFGs.size() >= 1);
 
-    // Make the function indirect if cloning exceeds the threshold
-    if (F.hasFnAttribute("visaStackCall") &&
-        m_FunctionCloningThreshold > 0 &&
-        CallerFGs.size() > m_FunctionCloningThreshold)
+    auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+
+    auto CanMakeIndirectFunc = [&](llvm::Function* F)->bool
+    {
+        // Don't convert subroutines, builtins, or invoke_simd_target
+        if (F->hasFnAttribute("visaStackCall") == false ||
+            F->hasFnAttribute("OclBuiltin") ||
+            F->hasFnAttribute("invoke_simd_target"))
+            return false;
+
+        // If SIMD Variant Compilation is not enabled, we have to make sure all callers
+        // have the same SIMD sizes, otherwise we cannot make it an indirect call
+        int simd_size = 0;
+        unsigned CallersPerSIMD[3] = { 0, 0, 0 };
+        for (auto iter : CallerFGs)
+        {
+            Function* callerKernel = iter.first->getHead();
+            auto funcInfoMD = pMdUtils->getFunctionsInfoItem(callerKernel);
+            int sz = funcInfoMD->getSubGroupSize()->getSIMD_size();
+            if (sz != 0) {
+                if (simd_size == 0)
+                    simd_size = sz;
+                CallersPerSIMD[(sz >> 4)]++;
+            }
+
+            // Callers have varying SIMD size requirements, do not promote
+            if (IGC_IS_FLAG_DISABLED(EnableSIMDVariantCompilation) && simd_size != sz)
+                return false;
+        }
+        if (IGC_IS_FLAG_ENABLED(EnableSIMDVariantCompilation))
+        {
+            // For MultiSIMD compile, check profitability.
+            // Since we need to clone to each SIMD variant kernel, only promote if the
+            // number of FGs per SIMD is greater than the threshold.
+            if (m_FunctionCloningThreshold > 0 &&
+                CallersPerSIMD[0] <= m_FunctionCloningThreshold &&
+                CallersPerSIMD[1] <= m_FunctionCloningThreshold &&
+                CallersPerSIMD[2] <= m_FunctionCloningThreshold)
+            {
+                return false;
+            }
+        }
+
+        // If CallWA is needed, we should not convert to indirect call when requiring SIMD32,
+        // as we can potentially avoid CallWA if there are no indirect calls.
+        if (pCtx->platform.requireCallWA() /* && simd_size == 32*/)
+        {
+            return false;
+        }
+
+        // Can't make indirect if threadgroupbarrier intrinsic is set
+        for (inst_iterator I = inst_begin(F), E = inst_end(F); I != E; ++I)
+        {
+            if (auto* GII = dyn_cast<GenIntrinsicInst>(&*I))
+            {
+                if (GII->getIntrinsicID() == GenISAIntrinsic::GenISA_threadgroupbarrier)
+                {
+                    return false;
+                }
+            }
+        }
+        return true;
+    };
+
+    // Make the function indirect if cloning is required to decrease compile time
+    if (m_FunctionCloningThreshold > 0 &&
+        CallerFGs.size() > m_FunctionCloningThreshold &&
+        CanMakeIndirectFunc(&F))
     {
         auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
         auto IFG = FGA->getOrCreateIndirectCallGroup(F.getParent());
         if (IFG)
         {
+            if (IGC_IS_FLAG_ENABLED(PrintStackCallDebugInfo))
+            {
+                dbgs() << "Make Indirect: " << F.getName().str() << "\n";
+            }
             F.addFnAttr("referenced-indirectly");
             pCtx->m_enableFunctionPointer = true;
             FGA->addToFunctionGroup(&F, IFG, &F);
@@ -177,7 +265,7 @@ void GenXCodeGenModule::processFunction(Function& F)
         else
         {
             // clone the function, add to this function group
-            auto FCloned = cloneFunc(&F);
+            auto FCloned = cloneFunc(pMdUtils, &F);
 
             // Copy F's property over
             copyFuncProperties(FCloned, &F);
@@ -227,25 +315,6 @@ void GenXCodeGenModule::processSCC(std::vector<llvm::CallGraphNode*>* SCCNodes)
     }
     IGC_ASSERT(CallerFGs.size() >= 1);
 
-    // To prevent cloning if it exceeds the threshold, make every function in the SCC an indirect call
-    if (m_FunctionCloningThreshold > 0 && CallerFGs.size() > m_FunctionCloningThreshold)
-    {
-        auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
-        auto Mod = (*SCCNodes).front()->getFunction()->getParent();
-        auto IFG = FGA->getOrCreateIndirectCallGroup(Mod);
-        if (IFG)
-        {
-            for (CallGraphNode* Node : (*SCCNodes))
-            {
-                Function* F = Node->getFunction();
-                F->addFnAttr("referenced-indirectly");
-                pCtx->m_enableFunctionPointer = true;
-                FGA->addToFunctionGroup(F, IFG, F);
-            }
-            return;
-        }
-    }
-
     bool FirstPair = true;
     for (auto FG : CallerFGs)
     {
@@ -265,7 +334,7 @@ void GenXCodeGenModule::processSCC(std::vector<llvm::CallGraphNode*>* SCCNodes)
             for (CallGraphNode* Node : (*SCCNodes))
             {
                 Function* F = Node->getFunction();
-                auto FCloned = cloneFunc(F);
+                auto FCloned = cloneFunc(pMdUtils, F);
 
                 // Copy properties
                 copyFuncProperties(FCloned, F);
@@ -363,7 +432,9 @@ static bool DeduceNonNullAttribute(Module& M)
             if (NotNull) {
                 // FIXME: Below lines possibly can be refactored to be simpler.
                 AttributeList attrSet = AttributeList::get(Arg.getParent()->getContext(), Arg.getArgNo() + 1, llvm::Attribute::NonNull);
-                Arg.addAttr(IGCLLVM::getAttribute(attrSet, Arg.getArgNo() + 1, llvm::Attribute::NonNull));
+                llvm::Attribute attr = IGCLLVM::getAttribute(attrSet, Arg.getArgNo() + 1, llvm::Attribute::NonNull);
+                IGC_ASSERT(IGCLLVM::hasParentContext(attr, Arg.getParent()->getContext()));
+                Arg.addAttr(attr);
                 Modifided = true;
             }
         }
@@ -383,15 +454,19 @@ bool GenXCodeGenModule::runOnModule(Module& M)
     // function groups can be cloned. If the number exceeds the threshold, instead of cloning the
     // function N times, make it an indirect call and use relocation instead. The function will only be
     // compiled once and runtime must relocate its address for each caller.
-    if (IGC_IS_FLAG_ENABLED(EnableZEBinary))
+    m_FunctionCloningThreshold = 0;
+    if (IGC_IS_FLAG_ENABLED(EnableFunctionCloningControl))
     {
-        // Enable by default for zebin
-        m_FunctionCloningThreshold = 1;
-    }
-    if (IGC_GET_FLAG_VALUE(FunctionCloningThreshold) != 0)
-    {
-        // Overwrite with debug flag
-        m_FunctionCloningThreshold = IGC_GET_FLAG_VALUE(FunctionCloningThreshold);
+        if (getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->enableZEBinary())
+        {
+            // Avoid cloning by default on zebin
+            m_FunctionCloningThreshold = 1;
+        }
+        if (IGC_GET_FLAG_VALUE(FunctionCloningThreshold) != 0)
+        {
+            // Overwrite with debug flag
+            m_FunctionCloningThreshold = IGC_GET_FLAG_VALUE(FunctionCloningThreshold);
+        }
     }
 
     pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
@@ -437,6 +512,9 @@ bool GenXCodeGenModule::runOnModule(Module& M)
         delete SCCNodes;
     }
 
+    // Clone indirect funcs if SIMD variants are required
+    FGA->CloneFunctionGroupForMultiSIMDCompile(&M);
+
     this->pMdUtils->save(M.getContext());
 
     // Check and set FG attribute flags
@@ -477,24 +555,6 @@ bool GenXCodeGenModule::runOnModule(Module& M)
         }
     }
 
-    auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
-    if (pCtx->platform.getMinDispatchMode() == SIMDMode::SIMD8)
-    {
-        // Changing simd size from 32 to 16 for function groups with function calls due to slicing
-        for (auto GI = FGA->begin(), GE = FGA->end(); GI != GE; ++GI)
-        {
-            FunctionGroup* FG = *GI;
-            if (!FG->isSingle() || FG->hasStackCall())
-            {
-                Function* Kernel = FG->getHead();
-                IGC::IGCMD::FunctionInfoMetaDataHandle funcInfoMD = pMdUtils->getFunctionsInfoItem(Kernel);
-                int simd_size = funcInfoMD->getSubGroupSize()->getSIMD_size();
-                if (simd_size == 32)
-                    funcInfoMD->getSubGroupSize()->setSIMD_size(16);
-            }
-        }
-    }
-
     IGC_ASSERT(FGA->verify());
 
     FGA->setModule(&M);
@@ -540,7 +600,8 @@ bool GenXFunctionGroupAnalysis::verify()
             for (auto FI = (*SubGI)->begin(), FE = (*SubGI)->end(); FI != FE; ++FI)
             {
                 Function* F = *FI;
-                if (F->hasFnAttribute("referenced-indirectly"))
+                if (F->hasFnAttribute("referenced-indirectly")
+                    )
                 {
                     continue;
                 }
@@ -569,29 +630,86 @@ bool GenXFunctionGroupAnalysis::verify()
     return true;
 }
 
-FunctionGroup* GenXFunctionGroupAnalysis::getOrCreateIndirectCallGroup(Module* pModule)
+// Returns the default SIMD size to compile Indirect Functions to
+static inline int getDefaultSIMDSize(CodeGenContext* ctx)
 {
-    if (IndirectCallGroup) return IndirectCallGroup;
-
-    auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
-
-    // Use the dummy kernel if it exists. Otherwise use the unique entry function.
-    // OCL shaders should always use the dummy kernel.
-    llvm::Function* defaultKernel = IGC::getIntelSymbolTableVoidProgram(pModule);
-    if (!defaultKernel && pCtx->type != ShaderType::OPENCL_SHADER)
+    if (ctx->getModuleMetaData()->csInfo.forcedSIMDSize != 0)
     {
-        defaultKernel = IGC::getUniqueEntryFunc(pCtx->getMetaDataUtils(), pCtx->getModuleMetaData());
+        // Forcing SIMD size
+        return ctx->getModuleMetaData()->csInfo.forcedSIMDSize;
     }
+
+    // By default, use the minimum allowed SIMD for the HW platform
+    int defaultSz = numLanes(ctx->platform.getMinDispatchMode());
+
+    // Library Compilation requires functions to be exported externally, so we should always compile to the default size.
+    // Otherwise, use the sub_group_size set for the dummy kernel (in InsertDummyKernelForSymbolTable pass) if available.
+    if (ctx->getModuleMetaData()->compOpt.IsLibraryCompilation == false)
+    {
+        Function* defaultDummyKernel = IGC::getIntelSymbolTableVoidProgram(ctx->getModule());
+        if (defaultDummyKernel)
+        {
+            auto subGrpSz = ctx->getMetaDataUtils()->getFunctionsInfoItem(defaultDummyKernel)->getSubGroupSize();
+            if (subGrpSz->isSIMD_sizeHasValue())
+                defaultSz = subGrpSz->getSIMD_size();
+        }
+    }
+    return defaultSz;
+}
+
+FunctionGroup* GenXFunctionGroupAnalysis::getOrCreateIndirectCallGroup(Module* pModule, int SimdSize)
+{
+    CodeGenContext* ctx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    auto pMdUtil = ctx->getMetaDataUtils();
+    auto pModMD = ctx->getModuleMetaData();
+    int defaultSimd = getDefaultSIMDSize(ctx);
+
+    if (SimdSize == 0)
+    {
+        SimdSize = defaultSimd;
+    }
+
+    IGC_ASSERT(SimdSize == 8 || SimdSize == 16 || SimdSize == 32);
+    int idx = SimdSize >> 4;
+
+    if (IndirectCallGroup[idx] != nullptr) return IndirectCallGroup[idx];
+
+    llvm::Function* defaultKernel = IGC::getIntelSymbolTableVoidProgram(pModule);
+
     // No default kernel found
     if (!defaultKernel) return nullptr;
 
-    IndirectCallGroup = getGroup(defaultKernel);
-    if (!IndirectCallGroup)
+    if (SimdSize != defaultSimd)
+    {
+        // Require a SIMD variant version of the dummy kernel.
+        // Create a new dummy kernel clone here to attach all functions with the required SimdSize.
+        std::string fName = std::string(IGC::INTEL_SYMBOL_TABLE_VOID_PROGRAM) + "_GenXSIMD" + std::to_string(SimdSize);
+        Function* pNewFunc = Function::Create(defaultKernel->getFunctionType(), GlobalValue::ExternalLinkage, fName, pModule);
+        BasicBlock* entry = BasicBlock::Create(pModule->getContext(), "entry", pNewFunc);
+        IRBuilder<> builder(entry);
+        builder.CreateRetVoid();
+
+        // Set spirv calling convention and kernel metadata
+        pNewFunc->setCallingConv(llvm::CallingConv::SPIR_KERNEL);
+        IGCMD::FunctionInfoMetaDataHandle fHandle = IGCMD::FunctionInfoMetaDataHandle(IGCMD::FunctionInfoMetaData::get());
+        FunctionMetaData* funcMD = &pModMD->FuncMD[pNewFunc];
+        funcMD->functionType = IGC::FunctionTypeMD::KernelFunction;
+        fHandle->setType(FunctionTypeMD::KernelFunction);
+        pMdUtil->setFunctionsInfoItem(pNewFunc, fHandle);
+        defaultKernel = pNewFunc;
+    }
+    // Set the requested sub_group_size value for this kernel
+    pMdUtil->getFunctionsInfoItem(defaultKernel)->getSubGroupSize()->setSIMD_size(SimdSize);
+    pMdUtil->save(pModule->getContext());
+
+    auto FG = getGroup(defaultKernel);
+    if (!FG)
     {
         setSubGroupMap(defaultKernel, defaultKernel);
-        IndirectCallGroup = createFunctionGroup(defaultKernel);
+        FG = createFunctionGroup(defaultKernel);
     }
-    return IndirectCallGroup;
+    IndirectCallGroup[idx] = FG;
+    return FG;
 }
 
 bool GenXFunctionGroupAnalysis::useStackCall(llvm::Function* F)
@@ -601,18 +719,41 @@ bool GenXFunctionGroupAnalysis::useStackCall(llvm::Function* F)
 
 void GenXFunctionGroupAnalysis::setGroupAttributes()
 {
+    auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+
     for (auto FG : Groups)
     {
+        if (isIndirectCallGroup(FG))
+        {
+            // The dummy kernel group is not a true function group, in that the functions in this group does not have
+            // a valid callgraph that connects them. It's a dummy group where all indirectly called functions are contained.
+            // Therefore, the group attributes are not valid here, since they are not connected to the real groupHead, which
+            // is the caller kernel. We don't set any of the FG attribute flags for this group.
+            //
+            // Note, indirect functions in this group can still directly call stackcalls or subroutines, which may also belong
+            // to this group due to cloning. However we still can't associate all functions in this group with a single callgraph.
+            continue;
+        }
+
         for (const Function* F : *FG)
         {
-            // Ignore indirect functions
             if (F->hasFnAttribute("referenced-indirectly"))
             {
+                IGC_ASSERT_MESSAGE(0, "Indirectly referenced function not moved to IndirectCallGroup!");
                 continue;
             }
-            else if (F->hasFnAttribute("visaStackCall"))
+
+            if (F->hasFnAttribute("visaStackCall"))
             {
                 FG->m_hasStackCall = true;
+                if (!isLeafFunc(F))
+                {
+                    FG->m_hasNestedCall = true;
+                }
+            }
+            else if (!isEntryFunc(pMdUtils, F))
+            {
+                FG->m_hasSubroutine = true;
             }
 
             // check all functions in the group to see if there's an vla alloca
@@ -635,30 +776,37 @@ void GenXFunctionGroupAnalysis::setGroupAttributes()
                 if (const CallInst* call = dyn_cast<CallInst>(&*ii))
                 {
                     Function* calledF = call->getCalledFunction();
+                    bool hasStackCall = calledF && calledF->hasFnAttribute("visaStackCall");
                     if (call->isInlineAsm())
                     {
                         // Uses inline asm call
                         FG->m_hasInlineAsm = true;
                     }
-                    else if (calledF && calledF->hasFnAttribute("referenced-indirectly"))
-                    {
-                        // This is the case where the callee has the "referenced-indirectly" attribute, but we still
-                        // see the callgraph. The callee may not belong to the same FG as the caller, but it still
-                        // counts as a stackcall.
-                        FG->m_hasStackCall = true;
-                    }
                     else if (!calledF || (calledF->isDeclaration() && calledF->hasFnAttribute("referenced-indirectly")))
                     {
                         // This is the true indirect call case, where either the callee's address is taken, or it belongs
-                        // to an external module. We do not know the callgraph in this case, so set the indirectcall flag.
-                        FG->m_hasStackCall = true;
+                        // to an external module. We do not know the callgraph in this case.
+                        hasStackCall = true;
+                        FG->m_hasIndirectCall = true;
+                        FG->m_hasPartialCallGraph = true;
+                    }
+                    else if (calledF && calledF->hasFnAttribute("referenced-indirectly"))
+                    {
+                        // This is the case where the callee has the "referenced-indirectly" attribute, but we still
+                        // see the callgraph. The callee may not belong to the same FG as the caller, but it's CG is still available.
+                        hasStackCall = true;
                         FG->m_hasIndirectCall = true;
                     }
                     else if (calledF && calledF->isDeclaration() && calledF->hasFnAttribute("invoke_simd_target"))
                     {
                         // Invoke_simd targets use stack call by convention.
-                        FG->m_hasStackCall = true;
+                        // Calling a func decl indicates unknown CG
+                        hasStackCall = true;
+                        FG->m_hasPartialCallGraph = true;
                     }
+
+                    FG->m_hasStackCall |= hasStackCall;
+                    FG->m_hasNestedCall |= (!isEntryFunc(pMdUtils, F) && hasStackCall);
                 }
             }
         }
@@ -676,7 +824,8 @@ void GenXFunctionGroupAnalysis::addIndirectFuncsToKernelGroup(llvm::Module* pMod
         Function* F = &(*I);
         if (F->isDeclaration() || isEntryFunc(pMdUtils, F)) continue;
 
-        if (F->hasFnAttribute("referenced-indirectly"))
+        if (F->hasFnAttribute("referenced-indirectly") ||
+            F->hasFnAttribute("variant-function-def"))
         {
             IGC_ASSERT(getGroup(F) == nullptr);
             indirectFunctions.push_back(F);
@@ -685,15 +834,168 @@ void GenXFunctionGroupAnalysis::addIndirectFuncsToKernelGroup(llvm::Module* pMod
     // Add them to the indirect call group
     if (!indirectFunctions.empty())
     {
-        auto IFG = getOrCreateIndirectCallGroup(pModule);
-        IGC_ASSERT(IFG);
+        FunctionGroup* IFG = nullptr;
         for (auto F : indirectFunctions)
         {
+            if (F->hasFnAttribute("variant-function-def"))
+            {
+                // If function variant is already created (meaning this is a rebuild), add them to the correct SIMD group
+                auto [symStr, fName, vecLen] = IGC::ParseVectorVariantFunctionString(F->getName().str());
+                IFG = getOrCreateIndirectCallGroup(pModule, vecLen);
+            }
+            else
+            {
+                IFG = getOrCreateIndirectCallGroup(pModule);
+            }
+            IGC_ASSERT(IFG);
             addToFunctionGroup(F, IFG, F);
         }
     }
 }
 
+void GenXFunctionGroupAnalysis::CloneFunctionGroupForMultiSIMDCompile(llvm::Module* pModule)
+{
+    CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+
+    // Don't clone if forcing SIMD size
+    if (pCtx->getModuleMetaData()->csInfo.forcedSIMDSize != 0)
+        return;
+
+    int default_size = getDefaultSIMDSize(pCtx);
+
+    // All "referenced-indirectly" funcs should already be added to the default group
+    FunctionGroup* ICG = getOrCreateIndirectCallGroup(pModule);
+    if (!ICG || ICG->isSingle())
+        return;
+
+    // For the first iteration, clone only functions called from other function groups
+    // that have multiple SIMD size requirements. This is to get a baseline of how many variant
+    // dummy kernels are required.
+    for (Function* F : *ICG)
+    {
+        // Check if there are multiple callers with variant SIMD size requirements.
+        int hasReqdSIMD = 0;
+        if (!F->hasFnAttribute("referenced-indirectly"))
+            continue;
+
+        // Assume that if a FG uses the address of a function in the IndirectCallGroup,
+        // then we should compile the IndirectCallGroup with the subgroup size of the user FG.
+        DenseMap<Instruction*, int> UsersMap;
+        for (auto U : F->users())
+        {
+            if (Instruction* inst = dyn_cast<Instruction>(U))
+            {
+                Function* callerF = inst->getParent()->getParent();
+                if (!isIndirectCallGroup(callerF))
+                {
+                    auto FG = getGroup(callerF);
+                    auto funcInfoMD = pMdUtils->getFunctionsInfoItem(FG->getHead());
+                    int sz = funcInfoMD->getSubGroupSize()->getSIMD_size();
+                    if (sz != 0)
+                    {
+                        IGC_ASSERT(sz == 8 || sz == 16 || sz == 32);
+                        hasReqdSIMD |= sz;
+                        UsersMap[inst] = sz;
+                    }
+                }
+            }
+        }
+
+        if (hasReqdSIMD > 0)
+        {
+            bool ReqMultipleSIMD = hasReqdSIMD != 8 && hasReqdSIMD != 16 && hasReqdSIMD != 32;
+            if (IGC_IS_FLAG_ENABLED(EnableSIMDVariantCompilation) && ReqMultipleSIMD)
+            {
+                for (int i = 0; i < 3; i++)
+                {
+                    int simdsz = 1 << (i + 3);
+                    if ((hasReqdSIMD & simdsz) && simdsz != default_size)
+                    {
+                        // Clone the function
+                        // Use the function vector variant syntax for mangling func name
+                        string prefix = "_ZGVxN" + std::to_string(simdsz) + "_";
+                        Function* FCloned = cloneFunc(pMdUtils, F, prefix, "");
+                        copyFuncProperties(FCloned, F);
+                        FCloned->addFnAttr("variant-function-def");
+                        auto pNewICG = getOrCreateIndirectCallGroup(pModule, simdsz);
+                        addToFunctionGroup(FCloned, pNewICG, FCloned);
+
+                        for (auto iter : UsersMap)
+                        {
+                            if (iter.second == simdsz)
+                            {
+                                iter.first->replaceUsesOfWith(F, FCloned);
+                            }
+                        }
+                    }
+                }
+            }
+            else
+            {
+                IGC_ASSERT_MESSAGE(!ReqMultipleSIMD, "SIMD variant compilation not supported");
+                continue;
+            }
+        }
+    }
+
+    auto hasIndirectCaller = [this](Function* F, FunctionGroup* ICG)->bool
+    {
+        for (auto U : F->users()) {
+            if (Instruction* I = dyn_cast<Instruction>(U)) {
+                Function* callerF = I->getParent()->getParent();
+                if (getGroup(callerF) == ICG)
+                    return true;
+            }
+        }
+        return false;
+    };
+
+    // In second iteration, clone functions that have callers inside the indirect call group.
+    // These functions may have been missed in the first iteration if they are not used outside
+    // the indirect call group, but need to be cloned to each variant dummy kernel we create.
+    for (Function* F : *ICG)
+    {
+        if (F == ICG->getHead() || !hasIndirectCaller(F, ICG))
+            continue;
+
+        for (int i = 0; i < 3; i++)
+        {
+            int simdsz = 1 << (i + 3);
+            auto ICG = IndirectCallGroup[i];
+            if (ICG && simdsz != default_size)
+            {
+                string prefix = "_ZGVxN" + std::to_string(simdsz) + "_";
+
+                // If mangled function already exist, don't re-clone it
+                if (pModule->getFunction(prefix + F->getName().str()) != nullptr)
+                    continue;
+
+                Function* FCloned = cloneFunc(pMdUtils, F, prefix, "");
+                copyFuncProperties(FCloned, F);
+                FCloned->addFnAttr("variant-function-def");
+
+                Function* SubGrpH = useStackCall(F) ? FCloned : ICG->getHead();
+                addToFunctionGroup(FCloned, ICG, SubGrpH);
+
+                // update the edge after clone
+                for (auto UI = F->use_begin(), UE = F->use_end(); UI != UE; /*empty*/)
+                {
+                    // Increment iterator after setting U to change the use.
+                    Use* U = &*UI++;
+                    Function* Caller = cast<CallInst>(U->getUser())->getParent()->getParent();
+                    FunctionGroup* InFG = getGroup(Caller);
+                    Function* InSubGrpH = useStackCall(F) ? FCloned : getSubGroupMap(Caller);
+                    if (InFG == ICG && InSubGrpH == SubGrpH)
+                    {
+                        *U = FCloned;
+                    }
+                }
+            }
+        }
+    }
+}
+
 bool GenXFunctionGroupAnalysis::rebuild(llvm::Module* Mod) {
     clear();
     auto pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
@@ -789,7 +1091,7 @@ void GenXFunctionGroupAnalysis::clear()
     for (auto I = begin(), E = end(); I != E; ++I)
         delete* I;
     Groups.clear();
-    IndirectCallGroup = nullptr;
+    for (auto FG : IndirectCallGroup) FG = nullptr;
     M = nullptr;
 }
 
@@ -950,7 +1252,7 @@ void SubroutineInliner::visitGetElementPtrInst(GetElementPtrInst& GEPI)
                 newGEPI->setDebugLoc(GEPI.getDebugLoc());
 
                 auto* newLoad = new LoadInst(loadInst->getType(), newGEPI, "", loadInst);
-                newLoad->setAlignment(IGCLLVM::getAlign(loadInst->getAlignment()));
+                newLoad->setAlignment(IGCLLVM::getAlign(*loadInst));
                 loadInst->replaceAllUsesWith(newLoad);
                 newLoad->setDebugLoc(loadInst->getDebugLoc());
             }
@@ -970,14 +1272,14 @@ void SubroutineInliner::visitMemCpyInst(MemCpyInst& I)
         if (origSrc->getType()->getPointerAddressSpace() != Src->getType()->getPointerAddressSpace())
         {
             Value* SrcCast = BitCastInst::Create(Instruction::BitCast, origSrc,
-                PointerType::get(Src->getType()->getPointerElementType(), origSrc->getType()->getPointerAddressSpace()),
+                PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(Src->getType()), origSrc->getType()->getPointerAddressSpace()),
                 "", &I);
             I.replaceUsesOfWith(Src, SrcCast);
         }
         if (origDst->getType()->getPointerAddressSpace() != Dst->getType()->getPointerAddressSpace())
         {
             Value* DstCast = BitCastInst::Create(Instruction::BitCast, origDst,
-                PointerType::get(Dst->getType()->getPointerElementType(), origDst->getType()->getPointerAddressSpace()),
+                PointerType::get(IGCLLVM::getNonOpaquePtrEltTy(Dst->getType()), origDst->getType()->getPointerAddressSpace()),
                 "", &I);
             I.replaceUsesOfWith(Dst, DstCast);
         }
@@ -1029,25 +1331,25 @@ InlineCost SubroutineInliner::getInlineCost(IGCLLVM::CallSiteRef CS)
         )
     {
         if (CS.hasFnAttr(llvm::Attribute::AlwaysInline))
-            return IGCLLVM::InlineCost::getAlways();
+            return llvm::InlineCost::getAlways("Per AlwaysInline function attribute");
 
         int FCtrl = getFunctionControl(pCtx);
 
         if (IGC::ForceAlwaysInline(pCtx))
-            return IGCLLVM::InlineCost::getAlways();
+            return llvm::InlineCost::getAlways("IGC set force always inline");
 
         if (pCtx->m_enableSubroutine == false)
-            return IGCLLVM::InlineCost::getAlways();
+            return llvm::InlineCost::getAlways("Disabled subroutines/stackcalls");
 
         if (pCtx->type == ShaderType::OPENCL_SHADER &&
             Callee->hasFnAttribute(llvm::Attribute::NoInline))
-            return IGCLLVM::InlineCost::getNever();
+            return llvm::InlineCost::getNever("Per NoInline function attribute");
 
         if (Callee->hasFnAttribute("KMPLOCK"))
-            return IGCLLVM::InlineCost::getNever();
+            return llvm::InlineCost::getNever("Has KMPLOCK function attribute");
 
         if (Callee->hasFnAttribute("igc-force-stackcall"))
-            return IGCLLVM::InlineCost::getNever();
+            return llvm::InlineCost::getNever("Has igc-force-stackcall function attribute");
 
         if (FCtrl == FLAG_FCALL_DEFAULT)
         {
@@ -1062,22 +1364,22 @@ InlineCost SubroutineInliner::getInlineCost(IGCLLVM::CallSiteRef CS)
 
             if (FSA->getExpandedSize(Caller) <= PerFuncThreshold)
             {
-                return IGCLLVM::InlineCost::getAlways();
+                return llvm::InlineCost::getAlways("Caller size smaller than per func. threshold");
             }
             else if (isTrivialCall(Callee) || FSA->onlyCalledOnce(Callee))
             {
-                return IGCLLVM::InlineCost::getAlways();
+                return llvm::InlineCost::getAlways("Callee is called only once");
             }
             else if (!FSA->shouldEnableSubroutine())
             {
                 // This function returns true if the estimated total inlining size exceeds some module threshold.
                 // If we don't exceed it, and there's no preference on inline vs noinline, we just inline.
-                return IGCLLVM::InlineCost::getAlways();
+                return llvm::InlineCost::getAlways("Did not meet inline per function size threshold");
             }
         }
     }
 
-    return IGCLLVM::InlineCost::getNever();
+    return llvm::InlineCost::getNever("Did not meet any inlining conditions");
 }
 
 Pass* IGC::createSubroutineInlinerPass()
