@@ -1,39 +1,19 @@
-/*===================== begin_copyright_notice ==================================
+/*========================== begin_copyright_notice ============================
 
-Copyright (c) 2017 Intel Corporation
+Copyright (C) 2017-2021 Intel Corporation
 
-Permission is hereby granted, free of charge, to any person obtaining a
-copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+SPDX-License-Identifier: MIT
 
-The above copyright notice and this permission notice shall be included
-in all copies or substantial portions of the Software.
+============================= end_copyright_notice ===========================*/
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+/*========================== begin_copyright_notice ============================
 
+This file is distributed under the University of Illinois Open Source License.
+See LICENSE.TXT for details.
 
-======================= end_copyright_notice ==================================*/
+============================= end_copyright_notice ===========================*/
 
-//===-- llvm/CodeGen/LiveVariables.h - Live Variable Analysis ---*- C++ -*-===//
-//
-//                     The LLVM Compiler Infrastructure
-//
-// This file is distributed under the University of Illinois Open Source
-// License. See LICENSE.TXT for details.
-//
-//===----------------------------------------------------------------------===//
-
-//===------------ LiveVars.h - Live Variable Analysis ----------*- C++ -*-===//
+//===------------ LiveVars.h - Live Variable Analysis -----------*- C++ -*-===//
 //
 // Intel extension to LLVM core
 //
@@ -114,6 +94,7 @@ namespace IGC
             /// AliveBlocks - Set of blocks in which this value is alive completely
             /// through.
             llvm::SmallPtrSet<llvm::BasicBlock*, 16> AliveBlocks;
+            ///std::set<llvm::BasicBlock*> AliveBlocks;
 
             /// NumUses - Number of uses of this register across the entire function.
             ///
