@@ -24,14 +24,14 @@ namespace IGC
         class InternalOptions
         {
         public:
-            InternalOptions(const TC::STB_TranslateInputArgs* pInputArgs)
+            InternalOptions(const TC::STB_TranslateInputArgs* pInputArgs, OpenCLProgramContext& context)
             {
                 if (pInputArgs == nullptr)
                     return;
 
                 if (pInputArgs->pInternalOptions != nullptr)
                 {
-                    parseOptions(pInputArgs->pInternalOptions);
+                    parseOptions(pInputArgs->pInternalOptions, context);
                 }
             }
 
@@ -48,10 +48,6 @@ namespace IGC
             bool IntelHasPositivePointerOffset              = false;
             bool IntelHasBufferOffsetArg                    = false;
             bool IntelBufferOffsetArgOptional               = true;
-            bool IntelHasSubDWAlignedPtrArg                 = false;
-            // default: false, meaning kernel's sub-DW ptrArgs (char*, short*) are DW-aligned.
-            // This default is stronger than the natural alignment implied by char*/short*. But
-            // for historical reason, we have this.
 
             bool replaceGlobalOffsetsByZero                 = false;
             bool IntelEnablePreRAScheduling                 = true;
@@ -81,6 +77,7 @@ namespace IGC
 
             bool IntelExpGRFSize                            = false;
             uint32_t expGRFSize                             = 0;
+            bool IntelEnableAutoLargeGRF = false;
 
             // IntelForceInt32DivRemEmu is used only if fp64 is supported natively.
             // IntelForceInt32DivRemEmu wins if both are set and can be applied.
@@ -93,9 +90,6 @@ namespace IGC
             int FunctionControl                             = -1;
             // Fail comilation if spills are present in compiled kernel
             bool FailOnSpill                                = false;
-            // This option forces IGC to poison kernels using fp64
-            // operations on platforms without HW support for fp64.
-            bool EnableUnsupportedFP64Poisoning             = false;
             // This option enables FP64 emulation for platforms that
             // cannot HW support for double operations
             bool EnableFP64GenEmu                           = false;
@@ -120,55 +114,67 @@ namespace IGC
             bool EmitVisaOnly                               = false;
 
         private:
-            void parseOptions(const char* internalOpts);
+            void parseOptions(const char* internalOpts, OpenCLProgramContext& context);
         };
 
         class Options
         {
         public:
-            Options(const TC::STB_TranslateInputArgs* pInputArgs)
+            Options(const TC::STB_TranslateInputArgs* pInputArgs, OpenCLProgramContext& context)
             {
                 if (pInputArgs == nullptr)
                     return;
 
                 if (pInputArgs->pOptions != nullptr)
                 {
-                    parseOptions(pInputArgs->pOptions);
+                    parseOptions(pInputArgs->pOptions, context);
                 }
             }
 
-            bool CorrectlyRoundedSqrt           = false;
-            bool NoSubgroupIFP                  = false;
-            bool UniformWGS                     = false;
-            bool EnableTakeGlobalAddress        = false;
-            bool IsLibraryCompilation           = false;
-            bool IntelRequiredEUThreadCount     = false;
-            bool EmitErrorsForLibCompilation    = false;
-            uint32_t requiredEUThreadCount      = 0;
+            bool CorrectlyRoundedSqrt               = false;
+            bool NoSubgroupIFP                      = false;
+            bool UniformWGS                         = false;
+            bool EnableTakeGlobalAddress            = false;
+            bool IsLibraryCompilation               = false;
+            bool IntelRequiredEUThreadCount         = false;
+            bool EmitErrorsForLibCompilation        = false;
+            uint32_t requiredEUThreadCount          = 0;
 
-
-            bool GTPinReRA                      = false;
-            bool GTPinGRFInfo                   = false;
-            bool GTPinScratchAreaSize           = false;
-            bool GTPinIndirRef                  = false;
-            uint32_t GTPinScratchAreaSizeValue  = 0;
+            bool GTPinReRA                          = false;
+            bool GTPinGRFInfo                       = false;
+            bool GTPinScratchAreaSize               = false;
+            bool GTPinIndirRef                      = false;
+            // Skip emission of Frame Descriptor Entry in VISA.
+            bool SkipFDE                            = false;
+            uint32_t GTPinScratchAreaSizeValue      = 0;
 
             std::vector<std::string> LargeGRFKernels;
             std::vector<std::string> RegularGRFKernels;
 
-            // Enable compiler heuristics ("regSharingHeuristics" in VISA) for large GRF selection.
-            bool IntelEnableAutoLargeGRF        = false;
+            // Enable compiler heuristics ("-autoGRFSelection" in VISA) for large GRF selection.
+            bool IntelEnableAutoLargeGRF            = false;
 
-            bool IntelLargeRegisterFile         = false;
+            bool IntelLargeRegisterFile             = false;
 
-            bool IntelExpGRFSize                = false;
-            uint32_t expGRFSize                 = 0;
+            bool Intel128GRFPerThread               = false;
+            bool Intel256GRFPerThread               = false;
+            bool IntelGreaterThan4GBBufferRequired  = false;
+
+            bool IntelExpGRFSize                    = false;
+            uint32_t expGRFSize                     = 0;
 
             // Generic address related
-            bool NoLocalToGeneric               = false;
+            bool NoLocalToGeneric                   = false;
+
+            // This option forces IGC to poison kernels using fp64
+            // operations on platforms without HW support for fp64.
+            bool EnableUnsupportedFP64Poisoning = false;
+            // This option enables FP64 emulation for platforms that
+            // cannot HW support for double operations
+            bool EnableFP64GenEmu = false;
 
         private:
-            void parseOptions(const char* opts);
+            void parseOptions(const char* opts, OpenCLProgramContext& context);
         };
 
         // output: shader information
@@ -210,8 +216,8 @@ namespace IGC
             const bool createResourceDimTypes = true)
             : CodeGenContext(ShaderType::OPENCL_SHADER, btiLayout, platform, driverInfo, createResourceDimTypes, llvmContext),
             m_programOutput(platform.getPlatformInfo(), *this),
-            m_InternalOptions(pInputArgs),
-            m_Options(pInputArgs),
+            m_InternalOptions(pInputArgs, *this),
+            m_Options(pInputArgs, *this),
             isSpirV(false),
             m_ShouldUseNonCoherentStatelessBTI(shouldUseNonCoherentStatelessBTI)
         {
