@@ -111,9 +111,15 @@ namespace IGC
         // this is currently set only when we enable the A64 WA
         CVariable* m_activeMask = nullptr;   // flag var
         CVariable* m_numActiveLanes = nullptr;  // general var, #lanes for the entire dispatch size.
+
+        // caches combined current+next sources for SimdShuffleDown intrinsic
+        typedef std::pair<llvm::Value*, llvm::Value*> SimdShuffleDownSrcTy;
+        std::map<SimdShuffleDownSrcTy, CVariable*> m_simdShuffleDownSrc;
+
         void clearCaching() {
             m_activeMask = nullptr;
             m_numActiveLanes = nullptr;
+            m_simdShuffleDownSrc.clear();
         }
     };
 
@@ -132,6 +138,7 @@ namespace IGC
         virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const override
         {
             AU.addRequired<llvm::DominatorTreeWrapperPass>();
+            AU.addRequired<llvm::PostDominatorTreeWrapperPass>();
             AU.addRequired<llvm::LoopInfoWrapperPass>();
             AU.addRequired<WIAnalysis>();
             AU.addRequired<LiveVarsAnalysis>();
@@ -167,7 +174,6 @@ namespace IGC
         void visitFPToUIInst(llvm::FPToUIInst& I);
         void visitDbgInfoIntrinsic(llvm::DbgInfoIntrinsic& I);
         void visitExtractValueInst(llvm::ExtractValueInst& I);
-        void visitInsertValueInst(llvm::InsertValueInst& I);
         void visitBranchInst(llvm::BranchInst& I);
 
     public:
@@ -177,12 +183,12 @@ namespace IGC
         void ForceIsolate(llvm::Value* val) { m_forceIsolates.insert(val); }
         bool IsForceIsolated(llvm::Value* val) { return (m_forceIsolates.count(val) > 0) ? true : false; }
 
-        SSource GetSource(llvm::Value* v, bool modifier, bool regioning);
-        SSource GetSource(llvm::Value* value, e_modifier mod, bool regioning);
+        SSource GetSource(llvm::Value* v, bool modifier, bool regioning, bool isSampleDerivative);
+        SSource GetSource(llvm::Value* value, e_modifier mod, bool regioning, bool isSampleDerivative);
         bool GetRegionModifier(SSource& mod, llvm::Value*& source, bool regioning);
         bool BitcastSearch(SSource& mod, llvm::Value*& source, bool broadcast);
 
-        void MarkAsSource(llvm::Value* v);
+        void MarkAsSource(llvm::Value* v, bool isSampleDerivative);
 
         bool MatchFMA(llvm::IntrinsicInst& I);
         bool MatchFrc(llvm::BinaryOperator& I);
@@ -205,7 +211,6 @@ namespace IGC
         bool MatchAbsNeg(llvm::Instruction& I);
         bool MatchFloatingPointSatModifier(llvm::Instruction& I);
         bool MatchIntegerSatModifier(llvm::Instruction& I);
-        bool MatchPredicate(llvm::SelectInst& I);
         bool MatchSelectModifier(llvm::SelectInst& I);
         bool MatchPow(llvm::IntrinsicInst& I);
         bool MatchGenericPointersCmp(llvm::CmpInst& I);
@@ -220,7 +225,6 @@ namespace IGC
         bool MatchDp4a(llvm::GenIntrinsicInst& I);
         bool MatchLogicAlu(llvm::BinaryOperator& I);
         bool MatchRsqrt(llvm::BinaryOperator& I);
-        bool MatchLoadStorePointer(llvm::Instruction& I, llvm::Value& ptrVal);
         bool MatchBlockReadWritePointer(llvm::GenIntrinsicInst& I);
         bool MatchGradient(llvm::GenIntrinsicInst& I);
         bool MatchSampleDerivative(llvm::GenIntrinsicInst& I);
@@ -258,9 +262,12 @@ namespace IGC
         void SetPatternRoot(llvm::Instruction& inst);
         void CreateBasicBlocks(llvm::Function* pLLVMFunc);
         uint GetBlockId(llvm::BasicBlock* bb);
-        void HandleSubspanUse(llvm::Value* v);
+        void HandleSubspanUse(llvm::Value* v, bool isSampleSource);
         void HandleSampleDerivative(llvm::GenIntrinsicInst& I);
         bool IsSubspanUse(llvm::Value* v);
+        bool IsSourceOfSample(llvm::Value* v);
+        bool IsSourceOfSampleUnderCF(llvm::Value* v);
+        bool IsPHISourceOfSampleUnderCF(llvm::Value* v);
         bool HasUseOutsideLoop(llvm::Value* v);
         bool NeedVMask();
 
@@ -286,10 +293,16 @@ namespace IGC
             return m_WI && (m_WI->isUniform(V));
         };
 
+        bool supportsLSCImmediateGlobalBaseOffset();
+
+
     public:
         llvm::DenseSet<llvm::Instruction*>       m_usedInstructions;
         bool                  m_rootIsSubspanUse;
         llvm::DenseSet<llvm::Value*>             m_subSpanUse;
+        llvm::DenseSet<llvm::Value*>             m_sampleSource;
+        llvm::DenseSet<llvm::Value*>             m_sampleUnderCFsource;
+        llvm::DenseSet<llvm::Value*>             m_sampleUnderCFPHIsource;
         llvm::SmallPtrSet<llvm::Value*, 8>       m_forceIsolates;
         std::map<llvm::BasicBlock*, SBasicBlock*> m_blockMap;
         SBasicBlock* m_blocks;
@@ -316,6 +329,7 @@ namespace IGC
     private:
         CodeGenContext* m_ctx; // Caching codegen context, valid only within runOnFunction().
         llvm::DominatorTree* DT;
+        llvm::PostDominatorTree* PDT;
         llvm::LoopInfo* LI;
         const llvm::DataLayout* m_DL;
         WIAnalysis* m_WI;
