@@ -19,6 +19,8 @@ SPDX-License-Identifier: MIT
 #include "skuwa/iacm_g11_rev_id.h"
 #include "skuwa/iacm_g12_rev_id.h"
 
+#include "iStdLib/utility.h"
+
 namespace IGC
 {
 
@@ -59,7 +61,8 @@ bool hasNoInt64Inst() const {
         m_platformInfo.eProductFamily == IGFX_ALDERLAKE_N ||
         m_platformInfo.eProductFamily == IGFX_DG1 ||
         m_platformInfo.eProductFamily == IGFX_DG2 ||
-        m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+        m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
 }
 
 public:
@@ -695,7 +698,8 @@ bool hasHalfSIMDLSC() const
         GFX_IS_DG2_G12_CONFIG(m_platformInfo.usDeviceID) ||
         // false for PVC XL A0 RevID==0x0, true from PVC XT A0 RevID==0x3==REVISION_B
         (m_platformInfo.eProductFamily == IGFX_PVC && m_platformInfo.usRevId >= REVISION_B) ||
-        m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+        m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
 }
 
 bool NeedsLSCFenceUGMBeforeEOT() const
@@ -796,7 +800,8 @@ bool supports3DAndCubeSampleD() const
     return (
         m_platformInfo.eProductFamily != IGFX_XE_HP_SDV &&
         m_platformInfo.eProductFamily != IGFX_DG2 &&
-        m_platformInfo.eProductFamily != IGFX_METEORLAKE
+        m_platformInfo.eProductFamily != IGFX_METEORLAKE &&
+        m_platformInfo.eProductFamily != IGFX_ARROWLAKE
         ) ||
         IGC_IS_FLAG_DISABLED(EnableSampleDEmulation);
 }
@@ -847,7 +852,8 @@ uint32_t getMaxLSCBlockMsgSize(bool isD64 = true) const
 bool hasURBFence() const
 {
     return m_platformInfo.eProductFamily == IGFX_DG2 ||
-        m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+        m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
 }
 
 bool hasMultiTile() const
@@ -859,21 +865,16 @@ bool hasMultiTile() const
 // UGM LSC fence with GPU scope triggers L3 flush
 bool hasL3FlushOnGPUScopeInvalidate() const
 {
-    return m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
-           m_platformInfo.eProductFamily == IGFX_DG2;
+    return
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+        m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_DG2;
 }
 
 bool L3CacheCoherentCrossTiles() const {
     return isCoreChildOf(IGFX_XE_HPC_CORE);
 }
 
-bool AllowFenceOpt() const
-{
-    return ((m_platformInfo.eProductFamily == IGFX_DG2
-        || m_platformInfo.eProductFamily == IGFX_METEORLAKE
-        || m_platformInfo.eProductFamily == IGFX_PVC) &&
-        IGC_IS_FLAG_ENABLED(EnablePlatformFenceOpt));
-}
 
 //if RayTracing fence WA for LSCBackupMode (DG2.A0 only actually) is enabled.
 //In case of RayQuery of DG2.A0, we have to flush L1 after RTWrite and before shader read.
@@ -949,6 +950,7 @@ bool typedReadSupportsAllRenderableFormats() const
         (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G11Config && isChildOfDG2B0) ||
         (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G12Config) ||
         (m_platformInfo.eProductFamily == IGFX_METEORLAKE) ||
+        (m_platformInfo.eProductFamily == IGFX_ARROWLAKE) ||
         (m_platformInfo.eRenderCoreFamily == IGFX_XE_HPC_CORE))
     {
         return IGC_IS_FLAG_DISABLED(ForceFormatConversionDG2Plus);
@@ -957,10 +959,21 @@ bool typedReadSupportsAllRenderableFormats() const
     return false;
 }
 
+bool EnableNewTileYCheckDefault() const
+{
+    return (m_platformInfo.eProductFamily == IGFX_DG2);
+}
+
+bool EnableKeepTileYForFlattenedDefault() const
+{
+    return false;
+}
+
 bool needsWAForThreadsUtilization() const
 {
-    return m_platformInfo.eProductFamily == IGFX_DG2 ||
-           m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+    return (m_platformInfo.eProductFamily == IGFX_DG2 ||
+        m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE);
 }
 
 bool supportDualSimd8PS() const
@@ -1070,7 +1083,8 @@ bool supportAIParameterCombiningWithLODBiasEnabled() const
            (m_platformInfo.eProductFamily == IGFX_DG2 && SI_WA_FROM(m_platformInfo.usRevId, ACM_G10_GT_REV_ID_B0)) ||
            GFX_IS_DG2_G12_CONFIG(m_platformInfo.usDeviceID) ||
            GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID) ||
-            m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+           m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+           m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
 }
 
 bool useScratchSpaceForOCL() const
@@ -1121,12 +1135,14 @@ bool hasCorrectlyRoundedMacros() const {
         m_platformInfo.eProductFamily != IGFX_ALDERLAKE_P &&
         m_platformInfo.eProductFamily != IGFX_ALDERLAKE_N &&
         m_platformInfo.eProductFamily != IGFX_DG2 &&
-        m_platformInfo.eProductFamily != IGFX_METEORLAKE);
+        m_platformInfo.eProductFamily != IGFX_METEORLAKE) &&
+        m_platformInfo.eProductFamily != IGFX_ARROWLAKE;
 }
 
 // Has 64bit support but use 32bit for perf reasons
 bool preferFP32IntDivRemEmu() const {
-    return m_platformInfo.eProductFamily == IGFX_METEORLAKE;
+    return m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE;
 }
 
 // Platforms that haven't HW support for FP64 operations
@@ -1201,6 +1217,7 @@ bool canFuseTypedWrite() const
     bool isDG2G11Config = GFX_IS_DG2_G11_CONFIG(m_platformInfo.usDeviceID);
     bool isDG2G12Config = GFX_IS_DG2_G12_CONFIG(m_platformInfo.usDeviceID);
     bool canFuse = (m_platformInfo.eProductFamily == IGFX_METEORLAKE) ||
+        (m_platformInfo.eProductFamily == IGFX_ARROWLAKE) ||
         (m_platformInfo.eProductFamily == IGFX_DG2 && isChildOfDG2C0) ||
         (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G11Config && isChildOfDG2B0) ||
         (m_platformInfo.eProductFamily == IGFX_DG2 && isDG2G12Config);
@@ -1331,7 +1348,8 @@ bool supportSystemFence() const
 {
     return hasLSC()
         && m_platformInfo.eProductFamily != IGFX_DG2
-        && m_platformInfo.eProductFamily != IGFX_METEORLAKE;
+        && m_platformInfo.eProductFamily != IGFX_METEORLAKE
+        && m_platformInfo.eProductFamily != IGFX_ARROWLAKE;
 }
 
 
@@ -1403,6 +1421,7 @@ bool noNativeDwordMulSupport() const
         m_platformInfo.eProductFamily == IGFX_GEMINILAKE ||
         m_platformInfo.eProductFamily == IGFX_DG2 ||
         m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+        m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
         GetPlatformFamily() == IGFX_GEN11_CORE ||
         GetPlatformFamily() == IGFX_GEN12LP_CORE;
 }
@@ -1468,5 +1487,12 @@ bool supportsNonDefaultLSCCacheSetting() const
     return false;
 }
 
+unsigned int roundUpTgsmSize(DWORD size) const
+{
+    const DWORD blockSize = getSharedLocalMemoryBlockSize() ? getSharedLocalMemoryBlockSize() : sizeof(KILOBYTE);
+    size = iSTD::Round(size, blockSize) / blockSize;
+    return iSTD::RoundPower2(size) * blockSize;
+}
+
 };
 }//namespace IGC
