@@ -28,6 +28,8 @@ SPDX-License-Identifier: MIT
 #include "Probe/Assertion.h"
 #include <algorithm>
 #include <list>
+#include "llvm/IR/Constants.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -149,6 +151,49 @@ namespace
         Value* createSI16Value(Value* V) { return createI16Value(V, false); }
         Value* createUI16Value(Value* V) { return createI16Value(V, true); }
     };
+
+
+    ValueInfo* PromoteInt8Type::getValInfo(Value* V) const
+    {
+        if (!V || isa<Constant>(V))
+        {
+            return nullptr;
+        }
+        auto VI = m_valInfoMap.find(V);
+        if (VI != m_valInfoMap.end())
+        {
+            return VI->second;
+        }
+        return nullptr;
+    }
+
+    // Create an entry in m_valInfoMap for V
+    ValueInfo* PromoteInt8Type::addValInfo(Value* V)
+    {
+        auto VI = m_valInfoMap.find(V);
+        if (VI == m_valInfoMap.end())
+        {
+            ValueInfo* valinfo = new (Allocator) ValueInfo(V);
+            m_valInfoMap.insert(std::make_pair(V, valinfo));
+            m_valInsertOrder.push_back(V);
+        }
+        return m_valInfoMap[V];
+    }
+
+    // If V is of i8 type, create an entry in m_valInfoMap (via
+    // addValInfo()) if it does not exist, and return that entry.
+    // If V is not i8 type, do not create an entry and just return
+    // nullptr.
+    ValueInfo* PromoteInt8Type::addValInfoIfI8(Value* V)
+    {
+        ValueInfo* valinfo = nullptr;
+        if (isI8Type(V))
+        {
+            valinfo = addValInfo(V);
+        }
+        return valinfo;
+    }
+
 }
 
 // Register pass to igc-opt
@@ -167,44 +212,6 @@ FunctionPass* IGC::createPromoteInt8TypePass()
     return new PromoteInt8Type();
 }
 
-ValueInfo* PromoteInt8Type::getValInfo(Value* V) const
-{
-    if (!V || isa<Constant>(V))
-    {
-        return nullptr;
-    }
-    auto VI = m_valInfoMap.find(V);
-    if (VI != m_valInfoMap.end()) {
-        return VI->second;
-    }
-    return nullptr;
-}
-
-// Create an entry in m_valInfoMap for V
-ValueInfo* PromoteInt8Type::addValInfo(Value* V)
-{
-    auto VI = m_valInfoMap.find(V);
-    if (VI == m_valInfoMap.end()) {
-        ValueInfo* valinfo = new (Allocator) ValueInfo(V);
-        m_valInfoMap.insert(std::make_pair(V, valinfo));
-        m_valInsertOrder.push_back(V);
-    }
-    return m_valInfoMap[V];
-}
-
-// If V is of i8 type, create an entry in m_valInfoMap (via
-// addValInfo()) if it does not exist, and return that entry.
-// If V is not i8 type, do not create an entry and just return
-// nullptr.
-ValueInfo* PromoteInt8Type::addValInfoIfI8(Value* V)
-{
-    ValueInfo* valinfo = nullptr;
-    if (isI8Type(V)) {
-        valinfo = addValInfo(V);
-    }
-    return valinfo;
-}
-
 bool PromoteInt8Type::runOnFunction(Function& F)
 {
     m_changed = false;
@@ -559,6 +566,7 @@ void PromoteInt8Type::promoteInstructions()
                 case Instruction::Or:
                 case Instruction::Xor:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     BinaryOperator* BinOp = cast<BinaryOperator>(Inst);
                     newVal = m_builder->CreateBinOp(
                         BinOp->getOpcode(),
@@ -570,6 +578,7 @@ void PromoteInt8Type::promoteInstructions()
                 case Instruction::SDiv:
                 case Instruction::SRem:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     // trunc first, then extend
                     Value* nV0 = m_builder->CreateSExt(getI8Value(v0), i16Ty, "b2s");
                     Value* nV1 = m_builder->CreateSExt(getI8Value(v1), i16Ty, "b2s");
@@ -584,6 +593,7 @@ void PromoteInt8Type::promoteInstructions()
                 case Instruction::UDiv:
                 case Instruction::URem:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     // trunc first, then extend
                     Value* nV0 = m_builder->CreateZExt(getI8Value(v0), i16Ty, "b2s");
                     Value* nV1 = m_builder->CreateZExt(getI8Value(v1), i16Ty, "b2s");
@@ -597,18 +607,21 @@ void PromoteInt8Type::promoteInstructions()
                 }
                 case Instruction::AShr:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     Value* nV0 = m_builder->CreateSExt(getI8Value(v0), i16Ty, "b2s");
                     newVal = m_builder->CreateAShr(nV0, getUI16Value(v1), "b2s");
                     break;
                 }
                 case Instruction::LShr:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     Value* nV0 = m_builder->CreateZExt(getI8Value(v0), i16Ty, "b2s");
                     newVal = m_builder->CreateLShr(nV0, getUI16Value(v1), "b2s");
                     break;
                 }
                 case Instruction::Shl:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     // upper 8-bit does not matter
                     newVal = m_builder->CreateShl(getSI16Value(v0), getUI16Value(v1), "b2s");
                     break;
@@ -638,6 +651,7 @@ void PromoteInt8Type::promoteInstructions()
                 }
                 case Instruction::Select:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     Value* v2 = Inst->getOperand(2);
                     Value* si16v1 = getSI16Value(v1);
                     Value* si16v2 = getSI16Value(v2);
@@ -666,6 +680,7 @@ void PromoteInt8Type::promoteInstructions()
                 }
                 case Instruction::ExtractElement:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     ValueInfo* vinfo = getValInfo(v0);
                     // vector operand
                     //    not promoted : if it isn't constant and marked to be not promoted;
@@ -694,6 +709,7 @@ void PromoteInt8Type::promoteInstructions()
                 }
                 case Instruction::InsertElement:
                 {
+                    IGC_ASSERT(nullptr != v1);
                     ValueInfo* vinfo = getValInfo(Inst->getOperand(0));
                     IGC_ASSERT_MESSAGE((!vinfo || vinfo->NeedPromote), "IEI's vector operands and dst shall be both promoted!");
                     Value* v2 = Inst->getOperand(2);
@@ -704,6 +720,13 @@ void PromoteInt8Type::promoteInstructions()
                         "b2s");
                     break;
                 }
+#if LLVM_VERSION_MAJOR >= 10
+                case Instruction::Freeze:
+                {
+                    newVal = m_builder->CreateFreeze(getSI16Value(v0), "b2s");
+                    break;
+                }
+#endif
                 default:
                     IGC_ASSERT_MESSAGE(0, "Unexpected inst of i8");
                 }  // Switch
@@ -865,6 +888,36 @@ Value* PromoteInt8Type::getI16Value(Value* V, bool IsZExt)
         return ConstantDataVector::get(m_F->getContext(), CElts);
     }
 
+    if (ConstantVector* CVI = dyn_cast<ConstantVector>(V))
+    {
+        Type* Ty = dyn_cast<VectorType>(CVI->getType())->getElementType();
+        IGC_ASSERT(nullptr != Ty);
+
+        if (!Ty->isIntegerTy()) {
+            IGC_ASSERT_MESSAGE(0, "unsupported constant expression");
+            return nullptr;
+        }
+
+        uint32_t nelts = CVI->getNumOperands();
+
+        SmallVector<Constant*, 16> Vals(nelts, nullptr);
+        for (unsigned idx = 0; idx < nelts; ++idx)
+        {
+            auto elem= CVI->getOperand(idx);
+            auto CstI = dyn_cast<ConstantInt>(elem);
+            if (CstI)
+            {
+                uint64_t n = IsZExt ? CstI->getZExtValue() : CstI->getSExtValue();
+                Vals[idx] = ConstantInt::get(i16Ty, n, IsZExt);
+            }
+            else
+            {
+                Vals[idx] = UndefValue::get(i16Ty);
+            }
+        }
+        return ConstantVector::get(Vals);
+    }
+
     if (isa<UndefValue>(V)) {
         if (auto* VTy = dyn_cast<IGCLLVM::FixedVectorType>(V->getType())) {
             auto* nVTy = IGCLLVM::FixedVectorType::get(i16Ty, (unsigned)VTy->getNumElements());
@@ -1058,13 +1111,15 @@ void PromoteInt8Type::promoteIntrinsic()
         if (!GII)
             continue;
         if (GII->isGenIntrinsic(GenISAIntrinsic::GenISA_WaveShuffleIndex) ||
+            GII->isGenIntrinsic(GenISAIntrinsic::GenISA_WaveBroadcast) ||
             GII->isGenIntrinsic(GenISAIntrinsic::GenISA_simdShuffleDown))
         {
             // Those are mov insts. Need to promote if its operand is
             // of type I8 and index is not uniform.
             Type* Ty = GII->getType();
             Value* IndexOrDelta;
-            if (GII->isGenIntrinsic(GenISAIntrinsic::GenISA_WaveShuffleIndex)) {
+            if (GII->isGenIntrinsic(GenISAIntrinsic::GenISA_WaveShuffleIndex) ||
+                GII->isGenIntrinsic(GenISAIntrinsic::GenISA_WaveBroadcast)) {
                 IndexOrDelta = GII->getArgOperand(1);
             }
             else {
@@ -1105,6 +1160,7 @@ void PromoteInt8Type::promoteIntrinsic()
                 gid == GenISAIntrinsic::GenISA_WavePrefix ||
                 gid == GenISAIntrinsic::GenISA_QuadPrefix ||
                 gid == GenISAIntrinsic::GenISA_WaveShuffleIndex ||
+                gid == GenISAIntrinsic::GenISA_WaveBroadcast ||
                 gid == GenISAIntrinsic::GenISA_simdShuffleDown)
             {
                 //
@@ -1161,6 +1217,7 @@ void PromoteInt8Type::promoteIntrinsic()
                 }
                 case GenISAIntrinsic::GenISA_QuadPrefix:
                 case GenISAIntrinsic::GenISA_WaveShuffleIndex:
+                case GenISAIntrinsic::GenISA_WaveBroadcast:
                 case GenISAIntrinsic::GenISA_WaveAll:
                 {
                     // prototype:
