@@ -38,10 +38,10 @@ namespace IGC
     {
     public:
         static char ID;
-        RegisterPressureEstimate()
+        RegisterPressureEstimate(bool RequireWIA = true, bool CountTemps = false)
             : FunctionPass(ID), m_DL(nullptr), m_pFunc(nullptr), LI(nullptr),
             WI(nullptr), m_available(false), MaxAssignedNumber(0),
-            OVERALL_PRESSURE_UPBOUND(512)
+              OVERALL_PRESSURE_UPBOUND(512), m_requireWIA(RequireWIA), m_countTemps(CountTemps)
         {
             initializeRegisterPressureEstimatePass(*llvm::PassRegistry::getPassRegistry());
         }
@@ -189,13 +189,12 @@ namespace IGC
         bool buildLiveIntervals(bool RemoveLR = false);
 
         /// \brief Return the register pressure for the whole function.
-        unsigned getMaxRegisterPressure() const;
+        unsigned getMaxRegisterPressure();
 
         /// \brief Return the register pressure for a basic block.
         unsigned getMaxRegisterPressure(llvm::BasicBlock* BB) const;
 
-        void printRegisterPressureInfo(bool Detailed = false,
-            const char* msg = "") const;
+        void printRegisterPressureInfo(bool Detailed = false, const char* msg = "");
 
         bool isAvailable() const { return m_available; }
 
@@ -270,5 +269,8 @@ namespace IGC
         unsigned int MaxAssignedNumber;
 
         llvm::DenseMap<unsigned, unsigned> m_pRegisterPressureByInstruction;
+
+        bool m_requireWIA;
+        bool m_countTemps;
     };
 } // namespace IGC
