@@ -69,11 +69,10 @@ typedef std::multimap<
 // @brief Get all RuntimeValue calls. The collection of RuntimeValue calls is sorted
 // according to offset and number of elements of RuntimeValue. RuntimeValue calls
 // representing single scalars have only one element unlike RuntimeValue calls
-// representing vectors of scalars. RuntimeValue vectors are made GRF-aligned if needed.
+// representing vectors of scalars.
 static bool GetAllRuntimeValueCalls(
     llvm::Module& module,
-    RuntimeValueCollection& runtimeValueCalls,
-    uint32_t dataGRFAlignmentInDwords)
+    RuntimeValueCollection& runtimeValueCalls)
 {
     bool legalizationCheckNeeded = false;
     for (llvm::Function& F : module)
@@ -99,42 +98,13 @@ static bool GetAllRuntimeValueCalls(
                             // Only vectors of 32-bit values are supported at the moment
                             if (fixedVectorTy->getElementType()->getPrimitiveSizeInBits() == 32)
                             {
-                                // Only vectors having corresponding extract element instructions
-                                // are subject of further processing.
-                                bool hasEEUser = false;
-                                for (llvm::User* user : intr->users())
-                                {
-                                    if (llvm::ExtractElementInst* EEI = llvm::dyn_cast<llvm::ExtractElementInst>(user))
-                                    {
-                                        hasEEUser = true;
-                                        break;
-                                    }
-                                }
-                                if (hasEEUser)
-                                {
-                                    uint32_t numElements = int_cast<uint32_t>(fixedVectorTy->getNumElements());
-                                    const uint32_t lastElementOffset = offset + numElements - 1;
+                                uint32_t numElements = int_cast<uint32_t>(fixedVectorTy->getNumElements());
+                                const uint32_t lastElementOffset = offset + numElements - 1;
 
-                                    // Check if vector needs to be GRF aligned
-                                    // Vector must be GRF aligned if it's size is larger than or equal to one GRF.
-                                    // Vector must fit in one GRF if its size is less than one GRF(it can not cross GRF boundary).
-                                    const uint32_t alignedVectorOffset =
-                                        int_cast<uint32_t>(llvm::alignTo(offset, dataGRFAlignmentInDwords));
-                                    // Offset can be already aligned to GRF boundary
-                                    if (offset != alignedVectorOffset)
-                                    {
-                                        // Check if vector crosses GRF boundary
-                                        if (lastElementOffset >= alignedVectorOffset)
-                                        {
-                                            // Align to GRF by changing vector's offset
-                                            offset = int_cast<uint32_t>(llvm::alignDown(offset, dataGRFAlignmentInDwords));
-                                        }
-                                    }
-                                    runtimeValueCalls.insert(std::make_pair(std::make_pair(offset, lastElementOffset), intr));
+                                runtimeValueCalls.insert(std::make_pair(std::make_pair(offset, lastElementOffset), intr));
 
-                                    // Having RuntimeValue vectors, further legalization checks are needed
-                                    legalizationCheckNeeded = true;
-                                }
+                                // Having RuntimeValue vectors, further legalization checks are needed
+                                legalizationCheckNeeded = true;
                             }
                             else
                             {
@@ -142,6 +112,10 @@ static bool GetAllRuntimeValueCalls(
                             }
                         }
                     }
+                    else if (intr->getType()->getPrimitiveSizeInBits() == 64)
+                    {
+                        runtimeValueCalls.insert(std::make_pair(std::make_pair(offset, offset + 1), intr));
+                    }
                     else
                     {
                         runtimeValueCalls.insert(std::make_pair(std::make_pair(offset, offset), intr));
@@ -153,11 +127,14 @@ static bool GetAllRuntimeValueCalls(
     return legalizationCheckNeeded;
 }
 
-////////////////////////////////////////////////////////////////////////////
-// @brief Creates a set of accessed RuntimeValue offsets (no duplicates).
-static void GetAccessedOffsets(
-    RuntimeValueCollection& runtimeValueCalls,
-    std::set<uint32_t>& accessedOffsetsSet)
+////////////////////////////////////////////////////////////////////////////////
+// @brief Creates a vector of accessed RuntimeValue offset ranges. Returned
+// ranges cannot overlap and must either not cross GRF boundaries or start
+// at GRF boundary.
+static void GetDisjointRegions(
+    std::vector<std::pair<uint32_t, uint32_t>>& disjointRegions,
+    const RuntimeValueCollection& runtimeValueCalls,
+    const uint32_t dataGRFAlignmentInDwords)
 {
     // Since input collection is already sorted according to offset
     // and number of elements of RuntimeValue, it's enough to process
@@ -167,17 +144,56 @@ static void GetAccessedOffsets(
     // ->{ {6, 6},  RuntimeValue* }
     // ->{ {8, 19}, RuntimeValue* }
     //   { {8, 8},  RuntimeValue* }
-    int prevOffset = -1;
     for (auto it : runtimeValueCalls)
     {
-        std::pair<uint32_t, uint32_t> offsets = it.first;
-        if (offsets.first != prevOffset)
+        std::pair<uint32_t, uint32_t> range = it.first;
+        if (disjointRegions.empty() ||
+            range.first > disjointRegions.back().second)
         {
-            for (uint32_t i = offsets.first; i <= offsets.second; i++)
+            disjointRegions.push_back(range);
+        }
+        else if (range.first != disjointRegions.back().first &&
+            range.second > disjointRegions.back().second)
+        {
+            disjointRegions.back().second = range.second;
+        }
+
+        // Lambda checks if a range of offsets accessed as a vector is correctly
+        // aligned:
+        // - vector must be GRF aligned if it's size is larger than or equal to
+        //   a single GRF.
+        // - vector must fit in one GRF if its size is less than a single GRF
+        //   (it can not cross GRF boundary).
+        auto IsUnaligned = [dataGRFAlignmentInDwords](
+            const std::pair<uint32_t, uint32_t>& range)
+        {
+            const uint64_t alignedRegionOffset =
+                llvm::alignTo(range.first, dataGRFAlignmentInDwords);
+            if (range.first != alignedRegionOffset &&
+                range.second >= alignedRegionOffset)
             {
-                accessedOffsetsSet.insert(i);
+                return true;
             }
-            prevOffset = offsets.first;
+            return false;
+        };
+        while (IsUnaligned(disjointRegions.back()))
+        {
+            auto current = disjointRegions.back();
+            // Align to GRF boundary
+            current.first = int_cast<uint32_t>(llvm::alignDown(current.first, dataGRFAlignmentInDwords));
+            // Check for overlapping regions already in the vector
+            while (!disjointRegions.empty() &&
+                disjointRegions.back().second >= current.first)
+            {
+                IGC_ASSERT(disjointRegions.back().second <= current.second);
+                if (disjointRegions.back().first < current.first)
+                {
+                    IGC_ASSERT(!IsUnaligned(disjointRegions.back()));
+                    current.first = disjointRegions.back().first;
+                }
+                disjointRegions.pop_back();
+            }
+            disjointRegions.push_back(current);
         }
     }
 }
@@ -188,44 +204,26 @@ static void GetAccessedOffsets(
 // for example: {0, {0, 2}}, {1, {0, 2}}, {4, {4, 1}}
 // Resulting ranges are disjoint and each spans the biggest continuous range of offsets.
 static void GetAccessedRegions(
-    RuntimeValueCollection& runtimeValueCalls,
-    std::map<uint32_t, std::pair<uint32_t, uint32_t>>& accessedRegions)
+    std::map<uint32_t, std::pair<uint32_t, uint32_t>>& accessedRegions,
+    const RuntimeValueCollection& runtimeValueCalls,
+    const uint32_t dataGRFAlignmentInDwords)
 {
-    // Create a set of accessed offsets without duplicates
-    std::set<uint32_t> accessedOffsetsSet;
-    GetAccessedOffsets(runtimeValueCalls, accessedOffsetsSet);
+    // Get disjoint offsets regions
+    std::vector<std::pair<uint32_t, uint32_t>> disjointRegions;
+    GetDisjointRegions(disjointRegions, runtimeValueCalls, dataGRFAlignmentInDwords);
 
-    // Traverse the accessed offsets to create map of disjoint RuntimeValue regions.
-    std::vector<uint32_t> accessedOffsets(accessedOffsetsSet.begin(), accessedOffsetsSet.end());
-    std::size_t numOffsets = accessedOffsets.size();
-    if (numOffsets > 1)
+    // Create final map of disjoint RuntimeValue regions
+    std::size_t disjointRegionsNum = disjointRegions.size();
+    for (std::size_t i = 0; i < disjointRegionsNum; i++)
     {
-        uint32_t numElementsInRange = 1;
-        for (std::size_t i = 1; i <= numOffsets; i++)
+        uint32_t beginIdx = disjointRegions[i].first;
+        uint32_t endIdx = disjointRegions[i].second;
+        uint32_t numOfElements = endIdx - beginIdx + 1;
+        for (uint32_t idx = beginIdx; idx <= endIdx; idx++)
         {
-            if ((i == numOffsets) || (accessedOffsets[i] - accessedOffsets[i - 1] != 1))
-            {
-                uint32_t beginIdx = accessedOffsets[i - numElementsInRange];
-                uint32_t endIdx = accessedOffsets[i - 1];
-                uint32_t numOfElements = endIdx - beginIdx + 1;
-                for (uint32_t idx = beginIdx; idx <= endIdx; idx++)
-                {
-                    accessedRegions.insert(std::make_pair(idx, std::make_pair(beginIdx, numOfElements)));
-                }
-
-                // Reset range element counter
-                numElementsInRange = 1;
-            }
-            else
-            {
-                numElementsInRange++;
-            }
+            accessedRegions.insert(std::make_pair(idx, std::make_pair(beginIdx, numOfElements)));
         }
     }
-    else
-    {
-        accessedRegions.insert(std::make_pair(accessedOffsets[0], std::make_pair(accessedOffsets[0], 1)));
-    }
 }
 
 ////////////////////////////////////////////////////////////////////////////
@@ -277,7 +275,7 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
 
     RuntimeValueCollection runtimeValueCalls(RuntimeValueComparator);
     bool legalizationCheckNeeded =
-        GetAllRuntimeValueCalls(module, runtimeValueCalls, dataGRFAlignmentInDwords);
+        GetAllRuntimeValueCalls(module, runtimeValueCalls);
 
     if (legalizationCheckNeeded)
     {
@@ -285,7 +283,7 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
         // {offset { enclosing_region_start_offset, enclosing_region_size }}
         // for example: {0, {0, 2}}, {1, {0, 2}}, {4, {4, 1}}
         std::map<uint32_t, std::pair<uint32_t, uint32_t>> accessedRegions;
-        GetAccessedRegions(runtimeValueCalls, accessedRegions);
+        GetAccessedRegions(accessedRegions, runtimeValueCalls, dataGRFAlignmentInDwords);
 
         // Loop through all RuntimeValue calls
         for (auto it : runtimeValueCalls)
@@ -297,7 +295,9 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
 
             uint32_t resolvedOffset = int_cast<uint32_t>(
                 cast<ConstantInt>(callToResolve->getArgOperand(0))->getZExtValue());
-            uint32_t resolvedSize = fixedVectorTy ? int_cast<uint32_t>(fixedVectorTy->getNumElements()) : 1;
+            uint32_t resolvedSize = int_cast<uint32_t>(fixedVectorTy
+                ? fixedVectorTy->getNumElements()
+                : callToResolve->getType()->getPrimitiveSizeInBits() / 32);
 
             // Find corresponding region
             auto regionIter = accessedRegions.find(resolvedOffset);
@@ -311,6 +311,13 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
                 llvm::IRBuilder<> builder(callToResolve);
 
                 llvm::Type* resolvedBaseType = fixedVectorTy ? fixedVectorTy->getElementType() : callToResolve->getType();
+                IGC_ASSERT(regionSize > 1);
+                bool is64bit = resolvedBaseType->getPrimitiveSizeInBits() == 64;
+                if (is64bit)
+                {
+                    IGC_ASSERT(fixedVectorTy == nullptr);
+                    resolvedBaseType = builder.getInt32Ty();
+                }
                 llvm::Type* vectorType = IGCLLVM::FixedVectorType::get(resolvedBaseType, regionSize);
                 Function* runtimeValueFunc = GenISAIntrinsic::getDeclaration(&module,
                     GenISAIntrinsic::GenISA_RuntimeValue,
@@ -319,24 +326,54 @@ bool RuntimeValueLegalizationPass::runOnModule(llvm::Module& module)
                 // Create new RuntimeValue call
                 Value* newValue = builder.CreateCall(runtimeValueFunc, builder.getInt32(regionOffset));
 
+                IGC_ASSERT(resolvedOffset >= regionOffset);
                 uint32_t eeOffset = resolvedOffset - regionOffset;
 
-                if (fixedVectorTy)
+                if (fixedVectorTy || is64bit)
                 {
                     // RuntimeValue calls representing vectors of scalars are rewritten due to offset/size change.
-                    // Thus related extract element instructions should be adjusted too.
+                    // Thus related instructions should be adjusted too.
                     std::vector<llvm::User*> users(callToResolve->user_begin(), callToResolve->user_end());
+
+                    bool EEOnly = true;
                     for (llvm::User* const user : users)
                     {
-                        llvm::ExtractElementInst* EEI = llvm::dyn_cast<llvm::ExtractElementInst>(user);
-                        IGC_ASSERT(EEI);
-                        if (EEI)
+                        if (!llvm::isa<llvm::ExtractElementInst>(user))
                         {
+                            EEOnly = false;
+                            break;
+                        }
+                    }
+
+                    if (EEOnly)
+                    {
+                        // Adjust all extract element instructions
+                        for (llvm::User* const user : users)
+                        {
+                            llvm::ExtractElementInst* EEI = llvm::cast<llvm::ExtractElementInst>(user);
                             builder.SetInsertPoint(EEI);
                             EEI->setOperand(0, newValue);
-                            EEI->setOperand(1, builder.CreateAdd(EEI->getIndexOperand(), builder.getInt32(eeOffset)));
+                            if (eeOffset > 0)
+                            {
+                                EEI->setOperand(1, builder.CreateAdd(EEI->getIndexOperand(), builder.getInt32(eeOffset)));
+                            }
                         }
                     }
+                    else
+                    {
+                        // Repack the vector and replace all uses with new one
+                        llvm::Value* repackedVectorVal = llvm::UndefValue::get(
+                            (is64bit ? IGCLLVM::FixedVectorType::get(resolvedBaseType, 2) : fixedVectorTy));
+                        for (unsigned i = 0; i < resolvedSize; i++)
+                        {
+                            repackedVectorVal = builder.CreateInsertElement(
+                                repackedVectorVal,
+                                builder.CreateExtractElement(newValue, builder.getInt32(eeOffset + i)),
+                                builder.getInt32(i));
+                        }
+                        callToResolve->replaceAllUsesWith(
+                            builder.CreateBitCast(repackedVectorVal, callToResolve->getType()));
+                    }
                 }
                 else
                 {
