@@ -79,7 +79,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OpenCLPasses/UndefinedReferences/UndefinedReferencesPass.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/StatelessToStateful/StatelessToStateful.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/DisableLoopUnrollOnRetry/DisableLoopUnrollOnRetry.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/TransformUnmaskedFunctionsPass.h"
+#include "Compiler/Optimizer/OpenCLPasses/TransformUnmaskedFunctionsPass/TransformUnmaskedFunctionsPass.h"
 #include "Compiler/Optimizer/OpenCLPasses/UnreachableHandling/UnreachableHandling.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/WIFuncs/WIFuncResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ScalarArgAsPointer/ScalarArgAsPointer.hpp"
@@ -397,8 +397,11 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         ctx.m_instrTypes.hasUnmaskedRegion) {
         IGC_SET_FLAG_VALUE(allowLICM, false);
     }
-
-    if (IGC_IS_FLAG_ENABLED(ForceAllPrivateMemoryToSLM) ||
+    bool disableConvergentInstructionsHoisting =
+        ctx.m_DriverInfo.DisableConvergentInstructionsHoisting() &&
+        ctx.m_instrTypes.numWaveIntrinsics > 0;
+    if (disableConvergentInstructionsHoisting ||
+        IGC_IS_FLAG_ENABLED(ForceAllPrivateMemoryToSLM) ||
         IGC_IS_FLAG_ENABLED(ForcePrivateMemoryToSLMOnBuffers))
     {
         DummyPass* dummypass = new DummyPass();
@@ -924,12 +927,16 @@ void AddLegalizationPasses(CodeGenContext& ctx, IGCPassManager& mpm, PSSignature
         mpm.add(createVectorBitCastOptPass());
     }
 
+    bool csForceUniformSurfaceSampler = ( ctx.type == ShaderType::COMPUTE_SHADER && ctx.getModuleMetaData()->csInfo.forceUniformSurfaceSampler );
+    bool csForceUniformBuffer = ( ctx.type == ShaderType::COMPUTE_SHADER && ctx.getModuleMetaData()->csInfo.forceUniformBuffer );
     if (ctx.m_instrTypes.hasUniformAssumptions ||
         IGC_IS_FLAG_ENABLED(ForceUniformSurfaceSampler) ||
-        IGC_IS_FLAG_ENABLED(ForceUniformBuffer)) {
+        csForceUniformSurfaceSampler ||
+        IGC_IS_FLAG_ENABLED(ForceUniformBuffer) ||
+        csForceUniformBuffer ) {
         mpm.add(new UniformAssumptions(
-                        IGC_IS_FLAG_ENABLED(ForceUniformSurfaceSampler),
-                        IGC_IS_FLAG_ENABLED(ForceUniformBuffer))
+                        IGC_IS_FLAG_ENABLED(ForceUniformSurfaceSampler) || csForceUniformSurfaceSampler,
+                        IGC_IS_FLAG_ENABLED(ForceUniformBuffer) || csForceUniformBuffer )
         );
     }
 
@@ -1492,12 +1499,12 @@ void OptimizeIR(CodeGenContext* const pContext)
                 }
 
                 if (pContext->type == ShaderType::OPENCL_SHADER &&
-                    pContext->platform.isCoreChildOf(IGFX_XE_HPC_CORE) &&
+                    pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC &&
                     !useStatelessToStateful(*pContext) &&
                     pContext->m_retryManager.IsFirstTry())
                 {
-                    mpm.add(createGEPLoopStrengthReductionPass(IGC_IS_FLAG_ENABLED(allowLICM) &&
-                            pContext->m_retryManager.AllowLICM()));
+                    //mpm.add(createGEPLoopStrengthReductionPass(IGC_IS_FLAG_ENABLED(allowLICM) &&
+                    //        pContext->m_retryManager.AllowLICM()));
                 }
             }
 
@@ -1554,7 +1561,8 @@ void OptimizeIR(CodeGenContext* const pContext)
             }
             GFX_ONLY_PASS { mpm.add(new GenUpdateCB()); }
 
-            if (!pContext->m_instrTypes.hasAtomics && !extensiveShader(pContext))
+            if (IGC_IS_FLAG_ENABLED(EnableJumpThreading) &&
+                !pContext->m_instrTypes.hasAtomics && !extensiveShader(pContext))
             {
                 if (pContext->type == ShaderType::OPENCL_SHADER)
                 {
@@ -1593,7 +1601,8 @@ void OptimizeIR(CodeGenContext* const pContext)
 
             // Conditions apply just as above due to problems with atomics
             // (see comment above for details).
-            if (!pContext->m_instrTypes.hasAtomics && !extensiveShader(pContext))
+            if (IGC_IS_FLAG_ENABLED(EnableJumpThreading) &&
+                !pContext->m_instrTypes.hasAtomics && !extensiveShader(pContext))
             {
                 // After lowering 'switch', run jump threading to remove redundant jumps.
                 mpm.add(llvm::createJumpThreadingPass());
