@@ -48,6 +48,7 @@ class DeSSA;
 class CoalescingEngine;
 class GenXFunctionGroupAnalysis;
 class VariableReuseAnalysis;
+class EmitPass;
 
 struct PushInfo;
 
@@ -104,9 +105,10 @@ public:
     virtual void PreAnalysisPass();
     virtual void ExtractGlobalVariables() {}
     void         EOTURBWrite();
-    void         EOTRenderTarget(CVariable* r1, bool isPerCoarse);
     CVariable* URBFence();
     void         EOTGateway(CVariable* payload = nullptr);
+    void         EOTRenderTarget(CVariable* r1,
+        bool isPerCoarse);
     virtual void AddEpilogue(llvm::ReturnInst* ret);
 
     virtual CVariable* GetURBOutputHandle()
@@ -121,11 +123,18 @@ public:
         IGC_ASSERT_MESSAGE(0, "Should be overridden in a derived class!");
         return nullptr;
     }
+    virtual OctEltUnit GetTotalURBReadLength() const {
+        return OctEltUnit(0);
+    }
+    unsigned GetMaxRegForThreadDispatch() const
+    {
+        return m_simdProgram.m_startReg + 8 * GetTotalURBReadLength().Count();
+    }
 
     // if true, HW will pass one GRF NOS of inlinedata to payload, (compute only right now)
     virtual bool passNOSInlineData() { return false; }
     virtual bool loadThreadPayload() { return false; }
-    virtual unsigned getAnnotatedNumThreads() { return 0; }
+    virtual int getAnnotatedNumThreads() { return -1; }
     virtual bool IsRegularGRFRequested() { return false; }
     virtual bool IsLargeGRFRequested() { return false; }
     virtual bool hasReadWriteImage(llvm::Function& F)
@@ -146,6 +155,7 @@ public:
     void        AddSetup(uint index, CVariable* var);
     bool        AppendPayloadSetup(CVariable* var);
     void        AddPatchTempSetup(CVariable* var);
+    void        AddPatchPredSetup(CVariable* var);
     void        AddPatchConstantSetup(uint index, CVariable* var);
 
     // TODO: simplify calls to GetNewVariable to these shorter and more
@@ -251,6 +261,7 @@ public:
 
     void        AllocateInput(CVariable* var, uint offset, uint instance = 0, bool forceLiveOut = false);
     void        AllocateOutput(CVariable* var, uint offset, uint instance = 0);
+    void        AllocatePred(CVariable* var, uint offset, bool forceLiveOut = false);
     CVariable* ImmToVariable(uint64_t immediate, VISA_Type type, bool isCodePatchCandidate = false);
     CVariable* GetConstant(llvm::Constant* C, CVariable* dstVar = nullptr);
     CVariable* GetScalarConstant(llvm::Value* c);
@@ -263,8 +274,8 @@ public:
     virtual void MapPushedInputs();
     void        CreateGatherMap();
     void        CreateConstantBufferOutput(SKernelProgram* pKernelProgram);
-    void        CreateFunctionSymbol(llvm::Function* pFunc);
-    void        CreateGlobalSymbol(llvm::GlobalVariable* pGlobal);
+    CVariable*  CreateFunctionSymbol(llvm::Function* pFunc);
+    CVariable*  CreateGlobalSymbol(llvm::GlobalVariable* pGlobal);
 
     CVariable*  GetStructVariable(llvm::Value* v);
 
@@ -277,12 +288,38 @@ public:
     void        SetCoalescingEngineHelper(CoalescingEngine* ce) { m_coalescingEngine = ce; }
     void        SetCodeGenHelper(CodeGenPatternMatch* CG) { m_CG = CG; }
     void        SetPushInfoHelper(PushInfo* PI) { pushInfo = *PI; }
+    void        SetEmitPassHelper(EmitPass* EP) { m_EmitPass = EP; }
     void        SetDominatorTreeHelper(llvm::DominatorTree* DT) { m_DT = DT; }
     void        SetDataLayout(const llvm::DataLayout* DL) { m_DL = DL; }
-    void        SetFunctionGroupAnalysis(GenXFunctionGroupAnalysis* FGA) { m_FGA = FGA; }
     void        SetVariableReuseAnalysis(VariableReuseAnalysis* VRA) { m_VRA = VRA; }
     void        SetMetaDataUtils(IGC::IGCMD::MetaDataUtils* pMdUtils) { m_pMdUtils = pMdUtils; }
     void        SetScratchSpaceSize(uint size) { m_ScratchSpaceSize = size; }
+
+    // Set FGA and also FunctionGroup attributes
+    void SetFunctionGroupAnalysis(GenXFunctionGroupAnalysis* FGA)
+    {
+        m_FGA = FGA;
+        FunctionGroup* FG = (FGA && entry) ? FGA->getGroupForHead(entry) : nullptr;
+        if (FG)
+        {
+            m_HasStackCall = FG->hasStackCall();
+            m_HasIndirectCall = FG->hasIndirectCall();
+            m_HasNestedCall = FG->hasNestedCall();
+            m_IsIntelSymbolTableVoidProgram = FGA->isIndirectCallGroup(FG);
+        }
+        if (IGC_IS_FLAG_ENABLED(ForceAddingStackcallKernelPrerequisites))
+        {
+            m_HasStackCall = true;
+        }
+    }
+
+    GenXFunctionGroupAnalysis* GetFGA() { return m_FGA; }
+    bool HasStackCalls() const { return m_HasStackCall; }
+    bool HasNestedCalls() const { return m_HasNestedCall; }
+    bool HasIndirectCalls() const { return m_HasIndirectCall; }
+    bool IsIntelSymbolTableVoidProgram() const { return m_IsIntelSymbolTableVoidProgram; }
+    int  PrivateMemoryPerWI() const { return m_PrivateMemoryPerWI;  }
+
     IGCMD::MetaDataUtils* GetMetaDataUtils() { return m_pMdUtils; }
 
     virtual  void SetShaderSpecificHelper(EmitPass* emitPass) { IGC_UNUSED(emitPass); }
@@ -353,6 +390,7 @@ public:
     SIMDMode m_SIMDSize;
     uint8_t m_numberInstance = 0;
     PushInfo pushInfo;
+    EmitPass* m_EmitPass;
     bool isInputsPulled; //true if any input is pulled, false otherwise
     bool isMessageTargetDataCacheDataPort;
     uint m_sendStallCycle = 0;
@@ -529,8 +567,10 @@ public:
     void SetHasSample() { m_HasSample = true; }
     void IncStatelessWritesCount() { ++m_StatelessWritesCount; }
     void IncIndirectStatelessCount() { ++m_IndirectStatelessCount; }
+    void IncNumSampleBallotLoops() { ++m_NumSampleBallotLoops; }
     uint32_t GetStatelessWritesCount() const { return m_StatelessWritesCount; }
     uint32_t GetIndirectStatelessCount() const { return m_IndirectStatelessCount; }
+    uint32_t GetNumSampleBallotLoops() const { return m_NumSampleBallotLoops; }
 
     // In bytes
     uint32_t getGRFSize() const { return m_Platform->getGRFSize(); }
@@ -573,11 +613,6 @@ public:
     unsigned int GetScalarTypeSizeInRegisterInBits(const llvm::Type* Ty) const;
     unsigned int GetScalarTypeSizeInRegister(const llvm::Type* Ty) const;
 
-    bool HasStackCalls() const { return m_HasStackCalls; }
-    void SetHasStackCalls() { m_HasStackCalls = true; }
-    bool IsIntelSymbolTableVoidProgram() const { return m_isIntelSymbolTableVoidProgram; }
-    void SetIsIntelSymbolTableVoidProgram() { m_isIntelSymbolTableVoidProgram = true; }
-
     ////////////////////////////////////////////////////////////////////
     // NOTE: for vector load/stores instructions pass the
     // optional instruction argument checks additional constraints
@@ -585,7 +620,7 @@ public:
         const CodeGenContext& Ctx,
         llvm::Instruction* vectorLdStInst = nullptr,
         SIMDMode Mode = SIMDMode::UNKNOWN);
-    bool shouldGenerateLSC(llvm::Instruction* vectorLdStInst = nullptr);
+    bool shouldGenerateLSC(llvm::Instruction* vectorLdStInst = nullptr, bool isTGM = false);
     bool forceCacheCtrl(llvm::Instruction* vectorLdStInst = nullptr);
     // Shader has LSC store messages with cache controls specified in `ops`
     void HasLscStoreCacheControls(const LSC_CACHE_OPTS& opts)
@@ -666,6 +701,7 @@ protected:
     std::vector<CVariable*> setup;
     std::vector<CVariable*> payloadLiveOutSetup;
     std::vector<CVariable*> payloadTempSetup;
+    std::vector<CVariable*> payloadPredSetup;
     std::vector<CVariable*> patchConstantSetup;
     std::vector<CVariable*> perPrimitiveSetup;
 
@@ -732,13 +768,20 @@ protected:
     uint32_t m_StatelessWritesCount = 0;
     uint32_t m_IndirectStatelessCount = 0;
 
+    uint32_t m_NumSampleBallotLoops = 0;
+
     DebugInfoData diData;
 
-    bool m_HasStackCalls = false;
-    bool m_isIntelSymbolTableVoidProgram = false;
     // Shader has LSC store messages with non-default L1 cache control
     bool m_HasLscStoresWithNonDefaultL1CacheControls = false;
     bool m_HasSample = false;
+
+    // Program function attributes
+    bool m_HasStackCall = false;
+    bool m_HasNestedCall = false;
+    bool m_HasIndirectCall = false;
+    bool m_IsIntelSymbolTableVoidProgram = false;
+    int m_PrivateMemoryPerWI = 0;
 };
 
 struct SInstContext
