@@ -22,6 +22,7 @@ namespace IGC
 {
     struct RetryState
     {
+        bool allowAddrArithCloning;
         bool allowLICM;
         bool allowCodeSinking;
         bool allowAddressArithmeticSinking;
@@ -36,9 +37,9 @@ namespace IGC
     };
 
     static const RetryState RetryTable[] = {
-        // licm  codSk AdrSk  Slice  PrivM  VISAP  URBWr  Coals  GRF    loadSk
-        { true,  true, false, false, true,  true,  true,  true,  false, false, 1 },
-        { false, true, true,  true,  false, false, false, false, true,  true, 500 }
+       // adrCl  licm  codSk AdrSk  Slice  PrivM  VISAP  URBWr  Coals  GRF    loadSk
+        { false, true,  true, false, false, true,  true,  true,  true,  false, false, 1 },
+        { true,  false, true, true,  true,  false, false, false, false, true,  true, 500 }
     };
 
     static constexpr size_t RetryTableSize = sizeof(RetryTable) / sizeof(RetryState);
@@ -116,6 +117,13 @@ namespace IGC
         return RetryTable[id].allowCodeSinking;
     }
 
+    bool RetryManager::AllowCloneAddressArithmetic(Function* F) const
+    {
+        unsigned id = GetPerFuncRetryStateId(F);
+        IGC_ASSERT(id < RetryTableSize);
+        return RetryTable[id].allowAddrArithCloning;
+    }
+
     bool RetryManager::AllowSimd32Slicing(Function* F) const
     {
         unsigned id = GetPerFuncRetryStateId(F);
@@ -239,7 +247,7 @@ namespace IGC
         return nullptr;
     }
 
-    bool RetryManager::IsBetterThanPrevious(CShaderProgram* pCurrent)
+    bool RetryManager::IsBetterThanPrevious(CShaderProgram* pCurrent, float threshold)
     {
         bool isBetter = true;
         auto pPrevious = GetPrevious(pCurrent);
@@ -253,12 +261,12 @@ namespace IGC
             // Get shaders
             for (auto simd : simdToAnalysis)
             {
-                if (!previousShader &&
+                if (!previousShader && pPrevious->GetShader(simd) &&
                     (pPrevious->GetShader(simd)->ProgramOutput()->m_programSize > 0))
                 {
                     previousShader = pPrevious->GetShader(simd);
                 }
-                if (!currentShader &&
+                if (!currentShader && pCurrent->GetShader(simd) &&
                     (pCurrent->GetShader(simd)->ProgramOutput()->m_programSize > 0))
                 {
                     currentShader = pCurrent->GetShader(simd);
@@ -268,9 +276,11 @@ namespace IGC
             IGC_ASSERT(currentShader);
             IGC_ASSERT(previousShader);
 
-            // Check if current shader has more than 200% of previous spill size
+            // Check if current shader spill is larger than previous shader spill
+            // Threshold flag controls comparison tolerance - i.e. A threshold of 2.0 means that the
+            // current shader spill must be 2x larger than previous spill to be considered "better".
             bool spillSizeBigger =
-                currentShader->m_spillSize > previousShader->m_spillSize << 1;
+                currentShader->m_spillSize > (unsigned int)(previousShader->m_spillSize * threshold);
 
             if (spillSizeBigger)
             {
@@ -335,6 +345,7 @@ namespace IGC
     }
 
     LLVMContextWrapper::LLVMContextWrapper(bool createResourceDimTypes)
+        : m_UserAddrSpaceMD(this)
     {
         if (createResourceDimTypes)
         {
