@@ -55,6 +55,7 @@ SPDX-License-Identifier: MIT
 #include <optional>
 #include <Metrics/IGCMetric.h>
 #include "llvmWrapper/IR/Module.h"
+#include "Compiler/UserAddrSpaceMD.hpp"
 
 /************************************************************************
 This file contains the interface structure and functions to communicate
@@ -147,6 +148,7 @@ namespace IGC
         bool            m_roundPower2KBytes = false;
         bool            m_UseScratchSpacePrivateMemory = true;
         bool            m_SeparatingSpillAndPrivateScratchMemorySpace = false;
+        bool            m_EnableSeparateScratchWA = true;
         unsigned int m_scratchSpaceSizeLimit = 0;
         unsigned int m_numGRFTotal = 128;
         unsigned int m_numGRFSpillFill = 0;
@@ -194,12 +196,14 @@ namespace IGC
             }
         }
 
-        void init(bool roundPower2KBytes, unsigned int scratchSpaceSizeLimitT, bool useScratchSpacePrivateMemory, bool SepSpillPvtSS)
+        void init(bool roundPower2KBytes, unsigned int scratchSpaceSizeLimitT, bool useScratchSpacePrivateMemory, bool SepSpillPvtSS,
+            bool SeparateScratchWA)
         {
             m_roundPower2KBytes = roundPower2KBytes;
             m_scratchSpaceSizeLimit = scratchSpaceSizeLimitT;
             m_UseScratchSpacePrivateMemory = useScratchSpacePrivateMemory;
             m_SeparatingSpillAndPrivateScratchMemorySpace = SepSpillPvtSS;
+            m_EnableSeparateScratchWA = SeparateScratchWA;
         }
 
         // if IGC needs scratch for private memory, we use slot0 for private
@@ -217,7 +221,7 @@ namespace IGC
             {
                 result += (m_scratchSpaceUsedBySpills + m_scratchSpaceUsedByGtpin);
             }
-            else
+            else if (m_EnableSeparateScratchWA)
             {
                 // \TODO: doubts about driver-compiler interface, conservatively set the size
                 // to the max of two slots
@@ -235,9 +239,16 @@ namespace IGC
             unsigned int result = 0;
             if (m_SeparatingSpillAndPrivateScratchMemorySpace && slot0_offset > 0)
             {
-                // \TODO: doubts about driver-compiler interface, conservatively set the size
-                // to the max of two slots
-                result = std::max(slot0_offset, m_scratchSpaceUsedBySpills + m_scratchSpaceUsedByGtpin);
+                if (m_EnableSeparateScratchWA)
+                {
+                    // \TODO: doubts about driver-compiler interface, conservatively set the size
+                    // to the max of two slots
+                    result = std::max(slot0_offset, m_scratchSpaceUsedBySpills + m_scratchSpaceUsedByGtpin);
+                }
+                else
+                {
+                    result = m_scratchSpaceUsedBySpills + m_scratchSpaceUsedByGtpin;
+                }
             }
             result = roundSize(result);
             IGC_ASSERT(result <= m_scratchSpaceSizeLimit);
@@ -544,12 +555,14 @@ namespace IGC
         // encounters a null shader. It is optional because there is
         // no need to compile it for collection state objects.
         llvm::Optional<SBindlessProgram> callStackHandler;
+
+        typedef llvm::SmallVector<SBindlessProgram, 8> BindlessShaderVec;
         // These are the raygen shaders
-        llvm::SmallVector<SBindlessProgram, 4> m_DispatchPrograms;
+        BindlessShaderVec m_DispatchPrograms;
         // Non raygen shaders
-        llvm::SmallVector<SBindlessProgram, 8> m_CallableShaders;
+        BindlessShaderVec m_CallableShaders;
         // Continuation shaders
-        llvm::SmallVector<SBindlessProgram, 8> m_Continuations;
+        BindlessShaderVec m_Continuations;
     };
 
     struct SRayTracingPipelineConfig
@@ -768,7 +781,7 @@ namespace IGC
         /// the set of OCL kernels that need to skip recompilation
         std::set<std::string> kernelSkip;
         // Check if current shader is better then previous one
-        bool IsBetterThanPrevious(CShaderProgram* pCurrent);
+        bool IsBetterThanPrevious(CShaderProgram* pCurrent, float threshold = 1.0f);
         // Get the previous compilation of the current kernel
         CShaderProgram* GetPrevious(CShaderProgram* pCurrent, bool ReleaseUPtr = false);
         // Collect compilation of the current kernel
@@ -818,7 +831,9 @@ namespace IGC
 
     };
 
-    /// this class adds intrinsic cache to LLVM context
+    /// This class:
+    ///    Add intrinsic cache to LLVM context
+    ///    Add llvm metadata cache
     class LLVMContextWrapper : public llvm::LLVMContext
     {
         LLVMContextWrapper(LLVMContextWrapper&) = delete;
@@ -832,6 +847,10 @@ namespace IGC
         /// requested in this context
         typedef llvm::ValueMap<const llvm::Function*, unsigned> SafeIntrinsicIDCacheTy;
         SafeIntrinsicIDCacheTy m_SafeIntrinsicIDCache;
+        /// metadata caching
+        UserAddrSpaceMD m_UserAddrSpaceMD;
+        // structType caching : for unique identified struct type
+        llvm::SmallVector<llvm::StructType*, 16> m_allLayoutStructTypes;
         void AddRef();
         void Release();
     };
@@ -902,6 +921,9 @@ namespace IGC
 
         IGCMetrics::IGCMetric metrics;
 
+        // Used scratch space for private variables
+        llvm::DenseMap<llvm::Function*, uint64_t> m_ScratchSpaceUsage;
+
         // shader stat for opt customization
         uint32_t     m_tempCount = 0;
         uint32_t     m_sampler = 0;
@@ -973,6 +995,8 @@ namespace IGC
         // Ignore per module fast math flag and use only per instruction fast math flags
         // Add few changes to CustomUnsafeOptPass related to fast flag propagation
         bool m_checkFastFlagPerInstructionInCustomUnsafeOptPass = false;
+        // Map to store global offsets in original global buffer
+        std::map<std::string, uint64_t> inlineProgramScopeGlobalOffsets;
 
     private:
         //For storing error message
@@ -1077,6 +1101,15 @@ namespace IGC
         virtual bool enableZEBinary() const;
         bool isPOSH() const;
 
+        UserAddrSpaceMD& getUserAddrSpaceMD() {
+            IGC_ASSERT(llvmCtxWrapper);
+            return llvmCtxWrapper->m_UserAddrSpaceMD;
+        }
+        llvm::SmallVector<llvm::StructType*, 16>& getLayoutStructTypes() {
+            IGC_ASSERT(llvmCtxWrapper);
+            return llvmCtxWrapper->m_allLayoutStructTypes;
+        }
+
         unsigned int GetSIMDInfoOffset(SIMDMode simd, ShaderDispatchMode mode)
         {
             unsigned int offset = 0;
