@@ -24,6 +24,8 @@ SPDX-License-Identifier: MIT
 #include "llvmWrapper/Transforms/Utils/LoopUtils.h"
 #include "common/LLVMWarningsPop.hpp"
 
+#include <algorithm>
+
 using namespace llvm;
 using namespace IGC;
 
@@ -155,9 +157,7 @@ namespace llvm {
     }
 
     void GenIntrinsicsTTIImpl::getUnrollingPreferences(Loop* L,
-#if LLVM_VERSION_MAJOR >= 7
         ScalarEvolution& SE,
-#endif
         TTI::UnrollingPreferences& UP
 #if LLVM_VERSION_MAJOR >= 14
         , OptimizationRemarkEmitter* ORE
@@ -182,6 +182,65 @@ namespace llvm {
                 LoopUnrollThreshold = ctx->getModuleMetaData()->compOpt.SetLoopUnrollThreshold;
             }
         }
+
+        // Special case when DP emulation is needed.
+        if (ctx->m_hasDPEmu && IGC_IS_FLAG_ENABLED(SelectiveLoopUnrollForDPEmu)) {
+            bool hasDPInst = false;
+            for (auto BB : L->blocks()) {
+                for (auto& I : *BB) {
+                    switch (I.getOpcode()) {
+                    case Instruction::FMul:
+                    case Instruction::FAdd:
+                    case Instruction::FSub:
+                    case Instruction::FDiv:
+                        hasDPInst = I.getType()->isDoubleTy();
+                        break;
+                    case Instruction::FCmp:
+                    case Instruction::FPToUI:
+                    case Instruction::FPToSI:
+                    case Instruction::FPTrunc:
+                        hasDPInst = I.getOperand(0)->getType()->isDoubleTy();
+                        break;
+                    case Instruction::UIToFP:
+                    case Instruction::SIToFP:
+                    case Instruction::FPExt:
+                        hasDPInst = I.getType()->isDoubleTy();
+                        break;
+                    case Instruction::Call:
+                    {
+                        if (isa<GenIntrinsicInst>(&I) ||
+                            isa<IntrinsicInst>(&I)) {
+                            CallInst* callI = cast<CallInst>(&I);
+                            hasDPInst = (callI->getType()->isDoubleTy() ||
+                                std::any_of(callI->arg_begin(), callI->arg_end(),
+                                    [](Value* v) {
+                                        return v->getType()->isDoubleTy();
+                                    }));
+                        }
+                        break;
+                    }
+                    default:
+                        break;
+                    }
+
+                    if (hasDPInst)
+                        break;
+                }
+                if (hasDPInst)
+                    break;
+            }
+            if (hasDPInst) {
+                // Disable unroll
+                UP.Threshold = 0;
+                UP.OptSizeThreshold = 0;
+                UP.Count = 1;
+                UP.MaxCount = 1;
+                UP.Partial = false;
+                UP.Runtime = false;
+                return;
+            }
+        }
+
         unsigned totalInstCountInShader = countTotalInstructions(L->getBlocks()[0]->getParent());
         uint32_t registerPressureEst = (uint32_t)(IGC_GET_FLAG_VALUE(SetRegisterPressureThresholdForLoopUnroll) * (ctx->getNumGRFPerThread() / 128.0));
         bool lowPressure = (this->ctx->m_tempCount < registerPressureEst) && (totalInstCountInShader < LoopUnrollThreshold);
@@ -245,12 +304,6 @@ namespace llvm {
             }
         }
 
-#if LLVM_VERSION_MAJOR == 4
-        ScalarEvolution* SE = &dummyPass->getAnalysisIfAvailable<ScalarEvolutionWrapperPass>()->getSE();
-        if (!SE)
-            return;
-#endif
-
         unsigned sendMessage = 0;
         unsigned TripCount = 0;
         BasicBlock* ExitingBlock = L->getLoopLatch();
@@ -438,15 +491,19 @@ namespace llvm {
             UP.MaxCount = UP.Count;
         }
 
-        // do not enable runtime unrolling if the loop is long or trip count is already known.
-        if (instCount > 35 || TripCount)
+        unsigned int runtimeUnroll = IGC_GET_FLAG_VALUE(RuntimeLoopUnrolling); // 0: default, 1: on, 2: off
+        if (runtimeUnroll == 2)
         {
             return;
         }
-
-        if (IGC_IS_FLAG_ENABLED(DisableRuntimeLoopUnrolling))
+        else if (runtimeUnroll == 0)
         {
-            return;
+            // do not enable runtime unrolling if the loop is long or trip count is already known.
+            // skip this check if RuntimeLoopUnrolling is set to force on.
+            if (instCount > 35 || TripCount)
+            {
+                return;
+            }
         }
 
         UP.Runtime = true;
@@ -522,11 +579,7 @@ namespace llvm {
         if (auto* CI = dyn_cast<CallInst>(I))
         {
             if (CI->isConvergent() &&
-#if LLVM_VERSION_MAJOR >= 7
                 CI->onlyAccessesInaccessibleMemory()
-#else
-                CI->hasFnAttr(Attribute::InaccessibleMemOnly)
-#endif
                 )
             {
                 return false;
