@@ -86,10 +86,10 @@ IGC_IR_LLVM_INTRINSIC(canonicalize)
 IGC_IR_LLVM_INTRINSIC(stacksave)
 IGC_IR_LLVM_INTRINSIC(stackrestore)
 IGC_IR_LLVM_INTRINSIC(bitreverse)
-#if LLVM_VERSION_MAJOR >= 8
+IGC_IR_LLVM_INTRINSIC(lround)
+IGC_IR_LLVM_INTRINSIC(llround)
 IGC_IR_LLVM_INTRINSIC(fshl)
 IGC_IR_LLVM_INTRINSIC(fshr)
-#endif
 #if LLVM_VERSION_MAJOR >= 9
 IGC_IR_LLVM_INTRINSIC(usub_sat)
 IGC_IR_LLVM_INTRINSIC(ssub_sat)
@@ -100,6 +100,10 @@ IGC_IR_LLVM_INTRINSIC(sadd_sat)
 #endif
 #if LLVM_VERSION_MAJOR >= 12
 IGC_IR_LLVM_INTRINSIC(abs)
+IGC_IR_LLVM_INTRINSIC(smax)
+IGC_IR_LLVM_INTRINSIC(smin)
+IGC_IR_LLVM_INTRINSIC(umax)
+IGC_IR_LLVM_INTRINSIC(umin)
 IGC_IR_LLVM_INTRINSIC(experimental_noalias_scope_decl)
 #endif
 
