@@ -28,12 +28,12 @@ LegalizeAction InstLegalChecker::visitInstruction(Instruction& I) {
 #endif // LLVM_VERSION_MAJOR >= 10
 
     LLVM_DEBUG(dbgs() << "LEGAL-CHECK: " << I << '\n');
-    IGC_ASSERT_EXIT_MESSAGE(0, "UNKNOWN INSTRUCTION IS BEING LEGAL-CHECKED!");
+    IGC_ASSERT_UNREACHABLE(); // UNKNOWN INSTRUCTION IS BEING LEGAL-CHECKED!
 }
 
 /// Terminator instructions
 ///
-
+/*
 LegalizeAction InstLegalChecker::visitReturnInst(ReturnInst& I) {
     // It's legal iff the return value (if any) is legal.
     if (Value * V = I.getReturnValue())
@@ -47,7 +47,7 @@ LegalizeAction InstLegalChecker::visitTerminatorInst(IGCLLVM::TerminatorInst&) {
     // support 'indirectbr' or 'resume'.
     return Legal;
 }
-
+*/
 /// Standard binary operators
 ///
 
