@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2022 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -68,20 +68,23 @@ bool PeepholeTypeLegalizer::runOnFunction(Function& F) {
          ctx->platform.WaDisableD64ScratchMessage()) &&
         ctx->getModuleMetaData()->compOpt.UseScratchSpacePrivateMemory;
 
+    NonBitcastInstructionsLegalized = false;
+    CastInst_ZExtWithIntermediateIllegalsEliminated = false;
+    CastInst_TruncWithIntermediateIllegalsEliminated = false;
+    Bitcast_BitcastWithIntermediateIllegalsEliminated = false;
+
     IGCLLVM::IRBuilder<> builder(F.getContext());
     m_builder = &builder;
 
     Changed = false;
     visit(F);
-    if (Changed) {
-        NonBitcastInstructionsLegalized = true;
-        visit(F);
-        CastInst_ZExtWithIntermediateIllegalsEliminated = true;
-        visit(F);
-        CastInst_TruncWithIntermediateIllegalsEliminated = true;
-        visit(F);
-        Bitcast_BitcastWithIntermediateIllegalsEliminated = true;
-    }
+    NonBitcastInstructionsLegalized = true;
+    visit(F);
+    CastInst_ZExtWithIntermediateIllegalsEliminated = true;
+    visit(F);
+    CastInst_TruncWithIntermediateIllegalsEliminated = true;
+    visit(F);
+    Bitcast_BitcastWithIntermediateIllegalsEliminated = true;
     return Changed;
 }
 
@@ -110,37 +113,39 @@ void PeepholeTypeLegalizer::visitInstruction(Instruction& I) {
         return;
 
     if (!I.getOperand(0)->getType()->isIntOrIntVectorTy() &&
-        !dyn_cast<ExtractElementInst>(&I))
+        !isa<ExtractElementInst>(&I))
         return; // Legalization for int types only or for extractelements
 
     m_builder->SetInsertPoint(&I);
 
     //Depending on the phase of legalization pass, call appropriate function
     if (!NonBitcastInstructionsLegalized) { // LEGALIZE ALUs first
-        if (dyn_cast<PHINode>(&I)) {
+        if (isa<PHINode>(&I)) {
             legalizePhiInstruction(I);  // phi nodes and all incoming values
         }
-        else if (dyn_cast<UnaryInstruction>(&I)) {
+        else if (isa<UnaryInstruction>(&I)) {
             legalizeUnaryInstruction(I); // pointercast &/or load
         }
-        else if (dyn_cast<ICmpInst>(&I) || dyn_cast<BinaryOperator>(&I) || dyn_cast<SelectInst>(&I)) {
+        else if (isa<ICmpInst>(&I) || isa<BinaryOperator>(&I) || isa<SelectInst>(&I)) {
             legalizeBinaryOperator(I);    // Bitwise and Arithmetic Operations
         }
-        else if (dyn_cast<ExtractElementInst>(&I)) {
+        else if (isa<ExtractElementInst>(&I)) {
             legalizeExtractElement(I);
         }
     }
     else if (!CastInst_ZExtWithIntermediateIllegalsEliminated) { // Eliminate intermediate ILLEGAL operands in bitcast-zext or trunc-zext pairs
-        if (dyn_cast<ZExtInst>(&I))
+        if (isa<ZExtInst>(&I))
             cleanupZExtInst(I);
     }
     else if (!CastInst_TruncWithIntermediateIllegalsEliminated) { // Eliminate intermediate ILLEGAL operands in bitcast-zext or trunc-zext pairs
-        if (dyn_cast<TruncInst>(&I))
+        if (isa<TruncInst>(&I))
             cleanupTruncInst(I);
     }
     else if (!Bitcast_BitcastWithIntermediateIllegalsEliminated) { // Eliminate redundant bitcast-bitcast pairs and eliminate intermediate ILLEGAL operands in bitcast-bitcast pairs with src == dest OR src != dest
-        if (dyn_cast<BitCastInst>(&I))
+        if (isa<BitCastInst>(&I))
             cleanupBitCastInst(I);
+        else if (isa<TruncInst>(&I))
+            cleanupBitCastTruncInst(I);
     }
 }
 
@@ -199,11 +204,11 @@ void PeepholeTypeLegalizer::legalizePhiInstruction(Instruction& I)
     if (!I.getType()->isIntOrIntVectorTy() || isLegalInteger(srcWidth) || srcWidth == 1) // nothing to legalize
         return;
 
-    unsigned quotient, promoteToInt;
+    unsigned quotient = 0, promoteToInt = 0;
     promoteInt(srcWidth, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
-    PHINode* oldPhi = dyn_cast<PHINode>(&I);
-    Value* result;
+    PHINode* oldPhi = cast<PHINode>(&I);
+    Value* result = nullptr;
 
     if (quotient > 1)
     {
@@ -272,7 +277,7 @@ void PeepholeTypeLegalizer::legalizeExtractElement(Instruction& I)
     if (!isLegalInteger(elementWidth) && extract->getType()->isIntOrIntVectorTy())
     {
         unsigned numElements = (unsigned)cast<IGCLLVM::FixedVectorType>(extract->getOperand(0)->getType())->getNumElements();
-        unsigned quotient, promoteToInt;
+        unsigned quotient = 0, promoteToInt = 0;
         promoteInt(elementWidth, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
         m_builder->SetInsertPoint(&I);
@@ -360,7 +365,7 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
         Src2 = I.getOperand(1);
     }
 
-    unsigned quotient, promoteToInt, Src1width;
+    unsigned quotient = 0, promoteToInt = 0, Src1width = 0;
 
     if (!Src1->getType()->isIntOrIntVectorTy())
         return; // Legalization for int types only
@@ -408,25 +413,40 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
             {
                 if (auto val = dyn_cast<ConstantInt>(Src2))
                 {
-                    int64_t offset = val->getSExtValue();
-                    IGC_ASSERT(offset >= 0 && offset < (int64_t)Src1width);
-                    IGC_ASSERT(quotient == 2);
-                    if (offset < promoteToInt)
+                    int64_t ShiftAmt = val->getSExtValue();
+                    IGC_ASSERT(ShiftAmt >= 0 && ShiftAmt < (int64_t)Src1width);
+                    uint64_t EltIdx = ShiftAmt / promoteToInt + Idx;
+                    uint64_t NewShiftAmt = ShiftAmt % promoteToInt;
+
+                    auto getElt = [&](Value* Vec, uint64_t i) -> Value* {
+                        // if EltIdx is OOB of vector, return 0
+                        if (i < cast<IGCLLVM::FixedVectorType>(Vec->getType())->getNumElements())
+                            return m_builder->CreateExtractElement(Vec, i);
+                        else
+                            return ConstantInt::get(IntegerType::get(I.getContext(), promoteToInt), 0, false);
+                    };
+
+                    if (NewShiftAmt == 0)
                     {
-                        NewInst = m_builder->CreateLShr(m_builder->CreateExtractElement(NewLargeSrc1VecForm, Idx), offset);
-                        if (Idx == 0)
-                        {
-                            Value* shrVal = NewInst;
-                            Value* shlVal = m_builder->CreateShl(m_builder->CreateExtractElement(NewLargeSrc1VecForm, 1), promoteToInt - offset);
-                            NewInst = m_builder->CreateOr(shrVal, shlVal);
-                        }
+                        // Simple case: we can just extract parts
+                        NewInst = getElt(NewLargeSrc1VecForm, EltIdx);
                     }
                     else
                     {
-                        if (Idx == 0)
-                            NewInst = m_builder->CreateLShr(m_builder->CreateExtractElement(NewLargeSrc1VecForm, 1), offset - promoteToInt);
-                        else
-                            NewInst = ConstantInt::get(IntegerType::get(I.getContext(), promoteToInt), 0, false);
+                        auto lshr = [&](Value* Op0, uint64_t ShiftAmt) -> Value* {
+                            if (auto* C = dyn_cast<Constant>(Op0); C && C->isNullValue())
+                                return C;
+                            return m_builder->CreateLShr(Op0, ShiftAmt);
+                        };
+                        auto shl = [&](Value* Op0, uint64_t ShiftAmt) -> Value* {
+                            if (auto* C = dyn_cast<Constant>(Op0); C && C->isNullValue())
+                                return C;
+                            return m_builder->CreateShl(Op0, ShiftAmt);
+                        };
+                        // V[Idx] = (V[EltIdx] >> NewShiftAmt) | (V[EltIdx + 1] << (promoteToInt - NewShiftAmt))
+                        NewInst = m_builder->CreateOr(
+                            lshr(getElt(NewLargeSrc1VecForm, EltIdx), NewShiftAmt),
+                            shl(getElt(NewLargeSrc1VecForm, EltIdx + 1), promoteToInt - NewShiftAmt));
                     }
                 }
                 else
@@ -437,13 +457,12 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
                 break;
             }
             case Instruction::Mul:
-                IGC_ASSERT(quotient == 2);
                 if (Idx == 0)
                 {
                     NewInst = m_builder->CreateMul(m_builder->CreateExtractElement(NewLargeSrc1VecForm, Idx),
                         m_builder->CreateExtractElement(NewLargeSrc2VecForm, Idx));
                 }
-                else
+                else if (Idx == 1)
                 {
                     Type* type = llvm::Type::getIntNTy(I.getContext(), promoteToInt);
                     Function* MulHFunc = llvm::GenISAIntrinsic::getDeclaration(
@@ -461,8 +480,20 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
                     Value* MulLo2Hi1 = m_builder->CreateMul(Lo2, Hi1);
                     Value* AddLoHi = m_builder->CreateAdd(MulLo1Hi2, MulLo2Hi1);
                     Value* AddMulHi = m_builder->CreateAdd(AddLoHi, MulHiLo1Lo2);
-                    uint64_t mask = (1ULL << (Src1width - promoteToInt)) - 1;
-                    NewInst = m_builder->CreateAnd(AddMulHi, mask);
+                    if (Src1width < promoteToInt * 2)
+                    {
+                        uint64_t mask = (1ULL << (Src1width - promoteToInt)) - 1;
+                        NewInst = m_builder->CreateAnd(AddMulHi, mask);
+                    }
+                    else
+                    {
+                        NewInst = AddMulHi;
+                    }
+                }
+                else
+                {
+                    IGC_ASSERT_MESSAGE(0, "Mul legalization for width > 64 (quotient => 3) is not fully supported");
+                    NewInst = ConstantInt::get(IntegerType::get(I.getContext(), promoteToInt), 0, false);
                 }
                 break;
             case Instruction::Add:
@@ -613,7 +644,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
             if (isLegalInteger(DL->getPointerTypeSizeInBits(I.getType())) || DL->getPointerTypeSizeInBits(I.getType()) == 1)
                 return;
 
-            unsigned quotient, promoteToInt, Src1width = DL->getPointerTypeSizeInBits(I.getType());
+            unsigned quotient = 0, promoteToInt = 0, Src1width = DL->getPointerTypeSizeInBits(I.getType());
             promoteInt(Src1width, quotient, promoteToInt, 8);// byte level addressing
 
             if (quotient > 1)
@@ -674,7 +705,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
         if (isLegalInteger(I.getType()->getScalarSizeInBits()) || I.getType()->getScalarSizeInBits() == 1)
             return; // Nothing to legalize
 
-        unsigned loadQuotient, loadPromoteToInt, loadSrcWidth = DL->getPointerTypeSizeInBits(I.getOperand(0)->getType());
+        unsigned loadQuotient = 0, loadPromoteToInt = 0, loadSrcWidth = DL->getPointerTypeSizeInBits(I.getOperand(0)->getType());
         promoteInt(loadSrcWidth, loadQuotient, loadPromoteToInt, 8); // hard coded to 8 since our hardware is bte addressable.
 
         if (loadQuotient > 1) {
@@ -691,7 +722,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
             Value* newBitCastBackToScalar = m_builder->CreateBitCast(newMaskedLoad, Type::getIntNTy(I.getContext(), loadPromoteToInt * loadQuotient));
 
             //extend new large scalar to a scalar length of legal vector of MAX_LEGAL_INT
-            unsigned quotient, promoteToInt, SrcWidth = DL->getPointerTypeSizeInBits(I.getOperand(0)->getType());
+            unsigned quotient = 0, promoteToInt = 0, SrcWidth = DL->getPointerTypeSizeInBits(I.getOperand(0)->getType());
             promoteInt(SrcWidth, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
             Value* newZextInst = m_builder->CreateZExt(newBitCastBackToScalar, Type::getIntNTy(I.getContext(), promoteToInt * quotient));
@@ -725,7 +756,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
 
         if (I.getOperand(0)->getType()->getIntegerBitWidth() == 1)
         {
-            unsigned quotient, promoteToInt, Src1width = I.getOperand(0)->getType()->getIntegerBitWidth();
+            unsigned quotient = 0, promoteToInt = 0, Src1width = I.getOperand(0)->getType()->getIntegerBitWidth();
             promoteInt(Src1width, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
             Value* Src1 = I.getOperand(0);
@@ -753,6 +784,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
         // %4 = extractelement %1, 1
         // %5 = insertelement %3, %4, 1
         // %6 = bitcast <2 x i64> %5 to i128
+
         unsigned dstSize = I.getType()->getScalarSizeInBits();
         unsigned srcSize = I.getOperand(0)->getType()->getScalarSizeInBits();
 
@@ -775,18 +807,20 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
             return;
         }
 
-        unsigned numSrcElements = srcSize / promotedInt;
-        unsigned numDstElements = dstSize / promotedInt;
+        unsigned numSrcElements = static_cast<unsigned>(I.getOperand(0)->getType()->getPrimitiveSizeInBits() / promotedInt);
+        unsigned numDstElements = static_cast<unsigned>(I.getType()->getPrimitiveSizeInBits() / promotedInt);
         Type* srcVecTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numSrcElements);
         Type* dstVecTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numDstElements);
 
         // Bitcast the illegal src type to a legal vector
         Value* srcVec = m_builder->CreateBitCast(I.getOperand(0), srcVecTy);
         Value* dstVec = UndefValue::get(dstVecTy);
+        unsigned numElements = I.getType()->isVectorTy() ? (unsigned)cast<IGCLLVM::FixedVectorType>(I.getType())->getNumElements() : 1;
 
         for (unsigned i = 0; i < numDstElements; i++)
         {
-            Value* v = m_builder->CreateExtractElement(srcVec, m_builder->getInt32(i));
+            Value* v = m_builder->CreateExtractElement(srcVec, m_builder->getInt32((i / (numDstElements / numElements)) *
+                (numSrcElements / numElements) + (i % (numDstElements / numElements))));
             dstVec = m_builder->CreateInsertElement(dstVec, v, m_builder->getInt32(i));
         }
         // Cast back to original dst type
@@ -821,7 +855,7 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
     case Instruction::Trunc:
     {
         //then we only need to mask out the truncated bits
-        unsigned quotient, promoteToInt, Src1width = prevInst->getOperand(0)->getType()->getIntegerBitWidth();
+        unsigned quotient = 0, promoteToInt = 0, Src1width = prevInst->getOperand(0)->getType()->getIntegerBitWidth();
         promoteInt(Src1width, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
         int activeBits = prevInst->getType()->getIntegerBitWidth(); // or I.getoperand(0)->getType()->getIntegerBitWidth();
@@ -863,8 +897,8 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
                     Value* truncSrcAsVec = m_builder->CreateBitCast(prevInst->getOperand(0),
                         IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
                     Value* vecRes = UndefValue::get(IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
-                    Value* Elmt;
-                    Value* maskedElmt;
+                    Value* Elmt = nullptr;
+                    Value* maskedElmt = nullptr;
 
                     for (unsigned Idx = 0; Idx < quotient; ++Idx)
                     {
@@ -881,6 +915,7 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
                 }
                 else {
                     // this is a place holder, but DO NOT expect to need an implementation for this case.
+                    IGC_ASSERT_MESSAGE(0, "Not yet implemented");
                 }
             }
             else {
@@ -916,6 +951,7 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
         }
         else { // (promoteToInt*quotient != Src1width) case
                // No support yet
+            IGC_ASSERT_MESSAGE(0, "Not yet implemented");
         }
     }
     break;
@@ -938,7 +974,7 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
         ALU (%7)
         */
 
-        unsigned quotient, promoteToInt, srcWidth = I.getOperand(0)->getType()->getScalarSizeInBits();
+        unsigned quotient = 0, promoteToInt = 0, srcWidth = I.getOperand(0)->getType()->getScalarSizeInBits();
         promoteInt(srcWidth, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
         if (quotient * promoteToInt != I.getType()->getScalarSizeInBits()) {
@@ -955,7 +991,7 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
         unsigned Idx = 0;
 
         for (unsigned o = 0; o < quotient; ++o) {
-            Value* NewVal, * Hi;
+            Value* NewVal = nullptr, * Hi = nullptr;
             unsigned i = 0;
 
             NewVal = m_builder->CreateZExt(
@@ -1058,6 +1094,13 @@ void PeepholeTypeLegalizer::cleanupTruncInst(Instruction& I) {
             //%q = and i8 %in, 31
             //%out = zext i8 %q to i32
 
+            //Example 3:
+            //%a = trunc i32 %in to i4
+            //%out = uitofp i4 %a to float
+            //=>
+            //%q = and i32 %in, 15
+            //%out = uitofp i32 %q to float
+
             if (isa<SExtInst>(castInst))
             {
                 auto shiftAmt = castInst_bitsize - Trunc_bitsize;
@@ -1070,6 +1113,11 @@ void PeepholeTypeLegalizer::cleanupTruncInst(Instruction& I) {
                 auto inst1 = m_builder->CreateAnd(Src, (1 << Trunc_bitsize) - 1);
                 new_inst = m_builder->CreateZExt(inst1, castInst->getType());
             }
+            else if (isa<UIToFPInst>(castInst))
+            {
+                auto inst1 = m_builder->CreateAnd(Src, (1 << Trunc_bitsize) - 1);
+                new_inst = m_builder->CreateUIToFP(inst1, castInst->getType());
+            }
         }
         else if (Src_bitsize > castInst_bitsize)
         {
@@ -1095,10 +1143,28 @@ void PeepholeTypeLegalizer::cleanupTruncInst(Instruction& I) {
             Changed = true;
         }
     }
+    else
+    {
+        if (TruncInst* prevTrunc = dyn_cast<TruncInst>(I.getOperand(0)))
+        {
+            //Example:
+            //%1 = trunc i96 %in to i65
+            //%out = trunc i65 %1 to i64
+            //=>
+            //%out = trunc i96 %in to i64
+            IGC_ASSERT(prevTrunc->getType()->getScalarSizeInBits() > I.getType()->getScalarSizeInBits());
+            auto* newTrunc = cast<TruncInst>(m_builder->CreateTrunc(prevTrunc->getOperand(0), I.getType()));
+            I.replaceAllUsesWith(newTrunc);
+            Changed = true;
+        }
+    }
 
     if (I.use_empty())
     {
+        Instruction* prevInst = dyn_cast<Instruction>(I.getOperand(0));
         I.eraseFromParent();
+        if (prevInst && prevInst->use_empty())
+            prevInst->eraseFromParent();
         Changed = true;
     }
 
@@ -1107,23 +1173,22 @@ void PeepholeTypeLegalizer::cleanupTruncInst(Instruction& I) {
 
 void PeepholeTypeLegalizer::cleanupBitCastInst(Instruction& I) {
 
-    /*
-    Need to handle:
-    1.    bitcast
-    2.    bitcast addrspace*
-
-    a.    bitcast iSrc , iILLEGAL
-        bitcast iILLEGAL, iSrc
-    b.    bitcast iSrc, iILLEGAL
-        bitcast iILLEGAL, iLEGAL
-    */
-
     Instruction* prevInst = dyn_cast<Instruction>(I.getOperand(0));
     if (!prevInst)
         return;
     switch (prevInst->getOpcode()) {
     case Instruction::BitCast:
     {
+        /*
+        Need to handle:
+        1.    bitcast
+        2.    bitcast addrspace*
+
+        a.    bitcast iSrc , iILLEGAL
+            bitcast iILLEGAL, iSrc
+        b.    bitcast iSrc, iILLEGAL
+            bitcast iILLEGAL, iLEGAL
+        */
         Type* srcType = prevInst->getOperand(0)->getType();
         Type* dstType = I.getType();
         if (srcType == dstType)
@@ -1176,7 +1241,137 @@ void PeepholeTypeLegalizer::cleanupBitCastInst(Instruction& I) {
         }
         break;
     }
+    case Instruction::ZExt:
+    {
+        Type* srcType = prevInst->getOperand(0)->getType();
+        Type* midType = prevInst->getType();
+        Type* dstType = I.getType();
+        if (isLegalInteger(srcType->getScalarSizeInBits())
+            && !isLegalInteger(midType->getScalarSizeInBits())
+            && isLegalInteger(dstType->getScalarSizeInBits()))
+        {
+            m_builder->SetInsertPoint(&I);
+
+            IGC_ASSERT_MESSAGE(midType->getScalarSizeInBits() % 8 == 0, "Unexpected type");
+            int interimTypeBitWidth = DL->getLargestLegalIntTypeSizeInBits();
+            for (; interimTypeBitWidth >= 8; interimTypeBitWidth /= 2)
+            {
+                if (srcType->getScalarSizeInBits() % interimTypeBitWidth == 0
+                    && midType->getScalarSizeInBits() % interimTypeBitWidth == 0)
+                    break;
+            }
+            Value* newInVecValue = prevInst->getOperand(0);
+            if (srcType->getScalarSizeInBits() != interimTypeBitWidth)
+            {
+                Type* newInVecType = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), interimTypeBitWidth),
+                   static_cast<unsigned>(srcType->getPrimitiveSizeInBits() / interimTypeBitWidth));
+                newInVecValue = m_builder->CreateBitCast(newInVecValue, newInVecType);
+            }
+            Value* newExtVec = UndefValue::get(IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(),
+                interimTypeBitWidth), static_cast<unsigned>(dstType->getPrimitiveSizeInBits() / interimTypeBitWidth)));
+            unsigned numElements = srcType->isVectorTy() ? (unsigned)cast<IGCLLVM::FixedVectorType>(srcType)->getNumElements() : 1;
+            unsigned newInQuotient = srcType->getScalarSizeInBits() / interimTypeBitWidth;
+            unsigned extQuotient = static_cast<unsigned>(dstType->getPrimitiveSizeInBits() / numElements / interimTypeBitWidth);
+            auto zero = ConstantInt::get(IntegerType::get(I.getContext(), interimTypeBitWidth), 0, false);
+            for (unsigned i = 0; i < numElements; i++) {
+                for (unsigned k = 0; k < newInQuotient; k++) {
+                    Value* extractedVal = m_builder->CreateExtractElement(newInVecValue, m_builder->getInt32(newInQuotient * i + k));
+                    newExtVec = m_builder->CreateInsertElement(newExtVec, extractedVal, m_builder->getInt32(extQuotient * i + k));
+                }
+                for (unsigned k = newInQuotient; k < extQuotient; k++) {
+                    newExtVec = m_builder->CreateInsertElement(newExtVec, zero, m_builder->getInt32(extQuotient * i + k));
+                }
+            }
+            if (dstType->getScalarSizeInBits() != newExtVec->getType()->getScalarSizeInBits())
+            {
+                newExtVec = m_builder->CreateBitCast(newExtVec, dstType);
+            }
+            I.replaceAllUsesWith(newExtVec);
+            I.eraseFromParent();
+            if (prevInst->use_empty())
+            {
+                prevInst->eraseFromParent();
+            }
+            Changed = true;
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(isLegalInteger(midType->getScalarSizeInBits()),
+                "Unexpected illegal type width");
+        }
+        break;
+    }
     default:
+        IGC_ASSERT_MESSAGE(isLegalInteger(I.getOperand(0)->getType()->getScalarSizeInBits()),
+            "Unexpected illegal type width");
         break;
     }
 }
+
+void PeepholeTypeLegalizer::cleanupBitCastTruncInst(Instruction& I) {
+
+    BitCastInst* bitCastInst = dyn_cast<BitCastInst>(I.getOperand(0));
+    if (!bitCastInst)
+        return;
+
+    if (isLegalInteger(bitCastInst->getOperand(0)->getType()->getScalarSizeInBits())
+        && !isLegalInteger(bitCastInst->getType()->getScalarSizeInBits()) &&
+        isLegalInteger(I.getType()->getScalarSizeInBits()))
+    {
+        /*
+            Example:
+            %2 = bitcast <3 x i32> %0 to <2 x i48>
+            %3 = trunc <2 x i48> %2 to <2 x i16>
+            =>
+            %2 = bitcast <3 x i32> %0 to <6 x i16>
+            %3 = extractelement <6 x i16> %2, i32 0
+            %4 = insertelement <2 x i16> undef, i16 %3, i32 0
+            %5 = extractelement <6 x i16> %2, i32 3
+            %6 = insertelement <2 x i16> %4, i16 %5, i32 1
+        */
+
+        m_builder->SetInsertPoint(&I);
+
+        Type* srcType = bitCastInst->getOperand(0)->getType();
+        Type* midType = bitCastInst->getType();
+        Type* dstType = I.getType();
+
+        IGC_ASSERT_MESSAGE(midType->getScalarSizeInBits() % 8 == 0, "Unexpected type");
+        int interimTypeBitWidth = DL->getLargestLegalIntTypeSizeInBits();
+        for (; interimTypeBitWidth >= 8; interimTypeBitWidth /= 2)
+        {
+            if (midType->getScalarSizeInBits() % interimTypeBitWidth == 0
+                && dstType->getScalarSizeInBits() % interimTypeBitWidth == 0)
+                break;
+        }
+        Value* newInVecValue = bitCastInst->getOperand(0);
+        if (srcType->getScalarSizeInBits() != interimTypeBitWidth)
+        {
+            Type* newInVecType = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(),
+                interimTypeBitWidth), static_cast<unsigned>(midType->getPrimitiveSizeInBits() / interimTypeBitWidth));
+            newInVecValue = m_builder->CreateBitCast(newInVecValue, newInVecType);
+        }
+        Value* newTruncVec = UndefValue::get(IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(),
+            interimTypeBitWidth), static_cast<unsigned>(dstType->getPrimitiveSizeInBits() / interimTypeBitWidth)));
+        unsigned numElements = dstType->isVectorTy() ? (unsigned)cast<IGCLLVM::FixedVectorType>(dstType)->getNumElements() : 1;
+        unsigned newInQuotient = midType->getScalarSizeInBits() / interimTypeBitWidth;
+        unsigned truncQuotient = static_cast<unsigned>(dstType->getPrimitiveSizeInBits() / numElements / interimTypeBitWidth);
+        for (unsigned i = 0; i < numElements; i++) {
+            for (unsigned k = 0; k < truncQuotient; k++) {
+                Value* extractedVal = m_builder->CreateExtractElement(newInVecValue, m_builder->getInt32(newInQuotient * i + k));
+                newTruncVec = m_builder->CreateInsertElement(newTruncVec, extractedVal, m_builder->getInt32(truncQuotient * i + k));
+            }
+        }
+        if (dstType->getScalarSizeInBits() != newTruncVec->getType()->getScalarSizeInBits())
+        {
+            newTruncVec = m_builder->CreateBitCast(newTruncVec, dstType);
+        }
+        I.replaceAllUsesWith(newTruncVec);
+        I.eraseFromParent();
+        if (bitCastInst->use_empty())
+        {
+            bitCastInst->eraseFromParent();
+        }
+        Changed = true;
+    }
+}
