@@ -7,7 +7,7 @@ SPDX-License-Identifier: MIT
 ============================= end_copyright_notice ===========================*/
 
 #include "ModuleAllocaAnalysis.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/KernelArgs/KernelArgs.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
@@ -201,7 +201,10 @@ bool ModuleAllocaAnalysis::safeToUseScratchSpace() const
 
         // if one API doesn't support stateless, we should try to use smallest dispatch mode
         // which can hold more pvt_data to avoid error out.
-        if (Ctx.platform.hasScratchSurface() && Ctx.m_DriverInfo.supportsSeparatingSpillAndPrivateScratchMemorySpace() && !supportsStatelessSpacePrivateMemory)
+        if (Ctx.platform.hasScratchSurface() &&
+            Ctx.m_DriverInfo.supportsSeparatingSpillAndPrivateScratchMemorySpace() &&
+            !Ctx.getModuleMetaData()->disableSeparateSpillPvtScratchSpace &&
+            !supportsStatelessSpacePrivateMemory)
             simd_size = numLanes(Ctx.platform.getMinDispatchMode());
 
         unsigned maxScratchSpaceBytes = Ctx.platform.maxPerThreadScratchSpace();
