@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -62,7 +62,20 @@ namespace {
                 AS1 != ADDRESS_SPACE_GENERIC &&
                 AS2 != ADDRESS_SPACE_GENERIC &&
                 AS1 != AS2)
-                return IGCLLVM::AliasResultEnum::NoAlias;
+            {
+                bool isDisjointMemory = true;
+                if (CGC.allocatePrivateAsGlobalBuffer() && CGC.getModuleMetaData()->genericAccessesResolved)
+                {
+                    const bool isPrivateGlobal =
+                        (AS1 == ADDRESS_SPACE_PRIVATE && AS2 == ADDRESS_SPACE_GLOBAL) ||
+                        (AS1 == ADDRESS_SPACE_GLOBAL && AS2 == ADDRESS_SPACE_PRIVATE);
+
+                    isDisjointMemory = !isPrivateGlobal;
+                }
+
+                if(isDisjointMemory)
+                    return IGCLLVM::AliasResultEnum::NoAlias;
+            }
 
 
             // Shared local memory doesn't alias any stateful memory.
@@ -128,15 +141,17 @@ namespace {
             bool OrLocal) {
             // Pointers to constant address space memory, well, point to constant memory
             PointerType* ptrType = dyn_cast<PointerType>(Loc.Ptr->getType());
-            if (ptrType && ptrType->getAddressSpace() == ADDRESS_SPACE_CONSTANT)
-                return true;
+            if (ptrType) {
+                if (ptrType->getAddressSpace() == ADDRESS_SPACE_CONSTANT)
+                    return true;
 
-            bool DirectIdx;
-            unsigned BufId;
-            BufferType BufTy = DecodeAS4GFXResource(ptrType->getAddressSpace(),
-                DirectIdx, BufId);
-            if (BufTy == CONSTANT_BUFFER)
-                return true;
+                bool DirectIdx;
+                unsigned BufId;
+                BufferType BufTy = DecodeAS4GFXResource(ptrType->getAddressSpace(),
+                    DirectIdx, BufId);
+                if (BufTy == CONSTANT_BUFFER)
+                    return true;
+            }
 
             return AAResultBase::pointsToConstantMemory(Loc,
 #if LLVM_VERSION_MAJOR >= 9
@@ -163,12 +178,11 @@ namespace {
         bool doInitialization(Module& M) override {
             if(M.size() > 0)
             {
-                Result.reset(new AddressSpaceAAResult(
-                getAnalysis<TargetLibraryInfoWrapperPass>().getTLI(
 #if LLVM_VERSION_MAJOR >= 10
-                    *M.begin()
+                auto& F = *M.begin(); // see llvmWrapper/Analysis/TargetLibraryInfo.h
 #endif
-                    ),
+                Result.reset(new AddressSpaceAAResult(
+                    getAnalysis<TargetLibraryInfoWrapperPass>().getTLI(),
                     *getAnalysis<CodeGenContextWrapper>().getCodeGenContext()));
             }
             return false;
