@@ -133,7 +133,7 @@ auto AlignmentAnalysis::getAlignValue(Value* V) const
             if (!align)
             {
                 Type* gvType = GV->getType();
-                return m_DL->getABITypeAlignment(gvType->getPointerElementType());
+                return m_DL->getABITypeAlignment(IGCLLVM::getNonOpaquePtrEltTy(gvType));
             }
             else
             {
@@ -149,7 +149,7 @@ auto AlignmentAnalysis::getAlignValue(Value* V) const
         if (arg->getType()->isPointerTy())
         {
             // Pointer arguments are guaranteed to be aligned on the ABI alignment
-            Type* pointedTo = arg->getType()->getPointerElementType();
+            Type* pointedTo = IGCLLVM::getNonOpaquePtrEltTy(arg->getType());
             if (pointedTo->isSized())
             {
                 return m_DL->getABITypeAlignment(pointedTo);
@@ -187,7 +187,7 @@ bool AlignmentAnalysis::processInstruction(llvm::Instruction* I)
         // If a pointer is specifically given an 'align' field in the MD, use it.
         MDNode* alignmentMD = I->getMetadata("align");
         if (alignmentMD)
-            newAlign = (alignment_t)mdconst::dyn_extract<ConstantInt>(alignmentMD->getOperand(0))->getZExtValue();
+            newAlign = (alignment_t)mdconst::extract<ConstantInt>(alignmentMD->getOperand(0))->getZExtValue();
     }
     if (!newAlign)
     {
@@ -220,13 +220,13 @@ alignment_t AlignmentAnalysis::visitInstruction(Instruction& I)
 alignment_t AlignmentAnalysis::visitAllocaInst(AllocaInst& I)
 {
     // Return the alignment of the alloca, which ought to be correct
-    auto newAlign = (alignment_t)I.getAlignment();
+    auto newAlign = (alignment_t)IGCLLVM::getAlignmentValue(&I);
 
     // If the alloca uses the default alignment, pull it from the datalayout
     if (!newAlign)
     {
         Type* allocaType = I.getType();
-        newAlign = m_DL->getABITypeAlignment(allocaType->getPointerElementType());
+        newAlign = m_DL->getABITypeAlignment(IGCLLVM::getNonOpaquePtrEltTy(allocaType));
     }
 
     return newAlign;
@@ -286,14 +286,14 @@ void AlignmentAnalysis::SetInstAlignment(LoadInst& I)
 {
     // Set the align attribute of the load according to the detected
     // alignment of its operand.
-    I.setAlignment(IGCLLVM::getCorrectAlign(iSTD::Max(I.getAlignment(), getAlignValue(I.getPointerOperand()))));
+    I.setAlignment(IGCLLVM::getCorrectAlign(iSTD::Max(IGCLLVM::getAlignmentValue(&I), getAlignValue(I.getPointerOperand()))));
 }
 
 void AlignmentAnalysis::SetInstAlignment(StoreInst& I)
 {
     // Set the align attribute of the store according to the detected
     // alignment of its operand.
-    I.setAlignment(IGCLLVM::getCorrectAlign(iSTD::Max(I.getAlignment(), getAlignValue(I.getPointerOperand()))));
+    I.setAlignment(IGCLLVM::getCorrectAlign(iSTD::Max(IGCLLVM::getAlignmentValue(&I), getAlignValue(I.getPointerOperand()))));
 }
 
 alignment_t AlignmentAnalysis::visitAdd(BinaryOperator& I)
@@ -434,62 +434,6 @@ alignment_t AlignmentAnalysis::visitCallInst(CallInst& I)
         // return value does not matter
         return MinimumAlignment;
 
-// deprecated code. TODO: remove?
-#if LLVM_VERSION_MAJOR < 7
-    StringRef calleeName = callee->getName();
-    IntrinsicInst* Intri = dyn_cast<IntrinsicInst>(&I);
-    llvm::Intrinsic::ID ID = llvm::Intrinsic::ID::not_intrinsic;
-
-    if (Intri)
-        ID = Intri->getIntrinsicID();
-
-    if ((Intri && ID == Intrinsic::memcpy) ||
-        (!Intri && calleeName.startswith("__builtin_IB_memcpy_")))
-    {
-        unsigned int origAlign = MinimumAlignment;
-        if (ConstantInt * CI = dyn_cast<ConstantInt>(I.getArgOperand(3)))
-        {
-            origAlign = (unsigned int)CI->getZExtValue();
-        }
-        unsigned int dstAlign = getAlignValue(I.getArgOperand(0));
-        unsigned int srcAlign = getAlignValue(I.getArgOperand(1));
-
-        // Need to get the Min of dest alignment and src alignment
-        unsigned int minAlign = iSTD::Min(dstAlign, srcAlign);
-
-        // As the original intrinsic's alignemnt is the minimum one, use
-        // max to get a better alignment.
-        if (minAlign > origAlign)
-        {
-            I.setArgOperand(3, ConstantInt::get(I.getArgOperand(3)->getType(), minAlign));
-        }
-
-        // return value does not matter
-        return MinimumAlignment;
-    }
-
-    if (Intri && ID == Intrinsic::memset)
-    {
-        unsigned int origAlign = MinimumAlignment;
-        if (ConstantInt * CI = dyn_cast<ConstantInt>(I.getArgOperand(3)))
-        {
-            origAlign = (unsigned int)CI->getZExtValue();
-        }
-        unsigned int dstAlign = getAlignValue(I.getArgOperand(0));
-
-        // As the original intrinsic's alignemnt is the minimum one, use
-        // max to get a better alignment.
-        if (dstAlign > origAlign)
-        {
-            I.setArgOperand(3, ConstantInt::get(I.getArgOperand(3)->getType(), dstAlign));
-        }
-
-        // return value does not matter
-        return MinimumAlignment;
-    }
-    return MinimumAlignment;
-#elif LLVM_VERSION_MAJOR >= 7
-
     MemIntrinsic* memIntr = dyn_cast<MemIntrinsic>(&I);
     if (!memIntr)
         return MinimumAlignment;
@@ -511,6 +455,16 @@ alignment_t AlignmentAnalysis::visitCallInst(CallInst& I)
 
     return alignment;
 
+}
+
+// Add Max utilities here instead of the WrapperLLVM module so as to avoid scope pollution.
+namespace IGCLLVM {
+    // Covers alignment_t and llvm::Align
+    using iSTD::Max;
+#if LLVM_VERSION_MAJOR >= 10
+    inline llvm::MaybeAlign Max(llvm::MaybeAlign LHS, llvm::MaybeAlign RHS) {
+        return LHS && RHS && *LHS > *RHS ? *LHS : *RHS;
+    }
 #endif
 }
 
@@ -518,40 +472,22 @@ void AlignmentAnalysis::SetInstAlignment(MemSetInst& I)
 {
     // Set the align attribute of the memset according to the detected
     // alignment of its operand.
-#if LLVM_VERSION_MAJOR >= 14
-    uint64_t alignment_value = getAlignValue(I.getRawDest());
-    llvm::Align alignment = llvm::max(I.getDestAlign(), llvm::Align(alignment_value));
+    auto alignment = IGCLLVM::Max(IGCLLVM::getDestAlign(I), IGCLLVM::Align(getAlignValue(I.getRawDest())));
     I.setDestAlignment(alignment);
-#else
-    unsigned alignment = iSTD::Max(I.getDestAlignment(), getAlignValue(I.getRawDest()));
-    I.setDestAlignment(alignment);
-#endif
 }
 
 void AlignmentAnalysis::SetInstAlignment(MemCpyInst& I)
 {
     // Set the align attribute of the memcpy based on the minimum alignment of its source and dest fields
-#if LLVM_VERSION_MAJOR >= 14
-    uint64_t alignment_value = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
-    llvm::Align alignment = llvm::Align(alignment_value);
+    auto minRawAlignment = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
+    auto alignment = IGCLLVM::Max(IGCLLVM::getDestAlign(I), IGCLLVM::Align(minRawAlignment));
     I.setDestAlignment(alignment);
-#else
-    unsigned alignment = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
-    alignment = iSTD::Max(I.getDestAlignment(), alignment);
-    I.setDestAlignment(alignment);
-#endif
 }
 
 void AlignmentAnalysis::SetInstAlignment(MemMoveInst& I)
 {
     // Set the align attribute of the memmove based on the minimum alignment of its source and dest fields
-#if LLVM_VERSION_MAJOR >= 14
-    uint64_t alignment_value = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
-    llvm::Align alignment = llvm::max(I.getDestAlign(), llvm::Align(alignment_value));
+    auto minRawAlignment = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
+    auto alignment = IGCLLVM::Max(IGCLLVM::getDestAlign(I), IGCLLVM::Align(minRawAlignment));
     I.setDestAlignment(alignment);
-#else
-    unsigned alignment = iSTD::Min(getAlignValue(I.getRawDest()), getAlignValue(I.getRawSource()));
-    alignment = iSTD::Max(I.getDestAlignment(), alignment);
-    I.setDestAlignment(alignment);
-#endif
 }
