@@ -10,6 +10,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OpenCLPasses/LocalBuffers/InlineLocalsResolution.hpp"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
+#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/DebugInfo/Utils.h"
 #include "common/LLVMWarningsPush.hpp"
@@ -318,6 +319,7 @@ void InlineLocalsResolution::collectInfoOnSharedLocalMem(Module& M)
             }
             const unsigned int numThreadsPerEU = platform.ThreadCount / platform.EUCount;
             unsigned int simdSizeUsed = 8;
+            simdSizeUsed = numLanes(pCtx->platform.getMinDispatchMode());
             unsigned int maxWS = maxNumEUsPerSubSlice * numThreadsPerEU * simdSizeUsed;
             if (!iSTD::IsPowerOfTwo(maxWS))
             {
@@ -519,7 +521,7 @@ void InlineLocalsResolution::computeOffsetList(Module& M, llvm::MapVector<Functi
 
             // And the total size after this local is added
             PointerType* ptrType = dyn_cast<PointerType>(G->getType());
-            Type* varType = ptrType->getPointerElementType();
+            Type* varType = IGCLLVM::getNonOpaquePtrEltTy(ptrType);
             if (G == m_pGV)
             {
                 // it is GetMemPoolPtr usage
