@@ -156,8 +156,8 @@ void OpenCLPrintfResolution::visitCallInst(CallInst& callInst)
     }
 
     StringRef  funcName = callInst.getCalledFunction()->getName();
-    if (funcName == OpenCLPrintfAnalysis::OPENCL_PRINTF_FUNCTION_NAME)
-    {
+    if (funcName == OpenCLPrintfAnalysis::OPENCL_PRINTF_FUNCTION_NAME ||
+        funcName == OpenCLPrintfAnalysis::BUILTIN_PRINTF_FUNCTION_NAME) {
         m_printfCalls.push_back(&callInst);
     }
 }
@@ -226,11 +226,6 @@ std::string OpenCLPrintfResolution::getEscapedString(const ConstantDataSequentia
 
 Value* OpenCLPrintfResolution::processPrintfString(Value* arg, Function& F)
 {
-    if (m_CGContext->enableZEBinary())
-    {
-        return arg;
-    }
-
     GlobalVariable* formatString = nullptr;
 
     if (isa<GlobalVariable>(arg))
@@ -241,6 +236,12 @@ Value* OpenCLPrintfResolution::processPrintfString(Value* arg, Function& F)
             IGC_ASSERT_MESSAGE(0, "Unexpected printf argument (expected string literal)");
             return ConstantInt::get(m_int32Type, -1);
         }
+
+        if (m_CGContext->enableZEBinary())
+        {
+            return arg;
+        }
+
         ConstantDataArray* formatStringConst = dyn_cast<ConstantDataArray>(formatString->getInitializer());
         std::string escaped_string = getEscapedString(formatStringConst);
 
@@ -358,15 +359,39 @@ std::string OpenCLPrintfResolution::getPrintfStringsMDNodeName(Function& F)
     return "printf.strings";
 }
 
-static StoreInst* genStoreInternal(Value* Val, Value* Ptr, BasicBlock* InsertAtEnd, DebugLoc DL)
+static StoreInst* genStoreInternal(Value* Val, Value* Ptr, BasicBlock* InsertAtEnd, DebugLoc DL, bool isNontemporal)
 {
     bool isVolatile = false;
     unsigned Align = 4;
     auto SI = new llvm::StoreInst(Val, Ptr, isVolatile, IGCLLVM::getCorrectAlign(Align), InsertAtEnd);
     SI->setDebugLoc(DL);
+    if (isNontemporal) {
+        Constant *One = ConstantInt::get(Type::getInt32Ty(SI->getContext()), 1);
+        MDNode *Node =
+            MDNode::get(SI->getContext(), ConstantAsMetadata::get(One));
+        SI->setMetadata(LLVMContext::MD_nontemporal, Node);
+    }
     return SI;
 }
 
+void OpenCLPrintfResolution::removeExcessArgs()
+{
+    SPrintfArgDescriptor* formatStringArgDesc = &m_argDescriptors[0];
+
+    Value* formatString = formatStringArgDesc->value;
+    IGC::SHADER_PRINTF_TYPE dataType = formatStringArgDesc->argType;
+    IGC_ASSERT(dataType == SHADER_PRINTF_STRING_LITERAL);
+
+    if (auto GV = dyn_cast<GlobalVariable>(formatString))
+    {
+        IGC_ASSERT(GV->hasInitializer());
+
+        unsigned int numFormatSpecifiers = getNumFormatSpecifiers(cast<ConstantDataArray>(GV->getInitializer()));
+        if (m_argDescriptors.size() > numFormatSpecifiers + 1)
+            m_argDescriptors.erase(m_argDescriptors.begin() + numFormatSpecifiers + 1, m_argDescriptors.end());
+    }
+}
+
 void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
 {
     /* Replace a printf call with IR instructions that fill the printf
@@ -419,10 +444,23 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
               // printf returns -1 if failed                               |
               return_val = -1;                                             /
          }
+
+         We also support printf to any provided buffer.
+         This is done with special builtin with following signature:
+         int __builtin_IB_printf_to_buffer(global char* buf, global char* currentOffset, int bufSize, ...);
+           buf - pointer to the begging of the buffer.
+           currentOffset - pointer to the location with the current offset that will be atomically incremented.
+             In the case of regular printf this offset is on the first DWORD of printfBuffer.
+             E.g. in assert buffer it is on the third DWORD.
+            bufSize - total size of the buffer.
+         Note: in the case of builtin printf, all the stores will be nontemporal.
+
+
        ----------------------------------------------------------------------
     */
     MetaDataUtils* MdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     ImplicitArgs implicitArgs(F, MdUtils);
+    bool isPrintfBuiltin = OpenCLPrintfAnalysis::isBuiltinPrintf(printfCall.getCalledFunction());
 
     BasicBlock* currentBBlock = printfCall.getParent();
 
@@ -430,11 +468,17 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
     // Scalarize vector arguments and substitute string arguments by their indices.
     preprocessPrintfArgs(printfCall);
 
+    removeExcessArgs();
+
     // writeOffset = atomic_add(bufferPtr, dataSize)
-    Value* basebufferPtr = implicitArgs.getImplicitArgValue(F, ImplicitArg::PRINTF_BUFFER, MdUtils);
+    Value *basebufferPtr = isPrintfBuiltin
+                               ? printfCall.getArgOperand(0)
+                               : implicitArgs.getImplicitArgValue(
+                                     F, ImplicitArg::PRINTF_BUFFER, MdUtils);
 
     Value* dataSizeVal = ConstantInt::get(m_int32Type, getTotalDataSize());
-    Instruction* writeOffsetStart = genAtomicAdd(basebufferPtr, dataSizeVal, printfCall, "write_offset");
+    Value* currentOffsetPtr = isPrintfBuiltin ? printfCall.getArgOperand(1) : basebufferPtr;
+    Instruction* writeOffsetStart = genAtomicAdd(currentOffsetPtr, dataSizeVal, printfCall, "write_offset");
     writeOffsetStart->setDebugLoc(m_DL);
 
     Instruction* writeOffset = writeOffsetStart;
@@ -444,7 +488,11 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
     Instruction* endOffset = BinaryOperator::CreateAdd(writeOffset, dataSizeVal, "end_offset", &printfCall);
     endOffset->setDebugLoc(m_DL);
 
-    Value* bufferMaxSize = ConstantInt::get(m_int32Type, m_CGContext->m_DriverInfo.getPrintfBufferSize());
+    Value *bufferMaxSize =
+        isPrintfBuiltin
+            ? printfCall.getArgOperand(2)
+            : ConstantInt::get(m_int32Type,
+                               m_CGContext->m_DriverInfo.getPrintfBufferSize());
 
     // write_ptr = buffer_ptr + write_offset;
     if (m_ptrSizeIntType != writeOffset->getType())
@@ -509,7 +557,7 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
             writeOffsetPtr = CastInst::Create(Instruction::CastOps::IntToPtr, writeOffset,
                 m_int32Type->getPointerTo(ADDRESS_SPACE_GLOBAL), "write_offset_ptr", bblockTrue);
             writeOffsetPtr->setDebugLoc(m_DL);
-            genStoreInternal(argTypeVal, writeOffsetPtr, bblockTrue, m_DL);
+            genStoreInternal(argTypeVal, writeOffsetPtr, bblockTrue, m_DL, isPrintfBuiltin);
 
             // write_offset += 4
             writeOffset = BinaryOperator::CreateAdd(writeOffset, constVal4, "write_offset", bblockTrue);
@@ -521,7 +569,7 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
                 writeOffsetPtr = CastInst::Create(Instruction::CastOps::IntToPtr, writeOffset,
                     m_int32Type->getPointerTo(ADDRESS_SPACE_GLOBAL), "write_offset_ptr", bblockTrue);
                 writeOffsetPtr->setDebugLoc(m_DL);
-                genStoreInternal(vecSizeVal, writeOffsetPtr, bblockTrue, m_DL);
+                genStoreInternal(vecSizeVal, writeOffsetPtr, bblockTrue, m_DL, isPrintfBuiltin);
 
                 // write_offset += 4
                 writeOffset = BinaryOperator::CreateAdd(writeOffset, constVal4, "write_offset", bblockTrue);
@@ -542,7 +590,7 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
         }
 
         // *write_offset = argument[i].value
-        genStoreInternal(printfArg, writeOffsetPtr, bblockTrue, m_DL);
+        genStoreInternal(printfArg, writeOffsetPtr, bblockTrue, m_DL, isPrintfBuiltin);
 
         // write_offset += argument[i].size
         Value* offsetInc = ConstantInt::get(m_ptrSizeIntType, getArgTypeSize(dataType, argDesc->vecSize));
@@ -582,7 +630,7 @@ void OpenCLPrintfResolution::expandPrintfCall(CallInst& printfCall, Function& F)
         "write_offset_ptr",
         bblockErrorString);
     writeOffsetPtr->setDebugLoc(m_DL);
-    genStoreInternal(constValErrStringIdx, writeOffsetPtr, bblockErrorString, m_DL);
+    genStoreInternal(constValErrStringIdx, writeOffsetPtr, bblockErrorString, m_DL, isPrintfBuiltin);
     brInst = BranchInst::Create(bblockFalseJoin, bblockErrorString);
     brInst->setDebugLoc(m_DL);
 
@@ -678,9 +726,38 @@ Value* OpenCLPrintfResolution::fixupPrintfArg(CallInst& printfCall, Value* arg,
     return arg;
 }
 
+unsigned OpenCLPrintfResolution::getNumFormatSpecifiers(const ConstantDataArray* dataArray)
+{
+    unsigned int count = 0;
+    StringRef formatString = dataArray->getRawDataValues();
+
+    const size_t length = formatString.size();
+    for (size_t i = 0; i < length; i++)
+    {
+        if (formatString[i] == '%')
+        {
+            if (i + 1 < length && formatString[i + 1] == '%')
+            {
+                i++;
+                continue;
+            }
+
+            count++;
+        }
+    }
+
+    return count;
+}
+
 void OpenCLPrintfResolution::preprocessPrintfArgs(CallInst& printfCall)
 {
-    for (int i = 0, numArgs = IGCLLVM::getNumArgOperands(&printfCall); i < numArgs; ++i)
+    int i = 0;
+    if (OpenCLPrintfAnalysis::isBuiltinPrintf(printfCall.getCalledFunction())) {
+        // printf builtin function has buffer pointer, current offset pointer and buffer size as first three arguments.
+        // Skip them here, as we want to collect the arguments starting from format string.
+        i = 3;
+    }
+    for (int numArgs = IGCLLVM::getNumArgOperands(&printfCall); i < numArgs; ++i)
     {
         Value* arg = printfCall.getOperand(i);
         Type* argType = arg->getType();
