@@ -10,7 +10,7 @@ SPDX-License-Identifier: MIT
 #include "AdaptorCommon/RayTracing/RTBuilder.h"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryResolution.hpp"
 #include "Compiler/ModuleAllocaAnalysis.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/KernelArgs/KernelArgs.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CISACodeGen/EmitVISAPass.hpp"
@@ -77,6 +77,7 @@ namespace IGC {
             unsigned int operandIndex;
         };
 
+        void expandPrivateMemoryForVla(uint32_t &maxPrivateMem);
         static bool testTransposedMemory(const Type* pTmpType, const Type* const pTypeOfAccessedObject, uint64_t tmpAllocaSize, const uint64_t bufferSizeLimit);
 
         /// @brief  The module level alloca information
@@ -121,6 +122,21 @@ void PrivateMemoryResolution::getAnalysisUsage(llvm::AnalysisUsage& AU) const
     AU.addRequired<ModuleAllocaAnalysis>();
 }
 
+void PrivateMemoryResolution::expandPrivateMemoryForVla(uint32_t &maxPrivateMem)
+{
+    // Add another 4KB if there are VLAs
+    maxPrivateMem += 4096;
+    std::string maxPrivateMemValue = std::to_string(maxPrivateMem);
+    std::string fullWarningMessage = "VLA has been detected, the private memory size is set to " + maxPrivateMemValue + "B. "
+        "You can change the size by setting flag ForcePerThreadPrivateMemorySize to a value from [1024:20480]. "
+        "Greater values can affect performance, and lower ones may lead to incorrect results of your program.\n"
+        "To make sure your program runs correctly you can set flag StackOverflowDetection to 1. "
+        "This flag will print \"Stack overflow detected!\" if insufficient memory value has lead to stack overflow. "
+        "It should be used for debugging only as it affects performance.";
+
+    getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitWarning(fullWarningMessage.c_str());
+}
+
 bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
 {
     // Get the analysis
@@ -193,13 +209,8 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
                 if (!FGA && hasVLA)
                 {
                     uint32_t maxPrivateMem = funcMD->second.privateMemoryPerWI;
-                    // Add another 4KB if there are VLAs
-                    maxPrivateMem += 4096;
-                    std::string maxPrivateMemValue = std::to_string(maxPrivateMem);
-                    std::string fullWarningMessage = "VLA has been detected, the private memory size is set to " + maxPrivateMemValue + "B. You can change the size by setting flag ForcePerThreadPrivateMemorySize to a value from [1024:20480]. Greater values can affect performance, and lower ones may lead to incorrect results of your program";
+                    expandPrivateMemoryForVla(maxPrivateMem);
 
-                    // Now, you can pass the concatenated warning message (const char*) to the EmitWarning function
-                    getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitWarning(fullWarningMessage.c_str());
                     maxPrivateMem = std::max(maxPrivateMem, Ctx.getPrivateMemoryMinimalSizePerThread());
                     maxPrivateMem = std::max(maxPrivateMem, (uint32_t)(IGC_GET_FLAG_VALUE(ForcePerThreadPrivateMemorySize)));
                     modMD.PrivateMemoryPerFG[m_currFunction] = maxPrivateMem;
@@ -316,13 +327,7 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
             }
             if (FG->hasVariableLengthAlloca())
             {
-                // Add another 4KB if there are VLAs
-                maxPrivateMem += 4096;
-                std::string maxPrivateMemValue = std::to_string(maxPrivateMem);
-                std::string fullWarningMessage = "VLA has been detected, the private memory size is set to " + maxPrivateMemValue + "B. You can change the size by setting flag ForcePerThreadPrivateMemorySize to a value from [1024:20480]. Greater values can affect performance, and lower ones may lead to incorrect results of your program";
-
-                // Now, you can pass the concatenated warning message (const char*) to the EmitWarning function
-                getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitWarning(fullWarningMessage.c_str());
+                expandPrivateMemoryForVla(maxPrivateMem);
             }
             maxPrivateMem = std::max(maxPrivateMem, Ctx.getPrivateMemoryMinimalSizePerThread());
             maxPrivateMem = std::max(maxPrivateMem, (uint32_t)(IGC_GET_FLAG_VALUE(ForcePerThreadPrivateMemorySize)));
@@ -725,6 +730,21 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         return false;
     }
 
+    for (AllocaInst* aInst : allocaInsts)
+    {
+        auto allocationSize = aInst->getAllocationSizeInBits(m_currFunction->getParent()->getDataLayout());
+
+        if (allocationSize)
+        {
+            auto scratchUsage_i = Ctx.m_ScratchSpaceUsage.find(m_currFunction);
+            if (scratchUsage_i == Ctx.m_ScratchSpaceUsage.end())
+            {
+                Ctx.m_ScratchSpaceUsage.insert({ m_currFunction, 0 });
+            }
+            Ctx.m_ScratchSpaceUsage[m_currFunction] += allocationSize.getValue() / 8;
+        }
+    }
+
     if (Ctx.m_instrTypes.numAllocaInsts > IGC_GET_FLAG_VALUE(AllocaRAPressureThreshold))
     {
         sinkAllocaSingleUse(allocaInsts);
@@ -1144,6 +1164,9 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
         if (!safe32bitOffset)
             perLaneOffset = builder.CreateZExt(perLaneOffset, typeInt64);
         Value* totalOffset = builder.CreateAdd(bufferOffsetForThread, perLaneOffset, VALUE_NAME(pAI->getName() + ".totalOffset"));
+        if (m_currFunction->getParent()->getDataLayout().getPointerSize() == 8)
+            // Manually zero-extend the offset to 64-bits to prevent it from being sign-extended by InstructionCombining
+            totalOffset = builder.CreateZExt(totalOffset, typeInt64);
         Value* privateBufferGEP = builder.CreateGEP(privateMemPtr, totalOffset, VALUE_NAME(pAI->getName() + ".privateBufferGEP"));
         Value* privateBuffer = builder.CreatePointerCast(privateBufferGEP, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
 
