@@ -10,6 +10,7 @@ SPDX-License-Identifier: MIT
 
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/CodeGenContextWrapper.hpp"
+#include "Compiler/ModuleAllocaAnalysis.hpp"
 
 #include <llvm/Pass.h>
 
@@ -41,6 +42,7 @@ namespace IGC
             AU.setPreservesCFG();
             AU.addRequired<MetaDataUtilsWrapper>();
             AU.addRequired<CodeGenContextWrapper>();
+            AU.addRequired<ModuleAllocaAnalysis>();
         }
 
         virtual bool runOnModule(Module& M) override;
