@@ -101,9 +101,9 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
             }
             // Include the variable in 'inlineProgramScopeOffsets', otherwise
             // code gen will not emit relocation. In case of S_UNDEF/extenal
-            // variable offset is not expected to be used. Using -1 as a
-            // placeholder so the unexpected usage will fail visibly when
-            // trying to emit variable with 0xffffffff offset
+            // variable offset is not expected to be used.
+            // Using -1 to communicate to ProgramScopeConstantResolution that
+            // this offset shouldn't be resolved.
             inlineProgramScopeOffsets[globalVar] = -1;
             continue;
         }
@@ -187,22 +187,8 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
 
         if (initializer->isZeroValue())
         {
-            // For zero initialized values, we dont need to copy the data, just tell driver how much to allocate
-            // However, if it's used as a pointer value, we need to do patching and therefore cannot defer the offset calculation
-            bool hasPointerUser = false;
-            for (auto UI : globalVar->users())
-            {
-                if (isa<Constant>(UI) && UI->getType()->isPointerTy())
-                {
-                    hasPointerUser = true;
-                    break;
-                }
-            }
-            if (!hasPointerUser)
-            {
-                zeroInitializedGlobals.push_back(globalVar);
-                continue;
-            }
+            zeroInitializedGlobals.push_back(globalVar);
+            continue;
         }
 
         // Align the buffer.
@@ -245,6 +231,22 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         offset += (unsigned)(m_DL->getTypeAllocSize(IGCLLVM::getNonOpaquePtrEltTy(globalVar->getType())));
     }
 
+    // Patch the offsets for usages of zero initialized globals after those offsets have been calculated in the previous step.
+    // TODO: Remove this logic after enabling ZeBinary, since we will switch the patching to use relocation table instead.
+    for (ZeroInitPatchInfo &patchData : m_PatchLaterDataVector)
+    {
+        char* whereToPatch = std::get<0>(patchData);
+        unsigned patchSize = std::get<1>(patchData);
+        GlobalVariable* globalVar = std::get<2>(patchData);
+        uint64_t offset = std::get<3>(patchData);
+
+        auto iter = inlineProgramScopeOffsets.find(globalVar);
+        IGC_ASSERT(iter != inlineProgramScopeOffsets.end());
+
+        const uint64_t patchOffset = iter->second + offset;
+        memcpy_s(whereToPatch, patchSize, (char*)&patchOffset, patchSize);
+    }
+
     if (inlineProgramScopeOffsets.size())
     {
         // Add globals tracked in metadata to the "llvm.used" list so they won't be deleted by optimizations
@@ -264,12 +266,18 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
     // Stackcalls:
     //  Stackcall ABI does not allow implicit args, so rely on relocation for global variable access
 
+    // TODO: Plan to disable ConstBase and GlobalBase implicit arguments on PVC+.
+    // StatelessToStateful pass is not enabled for most of the cases so there
+    // is no benefit to add the implicit arguments. Const/Global variables
+    // access can just go through relocations.
+
     // Workaround: When there is stringConstants in the module, do not insert
     // implicit arguments to prevent const vars getting promoted
     // at statelessToStateful pass. In zebin path, stateful promotion
     // of const vars can't work well with printf strings.
     bool skipConstAndGlobalBaseArgs =
-             Ctx->enableZEBinary() && !m_pModuleMd->stringConstants.empty();
+          IGC_IS_FLAG_ENABLED(DisableConstBaseGlobalBaseArg) ||
+          (Ctx->enableZEBinary() && !m_pModuleMd->stringConstants.empty());
 
     if (!skipConstAndGlobalBaseArgs && hasInlineConstantBuffer)
     {
@@ -340,7 +348,17 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
     const bool changed = !inlineProgramScopeOffsets.empty();
     for (auto offset : inlineProgramScopeOffsets)
     {
-        m_pModuleMd->inlineProgramScopeOffsets[offset.first] = offset.second;
+        std::string globalName = offset.first->getName().str();
+        if (Ctx->m_retryManager.IsFirstTry())
+        {
+            m_pModuleMd->inlineProgramScopeOffsets[offset.first] = static_cast<uint64_t>(offset.second);
+            Ctx->inlineProgramScopeGlobalOffsets[globalName] = static_cast<uint64_t>(offset.second);
+        }
+        else
+        {
+            IGC_ASSERT_MESSAGE(Ctx->inlineProgramScopeGlobalOffsets[globalName], "No offset recorded for global during initial compilation");
+            m_pModuleMd->inlineProgramScopeOffsets[offset.first] = Ctx->inlineProgramScopeGlobalOffsets[globalName];
+        }
     }
 
     // Update LLVM metadata based on IGC MetadataUtils
@@ -470,20 +488,26 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
                 }
                 else
                 {
+                    // Add the patching info for runtime
+                    pointerOffsetInfoList.push_back(PointerOffsetInfo(addressSpace, inlineProgramScopeBuffer.size(), pointedToAddrSpace));
+
                     auto iter = inlineProgramScopeOffsets.find(ptrBase);
-                    IGC_ASSERT(iter != inlineProgramScopeOffsets.end());
+                    if (iter != inlineProgramScopeOffsets.end())
+                    {
+                        const uint64_t pointeeOffset = iter->second + offset;
+                        // For old patching logic, write the offset relative to the entire global/constant buffer where the base global resides.
+                        // The base address of the buffer will be added to it at runtime.
+                        inlineProgramScopeBuffer.insert(inlineProgramScopeBuffer.end(), (char*)&pointeeOffset, ((char*)&pointeeOffset) + pointerSize);
+                    }
+                    else
+                    {
+                        // If we can't find the base global variable, it must be a zero initialized value whose data is not directly copied.
+                        // Save the info for now, and patch it later once we have the offsets for this zeroinit global vars.
+                        unsigned patchIdx = inlineProgramScopeBuffer.size();
+                        inlineProgramScopeBuffer.insert(inlineProgramScopeBuffer.end(), pointerSize, 0);
 
-                    const uint64_t pointeeOffset = iter->second + offset;
-
-                    pointerOffsetInfoList.push_back(
-                        PointerOffsetInfo(
-                            addressSpace,
-                            inlineProgramScopeBuffer.size(),
-                            pointedToAddrSpace));
-
-                    // For old patching logic, write the offset relative to the entire global/constant buffer where the base global resides.
-                    // The base address of the buffer will be added to it at runtime.
-                    inlineProgramScopeBuffer.insert(inlineProgramScopeBuffer.end(), (char*)&pointeeOffset, ((char*)&pointeeOffset) + pointerSize);
+                        m_PatchLaterDataVector.push_back(std::make_tuple((char*)(&inlineProgramScopeBuffer[patchIdx]), pointerSize, ptrBase, offset));
+                    }
                 }
             }
             else
