@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2021 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -126,22 +126,16 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
             continue;
         }
 
-        // Get the offset of this constant from the base.
-        int offset = -1;
-
         auto bufferOffset = modMD->inlineProgramScopeOffsets.find(pGlobalVar);
-        if (bufferOffset != modMD->inlineProgramScopeOffsets.end())
-        {
-            offset = bufferOffset->second;
-        }
+        if (bufferOffset == modMD->inlineProgramScopeOffsets.end())
+            continue; // This constant is not used, so it didn't get an offset.
 
-        // This constant is not used, so it didn't get an offset.
+        // Get the offset of this constant from the base.
+        int64_t offset = bufferOffset->second;
         if (offset == -1)
-        {
             continue;
-        }
 
-        ConstantInt* pOffset = ConstantInt::get(Type::getInt32Ty(C), offset);
+        ConstantInt* pOffset = ConstantInt::get(Type::getInt64Ty(C), offset);
         const ImplicitArg::ArgType argType =
             AS == ADDRESS_SPACE_GLOBAL ? ImplicitArg::GLOBAL_BASE : ImplicitArg::CONSTANT_BASE;
 
