@@ -18,7 +18,7 @@ SPDX-License-Identifier: MIT
 #include "Probe/Assertion.h"
 #include "IGC/AdaptorCommon/RayTracing/RTBuilder.h"
 #include "IGC/AdaptorCommon/RayTracing/RTStackFormat.h"
-#include "IGC/AdaptorCommon/RayTracing/RayTracingRayDispatchGlobalData.h"
+#include "IGC/AdaptorCommon/RayTracing/API/RayDispatchGlobalData.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -38,7 +38,7 @@ namespace {
     {"__builtin_IB_intel_get_global_btd_stack",          &ResolveOCLRaytracingBuiltins::handleGetGlobalBTDStack     },
     {"__builtin_IB_intel_dispatch_trace_ray_query",      &ResolveOCLRaytracingBuiltins::handleDispatchTraceRayQuery },
     {"__builtin_IB_intel_rt_sync",                       &ResolveOCLRaytracingBuiltins::handleRTSync                },
-    {"__builtin_IB_intel_get_implicit_dispatch_globals", &ResolveOCLRaytracingBuiltins::handleGetImplicitDG         },
+    {"__builtin_IB_intel_get_rt_global_buffer",          &ResolveOCLRaytracingBuiltins::handleGetRTGlobalBuffer     },
 
     // Handling for builtins operating on intel_ray_query_t from intel_rt_production extension
       {"__builtin_IB_intel_init_ray_query",                &ResolveOCLRaytracingBuiltins::handleInitRayQuery          },
@@ -72,8 +72,7 @@ bool ResolveOCLRaytracingBuiltins::runOnModule(Module& M) {
     visit(M);
 
     if (m_callsToReplace.size() > 0) {
-        if (m_pCtx->platform.getPlatformInfo().eRenderCoreFamily != IGFX_XE_HPC_CORE &&
-            m_pCtx->platform.getPlatformInfo().eProductFamily != IGFX_DG2) {
+        if (!m_pCtx->platform.supportRayTracing()) {
             IGC_ASSERT_MESSAGE(0, "Raytracing extensions used on unsupported platform!");
             m_pCtx->EmitError("OCL raytracing extensions can be used only on supported platform", *m_callsToReplace.begin());
             return false;
@@ -375,12 +374,12 @@ void ResolveOCLRaytracingBuiltins::handleRTSync(CallInst& callInst) {
 
 /*
 Handler for
-void __builtin_IB_intel_get_implicit_dispatch_globals();
+void __builtin_IB_intel_get_rt_global_buffer();
 
 Description:
 Returns IMPLICIT_RT_GLOBAL_BUFFER implicit argument.
 */
-void ResolveOCLRaytracingBuiltins::handleGetImplicitDG(llvm::CallInst& callInst) {
+void ResolveOCLRaytracingBuiltins::handleGetRTGlobalBuffer(llvm::CallInst& callInst) {
     RTBuilder rtbuilder(m_builder->getContext(), *m_pCtx);
     rtbuilder.SetInsertPoint(&callInst);
     auto v = rtbuilder.getGlobalBufferPtr();
@@ -406,7 +405,7 @@ void ResolveOCLRaytracingBuiltins::handleInitRayQuery(llvm::CallInst& callInst)
     unsigned numArgs = IGCLLVM::getNumArgOperands(&callInst);
     IGC_ASSERT(numArgs == 5);
 
-    auto* allocaType = callInst.getType()->getPointerElementType();
+    auto* allocaType = IGCLLVM::getNonOpaquePtrEltTy(callInst.getType());
     auto* alloca = rtbuilder.CreateAlloca(allocaType);
 
     rtbuilder.SetInsertPoint(&callInst);
@@ -510,8 +509,8 @@ Instruction* ResolveOCLRaytracingBuiltins::loadFromOffset(Value* basePtr, const
 
 #define RT_DISPATCH_GETTER_DEF(FieldName) \
 Instruction* ResolveOCLRaytracingBuiltins::FieldName##Getter(Value* rtDispatchGlobalsValue) {  \
-  constexpr size_t offset = offsetof(RayDispatchGlobalData, FieldName);                        \
-  constexpr size_t typeSize = sizeof(((RayDispatchGlobalData*)0)->FieldName);                  \
+  constexpr size_t offset = offsetof(RayDispatchGlobalData::RT::Xe, FieldName);                \
+  constexpr size_t typeSize = sizeof(((RayDispatchGlobalData::RT::Xe*)0)->FieldName);          \
   return loadFromOffset(rtDispatchGlobalsValue, offset, typeSize, #FieldName);                 \
 }
 
