@@ -7,7 +7,8 @@ SPDX-License-Identifier: MIT
 ============================= end_copyright_notice ===========================*/
 
 #include "Compiler/Optimizer/OpenCLPasses/ResourceAllocator/ResourceAllocator.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/KernelArgs/KernelArgs.hpp"
+#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "Compiler/MetaDataApi/MetaDataApi.h"
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
@@ -163,6 +164,7 @@ static AllocationType getAllocationType(KernelArg::ArgType argType, BindlessAllo
     case KernelArg::ArgType::IMPLICIT_DEVICE_ENQUEUE_EVENT_POOL:
     case KernelArg::ArgType::IMPLICIT_DEVICE_ENQUEUE_DEFAULT_DEVICE_QUEUE:
     case KernelArg::ArgType::IMPLICIT_BINDLESS_OFFSET:
+    case KernelArg::ArgType::IMPLICIT_ASSERT_BUFFER:
         return AllocationType::Other;
 
     default:
@@ -327,9 +329,19 @@ bool ResourceAllocator::runOnFunction(llvm::Function& F)
             break;
 
         case AllocationType::Other:
-            argAlloc.type = ResourceTypeEnum::UAVResourceType;
-            argAlloc.indexType = numUAVs;
-            numUAVs++;
+            if (ctx->platform.supportDynamicBTIsAllocation())
+            {
+                // Use default arg allocator for UAV resources
+                // Intentionally not allocating index here, since it will
+                // be done in StatelessToSatefull or PromoteStatelessToBindless
+                argAlloc = defaultArgAlloc;
+            }
+            else
+            {
+                argAlloc.type = ResourceTypeEnum::UAVResourceType;
+                argAlloc.indexType = numUAVs;
+                numUAVs++;
+            }
             break;
 
         default:
