@@ -7,6 +7,7 @@ SPDX-License-Identifier: MIT
 ============================= end_copyright_notice ===========================*/
 
 #include "Compiler/Optimizer/OpenCLPasses/WIFuncs/WIFuncsAnalysis.hpp"
+#include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "AdaptorCommon/ImplicitArgs.hpp"
 #include "AdaptorCommon/AddImplicitArgs.hpp"
 #include "Compiler/IGCPassSupport.h"
@@ -46,6 +47,7 @@ const llvm::StringRef WIFuncsAnalysis::GET_ENQUEUED_LOCAL_SIZE = "__builtin_IB_g
 const llvm::StringRef WIFuncsAnalysis::GET_STAGE_IN_GRID_ORIGIN = "__builtin_IB_get_stage_in_grid_origin";
 const llvm::StringRef WIFuncsAnalysis::GET_STAGE_IN_GRID_SIZE = "__builtin_IB_get_stage_in_grid_size";
 const llvm::StringRef WIFuncsAnalysis::GET_SYNC_BUFFER = "__builtin_IB_get_sync_buffer";
+const llvm::StringRef WIFuncsAnalysis::GET_ASSERT_BUFFER = "__builtin_IB_get_assert_buffer";
 
 WIFuncsAnalysis::WIFuncsAnalysis() : ModulePass(ID)
 {
@@ -85,6 +87,7 @@ bool WIFuncsAnalysis::runOnFunction(Function& F)
     m_hasStageInGridOrigin = false;
     m_hasStageInGridSize = false;
     m_hasSyncBuffer = false;
+    m_hasAssertBuffer = false;
     m_hasStackCalls = false;
 
     // Visit the function
@@ -99,6 +102,7 @@ bool WIFuncsAnalysis::runOnFunction(Function& F)
     if (isEntryFunc(m_pMDUtils, &F))
     {
         implicitArgs.push_back(ImplicitArg::R0);
+
         if (RequirePayloadHeader)
             implicitArgs.push_back(ImplicitArg::PAYLOAD_HEADER);
 
@@ -182,6 +186,10 @@ bool WIFuncsAnalysis::runOnFunction(Function& F)
     {
         implicitArgs.push_back(ImplicitArg::SYNC_BUFFER);
     }
+    if (m_hasAssertBuffer)
+    {
+        implicitArgs.push_back(ImplicitArg::ASSERT_BUFFER_POINTER);
+    }
 
     // Create the metadata representing the implicit args needed by this function
     ImplicitArgs::addImplicitArgs(F, implicitArgs, m_pMDUtils);
@@ -251,4 +259,7 @@ void WIFuncsAnalysis::visitCallInst(CallInst& CI)
     else if (funcName.equals(GET_SYNC_BUFFER)) {
         m_hasSyncBuffer = true;
     }
+    else if (funcName.equals(GET_ASSERT_BUFFER)) {
+        m_hasAssertBuffer = true;
+    }
 }
