@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -634,9 +634,10 @@ bool PreCompiledFuncImport::runOnModule(Module& M)
     // When we test it, we need to set emuKind
     if (IGC_IS_FLAG_ENABLED(TestIGCPreCompiledFunctions))
     {
-        m_emuKind = EmuKind::EMU_DP;
+        m_emuKind = IGC_GET_FLAG_VALUE(ForceEmuKind) ? IGC_GET_FLAG_VALUE(ForceEmuKind) : EmuKind::EMU_DP;
         checkAndSetEnableSubroutine();
     }
+
     // sanity check
     if (m_emuKind == 0) {
         // Nothing to emulate
@@ -750,15 +751,15 @@ bool PreCompiledFuncImport::runOnModule(Module& M)
                 for (auto& I : BB) {
                     if (CallInst* CI = dyn_cast<CallInst>(&I)) {
                         if (Function* calledFunc = CI->getCalledFunction()) {
-                            if (calledFunc->getName().equals("GenISA_fma_rtz"))
+                            if (calledFunc->getName().startswith("GenISA_fma_rtz"))
                             {
                                 createIntrinsicCall(CI, GenISAIntrinsic::GenISA_fma_rtz);
                             }
-                            else if (calledFunc->getName().equals("GenISA_fma_rtp"))
+                            else if (calledFunc->getName().startswith("GenISA_fma_rtp"))
                             {
                                 createIntrinsicCall(CI, GenISAIntrinsic::GenISA_fma_rtp);
                             }
-                            else if (calledFunc->getName().equals("GenISA_fma_rtn"))
+                            else if (calledFunc->getName().startswith("GenISA_fma_rtn"))
                             {
                                 createIntrinsicCall(CI, GenISAIntrinsic::GenISA_fma_rtn);
                             }
@@ -788,15 +789,15 @@ bool PreCompiledFuncImport::runOnModule(Module& M)
                         {
                             if (Function* calledFunc = CI->getCalledFunction())
                             {
-                                if (calledFunc->getName().equals("GenISA_mul_rtz"))
+                                if (calledFunc->getName().startswith("GenISA_mul_rtz"))
                                 {
                                     createIntrinsicCall(CI, GenISAIntrinsic::GenISA_mul_rtz);
                                 }
-                                else if (calledFunc->getName().equals("GenISA_add_rtz"))
+                                else if (calledFunc->getName().startswith("GenISA_add_rtz"))
                                 {
                                     createIntrinsicCall(CI, GenISAIntrinsic::GenISA_add_rtz);
                                 }
-                                else if (calledFunc->getName().equals("GenISA_uitof_rtz"))
+                                else if (calledFunc->getName().startswith("GenISA_uitof_rtz"))
                                 {
                                     createIntrinsicCall(CI, GenISAIntrinsic::GenISA_uitof_rtz);
                                 }
@@ -2547,7 +2548,6 @@ void PreCompiledFuncImport::checkAndSetEnableSubroutine()
     bool SPDiv = isSPDiv();
     bool DPEmu = isDPEmu();
     bool DPDivSqrtEmu = isDPDivSqrtEmu();
-    bool I64DivRem = isI64DivRem();
 
     Module* M = m_pCtx->getModule();
     for (auto FI = M->begin(), FE = M->end(); FI != FE; ++FI)
@@ -2590,15 +2590,6 @@ void PreCompiledFuncImport::checkAndSetEnableSubroutine()
                     m_enableCallForEmulation = true;
                 }
                 break;
-            case Instruction::UDiv:
-            case Instruction::URem:
-            case Instruction::SDiv:
-            case Instruction::SRem:
-                if (I64DivRem && I->getOperand(0)->getType()->isIntegerTy(64))
-                {
-                    m_enableCallForEmulation = true;
-                }
-                break;
             }
 
             GenIntrinsicInst* GII = dyn_cast<GenIntrinsicInst>(I);
