@@ -277,7 +277,6 @@ Value* ValueTracker::findAllocaValue(Value* V, const uint depth)
 {
     if (!V) return nullptr;
 
-    visitedValues.insert(V);
     for (auto U : V->users())
     {
         if (visitedValues.find(U) != visitedValues.end()) continue;
@@ -402,8 +401,6 @@ Value* ValueTracker::trackValue(Value* I)
             workList.pop_back();
         }
 
-        visitedValues.insert(baseValue);
-
         if (auto* I = dyn_cast<Argument>(baseValue))
         {
             // If we are here, it means that baseValue is an argument of function not argument of kernel,
