@@ -44,6 +44,7 @@ See LICENSE.TXT for details.
 #include "Probe/Assertion.h"
 
 #include "llvmWrapper/IR/Instructions.h"
+#include "llvmWrapper/Support/Alignment.h"
 
 namespace llvm {
 /// IntrinsicInst - A useful wrapper class for inspecting calls to intrinsic
@@ -93,68 +94,59 @@ public:
     }
 };
 
-class RTWritIntrinsic : public GenIntrinsicInst {
+class RTWriteIntrinsic : public GenIntrinsicInst {
 protected:
     Value* getOperand(unsigned i) const { return GenIntrinsicInst::getOperand(i); }
     void setOperand(unsigned i, Value* v) { return GenIntrinsicInst::setOperand(i, v); }
 
 public:
-    inline Value* getSource0Alpha()     const { return getOperand(0); }
-    inline Value* getOMask()            const { return getOperand(1); }
-    inline Value* getPMask()            const { return getOperand(2); }
-    inline void setPMask(Value* pmask)        { setOperand(2, pmask); }
-    inline Value* getRed()              const { return getOperand(3); }
-    inline Value* getGreen()            const { return getOperand(4); }
-    inline Value* getBlue()             const { return getOperand(5); }
-    inline Value* getAlpha()            const { return getOperand(6); }
-    inline Value* getDepth()            const { return getOperand(7); }
-    inline Value* getStencil()          const { return getOperand(8); }
-    inline Value* getRTIndex()          const { return getOperand(9); }
-    inline Value* getBlendStateIndex()  const { return getOperand(10); }
-    inline Value* getSampleIndex()      const { return getOperand(15); }
+    Value* getSource0Alpha()     const { return getOperand(operandOffset(0)); }
+    Value* getOMask()            const { return getOperand(operandOffset(1)); }
+    Value* getPMask()            const { return getOperand(operandOffset(2)); }
+    void setPMask(Value* pmask)  { setOperand(operandOffset(2), pmask); }
+    Value* getRed()              const { return getOperand(operandOffset(3)); }
+    Value* getGreen()            const { return getOperand(operandOffset(4)); }
+    Value* getBlue()             const { return getOperand(operandOffset(5)); }
+    Value* getAlpha()            const { return getOperand(operandOffset(6)); }
+    Value* getDepth()            const { return getOperand(operandOffset(7)); }
+    Value* getStencil()          const { return getOperand(operandOffset(8)); }
+    Value* getRTIndex()          const { return getOperand(operandOffset(9)); }
+    Value* getBlendStateIndex()  const { return getOperand(operandOffset(10)); }
+    bool hasMask()               const { return valueToImm64(getOperand(operandOffset(11))) != 0; }
+    bool hasDepth()              const { return valueToImm64(getOperand(operandOffset(12))) != 0; }
+    bool hasStencil()            const { return valueToImm64(getOperand(operandOffset(13))) != 0; }
+    bool perSample()             const { return valueToImm64(getOperand(operandOffset(14))) != 0; }
+    void setPerSample()          { setOperand(operandOffset(14), ConstantInt::getTrue(getParent()->getContext())); }
+    Value* getSampleIndex()      const { return getOperand(operandOffset(15)); }
+    void setSampleIndex(Value* v){ setOperand(operandOffset(15), v); }
 
-    inline bool isImmRTIndex() const
-    { return llvm::isa<llvm::ConstantInt>(getRTIndex()); }
-
-    inline int getRTIndexImm()  const
-    { return (int)llvm::cast<llvm::ConstantInt>(getRTIndex())->getZExtValue(); }
-
-    inline bool hasMask()    const
-    {
-        return valueToImm64(getOperand(11)) != 0;
-    }
-    inline bool hasDepth()   const
-    {
-        return valueToImm64(getOperand(12)) != 0;
-    }
-    inline bool hasStencil() const
-    {
-        return valueToImm64(getOperand(13)) != 0;
-    }
-    inline bool perSample()  const
-    {
-        return valueToImm64(getOperand(14)) != 0;
-    }
-
-    void setPerSample()
-    {
-        Value* btrue = ConstantInt::get(
-            Type::getInt1Ty(this->getParent()->getContext()), 1);
-        setOperand(14, btrue);
-    }
-    void setSampleIndex(Value* v)       { setOperand(15, v); }
+    bool isImmRTIndex()          const { return isa<ConstantInt>(getRTIndex()); }
+    int getRTIndexImm()          const { return static_cast<int>(cast<ConstantInt>(getRTIndex())->getZExtValue()); }
 
     // Methods for support type inquiry through isa, cast, and dyn_cast:
-    static inline bool classof(const GenIntrinsicInst *I) {
+    static bool classof(const GenIntrinsicInst *I) {
         switch (I->getIntrinsicID()) {
         case GenISAIntrinsic::GenISA_RTWrite:
             return true;
         default: return false;
         }
     }
-    static inline bool classof(const Value *V) {
+    static bool classof(const Value *V) {
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
+
+    bool isRegular() const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_RTWrite:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+
+private:
+    unsigned operandOffset(unsigned offset) const { return offset + ((!isRegular() && offset >= 9) ? 1 : 0); }
 };
 
 class RTDualBlendSourceIntrinsic : public GenIntrinsicInst {
@@ -163,61 +155,112 @@ protected:
     void setOperand(unsigned i, Value* v) { return GenIntrinsicInst::setOperand(i, v); }
 
 public:
-    inline Value* getSource0Alpha()     const { return nullptr; }
-    inline Value* getOMask()            const { return getOperand(0); }
-    inline Value* getPMask()            const { return getOperand(1); }
-    inline void setPMask(Value* mask)         { setOperand(1, mask);  }
-    inline Value* getRed0()             const { return getOperand(2); }
-    inline Value* getGreen0()           const { return getOperand(3); }
-    inline Value* getBlue0()            const { return getOperand(4); }
-    inline Value* getAlpha0()           const { return getOperand(5); }
-    inline Value* getRed1()             const { return getOperand(6); }
-    inline Value* getGreen1()           const { return getOperand(7); }
-    inline Value* getBlue1()            const { return getOperand(8); }
-    inline Value* getAlpha1()           const { return getOperand(9); }
-    inline Value* getDepth()            const { return getOperand(10); }
-    inline Value* getStencil()          const { return getOperand(11); }
-    inline Value* getRTIndex()          const { return getOperand(12); }
-    inline Value* getSampleIndex()      const { return getOperand(17); }
+    Value* getSource0Alpha()     const { return nullptr; }
+    Value* getOMask()            const { return getOperand(operandOffset(0)); }
+    Value* getPMask()            const { return getOperand(operandOffset(1)); }
+    void setPMask(Value* mask)   { setOperand(operandOffset(1), mask);  }
+    Value* getRed0()             const { return getOperand(operandOffset(2)); }
+    Value* getGreen0()           const { return getOperand(operandOffset(3)); }
+    Value* getBlue0()            const { return getOperand(operandOffset(4)); }
+    Value* getAlpha0()           const { return getOperand(operandOffset(5)); }
+    Value* getRed1()             const { return getOperand(operandOffset(6)); }
+    Value* getGreen1()           const { return getOperand(operandOffset(7)); }
+    Value* getBlue1()            const { return getOperand(operandOffset(8)); }
+    Value* getAlpha1()           const { return getOperand(operandOffset(9)); }
+    Value* getDepth()            const { return getOperand(operandOffset(10)); }
+    Value* getStencil()          const { return getOperand(operandOffset(11)); }
+    Value* getRTIndex()          const { return getOperand(operandOffset(12)); }
+    bool hasMask()               const { return valueToImm64(getOperand(operandOffset(13))) != 0; }
+    bool hasDepth()              const { return valueToImm64(getOperand(operandOffset(14))) != 0; }
+    bool hasStencil()            const { return valueToImm64(getOperand(operandOffset(15))) != 0; }
+    bool perSample()             const { return valueToImm64(getOperand(operandOffset(16))) != 0; }
+    void setPerSample()          { setOperand(operandOffset(16), ConstantInt::getTrue(getParent()->getContext())); }
+    Value* getSampleIndex()      const { return getOperand(operandOffset(17)); }
+    void setSampleIndex(Value* v){ setOperand(operandOffset(17), v); }
 
-    inline int getRTIndexImm()  const
-    { return (int)llvm::cast<llvm::ConstantInt>(getRTIndex())->getZExtValue(); }
+    int getRTIndexImm()          const { return static_cast<int>(llvm::cast<llvm::ConstantInt>(getRTIndex())->getZExtValue()); }
 
-    inline bool hasMask()    const
-    {
-        return valueToImm64(getOperand(13)) != 0;
-    }
-    inline bool hasDepth()   const
-    {
-        return valueToImm64(getOperand(14)) != 0;
-    }
-    inline bool hasStencil() const
-    {
-        return valueToImm64(getOperand(15)) != 0;
-    }
-    inline bool perSample()  const
-    {
-        return valueToImm64(getOperand(16)) != 0;
-    }
-
-    void setPerSample()
-    {
-        Value* btrue = ConstantInt::get(
-            Type::getInt1Ty(this->getParent()->getContext()), 1);
-        setOperand(16, btrue);
-    }
-    void setSampleIndex(Value* v)       { setOperand(17, v); }
-
-    static inline bool classof(const GenIntrinsicInst *I) {
+    static bool classof(const GenIntrinsicInst *I) {
         switch (I->getIntrinsicID()) {
         case GenISAIntrinsic::GenISA_RTDualBlendSource:
             return true;
         default: return false;
         }
     }
-    static inline bool classof(const Value *V) {
+    static bool classof(const Value *V) {
         return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
     }
+
+private:
+    bool isRegular() const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_RTDualBlendSource:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+
+    unsigned operandOffset(unsigned offset) const { return offset + ((!isRegular() && offset >= 12) ? 1 : 0); }
+};
+
+class RenderTargetReadIntrinsic : public GenIntrinsicInst {
+protected:
+    Value* getOperand(unsigned i) const { return GenIntrinsicInst::getOperand(i); }
+    void setOperand(unsigned i, Value* v) { return GenIntrinsicInst::setOperand(i, v); }
+
+public:
+    Value* getRTIndex()          const { return getOperand(operandOffset(0)); }
+    bool hasSampleIndex()        const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+    Value* getSampleIndex()      const { IGC_ASSERT(hasSampleIndex()); return getOperand(operandOffset(1)); }
+    void setSampleIndex(Value* v) { IGC_ASSERT(hasSampleIndex()); setOperand(operandOffset(1), v); }
+
+    bool isImmRTIndex()          const { return isa<ConstantInt>(getRTIndex()); }
+    int getRTIndexImm()          const { return static_cast<int>(cast<ConstantInt>(getRTIndex())->getZExtValue()); }
+
+    std::vector<Value*> getArgs () const {
+        std::vector<Value*> v;
+        if (isRegular()) v.push_back(getOperand(0));
+        v.push_back(getRTIndex());
+        if (hasSampleIndex()) v.push_back(getSampleIndex());
+        return v;
+    }
+
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static bool classof(const GenIntrinsicInst* I) {
+        switch (I->getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_RenderTargetRead:
+        case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
+            return true;
+        default: return false;
+        }
+    }
+    static bool classof(const Value* V) {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+private:
+    bool isRegular() const {
+        switch (getIntrinsicID()) {
+        case GenISAIntrinsic::GenISA_RenderTargetRead:
+        case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
+            return true;
+        default:
+            break;
+        }
+        return false;
+    }
+
+    unsigned operandOffset(unsigned offset) const { return offset + (!isRegular() ? 1 : 0); }
 };
 
 
@@ -264,6 +307,7 @@ public:
         switch (I->getIntrinsicID()) {
         case GenISAIntrinsic::GenISA_sampleinfoptr:
         case GenISAIntrinsic::GenISA_resinfoptr:
+        case GenISAIntrinsic::GenISA_readsurfacetypeandformat:
             return true;
         default: return false;
         }
@@ -293,16 +337,24 @@ public:
         }
     }
     inline unsigned int getTextureIndex() const { return getNumOperands() - 5; }
-    inline unsigned int getOffsetIndex(unsigned int i) const
+    inline unsigned int getImmediateOffsetsIndex() const
     {
-        return 5 + i;
+        return 5;
     }
-
     inline Value* getTextureValue() const { return getOperand(getTextureIndex()); }
     inline Value* getCoordinateValue(unsigned int i) const { return getOperand(getCoordinateIndex(i)); }
     inline void   setCoordinateValue(unsigned int i, Value* val) { setOperand(getCoordinateIndex(i), val); }
-    inline Value* getOffsetValue(unsigned int i) const { return getOperand(getOffsetIndex(i)); }
-    inline void   setOffsetValue(unsigned int i, Value* val) { setOperand(getOffsetIndex(i), val); }
+
+    inline Value* getImmediateOffsetsValue(unsigned int coordIndex) const
+    {
+        unsigned int operandCoordIndex = getImmediateOffsetsIndex() + coordIndex;
+        return getOperand(operandCoordIndex);
+    }
+    inline void setImmediateOffsetsValue(unsigned int coordIndex, Value* val)
+    {
+        unsigned int operandCoordIndex = getImmediateOffsetsIndex() + coordIndex;
+        setOperand(operandCoordIndex, val);
+    }
 
     static inline bool classof(const GenIntrinsicInst *I) {
         switch(I->getIntrinsicID()) {
@@ -608,7 +660,7 @@ public:
     }
 
 
-    inline void setAlignment(unsigned int alignment)
+    inline void setAlignment(alignment_t alignment)
     {
         Value* newAlignment = ConstantInt::get(getOperand(2)->getType(), alignment);
         setOperand(2, newAlignment);
@@ -958,8 +1010,9 @@ public:
         return getOperand(6);
     }
     Value* getMissShaderIndex() const { return getOperand(7); }
+    static constexpr uint32_t getNumRayInfoFields() { return 8; }
     Value* getRayInfo(uint32_t Idx) const {
-        IGC_ASSERT_MESSAGE(Idx < 8, "Index out of range!");
+        IGC_ASSERT_MESSAGE(Idx < getNumRayInfoFields(), "Index out of range!");
         return getOperand(8 + Idx);
     }
     // Get the 0 - X, 1 - Y, or 2 - Z component of the ray origin
@@ -974,6 +1027,7 @@ public:
     }
     Value* getTMin() const { return getOperand(14); }
     Value* getTMax() const { return getOperand(15); }
+    void setTMax(Value* V) { setOperand(15, V); }
     Value* getPayload() const { return getOperand(16); }
 };
 
@@ -1017,6 +1071,8 @@ public:
     }
 
     Value* getGlobalBufferPointer() const { return getOperand(0); }
+    void setGlobalBufferPointer(Value* V) { return setOperand(0, V); }
+
     Value* getPayload()             const { return getOperand(1); }
 };
 
@@ -1361,8 +1417,9 @@ public:
     Value* getFlag() const { return getOperand(2); }
     Value* getMask() const { return getOperand(3); }
 
+    static constexpr uint32_t getNumRayInfoFields() { return 8; }
     Value* getRayInfo(uint32_t Idx) const {
-        IGC_ASSERT_MESSAGE(Idx < 8, "Index out of range!");
+        IGC_ASSERT_MESSAGE(Idx < getNumRayInfoFields(), "Index out of range!");
         return getOperand(4 + Idx);
     }
     // Get the 0 - X, 1 - Y, or 2 - Z component of the ray origin
@@ -1377,6 +1434,7 @@ public:
     }
     Value* getTMin() const { return getOperand(10); }
     Value* getTMax() const { return getOperand(11); }
+    void setTMax(Value* V) { setOperand(11, V); }
 };
 
 class TraceRaySyncProceedHLIntrinsic : public RayQueryInstrisicBase {
@@ -1588,21 +1646,6 @@ public:
     }
 };
 
-class SpillAnchorIntrinsic : public GenIntrinsicInst {
-public:
-    // Methods for support type inquiry through isa, cast, and dyn_cast:
-    static inline bool classof(const GenIntrinsicInst *I) {
-        GenISAIntrinsic::ID ID = I->getIntrinsicID();
-        return ID == GenISAIntrinsic::GenISA_rt_spill_anchor;
-    }
-
-    static inline bool classof(const Value *V) {
-        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
-    }
-
-    Value* getValue() const { return getOperand(0); }
-};
-
 class StaticConstantPatchIntrinsic : public GenIntrinsicInst {
 public:
     // Methods for support type inquiry through isa, cast, and dyn_cast:
@@ -1651,17 +1694,35 @@ public:
         return cast<ConstantInt>(getOperand(3))->isOne();
     }
 
-    enum class FPBinaryOperators : uint32_t
+    enum class FPBinaryOperators
     {
-        FAdd,
-        FSub,
-        FMul,
-        FDiv,
-        FRem
+#define FP_BINOP_INSTRUCTION(Name, Val) Name = Val,
+#include "IGC/common/igc_regkeys_enums_defs.h"
+        FP_BINOP_INSTRUCTIONS
+#undef FP_BINOP_INSTRUCTION
+#undef FP_BINOP_INSTRUCTIONS
     };
 
 };
 
+class EmitHitAttributesIntrinstic: public GenIntrinsicInst
+{
+public:
+    // Methods for support type inquiry through isa, cast, and dyn_cast:
+    static inline bool classof(const GenIntrinsicInst* I)
+    {
+        GenISAIntrinsic::ID ID = I->getIntrinsicID();
+        return ID == GenISAIntrinsic::GenISA_EmitHitAttributes;
+    }
+
+    static inline bool classof(const Value* V)
+    {
+        return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V));
+    }
+
+
+};
+
 template <class X, class Y>
 inline bool isa(const Y &Val, GenISAIntrinsic::ID id)
 {
