@@ -22,10 +22,10 @@ SPDX-License-Identifier: MIT
 #include "inc/common/Compiler/API/SurfaceFormats.h"
 #include "inc/common/igfxfmid.h"
 #include "common/IGCIRBuilder.h"
-#include "../common/EmUtils.h"
-#include "../../../skuwa/iacm_g10_rev_id.h"
-#include "../../../skuwa/iacm_g11_rev_id.h"
-#include "../../../skuwa/iacm_g12_rev_id.h"
+#include "common/EmUtils.h"
+#include "skuwa/iacm_g10_rev_id.h"
+#include "skuwa/iacm_g11_rev_id.h"
+#include "skuwa/iacm_g12_rev_id.h"
 
 namespace llvm {
     class GenIntrinsicInst;
@@ -233,7 +233,7 @@ public:
         llvm::Value* srcBuffer,
         llvm::Value* float_src_s_mip);
 
-    llvm::Value* CreateReadSurfaceInfo(llvm::Value* resource, llvm::Value* mimap);
+    llvm::Value* CreateReadSurfaceTypeAndFormat(llvm::Value* resource);
 
     llvm::Value* Create_typedwrite(
         llvm::Value* dstBuffer,
