@@ -1330,6 +1330,7 @@ namespace TC
                             (strcmp(pParam, "-igc_opts") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-gtpin-rera") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-256-GRF-per-thread") == 0) || //temporary options
+                            (strcmp(pParam, "-ze-opt-256-GRF-per-thread") == 0) || //temporary options
                             (strcmp(pParam, "-ze-opt-large-register-file") == 0) || //temporary options
                             (strcmp(pParam, "-ze-opt-disable-recompilation") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-num-thread-per-eu") == 0) || //temporary options
@@ -1347,8 +1348,6 @@ namespace TC
                             (strcmp(pParam, "-ze-opt-has-buffer-offset-arg") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-buffer-offset-arg-required") == 0) || //temporary options
                             (strcmp(pParam, "-ze-opt-buffer-offset-arg-required") == 0) || //temporary options
-                            (strcmp(pParam, "-cl-intel-has-subDW-aligned-ptr-arg") == 0) || //temporary options
-                            (strcmp(pParam, "-ze-opt-has-subDW-aligned-ptr-arg") == 0) || //temporary options
                             (strcmp(pParam, "-cl-force-global-mem-allocation") == 0) || // temp
                             (strcmp(pParam, "-ze-force-global-mem-allocation") == 0) || // temp
                             (strcmp(pParam, "-cl-no-local-to-generic") == 0) || // temp
@@ -1365,7 +1364,8 @@ namespace TC
                             (strcmp(pParam, "-ze-gtpin-grf-info") == 0) || //used by GTPin
                             (strcmp(pParam, "-ze-gtpin-scratch-area-size") == 0) || //used by GTPin
                             (strcmp(pParam, "-ze-opt-enable-auto-large-GRF-mode") == 0) ||
-                            (strcmp(pParam, "-cl-intel-enable-auto-large-GRF-mode") == 0);
+                            (strcmp(pParam, "-cl-intel-enable-auto-large-GRF-mode") == 0) ||
+                            (strcmp(pParam, "-ze-skip-fde") == 0);
 
                         if (isCommonOption)
                         {
