@@ -38,11 +38,8 @@ namespace TC {
 namespace {
   static const std::string ERROR_VLD = "VLD: Failed to compile SPIR-V with following error: \n";
 
-  llvm::Expected<std::vector<llvm::StringRef>> getZeBinSectionsData(llvm::StringRef ZeBinary, zebin::SHT_ZEBIN SectionType) {
-    using namespace llvm;
+  llvm::Expected<object::ELF64LEFile> getElfFile(llvm::StringRef ZeBinary) {
     MemoryBufferRef inputRef(ZeBinary, "zebin");
-    std::vector<StringRef> OutVec;
-
     auto ElfOrErr = object::ObjectFile::createELFObjectFile(inputRef);
     if (!ElfOrErr)
       return ElfOrErr.takeError();
@@ -53,6 +50,18 @@ namespace {
 #else
     auto ElfFile = cast<object::ELF64LEObjectFile>(*ElfOrErr.get()).getELFFile();
 #endif
+
+    return ElfFile;
+  }
+
+  llvm::Expected<std::vector<llvm::StringRef>> getZeBinSectionsData(llvm::StringRef ZeBinary, zebin::SHT_ZEBIN SectionType) {
+    using namespace llvm;
+    std::vector<StringRef> OutVec;
+
+    auto ElfFileOrErr = getElfFile(ZeBinary);
+    if (!ElfFileOrErr) return ElfFileOrErr.takeError();
+    auto ElfFile = ElfFileOrErr.get();
+
     auto ElfSections = ElfFile.sections();
     if (!ElfSections)
       return ElfSections.takeError();
@@ -87,15 +96,16 @@ llvm::Expected<zebin::zeInfoContainer> GetZeInfoFromZeBinary(llvm::StringRef ZeB
   if (!ZeInfoYAMLOrErr) {
     return ZeInfoYAMLOrErr.takeError();
   }
+
+  zebin::zeInfoContainer ZeInfo;
+
   if (ZeInfoYAMLOrErr->size() != 1) {
-    return llvm::createStringError(llvm::inconvertibleErrorCode(), "ZEBinary expected to contain exactly one .ze_info section!");
+    return llvm::createStringError(llvm::inconvertibleErrorCode(), "ZEBinary expected to contain one .ze_info section!");
   }
 
   // ZeBinary strings are not null-terminated, so copy it to std::string.
   std::string ZeInfoYAML = (*(ZeInfoYAMLOrErr->begin())).str();
-
   llvm::yaml::Input yin(ZeInfoYAML.c_str());
-  zebin::zeInfoContainer ZeInfo;
   yin >> ZeInfo;
   if (yin.error()) {
     return llvm::createStringError(llvm::inconvertibleErrorCode(), "Failed to parse .ze_info section!");
@@ -104,6 +114,23 @@ llvm::Expected<zebin::zeInfoContainer> GetZeInfoFromZeBinary(llvm::StringRef ZeB
   return ZeInfo;
 }
 
+llvm::Expected<bool> ZeBinaryContainsSection(llvm::StringRef ZeBinary,
+                                             zebin::SHT_ZEBIN SectionType) {
+  auto ElfFileOrErr = getElfFile(ZeBinary);
+  if (!ElfFileOrErr)
+    return ElfFileOrErr.takeError();
+  auto ElfSections = (*ElfFileOrErr).sections();
+  if (!ElfSections)
+    return ElfSections.takeError();
+
+  for (auto &Sect : *ElfSections) {
+    if (Sect.sh_type == SectionType) {
+      return true;
+    }
+  }
+  return false;
+}
+
 llvm::Expected<int> GetSIMDSizeFromZeInfo(const zebin::zeInfoContainer& ZeInfo) {
   std::vector<int> SimdSizes;
   for (auto& Kernel : ZeInfo.kernels) {
@@ -137,6 +164,128 @@ void DumpSPIRVFile(const char *programData, size_t programSizeInBytes,
   spvTextDestroy(spirvAsm);
 }
 
+// Given a vector of SPVTranslationPairs, the function moves the pair with
+// entry module to the last.
+// If entry module has subgroup size forced, it is returned in SimdSize param.
+llvm::Expected<std::vector<IGC::VLD::SPVTranslationPair>>
+MoveEntryPointModuleToTheEnd(
+    llvm::ArrayRef<IGC::VLD::SPVTranslationPair> InputModules,
+    uint32_t &SimdSize) {
+  using namespace IGC::VLD;
+  SpvSplitter splitter;
+  SPVTranslationPair EntryPointPair;
+  bool HasEntryPointModule = false;
+  std::vector<SPVTranslationPair> RetPairs;
+  for (auto &InputArgsPair : InputModules) {
+    if (InputArgsPair.first.HasEntryPoints) {
+      EntryPointPair = InputArgsPair;
+      if (EntryPointPair.first.SpirvType == SPIRVTypeEnum::SPIRV_SPMD) {
+        SimdSize = splitter.GetForcedSubgroupSize();
+      }
+      if (HasEntryPointModule) {
+        return llvm::createStringError(
+            llvm::inconvertibleErrorCode(),
+            "The list of SPIR-V modules contains more than one module with an "
+            "entry point!");
+      }
+      HasEntryPointModule = true;
+    } else {
+      RetPairs.push_back(InputArgsPair);
+    }
+  }
+
+  bool IsLibraryCompilation = false;
+  auto OptionsOfLastModule = RetPairs[RetPairs.size() - 1].second.pOptions;
+  if (OptionsOfLastModule &&
+      strstr(OptionsOfLastModule, "-library-compilation")) {
+    IsLibraryCompilation = true;
+  }
+
+  if (!HasEntryPointModule && !IsLibraryCompilation) {
+    return llvm::createStringError(llvm::inconvertibleErrorCode(),
+                                   "The list of SPIR-V modules did not contain "
+                                   "any module with an entry point!");
+  }
+
+  if (HasEntryPointModule) {
+    RetPairs.push_back(EntryPointPair);
+  }
+
+  return RetPairs;
+}
+
+// Returns a list of functions that must be called directly:
+// functions that are exported in VC modules and imported in SPMD and
+// vice-versa.
+std::vector<const char *> GetDirectCallFunctions(
+    llvm::ArrayRef<IGC::VLD::SPVTranslationPair> InputModules) {
+  set<StringRef> exportedFunctionsSPMD;
+  set<StringRef> exportedFunctionsESIMD;
+  set<StringRef> importedFunctionsSPMD;
+  set<StringRef> importedFunctionsESIMD;
+
+  auto insertToVec = [](const auto &InputVec, auto &OutSPMD, auto &OutESIMD,
+                        auto SpirvType) {
+    for (auto &FuncName : InputVec) {
+      if (SpirvType == IGC::VLD::SPIRVTypeEnum::SPIRV_SPMD) {
+        OutSPMD.insert(FuncName);
+      } else {
+        OutESIMD.insert(FuncName);
+      }
+    }
+  };
+
+  for (auto &InputArgsPair : InputModules) {
+    IGC_ASSERT(InputArgsPair.first.SpirvType ==
+                   IGC::VLD::SPIRVTypeEnum::SPIRV_SPMD ||
+               InputArgsPair.first.SpirvType == IGC::VLD::SPIRVTypeEnum::SPIRV_ESIMD);
+    insertToVec(InputArgsPair.first.ExportedFunctions, exportedFunctionsSPMD,
+                exportedFunctionsESIMD, InputArgsPair.first.SpirvType);
+    insertToVec(InputArgsPair.first.ImportedFunctions, importedFunctionsSPMD,
+                importedFunctionsESIMD, InputArgsPair.first.SpirvType);
+  }
+
+  std::vector<StringRef> DirectCallFunctions;
+  set_intersection(exportedFunctionsESIMD.begin(), exportedFunctionsESIMD.end(),
+                   importedFunctionsSPMD.begin(), importedFunctionsSPMD.end(),
+                   back_inserter(DirectCallFunctions));
+  set_intersection(exportedFunctionsSPMD.begin(), exportedFunctionsSPMD.end(),
+                   importedFunctionsESIMD.begin(), importedFunctionsESIMD.end(),
+                   back_inserter(DirectCallFunctions));
+  // Special case: when SPMD+ESIMD module is provided, the invoke_simd callee will not
+  // be in the imported SPMD function list, as it is just Exported in the single module.
+  // It will be present in both SPMD and ESIMD exports.
+  set_intersection(exportedFunctionsSPMD.begin(), exportedFunctionsSPMD.end(),
+                   exportedFunctionsESIMD.begin(), exportedFunctionsESIMD.end(),
+                   back_inserter(DirectCallFunctions));
+
+
+  std::vector<const char *> OutVec;
+  std::transform(DirectCallFunctions.begin(), DirectCallFunctions.end(),
+                 back_inserter(OutVec), [](auto &el) { return el.data(); });
+  return OutVec;
+}
+
+// Helper function to create a SPVTranslationPair with given TranslateInputArgs
+// as base and binary SPV program. The caller needs to be the owner of the
+// program and options, as TranslateInputArgs structure keeps a pointer to them.
+llvm::Expected<IGC::VLD::SPVTranslationPair>
+MakeSPVTranslationPair(const TC::STB_TranslateInputArgs *pInputArgs,
+                       const IGC::VLD::ProgramStreamType &program,
+                       const std::string &options) {
+
+  TC::STB_TranslateInputArgs NewArgs = *pInputArgs;
+  NewArgs.pInput = (char*)(program.data());
+  NewArgs.InputSize = program.size() * sizeof(*program.begin());
+  NewArgs.pOptions = options.data();
+  NewArgs.OptionsSize = options.size();
+  auto VLDMetadata = IGC::VLD::GetVLDMetadata(NewArgs.pInput, NewArgs.InputSize);
+  if (!VLDMetadata) {
+    return VLDMetadata.takeError();
+  }
+  return IGC::VLD::SPVTranslationPair(*VLDMetadata, NewArgs);
+}
+
 } // namespace
 
 namespace IGC {
@@ -228,19 +377,20 @@ bool TranslateBuildSPMDAndESIMD(const TC::STB_TranslateInputArgs *pInputArgs,
       DumpSPIRVFile((const char*)esimdProg.data(), esimdProg.size() * sizeof(uint32_t), inputShHash, ".esimd_split.spv");
   }
 
-  STB_TranslateInputArgs newArgsSPMD = *pInputArgs;
-  newArgsSPMD.pInput = reinterpret_cast<char *>(spmdProg.data());
-  newArgsSPMD.InputSize = spmdProg.size() * sizeof(*spmdProg.begin());
-
-  STB_TranslateInputArgs newArgsESIMD = *pInputArgs;
-  newArgsESIMD.pInput = reinterpret_cast<char *>(esimdProg.data());
-  newArgsESIMD.InputSize = esimdProg.size() * sizeof(*esimdProg.begin());
-  newArgsESIMD.pOptions = esimdOptions.data();
-  newArgsESIMD.OptionsSize = esimdOptions.size();
+  auto SpmdTPOrErr = MakeSPVTranslationPair(pInputArgs, spmdProg, newOptions);
+  auto EsimdTPOrErr = MakeSPVTranslationPair(pInputArgs, esimdProg, esimdOptions);
+  if (!SpmdTPOrErr) {
+    errorMessage = ERROR_VLD + llvm::toString(SpmdTPOrErr.takeError());
+    return false;
+  }
+  if (!EsimdTPOrErr) {
+    errorMessage = ERROR_VLD + llvm::toString(EsimdTPOrErr.takeError());
+    return false;
+  }
 
   std::array<SPVTranslationPair, 2> SpvArr{
-      std::make_pair(SPIRVTypeEnum::SPIRV_ESIMD, newArgsESIMD),
-      std::make_pair(SPIRVTypeEnum::SPIRV_SPMD, newArgsSPMD)
+    *EsimdTPOrErr,
+    *SpmdTPOrErr
   };
 
   return TranslateBuildSPMDAndESIMD(
@@ -256,13 +406,72 @@ bool TranslateBuildSPMDAndESIMD(
     std::string &errorMessage) {
 #if defined(IGC_VC_ENABLED)
 
-  std::vector<std::string> ownerStrings;
-  std::vector<const char*> visaCStrings;
-  std::vector<const char*> DirectCallFunctions;
-  int SimdSize = 0;
+  std::vector<std::string> OwnerStrings;
+  std::vector<const char*> VisaCStrings;
+  uint32_t SimdSize = 0;
 
-  for (auto& InputArgsPair : InputModules) {
-    auto& InputArgs = InputArgsPair.second;
+  std::vector<SPVTranslationPair> SplitInputModules;
+  std::vector<ProgramStreamType> OwnerSplitBinaries;
+
+  auto SetVLDErrorMessage = [&errorMessage](llvm::Error Err) {
+    errorMessage = ERROR_VLD + llvm::toString(std::move(Err));
+  };
+
+  // Split any SPMD+ESIMD modules.
+  for (auto &InputModule : InputModules) {
+    if (InputModule.first.SpirvType ==
+        VLD::SPIRVTypeEnum::SPIRV_SPMD_AND_ESIMD) {
+      auto SPMPAndESIMDOrErr = VLD::SplitSPMDAndESIMD(
+        InputModule.second.pInput, InputModule.second.InputSize);
+      if (!SPMPAndESIMDOrErr) {
+        SetVLDErrorMessage(SPMPAndESIMDOrErr.takeError());
+        return false;
+      }
+      OwnerSplitBinaries.push_back(std::move(SPMPAndESIMDOrErr.get().first));
+
+      std::string newOptions{InputModule.second.pOptions ? InputModule.second.pOptions : ""};
+      std::string esimdOptions{ newOptions };
+      esimdOptions += " -vc-codegen";
+
+      OwnerStrings.push_back(std::move(newOptions));
+
+      auto SpmdTPOrErr = MakeSPVTranslationPair(&InputModule.second, OwnerSplitBinaries.back(), OwnerStrings.back());
+      if (!SpmdTPOrErr) {
+        SetVLDErrorMessage(SpmdTPOrErr.takeError());
+      }
+
+      OwnerSplitBinaries.push_back(std::move(SPMPAndESIMDOrErr.get().second));
+      OwnerStrings.push_back(std::move(esimdOptions));
+      auto EsimdTPOrErr = MakeSPVTranslationPair(&InputModule.second, OwnerSplitBinaries.back(), OwnerStrings.back());
+      if (!EsimdTPOrErr) {
+        SetVLDErrorMessage(EsimdTPOrErr.takeError());
+      }
+
+      SplitInputModules.push_back(*EsimdTPOrErr);
+      SplitInputModules.push_back(*SpmdTPOrErr);
+
+    } else {
+      SplitInputModules.push_back(InputModule);
+    }
+  }
+
+  auto DirectCallFunctions = GetDirectCallFunctions(SplitInputModules);
+
+  // Module with entry points should be compiled as the last one.
+  // We currently support the use-case, where only one of the SPIR-V modules contain entry points.
+  auto NewInputModulesOrErr = MoveEntryPointModuleToTheEnd(SplitInputModules, SimdSize);
+  if (!NewInputModulesOrErr)
+  {
+    SetVLDErrorMessage(NewInputModulesOrErr.takeError());
+    return false;
+  }
+
+  auto& NewInputModules = NewInputModulesOrErr.get();
+
+  for (auto &InputArgsPair : NewInputModules) {
+    bool IsLast = &InputArgsPair == &NewInputModules.back();
+
+    auto &InputArgs = InputArgsPair.second;
 
     TC::STB_TranslateOutputArgs NewOutputArgs;
     CIF::SafeZeroOut(NewOutputArgs);
@@ -271,71 +480,106 @@ bool TranslateBuildSPMDAndESIMD(
     auto debugData = std::unique_ptr<char[]>(NewOutputArgs.pDebugData);
 
     STB_TranslateInputArgs NewInputArgs = InputArgs;
-    std::string NewInternalOptions{InputArgs.pInternalOptions
-      ? InputArgs.pInternalOptions
-      : ""};
+    std::string NewInternalOptions{
+        InputArgs.pInternalOptions ? InputArgs.pInternalOptions : ""};
+    std::string NewOptions{InputArgs.pOptions ? InputArgs.pOptions : ""};
+    std::string NewEsimdOptions{ NewOptions };
 
-    switch (InputArgsPair.first) {
+    switch (InputArgsPair.first.SpirvType) {
     case VLD::SPIRVTypeEnum::SPIRV_SPMD:
-      NewInternalOptions += " -ze-emit-zebin-visa-sections";
+      if (!IsLast)
+        NewInternalOptions += " -ze-emit-visa-only -ze-emit-zebin-visa-sections";
       break;
     case VLD::SPIRVTypeEnum::SPIRV_ESIMD:
-      NewInternalOptions += " -emit-zebin-visa-sections";
+      if (!IsLast)
+        NewInternalOptions += " -emit-visa-only -emit-zebin-visa-sections";
       NewInternalOptions += " -binary-format=ze";
       if (SimdSize != 0) {
         NewInternalOptions += " -vc-interop-subgroup-size ";
         NewInternalOptions += std::to_string(SimdSize);
       }
       break;
+    case VLD::SPIRVTypeEnum::SPIRV_SPMD_AND_ESIMD:
+      IGC_ASSERT("SPMD+ESIMD module assumed to be split before this code.");
+      errorMessage = "VisaLinkerDriver logic error!";
+      return false;
     default:
       errorMessage = "Unsupported SPIR-V flavour detected!";
       return false;
     }
 
-    NewInputArgs.pVISAAsmToLinkArray = (!visaCStrings.empty()) ? visaCStrings.data() : nullptr;
-    NewInputArgs.NumVISAAsmsToLink = visaCStrings.size();
+    if (InputArgsPair.second.pVISAAsmToLinkArray) {
+      for (uint32_t i = 0; i < InputArgsPair.second.NumVISAAsmsToLink; ++i) {
+        VisaCStrings.push_back(InputArgsPair.second.pVISAAsmToLinkArray[i]);
+      }
+    }
+
+    NewInputArgs.pVISAAsmToLinkArray =
+        (!VisaCStrings.empty()) ? VisaCStrings.data() : nullptr;
+    NewInputArgs.NumVISAAsmsToLink = VisaCStrings.size();
     NewInputArgs.pDirectCallFunctions = DirectCallFunctions.data();
     NewInputArgs.NumDirectCallFunctions = DirectCallFunctions.size();
     NewInputArgs.pInternalOptions = NewInternalOptions.data();
     NewInputArgs.InternalOptionsSize = NewInternalOptions.size();
     bool success = false;
-    if (InputArgsPair.first == VLD::SPIRVTypeEnum::SPIRV_SPMD) {
+    if (InputArgsPair.first.SpirvType == VLD::SPIRVTypeEnum::SPIRV_SPMD) {
       success = TranslateBuildSPMD(&NewInputArgs, &NewOutputArgs,
                                    inputDataFormatTemp, IGCPlatform,
                                    profilingTimerResolution, inputShHash);
-    } else {
-      IGC_ASSERT(InputArgsPair.first == VLD::SPIRVTypeEnum::SPIRV_ESIMD);
+    } else if (InputArgsPair.first.SpirvType == VLD::SPIRVTypeEnum::SPIRV_ESIMD) {
+      NewEsimdOptions += " -vc-codegen";
+      NewInputArgs.pOptions = NewEsimdOptions.data();
+      NewInputArgs.OptionsSize = NewEsimdOptions.size();
       success =
           TranslateBuildVC(&NewInputArgs, &NewOutputArgs, inputDataFormatTemp,
                            IGCPlatform, profilingTimerResolution, inputShHash);
-    }
-
-    if (!success) {
-      errorMessage = "VLD: Failed to compile SPIR-V with following error: \n";
-      errorMessage += NewOutputArgs.pErrorString;
+    } else {
+      IGC_ASSERT("SPMD+ESIMD module assumed to be split before this code.");
+      errorMessage = "VisaLinkerDriver logic error!";
       return false;
     }
 
-    // If this is the last SPIR-V to compile, stop here. The rest of the code handles
-    // extracting information for further compilations.
-    if(&InputArgsPair == &InputModules.back()) {
+    if (!success) {
+      if (errorMessage.empty() && NewOutputArgs.pErrorString) {
+        errorMessage = "VLD: Failed to compile SPIR-V with following error: \n";
+        errorMessage += NewOutputArgs.pErrorString;
+      }
+      return false;
+    }
+
+    // If this is the last SPIR-V to compile, stop here. The rest of the code
+    // handles extracting information for further compilations.
+    if (IsLast) {
       *pOutputArgs = NewOutputArgs;
       break;
     }
 
     llvm::StringRef ZeBinary(NewOutputArgs.pOutput, NewOutputArgs.OutputSize);
+
+    // Check if the output has ZEINFO section. If not, it can mean that the
+    // module didn't contain any exported functions.
+    auto HasZeInfoSectionOrErr =
+        ZeBinaryContainsSection(ZeBinary, zebin::SHT_ZEBIN_ZEINFO);
+    if (!HasZeInfoSectionOrErr) {
+      SetVLDErrorMessage(HasZeInfoSectionOrErr.takeError());
+      return false;
+    }
+    if (!*HasZeInfoSectionOrErr) {
+      continue;
+    }
+
     auto ZeInfoOrErr = GetZeInfoFromZeBinary(ZeBinary);
-    if(!ZeInfoOrErr) {
-      errorMessage = ERROR_VLD + llvm::toString(ZeInfoOrErr.takeError());
+    if (!ZeInfoOrErr) {
+      SetVLDErrorMessage(ZeInfoOrErr.takeError());
       return false;
     }
 
     // Set SimdSize based on first SPMD module, as ESIMD always returns 1.
-    if (InputArgsPair.first == SPIRVTypeEnum::SPIRV_SPMD) {
+    if (InputArgsPair.first.SpirvType == SPIRVTypeEnum::SPIRV_SPMD) {
 
       auto SimdSizeOrErr = GetSIMDSizeFromZeInfo(*ZeInfoOrErr);
       if (!SimdSizeOrErr) {
-        errorMessage = ERROR_VLD + llvm::toString(SimdSizeOrErr.takeError());
+        SetVLDErrorMessage(SimdSizeOrErr.takeError());
         return false;
       }
       if (SimdSize != 0 && SimdSize != *SimdSizeOrErr) {
@@ -348,33 +592,30 @@ bool TranslateBuildSPMDAndESIMD(
         SimdSize = *SimdSizeOrErr;
     }
 
-    auto VISAAsm =
-      GetVISAAsmFromZEBinary(ZeBinary);
+    for (auto &F : ZeInfoOrErr->functions) {
+      OwnerStrings.push_back(std::move(F.name));
+      DirectCallFunctions.push_back(OwnerStrings.back().c_str());
+    }
+
+    auto VISAAsm = GetVISAAsmFromZEBinary(ZeBinary);
 
     if (!VISAAsm) {
-      errorMessage =
-        "VLD: Failed to compile SPIR-V with following error: \n" +
-        llvm::toString(VISAAsm.takeError());
+      SetVLDErrorMessage(VISAAsm.takeError());
       return false;
     }
 
     if (VISAAsm->empty()) {
-      errorMessage = "VLD: ZeBinary did not contain any .visaasm sections!";
+      errorMessage =
+          ERROR_VLD + "ZeBinary did not contain any .visaasm sections!";
       return false;
     }
 
-    // ZeBinary contains non-null terminated strings, add the null via std::string ownership.
-    for (auto& s : *VISAAsm) {
-      ownerStrings.push_back(s.str());
-      visaCStrings.push_back(ownerStrings.back().c_str());
+    // ZeBinary contains non-null terminated strings, add the null via
+    // std::string ownership.
+    for (auto &s : *VISAAsm) {
+      OwnerStrings.push_back(s.str());
+      VisaCStrings.push_back(OwnerStrings.back().c_str());
     }
-
-    for (auto& F : ZeInfoOrErr->functions)
-    {
-      ownerStrings.push_back(std::move(F.name));
-      DirectCallFunctions.push_back(ownerStrings.back().c_str());
-    }
-
   }
 
   return true;
