@@ -46,7 +46,12 @@ vector_impl<T, dst_width>
 __cm_cl_wrregion_float(vector_impl<T, dst_width> dst,
                        vector_impl<T, src_width> src, int vstride, int width,
                        int stride, vector_offset_type offset);
+
+void __cm_cl_debugtrap();
+_Noreturn void __cm_cl_trap();
+
 // FIXME: For legacy issues 64-bit pointer is always returned.
+uint64_t __cm_cl_assert_buffer();
 uint64_t __cm_cl_printf_buffer();
 
 int __cm_cl_printf_format_index(__constant const char *str);
@@ -59,6 +64,45 @@ int __cm_cl_printf_format_index(__private const char *str);
 template <int width> bool __cm_cl_all(vector_impl<char, width> src);
 template <int width> bool __cm_cl_any(vector_impl<char, width> src);
 
+// clang does not allow vector of pointers so passing vector of uint64_t(s)
+// and creating vector of pointers with inttoptr llvm instruction
+template <typename T, int width>
+vector_impl<T, width>
+__cm_cl_gather(int addrspace, vector_impl<uint64_t, width> ptrs, int alignment,
+               vector_impl<char, width> mask, vector_impl<T, width> passthru);
+
+template <typename T, int width>
+void __cm_cl_scatter(vector_impl<T, width> data, int addrspace,
+                     vector_impl<uint64_t, width> ptrs, int alignment,
+                     vector_impl<char, width> mask);
+
+// vector BTI atomic
+template <typename T, int width>
+vector_impl<T, width> __cm_cl_vector_atomic_bti(
+    vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
+    char l1cachecontrol, char l3cachecontrol, int bti,
+    vector_impl<int, width> index, short scale, int offset,
+    vector_impl<T, width> src1, vector_impl<T, width> src2,
+    vector_impl<T, width> passthru);
+
+// vector SLM atomic
+template <typename T, int width>
+vector_impl<T, width> __cm_cl_vector_atomic_slm(
+    vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
+    char l1cachecontrol, char l3cachecontrol, int base,
+    vector_impl<int, width> index, short scale, int offset,
+    vector_impl<T, width> src1, vector_impl<T, width> src2,
+    vector_impl<T, width> passthru);
+
+// vector UGM atomic
+template <typename T, int width>
+vector_impl<T, width> __cm_cl_vector_atomic_ugm(
+    vector_impl<char, width> mask, char opcode, char addrsize, char elementsize,
+    char l1cachecontrol, char l3cachecontrol, long base,
+    vector_impl<long, width> index, short scale, int offset,
+    vector_impl<T, width> src1, vector_impl<T, width> src2,
+    vector_impl<T, width> passthru);
+
 uint32_t __cm_cl_lzd(uint32_t src);
 template <int width>
 vector_impl<uint32_t, width> __cm_cl_lzd(vector_impl<uint32_t, width> src);
@@ -228,8 +272,27 @@ void write_region(vector_impl<T, dst_width> &dst, vector_impl<T, src_width> src,
   }
 }
 
+struct __assert_buffer_t {
+  uint32_t size;
+  uint32_t flags;
+  uint32_t begin;
+};
+
+inline __global uint32_t *assert_flags() {
+  auto addr = static_cast<uintptr_t>(__cm_cl_assert_buffer());
+  __global __assert_buffer_t *p =
+      reinterpret_cast<__global __assert_buffer_t *>(addr);
+  return &p->flags;
+}
+
+inline __global void *assert_buffer() {
+  auto addr = static_cast<uintptr_t>(__cm_cl_assert_buffer());
+  __global __assert_buffer_t *p =
+      reinterpret_cast<__global __assert_buffer_t *>(addr);
+  return &p->begin;
+}
+
 inline __global void *printf_buffer() {
-  // FIXME: for legacy issues 64-bit pointer is always returned.
   auto ptr = static_cast<uintptr_t>(__cm_cl_printf_buffer());
   return reinterpret_cast<__global void *>(ptr);
 }
@@ -345,7 +408,7 @@ template <typename T> T trunc(T src) {
 
 template <typename T> T roundne(T src) {
   static_assert(cl::is_floating_point<T>::value,
-    "Roundne function expects floating poing type.");
+                "Roundne function expects floating poing type.");
   return __cm_cl_roundne(src);
 }
 
