@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021 Intel Corporation
+Copyright (C) 2021-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -9,6 +9,8 @@ SPDX-License-Identifier: MIT
 #include "cmcl/Support/BuiltinTranslator.h"
 #include "cmcl/Support/AtomicsIface.h"
 
+#include "vc/InternalIntrinsics/InternalIntrinsics.h"
+
 #include <llvm/GenXIntrinsics/GenXIntrinsics.h>
 
 #include <llvm/ADT/STLExtras.h>
@@ -24,6 +26,8 @@ SPDX-License-Identifier: MIT
 
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/IRBuilder.h"
+#include "llvmWrapper/IR/Type.h"
+#include "llvmWrapper/Support/Alignment.h"
 
 #include <algorithm>
 #include <array>
@@ -73,8 +77,7 @@ constexpr const char BuiltinPrefix[] = "__cm_cl_";
 #include "TranslationInfo.inc"
 #undef CMCL_AUTOGEN_BUILTIN_DESCS
 
-template <BuiltinID::Enum BiID>
-static void handleBuiltinCall(CallInst &BiCall);
+template <BuiltinID::Enum BiID> static void handleBuiltinCall(CallInst &BiCall);
 
 // Imports:
 //
@@ -121,6 +124,49 @@ Function *getGenXDeclarationForIdFromArgs(Type &RetTy, Range &&Args,
   return GenXIntrinsic::getGenXDeclaration(&M, Id, Types);
 }
 
+template <typename Range>
+Function *getInternalDeclarationForIdFromArgs(Type &RetTy, Range &&Args,
+                                              vc::InternalIntrinsic::ID Id,
+                                              Module &M) {
+  assert(vc::InternalIntrinsic::isInternalIntrinsic(Id) &&
+         "Expected vc internal intrinsic id");
+
+  SmallVector<Type *, 4> Types;
+  if (vc::InternalIntrinsic::isOverloadedRet(Id)) {
+    if (isa<StructType>(RetTy))
+      llvm::copy(cast<StructType>(RetTy).elements(), std::back_inserter(Types));
+    else
+      Types.push_back(&RetTy);
+  }
+  for (auto &&EnumArg : llvm::enumerate(Args)) {
+    if (vc::InternalIntrinsic::isOverloadedArg(Id, EnumArg.index()))
+      Types.push_back(EnumArg.value()->getType());
+  }
+
+  return vc::InternalIntrinsic::getInternalDeclaration(&M, Id, Types);
+}
+
+template <typename Range>
+Function *getAnyDeclarationForIdFromArgs(Type &RetTy, Range &&Args, unsigned Id,
+                                         Module &M) {
+  if (vc::InternalIntrinsic::isInternalIntrinsic(Id))
+    return getInternalDeclarationForIdFromArgs(
+        RetTy, std::forward<Range>(Args),
+        static_cast<vc::InternalIntrinsic::ID>(Id), M);
+  if (GenXIntrinsic::isGenXIntrinsic(Id))
+    return getGenXDeclarationForIdFromArgs(RetTy, std::forward<Range>(Args),
+                                           static_cast<GenXIntrinsic::ID>(Id),
+                                           M);
+
+  auto IID = static_cast<Intrinsic::ID>(Id);
+
+  SmallVector<Type *, 1> Types;
+  if (Intrinsic::isOverloaded(IID))
+    Types.push_back(&RetTy);
+
+  return Intrinsic::getDeclaration(&M, IID, Types);
+}
+
 static bool isCMCLBuiltin(const Function &F) {
   return F.getName().contains(BuiltinPrefix);
 }
@@ -188,7 +234,7 @@ Type &getTypeFromBuiltinOperand(CallInst &BiCall, int OpIdx) {
   Value &BiOp = *BiCall.getArgOperand(OpIdx);
   switch (BuiltinOperandKind[BiID][OpIdx]) {
   case OperandKind::Output:
-    return *BiOp.getType()->getPointerElementType();
+    return *IGCLLVM::getNonOpaquePtrEltTy(BiOp.getType());
   case OperandKind::Input:
   case OperandKind::Constant:
     return *BiOp.getType();
@@ -204,7 +250,7 @@ static int getVectorWidth(Type &Ty) {
 }
 
 // A helper function to get structure type from its element types.
-template <typename... ArgTys> Type &getStructureOf(ArgTys &... ElementTys) {
+template <typename... ArgTys> Type &getStructureOf(ArgTys &...ElementTys) {
   return *StructType::create("", &ElementTys...);
 }
 
@@ -247,10 +293,9 @@ std::vector<Value *> getTranslatedBuiltinOperands(CallInst &BiCall,
 template <BuiltinID::Enum BiID>
 Value &createMainInst(const std::vector<Value *> &Operands, Type &RetTy,
                       IRBuilder<> &IRB) {
-  auto *Decl = getGenXDeclarationForIdFromArgs(
-      RetTy, Operands,
-      static_cast<GenXIntrinsic::ID>(IntrinsicForBuiltin[BiID]),
-      *IRB.GetInsertBlock()->getModule());
+  auto *Decl =
+      getAnyDeclarationForIdFromArgs(RetTy, Operands, IntrinsicForBuiltin[BiID],
+                                     *IRB.GetInsertBlock()->getModule());
   auto *CI =
       IRB.CreateCall(Decl, Operands, RetTy.isVoidTy() ? "" : "cmcl.builtin");
   return *CI;
@@ -276,10 +321,149 @@ Value &createMainInst<BuiltinID::AbsFloat>(const std::vector<Value *> &Operands,
   return createLLVMIntrinsic<BuiltinID::AbsFloat>(Operands, RetTy, IRB);
 }
 
+//------------------------ Memory operations -----------------------------//
+
+template <>
+Value &createMainInst<BuiltinID::Gather>(const std::vector<Value *> &Operands,
+                                         Type &RetTy, IRBuilder<> &IRB) {
+  assert(Operands.size() == GatherOperand::Size &&
+         "builtin operands should be trasformed into LLVM masked.gather "
+         "intrinsic operands without changes");
+  auto AddrSpace =
+      cast<ConstantInt>(Operands[GatherOperand::AddressSpace])->getZExtValue();
+
+  auto *RetVTy = cast<IGCLLVM::FixedVectorType>(&RetTy);
+  auto *RetETy = RetVTy->getElementType();
+  auto *PtrETy = RetETy->getPointerTo(AddrSpace);
+  auto *PtrVTy =
+      IGCLLVM::FixedVectorType::get(PtrETy, RetVTy->getNumElements());
+  auto *MaskVTy =
+      IGCLLVM::FixedVectorType::get(IRB.getInt1Ty(), RetVTy->getNumElements());
+
+  auto *PtrV = IRB.CreateIntToPtr(Operands[GatherOperand::Pointers], PtrVTy);
+  auto Align = IGCLLVM::getCorrectAlign(
+      cast<ConstantInt>(Operands[GatherOperand::Alignment])->getZExtValue());
+  auto *MaskV = IRB.CreateTrunc(Operands[GatherOperand::Mask], MaskVTy);
+  auto *PassthruV = Operands[GatherOperand::Passthru];
+
+#if LLVM_VERSION_MAJOR > 12
+  auto *GatherV = IRB.CreateMaskedGather(RetVTy, PtrV, Align, MaskV, PassthruV);
+#else
+  auto *GatherV = IRB.CreateMaskedGather(PtrV, Align, MaskV, PassthruV);
+#endif
+
+  return *GatherV;
+}
+
+template <>
+Value &createMainInst<BuiltinID::Scatter>(const std::vector<Value *> &Operands,
+                                          Type &RetTy, IRBuilder<> &IRB) {
+  assert(Operands.size() == ScatterOperand::Size &&
+         "builtin operands should be trasformed into LLVM masked.scatter "
+         "intrinsic operands without changes");
+  auto AddrSpace =
+      cast<ConstantInt>(Operands[ScatterOperand::AddressSpace])->getZExtValue();
+
+  auto *DataV = Operands[ScatterOperand::Data];
+
+  auto *DataVTy = cast<IGCLLVM::FixedVectorType>(DataV->getType());
+  auto *DataETy = DataVTy->getElementType();
+  auto *PtrETy = DataETy->getPointerTo(AddrSpace);
+  auto *PtrVTy =
+      IGCLLVM::FixedVectorType::get(PtrETy, DataVTy->getNumElements());
+  auto *MaskVTy =
+      IGCLLVM::FixedVectorType::get(IRB.getInt1Ty(), DataVTy->getNumElements());
+
+  auto *PtrV = IRB.CreateIntToPtr(Operands[ScatterOperand::Pointers], PtrVTy);
+  auto Align = IGCLLVM::getCorrectAlign(
+      cast<ConstantInt>(Operands[ScatterOperand::Alignment])->getZExtValue());
+  auto *MaskV = IRB.CreateTrunc(Operands[ScatterOperand::Mask], MaskVTy);
+
+  auto *ScatterV = IRB.CreateMaskedScatter(DataV, PtrV, Align, MaskV);
+
+  return *ScatterV;
+}
+
+template <>
+Value &
+createMainInst<BuiltinID::AtomicBti>(const std::vector<Value *> &Operands,
+                                     Type &RetTy, IRBuilder<> &IRB) {
+  assert(Operands.size() == AtomicBtiOperand::Size &&
+         "builtin operands should be trasformed into VC intrinsic "
+         "vector_atomic_bti "
+         "intrinsic operands without changes");
+
+  auto *RetVTy = cast<IGCLLVM::FixedVectorType>(&RetTy);
+  auto *MaskVTy =
+      IGCLLVM::FixedVectorType::get(IRB.getInt1Ty(), RetVTy->getNumElements());
+  auto *MaskV = IRB.CreateTrunc(Operands[AtomicBtiOperand::Mask], MaskVTy);
+
+  std::vector<Value *> Args = Operands;
+
+  Args[AtomicBtiOperand::Mask] = MaskV;
+
+  auto *Decl = getAnyDeclarationForIdFromArgs(
+      RetTy, Args, IntrinsicForBuiltin[BuiltinID::AtomicBti],
+      *IRB.GetInsertBlock()->getModule());
+  auto *CI = IRB.CreateCall(Decl, Args, RetTy.isVoidTy() ? "" : "cmcl.builtin");
+  return *CI;
+}
+
+template <>
+Value &
+createMainInst<BuiltinID::AtomicSlm>(const std::vector<Value *> &Operands,
+                                     Type &RetTy, IRBuilder<> &IRB) {
+  assert(Operands.size() == AtomicSlmOperand::Size &&
+         "builtin operands should be trasformed into VC intrinsic "
+         "vector_atomic_slm "
+         "intrinsic operands without changes");
+
+  auto *RetVTy = cast<IGCLLVM::FixedVectorType>(&RetTy);
+  auto *MaskVTy =
+      IGCLLVM::FixedVectorType::get(IRB.getInt1Ty(), RetVTy->getNumElements());
+  auto *MaskV = IRB.CreateTrunc(Operands[AtomicSlmOperand::Mask], MaskVTy);
+
+  std::vector<Value *> Args = Operands;
+
+  Args[AtomicSlmOperand::Mask] = MaskV;
+
+  auto *Decl = getAnyDeclarationForIdFromArgs(
+      RetTy, Args, IntrinsicForBuiltin[BuiltinID::AtomicSlm],
+      *IRB.GetInsertBlock()->getModule());
+  auto *CI = IRB.CreateCall(Decl, Args, RetTy.isVoidTy() ? "" : "cmcl.builtin");
+  return *CI;
+}
+
+template <>
+Value &
+createMainInst<BuiltinID::AtomicUgm>(const std::vector<Value *> &Operands,
+                                     Type &RetTy, IRBuilder<> &IRB) {
+  assert(Operands.size() == AtomicUgmOperand::Size &&
+         "builtin operands should be trasformed into VC intrinsic "
+         "vector_atomic_ugm "
+         "intrinsic operands without changes");
+
+  auto *RetVTy = cast<IGCLLVM::FixedVectorType>(&RetTy);
+  auto *MaskVTy =
+      IGCLLVM::FixedVectorType::get(IRB.getInt1Ty(), RetVTy->getNumElements());
+  auto *MaskV = IRB.CreateTrunc(Operands[AtomicUgmOperand::Mask], MaskVTy);
+
+  std::vector<Value *> Args = Operands;
+
+  Args[AtomicUgmOperand::Mask] = MaskV;
+
+  auto *Decl = getAnyDeclarationForIdFromArgs(
+      RetTy, Args, IntrinsicForBuiltin[BuiltinID::AtomicUgm],
+      *IRB.GetInsertBlock()->getModule());
+  auto *CI = IRB.CreateCall(Decl, Args, RetTy.isVoidTy() ? "" : "cmcl.builtin");
+  return *CI;
+}
+
+
 //----------------------- Rounding operations ----------------------------//
 template <>
 Value &createMainInst<BuiltinID::Ceil>(const std::vector<Value *> &Operands,
-                                           Type &RetTy, IRBuilder<> &IRB) {
+                                       Type &RetTy, IRBuilder<> &IRB) {
   assert(Operands.size() == CeilOperand::Size &&
          "builtin operands should be trasformed into LLVM ceil "
          "intrinsic operands without changes");
@@ -288,7 +472,7 @@ Value &createMainInst<BuiltinID::Ceil>(const std::vector<Value *> &Operands,
 
 template <>
 Value &createMainInst<BuiltinID::Floor>(const std::vector<Value *> &Operands,
-                                            Type &RetTy, IRBuilder<> &IRB) {
+                                        Type &RetTy, IRBuilder<> &IRB) {
   assert(Operands.size() == FloorOperand::Size &&
          "builtin operands should be trasformed into LLVM floor "
          "intrinsic operands without changes");
@@ -297,7 +481,7 @@ Value &createMainInst<BuiltinID::Floor>(const std::vector<Value *> &Operands,
 
 template <>
 Value &createMainInst<BuiltinID::Trunc>(const std::vector<Value *> &Operands,
-                                            Type &RetTy, IRBuilder<> &IRB) {
+                                        Type &RetTy, IRBuilder<> &IRB) {
   assert(Operands.size() == TruncOperand::Size &&
          "builtin operands should be trasformed into LLVM trunc "
          "intrinsic operands without changes");
@@ -396,10 +580,10 @@ Value &createMainInst<BuiltinID::Powr>(const std::vector<Value *> &Operands,
   assert(Operands.size() == PowrOperand::Size &&
          "builtin operands should be trasformed into LLVM pow "
          "intrinsic operands without changes");
-  std::vector<Value*> Args{ Operands[PowrOperand::Base],
-                            Operands[PowrOperand::Exponent] };
-  auto *InstPow = cast<Instruction>(&createLLVMIntrinsic<BuiltinID::Powr>(
-      Args, RetTy, IRB));
+  std::vector<Value *> Args{Operands[PowrOperand::Base],
+                            Operands[PowrOperand::Exponent]};
+  auto *InstPow = cast<Instruction>(
+      &createLLVMIntrinsic<BuiltinID::Powr>(Args, RetTy, IRB));
   if (cast<ConstantInt>(Operands[PowrOperand::IsFast])->getSExtValue())
     InstPow->setFast(true);
   return *InstPow;
@@ -407,7 +591,7 @@ Value &createMainInst<BuiltinID::Powr>(const std::vector<Value *> &Operands,
 
 template <>
 Value &createMainInst<BuiltinID::Sin>(const std::vector<Value *> &Operands,
-                                       Type &RetTy, IRBuilder<> &IRB) {
+                                      Type &RetTy, IRBuilder<> &IRB) {
   assert(Operands.size() == SinOperand::Size &&
          "builtin operands should be trasformed into LLVM sin "
          "intrinsic operands without changes");
@@ -420,7 +604,7 @@ Value &createMainInst<BuiltinID::Sin>(const std::vector<Value *> &Operands,
 
 template <>
 Value &createMainInst<BuiltinID::Cos>(const std::vector<Value *> &Operands,
-                                       Type &RetTy, IRBuilder<> &IRB) {
+                                      Type &RetTy, IRBuilder<> &IRB) {
   assert(Operands.size() == CosOperand::Size &&
          "builtin operands should be trasformed into LLVM cos "
          "intrinsic operands without changes");
@@ -585,8 +769,7 @@ static bool handleBuiltin(Function &Builtin) {
     return false;
   auto BiID = decodeBuiltin(Builtin.getName());
   for (User *Usr : Builtin.users()) {
-    assert((BiID >= 0 && BiID < BuiltinID::Size &&
-            BuiltinCallHandlers[BiID]) &&
+    assert((BiID >= 0 && BiID < BuiltinID::Size && BuiltinCallHandlers[BiID]) &&
            "no handler for such builtin ID");
     BuiltinCallHandlers[BiID](*cast<CallInst>(Usr));
   }
