@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2021 Intel Corporation
+Copyright (C) 2019-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -41,11 +41,13 @@ public:
 
   explicit CMKernel(const PLATFORM &platform);
   ~CMKernel();
+  CMKernel(const CMKernel &) = delete;
+  CMKernel &operator=(const CMKernel &) = delete;
 
   PLATFORM m_platform;
   IGC::SOpenCLKernelInfo m_kernelInfo;
   IGC::COCLBTILayout m_btiLayout;
-  uint32_t m_GRFSizeInBytes;
+  uint32_t m_GRFSizeInBytes = 0;
   bool m_SupportsDebugging = false;
 
   // getter for convenience
@@ -79,6 +81,9 @@ public:
                                      ArgAccessKind access, bool isBindless,
                                      bool isStateful);
 
+  void createPointerLocalAnnotation(unsigned index, unsigned offset,
+                                    unsigned sizeInBytes, unsigned alignment);
+
   void createPrivateBaseAnnotation(unsigned argNo, unsigned byteSize,
                                    unsigned payloadPosition, int BTI,
                                    unsigned statelessPrivateMemSize,
@@ -97,6 +102,8 @@ public:
   // Sampler
   void createSamplerAnnotation(unsigned argNo, unsigned BTI);
 
+  void createAssertBufferArgAnnotation(unsigned Index, unsigned BTI,
+                                       unsigned Size, unsigned ArgOffset);
   void createPrintfBufferArgAnnotation(unsigned Index, unsigned BTI,
                                        unsigned Size, unsigned ArgOffset);
 
@@ -128,13 +135,13 @@ public:
     bool KernelIsDebuggable = false;
   };
 
-  explicit CGen8CMProgram(PLATFORM platform, const WA_TABLE &WATable,
-                          llvm::ArrayRef<char> SPIRV = llvm::None,
-                          llvm::Optional<llvm::StringRef> Opts = llvm::None);
+  explicit CGen8CMProgram(const CompileOptions &Opts, PLATFORM platform,
+                          const WA_TABLE &WATable,
+                          llvm::ArrayRef<char> SPIRV = llvm::None);
 
   // Produce the final ELF binary with the given CM kernels
   // in OpenCL format.
-  void CreateKernelBinaries(CompileOptions& Opts);
+  void CreateKernelBinaries();
   void GetZEBinary(llvm::raw_pwrite_stream &programBinary,
                    unsigned pointerSizeInBytes) override;
   bool HasErrors() const { return !m_ErrorLog.empty(); };
@@ -152,8 +159,8 @@ public:
   std::string m_ErrorLog;
 
 private:
-  llvm::ArrayRef<char> m_spirv;
-  llvm::Optional<llvm::StringRef> m_opts;
+  const CompileOptions &m_opts;
+  const llvm::ArrayRef<char> m_spirv;
 
   TmpFilesStorage extractRawDebugInfo(llvm::raw_ostream &ErrStream);
   std::unique_ptr<llvm::MemoryBuffer> buildZeDebugInfo();
