@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2022 Intel Corporation
+Copyright (C) 2020-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -142,14 +142,52 @@ std::unique_ptr<llvm::MemoryBuffer> getVCModuleBuffer() {
                                           false /* RequiresNullTerminator */);
 }
 
+static std::pair<std::string, unsigned>
+getPlatformName(const PLATFORM &Platform) {
+  constexpr unsigned ComputeTileMaskPVC = 7;
+  auto Core = Platform.eRenderCoreFamily;
+  auto Product = Platform.eProductFamily;
+  unsigned DevId = Platform.usDeviceID;
+  unsigned RevId = Platform.usRevId;
+
+  switch (Core) {
+  case IGFX_GEN8_CORE:
+    return {"Gen8", RevId};
+  case IGFX_GEN9_CORE:
+    if (Product == IGFX_BROXTON || Product == IGFX_GEMINILAKE)
+      return {"Gen9LP", RevId};
+    return {"Gen9", RevId};
+  case IGFX_GEN11_CORE:
+  case IGFX_GEN11LP_CORE:
+    return {"Gen11", RevId};
+  case IGFX_GEN12_CORE:
+  case IGFX_GEN12LP_CORE:
+    return {"XeLP", RevId};
+  case IGFX_XE_HP_CORE:
+    return {"XeHP", RevId};
+  case IGFX_XE_HPG_CORE:
+    if (Product == IGFX_DG2)
+      return {"XeHPG", RevId};
+    if (Product == IGFX_METEORLAKE)
+      return {"XeLPG", RevId};
+    break;
+  case IGFX_XE_HPC_CORE:
+    if (Product == IGFX_PVC)
+      return {"XeHPC", RevId & ComputeTileMaskPVC};
+    break;
+  default:
+    break;
+  }
+  IGC_ASSERT_EXIT_MESSAGE(0, "Unsupported platform");
+  return {"Invalid", -1};
+}
+
 static void adjustPlatform(const IGC::CPlatform &IGCPlatform,
                            vc::CompileOptions &Opts) {
   auto &PlatformInfo = IGCPlatform.getPlatformInfo();
-  unsigned RevId = PlatformInfo.usRevId;
-  const char *PlatformStr =
-      cmc::getPlatformStr(PlatformInfo, /* inout */ RevId);
-  Opts.CPUStr = PlatformStr ? PlatformStr : "";
-  Opts.RevId = RevId;
+
+  std::tie(Opts.CPUStr, Opts.RevId) = getPlatformName(PlatformInfo);
+
   Opts.HasL1ReadOnlyCache = IGCPlatform.hasL1ReadOnlyCache();
   Opts.HasLocalMemFenceSupress = IGCPlatform.localMemFenceSupress();
   Opts.HasMultiTile = IGCPlatform.hasMultiTile();
@@ -292,9 +330,19 @@ static void adjustDumpOptions(vc::CompileOptions &Opts) {
   }
 }
 
+static void adjustHashOptions(vc::CompileOptions &Opts,
+                              const ShaderHash &Hash) {
+  Opts.EnableHashMovs =
+      IGC_IS_FLAG_DISABLED(ForceDisableShaderDebugHashCodeInKernel) &&
+      IGC_IS_FLAG_ENABLED(ShaderDebugHashCodeInKernel);
+  Opts.EnableHashMovsAtPrologue = IGC_IS_FLAG_ENABLED(EnableHashMovsAtPrologue);
+  Opts.AsmHash = Hash.getAsmHash();
+}
+
 static void adjustOptions(const IGC::CPlatform &IGCPlatform,
                           TC::TB_DATA_FORMAT DataFormat,
-                          vc::CompileOptions &Opts, BuildDiag &Diag) {
+                          vc::CompileOptions &Opts, BuildDiag &Diag,
+                          const ShaderHash &Hash) {
   adjustPlatform(IGCPlatform, Opts);
   adjustFileType(DataFormat, Opts);
   adjustOptLevel(Opts);
@@ -302,6 +350,7 @@ static void adjustOptions(const IGC::CPlatform &IGCPlatform,
   adjustDumpOptions(Opts);
   adjustStackCalls(Opts, Diag);
   adjustDebugStrippingPolicy(Opts);
+  adjustHashOptions(Opts, Hash);
 
   adjustTransformationsAndOptimizations(Opts);
 }
@@ -372,10 +421,10 @@ static void dumpInputData(vc::ShaderDumper &Dumper, llvm::StringRef ApiOptions,
   if (!IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
     return;
 
-  Dumper.dumpText(ApiOptions, IsRaw ? "options_raw.txt" : "options.txt");
+  Dumper.dumpText(ApiOptions, IsRaw ? "options_raw" : "options");
   Dumper.dumpText(InternalOptions,
-                  IsRaw ? "internal_options_raw.txt" : "internal_options.txt");
-  Dumper.dumpBinary(Input, IsRaw ? "igc_input_raw.spv" : "igc_input.spv");
+                  IsRaw ? "internal_options_raw" : "internal_options");
+  Dumper.dumpBinary(Input, IsRaw ? "igc_input_raw" : "igc_input", "spv");
 }
 
 static bool tryAddAuxiliaryOptions(llvm::StringRef AuxOpt,
@@ -440,6 +489,8 @@ parseOptions(vc::ShaderDumper &Dumper, llvm::StringRef ApiOptions,
     RawInputDumper.release();
     InputDumper.release();
   }
+  if (ExpOptions)
+    ExpOptions->ApiOptions = ApiOptions.str();
   return std::move(ExpOptions);
 }
 
@@ -480,9 +531,9 @@ fillExternalData(vc::BinaryKind Binary, llvm::StringRef CPUStr,
   default:
     IGC_ASSERT_EXIT_MESSAGE(0, "Unknown binary format");
   }
-  ExtData.VCEmulationBIFModule =
-      getVCModuleBufferForArch<vc::bif::RawKind::Emulation>(CPUStr);
-  if (!ExtData.VCEmulationBIFModule)
+  ExtData.VCBuiltinsBIFModule =
+      getVCModuleBufferForArch<vc::bif::RawKind::Builtins>(CPUStr);
+  if (!ExtData.VCBuiltinsBIFModule)
     return {};
   ExtData.VCSPIRVBuiltinsBIFModule =
       getVCModuleBuffer<vc::bif::RawKind::SPIRVBuiltins>();
@@ -513,7 +564,7 @@ static void dumpPlatform(const vc::CompileOptions &Opts, PLATFORM Platform,
      << (Opts.CPUStr.empty() ? "(empty)" : Opts.CPUStr) << ", " << Opts.RevId
      << "\n";
 
-  Dumper.dumpText(Os.str(), "platform.be.txt");
+  Dumper.dumpText(Os.str(), "platform.be");
 #endif
 }
 
@@ -577,7 +628,7 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
 
   BuildDiag Diag;
   vc::CompileOptions &Opts = ExpOptions.get();
-  adjustOptions(IGCPlatform, InputDataFormatTemp, Opts, Diag);
+  adjustOptions(IGCPlatform, InputDataFormatTemp, Opts, Diag, Hash);
 
   // here we have Opts set and can dump what we got from runtime and how
   // we understood it. We need to do it before output error on unknown platform
@@ -610,21 +661,16 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
                                SpecConstValues);
   if (!ExpOutput)
     return getError(ExpOutput.takeError(), OutputArgs);
-  vc::CompileOutput &Res = ExpOutput.get();
+  auto &CompileResult = ExpOutput.get();
+
+  vc::CGen8CMProgram CMProgram{Opts, IGCPlatform.getPlatformInfo(),
+                               IGCPlatform.getWATable(), Input};
+  vc::createBinary(CMProgram, CompileResult);
 
   switch (Opts.Binary) {
-  case vc::BinaryKind::CM: {
-    auto &CompileResult = std::get<vc::cm::CompileOutput>(Res);
-    outputBinary(CompileResult.IsaBinary, llvm::StringRef(), Diag, OutputArgs);
-    break;
-  }
   case vc::BinaryKind::OpenCL: {
-    auto &CompileResult = std::get<vc::ocl::CompileOutput>(Res);
-    vc::CGen8CMProgram CMProgram{IGCPlatform.getPlatformInfo(),
-                                 IGCPlatform.getWATable()};
-    vc::createBinary(CMProgram, CompileResult);
     validateCMProgramForOCLBin(CMProgram);
-    CMProgram.CreateKernelBinaries(Opts);
+    CMProgram.CreateKernelBinaries();
     Util::BinaryStream ProgramBinary;
     CMProgram.GetProgramBinary(ProgramBinary, CompileResult.PointerSizeInBytes);
     llvm::StringRef BinaryRef{ProgramBinary.GetLinearPointer(),
@@ -643,10 +689,6 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
     break;
   }
   case vc::BinaryKind::ZE: {
-    auto &CompileResult = std::get<vc::ocl::CompileOutput>(Res);
-    vc::CGen8CMProgram CMProgram{IGCPlatform.getPlatformInfo(),
-                                 IGCPlatform.getWATable()};
-    vc::createBinary(CMProgram, CompileResult);
     llvm::SmallVector<char, 0> ProgramBinary;
     llvm::raw_svector_ostream ProgramBinaryOS{ProgramBinary};
     CMProgram.GetZEBinary(ProgramBinaryOS, CompileResult.PointerSizeInBytes);
@@ -654,6 +696,9 @@ std::error_code vc::translateBuild(const TC::STB_TranslateInputArgs *InputArgs,
     if (CMProgram.HasErrors())
       return getError(CMProgram.GetError(), OutputArgs);
 
+    if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
+      Opts.Dumper->dumpBinary(ProgramBinary, "", "progbin");
+
     llvm::StringRef BinaryRef{ProgramBinary.data(), ProgramBinary.size()};
     outputBinary(BinaryRef, {}, Diag, OutputArgs);
     break;
