@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2022 Intel Corporation
+Copyright (C) 2019-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -120,7 +120,7 @@ class KernelArgInfoBuilder
             switch (ArgKindID)
             {
                 case ArgKind::General:
-                    return Private;
+                  return Private;
                 case ArgKind::Buffer:
                 case ArgKind::SVM:
                 case ArgKind::Image1D:
@@ -130,11 +130,14 @@ class KernelArgInfoBuilder
                 case ArgKind::Image2DMediaBlock:
                 case ArgKind::Image3D:
                 case ArgKind::BindlessBuffer:
-                    return Global;
+                  return Global;
                 case ArgKind::Sampler:
-                    return Constant;
+                  return Constant;
+                case ArgKind::SLM:
+                  return Local;
                 default:
-                    IGC_ASSERT_EXIT_MESSAGE(0, "implicit args cannot appear in kernel arg info");
+                  IGC_ASSERT_EXIT_MESSAGE(
+                      0, "implicit args cannot appear in kernel arg info");
             }
             return NotSpecified;
         }
@@ -218,7 +221,7 @@ void CMKernel::createConstArgumentAnnotation(unsigned argNo,
 
     // EnableZEBinary: ZEBinary related code
     zebin::ZEInfoBuilder::addPayloadArgumentByValue(m_kernelInfo.m_zePayloadArgs,
-        payloadPosition, sizeInBytes, argNo, offsetInArg);
+        payloadPosition, sizeInBytes, argNo, offsetInArg, false);
 }
 
 // TODO: this is incomplete. Media sampler types are not supported now.
@@ -275,6 +278,13 @@ getOCLImageType(llvm::GenXOCLRuntimeInfo::KernelArgInfo::KindType Kind)
     }
 }
 
+static inline bool checkStateful(unsigned int BTI) {
+  static constexpr unsigned int STATELESS_NONCOHERENT_BTI = 253;
+  static constexpr unsigned int STATELESS_BTI = 255;
+  // BTI > 255 - will be mapped to entry 0, it is still stateful access
+  return (BTI < STATELESS_NONCOHERENT_BTI || BTI > STATELESS_BTI);
+}
+
 void CMKernel::createImageAnnotation(
     unsigned argNo, unsigned BTI,
     llvm::GenXOCLRuntimeInfo::KernelArgInfo::KindType Kind,
@@ -316,75 +326,109 @@ void CMKernel::createImplicitArgumentsAnnotation(unsigned payloadPosition)
 
 void CMKernel::createPointerGlobalAnnotation(unsigned index, unsigned offset,
                                              unsigned sizeInBytes, unsigned BTI,
-                                             ArgAccessKind access, bool isBindless)
-{
-    auto ptrAnnotation = std::make_unique<iOpenCL::PointerArgumentAnnotation>();
-    ptrAnnotation->IsStateless = !isBindless;
-    ptrAnnotation->IsBindlessAccess = isBindless;
-    ptrAnnotation->AddressSpace = iOpenCL::KERNEL_ARGUMENT_ADDRESS_SPACE_GLOBAL;
-    ptrAnnotation->ArgumentNumber = index;
-    ptrAnnotation->BindingTableIndex = BTI;
-    ptrAnnotation->PayloadPosition = offset;
-    ptrAnnotation->PayloadSizeInBytes = sizeInBytes;
-    ptrAnnotation->LocationIndex = 0;
-    ptrAnnotation->LocationCount = 0;
-    ptrAnnotation->IsEmulationArgument = false;
-    m_kernelInfo.m_pointerArgument.push_back(std::move(ptrAnnotation));
+                                             ArgAccessKind access,
+                                             bool isBindless, bool isStateful) {
+  auto ptrAnnotation = std::make_unique<iOpenCL::PointerArgumentAnnotation>();
+  // must be only one true or all false
+  IGC_ASSERT(!(isBindless && isStateful));
+  // TODO igorban: next line must be (!isBindless && !isStateful)
+  ptrAnnotation->IsStateless = !isBindless;
+  ptrAnnotation->IsBindlessAccess = isBindless;
+  ptrAnnotation->HasStatefulAccess = isStateful;
+  ptrAnnotation->AddressSpace = iOpenCL::KERNEL_ARGUMENT_ADDRESS_SPACE_GLOBAL;
+  ptrAnnotation->ArgumentNumber = index;
+  ptrAnnotation->BindingTableIndex = BTI;
+  ptrAnnotation->PayloadPosition = offset;
+  ptrAnnotation->PayloadSizeInBytes = sizeInBytes;
+  ptrAnnotation->LocationIndex = 0;
+  ptrAnnotation->LocationCount = 0;
+  ptrAnnotation->IsEmulationArgument = false;
+  m_kernelInfo.m_pointerArgument.push_back(std::move(ptrAnnotation));
 
-    const auto zeAddrMode = isBindless ?
-      zebin::PreDefinedAttrGetter::ArgAddrMode::bindless :
-      zebin::PreDefinedAttrGetter::ArgAddrMode::stateless;
+  zebin::PreDefinedAttrGetter::ArgAddrMode zeAddrMode;
+  if (isBindless) {
+    zeAddrMode = zebin::PreDefinedAttrGetter::ArgAddrMode::bindless;
+  } else if (isStateful) {
+    zeAddrMode = zebin::PreDefinedAttrGetter::ArgAddrMode::stateful;
+  } else {
+    zeAddrMode = zebin::PreDefinedAttrGetter::ArgAddrMode::stateless;
+  }
 
-    // EnableZEBinary: ZEBinary related code
-    zebin::ZEInfoBuilder::addPayloadArgumentByPointer(
-        m_kernelInfo.m_zePayloadArgs, offset, sizeInBytes, index,
-        zeAddrMode, zebin::PreDefinedAttrGetter::ArgAddrSpace::global,
-        getZEArgAccessType(access));
+  // EnableZEBinary: ZEBinary related code
+  zebin::ZEInfoBuilder::addPayloadArgumentByPointer(
+      m_kernelInfo.m_zePayloadArgs, offset, sizeInBytes, index, zeAddrMode,
+      zebin::PreDefinedAttrGetter::ArgAddrSpace::global,
+      getZEArgAccessType(access));
+  if (isStateful)
     zebin::ZEInfoBuilder::addBindingTableIndex(m_kernelInfo.m_zeBTIArgs, BTI,
                                                index);
 }
 
-void CMKernel::createPrivateBaseAnnotation(
-    unsigned argNo, unsigned byteSize, unsigned payloadPosition, int BTI,
-    unsigned statelessPrivateMemSize) {
-  auto ptrAnnotation = std::make_unique<iOpenCL::PrivateInputAnnotation>();
+void CMKernel::createPointerLocalAnnotation(unsigned index, unsigned offset,
+                                            unsigned sizeInBytes,
+                                            unsigned alignment) {
+  auto locAnnotation = std::make_unique<iOpenCL::LocalArgumentAnnotation>();
 
+  locAnnotation->Alignment = alignment;
+
+  // TODO: offset must be aligned.
+  locAnnotation->PayloadPosition = offset;
+  locAnnotation->PayloadSizeInBytes = sizeInBytes;
+  locAnnotation->ArgumentNumber = index;
+  locAnnotation->LocationIndex = 0;
+  locAnnotation->LocationCount = 0;
+  m_kernelInfo.m_localPointerArgument.push_back(std::move(locAnnotation));
+
+  // EnableZEBinary: ZEBinary related code
+  zebin::ZEInfoBuilder::addPayloadArgumentByPointer(
+      m_kernelInfo.m_zePayloadArgs, offset, sizeInBytes, index,
+      zebin::PreDefinedAttrGetter::ArgAddrMode::slm,
+      zebin::PreDefinedAttrGetter::ArgAddrSpace::local,
+      zebin::PreDefinedAttrGetter::ArgAccessType::readwrite, alignment);
+}
+
+void CMKernel::createPrivateBaseAnnotation(unsigned argNo, unsigned byteSize,
+                                           unsigned payloadPosition, int BTI,
+                                           unsigned statelessPrivateMemSize,
+                                           bool isStateful) {
+  auto ptrAnnotation = std::make_unique<iOpenCL::PrivateInputAnnotation>();
   ptrAnnotation->AddressSpace = iOpenCL::KERNEL_ARGUMENT_ADDRESS_SPACE_PRIVATE;
   ptrAnnotation->ArgumentNumber = argNo;
   // PerThreadPrivateMemorySize is defined as "Total private memory requirements for each OpenCL work-item."
   ptrAnnotation->PerThreadPrivateMemorySize = statelessPrivateMemSize;
   ptrAnnotation->BindingTableIndex = BTI;
-  ptrAnnotation->IsStateless = true;
+  ptrAnnotation->IsStateless = !isStateful;
   ptrAnnotation->PayloadPosition = payloadPosition;
   ptrAnnotation->PayloadSizeInBytes = byteSize;
+  ptrAnnotation->HasStatefulAccess = isStateful;
   m_kernelInfo.m_pointerInput.push_back(std::move(ptrAnnotation));
 
-    // EnableZEBinary: ZEBinary related code
-    zebin::ZEInfoBuilder::addPayloadArgumentImplicit(m_kernelInfo.m_zePayloadArgs,
-        zebin::PreDefinedAttrGetter::ArgType::private_base_stateless,
-        payloadPosition, byteSize);
+  // EnableZEBinary: ZEBinary related code
+  zebin::ZEInfoBuilder::addPayloadArgumentImplicit(
+      m_kernelInfo.m_zePayloadArgs,
+      zebin::PreDefinedAttrGetter::ArgType::private_base_stateless,
+      payloadPosition, byteSize);
 }
 
 void CMKernel::createBufferStatefulAnnotation(unsigned argNo,
-                                              ArgAccessKind accessKind)
-{
-    auto constInput = std::make_unique<iOpenCL::ConstantInputAnnotation>();
+                                              ArgAccessKind accessKind) {
+  auto constInput = std::make_unique<iOpenCL::ConstantInputAnnotation>();
 
-    constInput->ConstantType = iOpenCL::DATA_PARAMETER_BUFFER_STATEFUL;
-    constInput->Offset = 0;
-    constInput->PayloadPosition = 0;
-    constInput->PayloadSizeInBytes = 0;
-    constInput->ArgumentNumber = argNo;
-    constInput->LocationIndex = 0;
-    constInput->LocationCount = 0;
-    m_kernelInfo.m_constantInputAnnotation.push_back(std::move(constInput));
+  constInput->ConstantType = iOpenCL::DATA_PARAMETER_BUFFER_STATEFUL;
+  constInput->Offset = 0;
+  constInput->PayloadPosition = 0;
+  constInput->PayloadSizeInBytes = 0;
+  constInput->ArgumentNumber = argNo;
+  constInput->LocationIndex = 0;
+  constInput->LocationCount = 0;
+  m_kernelInfo.m_constantInputAnnotation.push_back(std::move(constInput));
 
-    // EnableZEBinary: ZEBinary related code
-    zebin::ZEInfoBuilder::addPayloadArgumentByPointer(m_kernelInfo.m_zePayloadArgs,
-        0, 0, argNo,
-        zebin::PreDefinedAttrGetter::ArgAddrMode::stateful,
-        zebin::PreDefinedAttrGetter::ArgAddrSpace::global,
-        getZEArgAccessType(accessKind));
+  // EnableZEBinary: ZEBinary related code
+  zebin::ZEInfoBuilder::addPayloadArgumentByPointer(
+      m_kernelInfo.m_zePayloadArgs, 0, 0, argNo,
+      zebin::PreDefinedAttrGetter::ArgAddrMode::stateful,
+      zebin::PreDefinedAttrGetter::ArgAddrSpace::global,
+      getZEArgAccessType(accessKind));
 }
 
 void CMKernel::createSizeAnnotation(unsigned initPayloadPosition,
@@ -412,6 +456,15 @@ void CMKernel::createSizeAnnotation(unsigned initPayloadPosition,
         iOpenCL::DATA_PARAMETER_DATA_SIZE * 3);
 }
 
+void CMKernel::createAssertBufferArgAnnotation(unsigned Index, unsigned BTI,
+                                               unsigned Size,
+                                               unsigned ArgOffset) {
+    // EnableZEBinary: ZEBinary related code
+    zebin::ZEInfoBuilder::addPayloadArgumentImplicit(
+        m_kernelInfo.m_zePayloadArgs,
+        zebin::PreDefinedAttrGetter::ArgType::assert_buffer, ArgOffset, Size);
+}
+
 void CMKernel::createPrintfBufferArgAnnotation(unsigned Index, unsigned BTI,
                                                unsigned Size,
                                                unsigned ArgOffset) {
@@ -536,6 +589,7 @@ static void generateKernelArgInfo(
     case KindType::General:
     case KindType::Buffer:
     case KindType::SVM:
+    case KindType::SLM:
     case KindType::Sampler:
     case KindType::Image1D:
     case KindType::Image1DArray:
@@ -603,24 +657,33 @@ static void setArgumentsInfo(const GenXOCLRuntimeInfo::KernelInfo &Info,
                                   iOpenCL::DATA_PARAMETER_NUM_WORK_GROUPS);
       break;
     case ArgKind::Buffer:
-      Kernel.createPointerGlobalAnnotation(Arg.getIndex(), ArgOffset,
-                                           Arg.getSizeInBytes(), Arg.getBTI(),
-                                           Arg.getAccessKind(), /*isBindless=*/false);
-      Kernel.createBufferStatefulAnnotation(Arg.getIndex(),
-                                            Arg.getAccessKind());
+      Kernel.createPointerGlobalAnnotation(
+          Arg.getIndex(), ArgOffset, Arg.getSizeInBytes(), Arg.getBTI(),
+          Arg.getAccessKind(), /*isBindless=*/false,
+          checkStateful(Arg.getBTI()));
+      if (checkStateful(Arg.getBTI()))
+        Kernel.createBufferStatefulAnnotation(Arg.getIndex(),
+                                              Arg.getAccessKind());
       Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
       break;
     case ArgKind::BindlessBuffer:
-      Kernel.createPointerGlobalAnnotation(Arg.getIndex(), ArgOffset,
-                                           Arg.getSizeInBytes(), Arg.getBTI(),
-                                           Arg.getAccessKind(), /*isBindless=*/true);
+      Kernel.createPointerGlobalAnnotation(
+          Arg.getIndex(), ArgOffset, Arg.getSizeInBytes(), Arg.getBTI(),
+          Arg.getAccessKind(), /*isBindless=*/true,
+          /*isStateful=*/false);
       Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
       break;
     case ArgKind::SVM:
-      Kernel.createPointerGlobalAnnotation(Arg.getIndex(), ArgOffset,
-                                           Arg.getSizeInBytes(), Arg.getBTI(),
-                                           Arg.getAccessKind(), /*isBindless=*/false);
+      Kernel.createPointerGlobalAnnotation(
+          Arg.getIndex(), ArgOffset, Arg.getSizeInBytes(), Arg.getBTI(),
+          Arg.getAccessKind(), /*isBindless=*/false,
+          /*isStateful=*/false);
       Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
+
+      break;
+    case ArgKind::SLM:
+      Kernel.createPointerLocalAnnotation(
+          Arg.getIndex(), ArgOffset, Arg.getSizeInBytes(), Arg.getAlignment());
       break;
     case ArgKind::Sampler:
       Kernel.createSamplerAnnotation(Arg.getIndex(), Arg.getBTI());
@@ -636,21 +699,23 @@ static void setArgumentsInfo(const GenXOCLRuntimeInfo::KernelInfo &Info,
                                    Arg.getAccessKind());
       Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
       break;
+    case ArgKind::AssertBuffer:
+      Kernel.createAssertBufferArgAnnotation(Arg.getIndex(), Arg.getBTI(),
+                                             Arg.getSizeInBytes(), ArgOffset);
+      break;
     case ArgKind::PrintBuffer:
       Kernel.createPrintfBufferArgAnnotation(Arg.getIndex(), Arg.getBTI(),
                                              Arg.getSizeInBytes(), ArgOffset);
       break;
-    case ArgKind::PrivateBase:
-      if (Info.getStatelessPrivMemSize() != 0) {
-        auto PrivMemSize = Info.getStatelessPrivMemSize();
-        Kernel.createPrivateBaseAnnotation(Arg.getIndex(), Arg.getSizeInBytes(),
-                                           ArgOffset, Arg.getBTI(),
-                                           PrivMemSize);
-        Kernel.m_kernelInfo.m_executionEnvironment
-            .PerThreadPrivateOnStatelessSize = PrivMemSize;
-        Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
-      }
-      break;
+    case ArgKind::PrivateBase: {
+      auto PrivMemSize = Info.getStatelessPrivMemSize();
+      Kernel.createPrivateBaseAnnotation(Arg.getIndex(), Arg.getSizeInBytes(),
+                                         ArgOffset, Arg.getBTI(), PrivMemSize,
+                                         checkStateful(Arg.getBTI()));
+      Kernel.m_kernelInfo.m_executionEnvironment
+          .PerThreadPrivateOnStatelessSize = PrivMemSize;
+      Kernel.m_kernelInfo.m_argIndexMap[Arg.getIndex()] = Arg.getBTI();
+    } break;
     case ArgKind::ByValSVM:
       // Do nothing because it has already been linearized and implicit args
       // will be set instead of it.
@@ -674,8 +739,22 @@ static void setArgumentsInfo(const GenXOCLRuntimeInfo::KernelInfo &Info,
       ConstantBufferLengthInGRF;
 }
 
+// a helper function to get the conservative vISA stack size when having
+// stack call
+static uint32_t getConservativeVISAStackSize(const PLATFORM& platform) {
+  auto isPVCXT = [](const PLATFORM &platform) {
+    return platform.eProductFamily == IGFX_PVC &&
+           platform.usRevId >= REVISION_B;
+  };
+  // PVC-XT needs a smaller default size to avoid stack OOM
+  // Ref: IGC::CEncoder::getSpillMemSizeWithFG
+  if (isPVCXT(platform))
+    return 64 * 1024;
+  return 128 * 1024;
+}
+
 static void setExecutionInfo(const GenXOCLRuntimeInfo::KernelInfo &BackendInfo,
-                             const FINALIZER_INFO &JitterInfo,
+                             const vISA::FINALIZER_INFO &JitterInfo,
                              CMKernel &Kernel) {
   Kernel.m_SupportsDebugging = BackendInfo.supportsDebugging();
   Kernel.m_GRFSizeInBytes = BackendInfo.getGRFSizeInBytes();
@@ -690,16 +769,22 @@ static void setExecutionInfo(const GenXOCLRuntimeInfo::KernelInfo &BackendInfo,
   // HasBarriers isn't bool and preserves number of barriers for PVC+ targets
   // dispite misleading naming.
   ExecEnv.HasBarriers = BackendInfo.getNumBarriers();
+  ExecEnv.HasSample = BackendInfo.usesSample();
   ExecEnv.HasDPAS = BackendInfo.usesDPAS();
   ExecEnv.numThreads = BackendInfo.getNumThreads();
   ExecEnv.HasReadWriteImages = BackendInfo.usesReadWriteImages();
   ExecEnv.SubgroupIndependentForwardProgressRequired = true;
-  ExecEnv.NumGRFRequired = JitterInfo.numGRFTotal;
+  ExecEnv.NumGRFRequired = JitterInfo.stats.numGRFTotal;
   ExecEnv.RequireDisableEUFusion = BackendInfo.requireDisableEUFusion();
 
   // Allocate spill-fill buffer
-  if (JitterInfo.isSpill || JitterInfo.hasStackcalls)
-    ExecEnv.PerThreadScratchSpace += JitterInfo.spillMemUsed;
+  if (JitterInfo.hasStackcalls) {
+    ExecEnv.PerThreadScratchSpace +=
+        getConservativeVISAStackSize(Kernel.m_platform);
+  } else {
+    ExecEnv.PerThreadScratchSpace += JitterInfo.stats.spillMemUsed;
+  }
+
   if (!JitterInfo.hasStackcalls && BackendInfo.getTPMSize() != 0)
     // CM stack calls and thread-private memory use the same value to control
     // scratch space. Consequently, if we have stack calls, there is no need
@@ -740,7 +825,7 @@ static void setExecutionInfo(const GenXOCLRuntimeInfo::KernelInfo &BackendInfo,
   Kernel.RecomputeBTLayout(NumUAVs, NumResources);
 }
 
-static void setGenBinary(const FINALIZER_INFO &JitterInfo,
+static void setGenBinary(const vISA::FINALIZER_INFO &JitterInfo,
                          const std::vector<uint8_t> &GenBinary,
                          CMKernel &Kernel) {
   // Kernel binary, padding is hard-coded.
@@ -756,7 +841,7 @@ static void setGenBinary(const FINALIZER_INFO &JitterInfo,
   PO.m_programBin = KernelBin;
   PO.m_programSize = Size + Padding;
   PO.m_unpaddedProgramSize = Size;
-  PO.m_InstructionCount = JitterInfo.numAsmCount;
+  PO.m_InstructionCount = JitterInfo.stats.numAsmCountUnweighted;
 }
 
 static void setVISAAsm(
@@ -776,7 +861,7 @@ static void setDebugInfo(const std::vector<char> &DebugInfo, CMKernel &Kernel) {
   Kernel.getProgramOutput().m_debugDataSize = DebugInfoSize;
 }
 
-static void setGtpinInfo(const FINALIZER_INFO &JitterInfo,
+static void setGtpinInfo(const vISA::FINALIZER_INFO &JitterInfo,
                          const GenXOCLRuntimeInfo::GTPinInfo &GtpinInfo,
                          CMKernel &Kernel) {
   Kernel.getProgramOutput().m_scratchSpaceUsedByGtpin =
