@@ -31,17 +31,7 @@ SPDX-License-Identifier: MIT
 
 namespace vc {
 
-namespace ocl {
 using CompileOutput = llvm::GenXOCLRuntimeInfo::CompiledModuleT;
-} // namespace ocl
-
-namespace cm {
-struct CompileOutput {
-  std::string IsaBinary;
-};
-} // namespace cm
-
-using CompileOutput = std::variant<cm::CompileOutput, ocl::CompileOutput>;
 
 enum class FileType { SPIRV, LLVM_TEXT, LLVM_BINARY };
 
@@ -109,6 +99,8 @@ struct CompileOptions {
   llvm::Optional<unsigned> StackMemSize;
   llvm::Optional<unsigned> GRFSize;
 
+  bool EnableAutoLargeGRF = false;
+
   bool ForceLiveRangesLocalizationForAccUsage = false;
   bool ForceDisableNonOverlappingRegionOpt = false;
   bool ForceDisableIndvarsOpt = false;
@@ -161,6 +153,8 @@ struct CompileOptions {
   bool IgnoreLoopUnrollThresholdOnPragma = false;
   unsigned InteropSubgroupSize = 16;
 
+  bool CheckGVClobbering = false;
+
   bool EnableHashMovs = false;
   bool EnableHashMovsAtPrologue = false;
   uint64_t AsmHash = 0;
@@ -170,7 +164,7 @@ struct ExternalData {
   std::unique_ptr<llvm::MemoryBuffer> OCLGenericBIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCPrintf32BIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCPrintf64BIFModule;
-  std::unique_ptr<llvm::MemoryBuffer> VCEmulationBIFModule;
+  std::unique_ptr<llvm::MemoryBuffer> VCBuiltinsBIFModule;
   std::unique_ptr<llvm::MemoryBuffer> VCSPIRVBuiltinsBIFModule;
   llvm::ArrayRef<const char*> VISALTOStrings;
   llvm::ArrayRef<const char*> DirectCallFunctions;
