@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2019-2021 Intel Corporation
+Copyright (C) 2019-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -26,13 +26,13 @@ class PassRegistry;
 //
 // CMImpParam - Transforms to enable implicit parameters
 //
-Pass *createCMImpParamPass(bool, bool);
+Pass *createCMImpParamPass(bool);
 
 //===----------------------------------------------------------------------===//
 //
 // CMKernelArgOffset - Determine offset of each CM kernel argument
 //
-Pass *createCMKernelArgOffsetPass(unsigned GrfByteSize, bool OCLCodeGen);
+Pass *createCMKernelArgOffsetPass(unsigned GrfByteSize);
 
 //===----------------------------------------------------------------------===//
 //
@@ -82,6 +82,9 @@ void initializeGenXTrampolineInsertionPass(PassRegistry &);
 ModulePass *createGenXLinkageCorruptorPass();
 void initializeGenXLinkageCorruptorPass(PassRegistry &);
 
+FunctionPass *createGenXTranslateIntrinsicsPass();
+void initializeGenXTranslateIntrinsicsPass(PassRegistry &);
+
 } // End llvm namespace
 
 #endif // VC_GENXOPTS_GENXOPTS_H
