@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -50,9 +50,8 @@ inline int getStackAmount(const llvm::Function *F,
   return Result;
 }
 
-// Check if a function is to emulate instructions.
-inline bool isEmulationFunction(const llvm::Function &F) {
-  return F.hasFnAttribute(FunctionMD::VCEmulationRoutine);
+inline bool isBuiltinFunction(const llvm::Function &F) {
+  return F.hasFnAttribute(FunctionMD::VCBuiltinFunction);
 }
 
 // Utility function to tell whether a Function is a vISA kernel.
@@ -94,7 +93,7 @@ inline bool isIndirect(const llvm::Function *F) {
     return false;
   if (vc::isKernel(F))
     return false;
-  if (vc::isEmulationFunction(*F))
+  if (vc::isBuiltinFunction(*F))
     return false;
   if (!F->hasAddressTaken() && F->hasLocalLinkage())
     return false;
@@ -294,7 +293,8 @@ public:
     // payload in memory (for those platforms r0.0 is used to obtain the
     // pointer).
     IMP_IMPL_ARGS_BUFFER = 0xF << AKBitsForCategory,
-    IMP_PSEUDO_INPUT = 0x10 << AKBitsForCategory
+    IMP_PSEUDO_INPUT = 0x10 << AKBitsForCategory,
+    IMP_OCL_ASSERT_BUFFER = 0x11 << AKBitsForCategory,
   };
 
   enum { SKIP_OFFSET_VAL = -1 };
@@ -355,6 +355,10 @@ inline bool isPrintBufferKind(uint32_t ArgKind) {
   return isImplicitArgKind(ArgKind, KernelMetadata::IMP_OCL_PRINTF_BUFFER);
 }
 
+inline bool isAssertBufferKind(uint32_t ArgKind) {
+  return isImplicitArgKind(ArgKind, KernelMetadata::IMP_OCL_ASSERT_BUFFER);
+}
+
 inline bool isPrivateBaseKind(uint32_t ArgKind) {
   return isImplicitArgKind(ArgKind, KernelMetadata::IMP_OCL_PRIVATE_BASE);
 }
