@@ -118,12 +118,12 @@ ModulePass *createGenXNumberingWrapperPass();
 ModulePass *createGenXLiveRangesWrapperPass();
 ModulePass *createGenXRematerializationWrapperPass();
 ModulePass *createGenXCoalescingWrapperPass();
-ModulePass *createGenXGVClobberCheckerWrapperPass();
+ModulePass *createGenXGVClobberCheckerPass();
 ModulePass *createGenXAddressCommoningWrapperPass();
 ModulePass *createGenXArgIndirectionWrapperPass();
 FunctionPass *createGenXTidyControlFlowPass();
 ModulePass *createGenXVisaRegAllocWrapperPass();
-ModulePass *createGenXCisaBuilderWrapperPass();
+ModulePass *createGenXCisaBuilderPass();
 ModulePass *createGenXFinalizerPass();
 ModulePass *createGenXDebugInfoPass();
 ModulePass *createGenXGlobalValueLoweringPass();
