@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2022 Intel Corporation
+Copyright (C) 2017-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -918,7 +918,7 @@ bool GenXAddressCommoning::vectorizeAddrsFromOneVector(
   Type *FirstType = Extracts[0].Addr->getOperand(0)->getType();
   IGC_ASSERT(FirstType);
 
-  for (auto e : Extracts) {
+  for (auto &e : Extracts) {
     Type *Tp = e.Addr->getOperand(0)->getType();
     if (ConvertWholeRegion && (Tp != FirstType))
       return false;
@@ -950,7 +950,7 @@ bool GenXAddressCommoning::vectorizeAddrsFromOneVector(
             ->getNumElements();
     IGC_ASSERT(ST);
     Num = getLegalRegionSizeForTarget(*ST, R, 0, true /*Allow2D*/,
-                                      InputNumElements);
+                                      true /*UseRealIdx*/, InputNumElements);
     if (Num == 1)
       continue;
     // Even after legalizing the region, we can still vectorize to more than
