@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2020-2021 Intel Corporation
+Copyright (C) 2020-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -683,6 +683,8 @@ public:
 
   GenObjectWrapper(VISAKernel &VK, const Function &F);
   ~GenObjectWrapper() { releaseDebugInfoResources(); }
+  GenObjectWrapper(const GenObjectWrapper &) = delete;
+  GenObjectWrapper &operator=(const GenObjectWrapper &) = delete;
 
   bool hasErrors() const { return !ErrMsg.empty(); }
 
@@ -857,6 +859,8 @@ public:
   ~GenXFunction() {
     LLVM_DEBUG(dbgs() << "~GenXFunction() called for " << F.getName() << "\n");
   }
+  GenXFunction(const GenXFunction &) = delete;
+  GenXFunction &operator=(const GenXFunction &) = delete;
 
   llvm::StringRef GetVISAFuncName() const override {
     // TODO: this is not quite correct since VISA names is defined by VISA label
@@ -1358,8 +1362,9 @@ getKernelAndVisaOwners(const GenXFunctionConstPtrList &GFs) {
   GenXFunctionConstPtrList StackCalls;
   std::copy_if(GFs.begin(), GFs.end(), std::back_inserter(StackCalls),
                isStackCallGF);
-  auto KGFIt = std::find_if(GFs.begin(), GFs.end(), isKernelGF);
   IGC_ASSERT(std::count_if(GFs.begin(), GFs.end(), isKernelGF) == 1);
+  auto KGFIt = std::find_if(GFs.begin(), GFs.end(), isKernelGF);
+  IGC_ASSERT_EXIT(KGFIt != GFs.end());
   return {*KGFIt, std::move(StackCalls)};
 }
 
