@@ -332,16 +332,16 @@ class GenXDepressurizer : public FGPassImplInterface,
                           public IDMixin<GenXDepressurizer> {
   enum { FlagThreshold = 6, AddrThreshold = 32, GRFThreshold = 2560,
          FlagGRFTolerance = 3840 };
-  bool Modified;
-  GenXGroupBaling *Baling;
-  DominatorTree *DT;
-  LoopInfoBase<BasicBlock, Loop> *LI;
-  PseudoCFG *PCFG;
-  unsigned MaxPressure;
+  bool Modified = false;
+  GenXGroupBaling *Baling = nullptr;
+  DominatorTree *DT = nullptr;
+  LoopInfoBase<BasicBlock, Loop> *LI = nullptr;
+  PseudoCFG *PCFG = nullptr;
+  unsigned MaxPressure = 0;
   std::map<Function *, unsigned> SubroutinePressures;
   std::map<BasicBlock *, Liveness> LiveIn;
   std::map<BasicBlock *, Liveness> LiveOut;
-  Liveness *Live;
+  Liveness *Live = nullptr;
   // A numbering of instructions. Because of the way the basic block ordering
   // is constructed, if instruction I2 is reachable from instruction I1, then
   // InstNumbers[I1] < InstNumbers[I2], unless the reachability is via a
@@ -500,6 +500,21 @@ void GenXDepressurizer::orderAndNumber(Function *F) {
           default:
             break;
           }
+          // Surfaces are created after GenXCategory, placed before
+          // the dependend instruction and are not baled with anything.
+          // It is better to move the surface next to its user since
+          // it increases chances to get an immediate descriptor.
+          for (auto &U : Inst->operands()) {
+            auto *UInst = dyn_cast<Instruction>(U);
+            auto IID = GenXIntrinsic::getGenXIntrinsicID(UInst);
+            if (IID == GenXIntrinsic::genx_convert &&
+                isa<Constant>(UInst->getOperand(0))) {
+              // UInst will be numbered in the next iterations.
+              IGC_ASSERT(UInst->hasOneUse());
+              UInst->removeFromParent();
+              UInst->insertBefore(InsertBefore);
+            }
+          }
           Inst->removeFromParent();
           Inst->insertBefore(InsertBefore);
         }
@@ -638,6 +653,7 @@ void GenXDepressurizer::getLiveOut(BasicBlock *BB, Liveness *Live) {
       if (!Phi)
         break;
       Live->removeValue(Phi);
+      IGC_ASSERT_EXIT(Phi->getBasicBlockIndex(BB) >= 0);
       Live->addValue(Phi->getIncomingValue(Phi->getBasicBlockIndex(BB)));
     }
   }
