@@ -76,9 +76,6 @@ static cl::opt<bool> OptStricterOword(
 static cl::opt<bool> OptStricterAlloc(
     "vc-i64emu-strict-report-alloc", cl::init(false), cl::Hidden,
     cl::desc("strict check will break on 64-bit alloc"));
-static cl::opt<bool> OptStricterFaddr(
-    "vc-i64emu-strict-report-faddr", cl::init(false), cl::Hidden,
-    cl::desc("strict check will break on 64-bit faddr"));
 static cl::opt<bool>
     OptStricterConst("vc-i64emu-strict-const", cl::init(false), cl::Hidden,
                      cl::desc("strict check will break on 64-bit constanti"));
@@ -1053,6 +1050,7 @@ Value *GenXEmulate::Emu64Expander::visitGenxTrunc(CallInst &CI) {
     }
   }
 
+  IGC_ASSERT_EXIT(Result);
   if (Result->getType() != CI.getType())
     Result = Builder.CreateBitCast(Result, CI.getType());
 
@@ -1084,7 +1082,7 @@ Value *GenXEmulate::Emu64Expander::visitGenxMinMax(CallInst &CI) {
     CondVal = Builder.CreateICmpSLT(Lhs, Rhs);
     break;
   }
-  IGC_ASSERT(CondVal);
+  IGC_ASSERT_EXIT(CondVal);
   CondVal = ensureEmulated(CondVal);
   return ensureEmulated(Builder.CreateSelect(CondVal, Lhs, Rhs));
 }
@@ -1647,8 +1645,6 @@ bool GenXEmulate::Emu64Expander::hasStrictEmulationRequirement(
     return OptStricterOword;
   case GenXIntrinsic::genx_alloca:
     return OptStricterAlloc;
-  case GenXIntrinsic::genx_faddr:
-    return OptStricterFaddr;
   }
 
   switch (Inst->getOpcode()) {
