@@ -160,11 +160,11 @@ Value *GenXGEPLowering::visitPtrToIntInst(PtrToIntInst &PTI) {
   auto *Ty = PTI.getType();
   Value *NewI = nullptr;
 
-  if (SrcTy->isPointerTy()) {
+  if (SrcTy->isPtrOrPtrVectorTy()) {
     NewI = Builder->CreatePtrToInt(Src, Ty);
     NewI->takeName(&PTI);
   } else {
-    IGC_ASSERT(SrcTy->isIntegerTy());
+    IGC_ASSERT(SrcTy->isIntOrIntVectorTy());
     NewI = Builder->CreateIntCast(Src, Ty, false);
   }
 
@@ -222,8 +222,6 @@ Value *GenXGEPLowering::visitGetElementPtrInst(GetElementPtrInst &GEP) {
           PointerValue->getType(), APInt(PtrMathSizeInBits, Offset));
         PointerValue = Builder->CreateAdd(PointerValue, OffsetVal);
       }
-
-      Ty = StTy->getElementType(Field);
     } else {
       Ty = GTI.getIndexedType();
       if (const ConstantInt *CI = dyn_cast<ConstantInt>(Idx)) {
