@@ -20,11 +20,10 @@ SPDX-License-Identifier: MIT
 #include "GenXGotoJoin.h"
 #include "GenXIntrinsics.h"
 #include "GenXModule.h"
-#include "GenXRegion.h"
 #include "GenXSubtarget.h"
 #include "GenXUtil.h"
 #include "GenXVisa.h"
-#include "Probe/Assertion.h"
+
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
 #include "visa_igc_common_header.h"
@@ -46,6 +45,8 @@ SPDX-License-Identifier: MIT
 #include "llvm/Support/Debug.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
 
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 using namespace genx;
 
@@ -211,7 +212,8 @@ Type *GenXInlineAsmLowering::rewriteTypeForCR(Type *CRType) const {
   Type *Int1Ty = Type::getInt1Ty(*Context);
   return CRType->isVectorTy()
              ? IGCLLVM::FixedVectorType::get(
-                   Int1Ty, cast<VectorType>(CRType)->getNumElements())
+                   Int1Ty,
+                   cast<IGCLLVM::FixedVectorType>(CRType)->getNumElements())
              : Int1Ty;
 }
 
