@@ -432,7 +432,6 @@ private:
   std::pair<BranchInst *, Value *>
   findSimdCFLoopBranchAndCondition(const Loop &L);
   SimdCFRegionPtr tryMatchLoop(BasicBlock &BB);
-  bool CheckForBranching(Function *F, SimdCFRegionsT *Regions);
 };
 
 Value *GenXPredToSimdCF::getEMAddr(Function *F) {
@@ -591,19 +590,17 @@ bool GenXPredToSimdCF::areOppositeSimdCFConditions(Value *IfCond,
     LLVM_DEBUG(dbgs() << "Comparing "; IfMask->dump(); dbgs() << "against   ";
                ElseMask->getOperand(0)->dump());
     return IfMask == ElseMask->getOperand(0);
-  } else if (isIfCondReduceOr && !isElseCondReduceOr) {
-    return false; // not implemented
-  } else if (!isIfCondReduceOr && isElseCondReduceOr) {
-    return false; // not implemented
-  } else if (!isIfCondReduceOr && !isElseCondReduceOr) {
-    if (!genx::isPredicate(IfMask) || !genx::isPredNot(ElseMask))
-      return false;
-    LLVM_DEBUG(dbgs() << "Comparing "; IfMask->dump(); dbgs() << "against   ";
-               ElseMask->getOperand(0)->dump());
-    return IfMask == ElseMask->getOperand(0);
   }
-
-  return false;
+  if (isIfCondReduceOr && !isElseCondReduceOr)
+    return false; // not implemented
+  if (!isIfCondReduceOr && isElseCondReduceOr)
+    return false; // not implemented
+  // last variant - if (!isIfCondReduceOr && !isElseCondReduceOr)
+  if (!genx::isPredicate(IfMask) || !genx::isPredNot(ElseMask))
+    return false;
+  LLVM_DEBUG(dbgs() << "Comparing "; IfMask->dump(); dbgs() << "against   ";
+             ElseMask->getOperand(0)->dump());
+  return IfMask == ElseMask->getOperand(0);
 }
 
 BasicBlock *GenXPredToSimdCF::getIfExitFromIfBranch(BranchInst &BrInst) {
@@ -948,56 +945,6 @@ SimdCFRegionPtr GenXPredToSimdCF::tryMatchLoop(BasicBlock &BB) {
       &getAnalysis<DominatorTreeWrapperPass>().getDomTree());
 }
 
-bool GenXPredToSimdCF::CheckForBranching(Function *F, SimdCFRegionsT *Regions) {
-  SmallVector<BasicBlock *, 10> Terms;
-  for (auto &BB : F->getBasicBlockList()) {
-    auto *Terminator = dyn_cast<BranchInst>(BB.getTerminator());
-    if (Terminator && Terminator->isConditional()) {
-      Terms.push_back(&BB);
-      LLVM_DEBUG(Terminator->dump());
-    }
-  }
-  auto CheckTerms = [&](Region *R) {
-    for (auto *It : R->blocks()) {
-      LLVM_DEBUG(It->getTerminator()->dump());
-      Terms.erase(std::remove(Terms.begin(), Terms.end(), &*It), Terms.end());
-    }
-  };
-  LLVM_DEBUG(dbgs() << " Check now:\n");
-  for (auto &R : *Regions) {
-    if (auto *IFThen = dyn_cast_or_null<SimdCFIfRegion>(R.get())) {
-      CheckTerms(IFThen);
-      CheckTerms(IFThen->getIfThenRegion());
-
-      auto It = IFThen->getIfSimdBranch()->getParent();
-      LLVM_DEBUG(It->dump());
-      Terms.erase(std::remove(Terms.begin(), Terms.end(), It), Terms.end());
-      if (!IFThen->hasElse())
-        continue;
-      CheckTerms(IFThen->getIfElseRegion());
-    }
-    if (auto *Loop = dyn_cast_or_null<SimdCFLoopRegion>(R.get())) {
-      CheckTerms(Loop);
-      auto It = Loop->getSimdBranch()->getParent();
-      Terms.erase(std::remove(Terms.begin(), Terms.end(), It), Terms.end());
-    }
-  }
-  // Not supported any non-simd conditionals for now
-  if (Terms.empty()) {
-    LLVM_DEBUG(if (!Regions->empty()) dbgs() << "Success check\n");
-
-    return true;
-  }
-
-  LLVM_DEBUG(if (!Regions->empty()) {
-    dbgs() << "Failed to check Regions for conditional\n";
-    for (auto *Term : Terms)
-      Term->dump();
-  });
-
-  return false;
-}
-
 // TODO: rewrite this algo to use Region as arg instead of Fuction to allow
 // reuse it for matching subregions
 SimdCFRegionsT GenXPredToSimdCF::findSimdCFRegions(Function &F) {
@@ -1046,8 +993,6 @@ SimdCFRegionsT GenXPredToSimdCF::findSimdCFRegions(Function &F) {
     Visited.insert(&BB);
     LLVM_DEBUG(dbgs() << "Marking BB '" << BB.getName() << "' as visited\n");
   }
-  if (!CheckForBranching(&F, &Regions))
-    Regions.clear();
 
   return Regions;
 }
@@ -1611,8 +1556,8 @@ bool checkSelects(PHINode &PHINode, SimdCFIfRegion *R,
   auto *IV2 = PHINode.getIncomingValue(1);
   // R.getIfThen vs R.getIfElse
   if (isa<SelectInst>(IV1) && isa<SelectInst>(IV2)) {
-    SelectInst *V1 = dyn_cast<SelectInst>(IV1);
-    SelectInst *V2 = dyn_cast<SelectInst>(IV2);
+    auto *V1 = cast<SelectInst>(IV1);
+    auto *V2 = cast<SelectInst>(IV2);
 
     // TODO: support check for each block in region
     IGC_ASSERT(R->getIfThenRegion()->getEntry() ==
