@@ -76,8 +76,8 @@ static cl::opt<bool> EmitVLoadStore(
 
 // There's another copy of DL string in clang/lib/Basic/Targets.cpp
 static std::string getDL(bool Is64Bit) {
-  return Is64Bit ? "e-p:64:64-p6:32:32-i64:64-n8:16:32:64"
-                 : "e-p:32:32-p6:32:32-i64:64-n8:16:32";
+  return Is64Bit ? "e-p:64:64-p3:32:32-p6:32:32-i64:64-n8:16:32:64"
+                 : "e-p:32:32-p3:32:32-p6:32:32-i64:64-n8:16:32";
 }
 
 namespace llvm {
@@ -91,9 +91,9 @@ void initializeGenXPasses(PassRegistry &registry) {
   initializeGenXArgIndirectionWrapperPass(registry);
   initializeGenXCategoryWrapperPass(registry);
   initializeGenXCFSimplificationPass(registry);
-  initializeGenXCisaBuilderWrapperPass(registry);
+  initializeGenXCisaBuilderPass(registry);
   initializeGenXCoalescingWrapperPass(registry);
-  initializeGenXGVClobberCheckerWrapperPass(registry);
+  initializeGenXGVClobberCheckerPass(registry);
   initializeGenXDeadVectorRemovalPass(registry);
   initializeGenXDepressurizerWrapperPass(registry);
   initializeGenXEarlySimdCFConformancePass(registry);
@@ -609,7 +609,7 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   vc::addPass(PM, createGenXLiveRangesWrapperPass());
   /// .. include:: GenXGVClobberChecker.cpp
   if (BackendConfig.checkGVClobbering())
-    vc::addPass(PM, createGenXGVClobberCheckerWrapperPass());
+    vc::addPass(PM, createGenXGVClobberCheckerPass());
   /// .. include:: GenXCoalescing.cpp
   vc::addPass(PM, createGenXCoalescingWrapperPass());
   /// .. include:: GenXAddressCommoning.cpp
@@ -619,7 +619,7 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   /// .. include:: GenXTidyControlFlow.cpp
   vc::addPass(PM, createGenXTidyControlFlowPass());
   /// .. include:: GenXVisaRegAlloc.h
-  auto RegAlloc = createGenXVisaRegAllocWrapperPass();
+  auto *RegAlloc = createGenXVisaRegAllocWrapperPass();
   vc::addPass(PM, RegAlloc);
   if (BackendConfig.enableRegAllocDump())
     vc::addPass(PM, createGenXModuleAnalysisDumperPass(RegAlloc, FGDumpsPrefix,
@@ -627,7 +627,7 @@ bool GenXTargetMachine::addPassesToEmitFile(PassManagerBase &PM,
   if (!DisableVerify)
     vc::addPass(PM, createVerifierPass());
   /// .. include:: GenXCisaBuilder.cpp
-  vc::addPass(PM, createGenXCisaBuilderWrapperPass());
+  vc::addPass(PM, createGenXCisaBuilderPass());
   vc::addPass(PM, createGenXFinalizerPass());
   vc::addPass(PM, createGenXDebugInfoPass());
 
