@@ -1140,15 +1140,23 @@ Region IVSplitter::createSplitRegion(Type *SrcTy, IVSplitter::RegionType RT) {
 // function takes 64-bit constant value (vector or scalar) and splits it
 // into an equivalent vector of 32-bit constant (as if it was Bitcast-ed)
 static void convertI64ToI32(Constant &K, SmallVectorImpl<Constant *> &K32) {
-  auto I64To32 = [](const Constant &K) {
+  auto I64To32 = [](Constant &K) {
     // we expect only scalar types here
     IGC_ASSERT(!isa<VectorType>(K.getType()));
     IGC_ASSERT(K.getType()->isIntegerTy(64));
-    auto *Ty32 = K.getType()->getInt32Ty(K.getContext());
+    auto *Ty32 = Type::getInt32Ty(K.getContext());
     if (isa<UndefValue>(K)) {
       Constant *Undef = UndefValue::get(Ty32);
       return std::make_pair(Undef, Undef);
     }
+    if (isa<ConstantExpr>(K)) {
+      auto *Lo = ConstantExpr::getTrunc(&K, Ty32);
+      auto *Amount = ConstantInt::get(K.getType(), 32);
+      auto *Shift = ConstantExpr::getLShr(&K, Amount);
+      auto *Hi = ConstantExpr::getTrunc(Shift, Ty32);
+      return std::make_pair(Lo, Hi);
+    }
+    IGC_ASSERT_EXIT(isa<ConstantInt>(K));
     auto *KI = cast<ConstantInt>(&K);
     uint64_t Val64 = KI->getZExtValue();
     const auto UI32ValueMask = std::numeric_limits<uint32_t>::max();
@@ -1293,7 +1301,7 @@ void genx::adjustPhiNodesForBlockRemoval(BasicBlock *Succ, BasicBlock *BB)
       break;
     // For this phi node, get the incoming for BB.
     int Idx = Phi->getBasicBlockIndex(BB);
-    IGC_ASSERT(Idx >= 0);
+    IGC_ASSERT_EXIT(Idx >= 0);
     Value *Incoming = Phi->getIncomingValue(Idx);
     // Iterate through BB's predecessors. For the first one, replace the
     // incoming block with the predecessor. For subsequent ones, we need
@@ -1575,8 +1583,11 @@ void genx::LayoutBlocks(Function &func)
 }
 
 Value *genx::getBitCastedValue(Value *V) {
-  while (auto *BCI = dyn_cast<BitCastInst>(V))
-    V = BCI->getOperand(0);
+  while (isa<BitCastInst>(V) ||
+         (isa<ConstantExpr>(V) &&
+          cast<ConstantExpr>(V)->getOpcode() == CastInst::BitCast))
+    V = isa<BitCastInst>(V) ? cast<Instruction>(V)->getOperand(0)
+                            : cast<ConstantExpr>(V)->getOperand(0);
   return V;
 }
 
@@ -2037,15 +2048,11 @@ bool genx::isPredefRegDestination(const Value *V) {
   return GenXIntrinsic::isWritePredefReg(V);
 }
 
-// info is at main template function
-CastInst *genx::scalarizeOrVectorizeIfNeeded(Instruction *Inst, Type *RefType) {
-  return scalarizeOrVectorizeIfNeeded(Inst, &RefType, std::next(&RefType));
-}
-
 // info is at main template function
 CastInst *genx::scalarizeOrVectorizeIfNeeded(Instruction *Inst,
   Instruction *InstToReplace) {
-  return scalarizeOrVectorizeIfNeeded(Inst, &InstToReplace, std::next(&InstToReplace));
+  SmallVector<Type *, 1> Types = {InstToReplace->getType()};
+  return scalarizeOrVectorizeIfNeeded(Inst, Types.begin(), Types.end());
 }
 
 Function *genx::getFunctionPointerFunc(Value *V) {
@@ -2147,7 +2154,7 @@ bool genx::isWrPredRegionLegalSetP(const CallInst &WrPredRegion) {
   auto Offset = cast<ConstantInt>(
                     WrPredRegion.getOperand(vc::WrPredRegionOperand::Offset))
                     ->getZExtValue();
-  if (ExecSize >= 32 || !isPowerOf2_64(ExecSize))
+  if (ExecSize > 32 || !isPowerOf2_64(ExecSize))
     return false;
   if (ExecSize == 32)
     return Offset == 0;
@@ -2401,7 +2408,7 @@ genx::getInterveningGVStoreOrNull(Instruction *LI, Instruction *UIOrPos,
 
 void genx::collectRelatedCallSitesPerFunction(
     Instruction *SI, FunctionGroup *FG,
-    std::map<Function *, llvm::SetVector<Instruction *>>
+    std::unordered_map<Function *, llvm::SetVector<Instruction *>>
         &CallSitesPerFunction) {
   using FuncsSeenT = llvm::SetVector<Function *>;
   auto collectRelatedCallSites = [&](Function *Func, FuncsSeenT *FuncsSeen,
@@ -2413,7 +2420,7 @@ void genx::collectRelatedCallSitesPerFunction(
       if (isa<CallBase>(FuncUser)) {
         auto *Call = dyn_cast<Instruction>(FuncUser);
         auto *curFunction = Call->getFunction();
-        if (llvm::find(*FG, curFunction) == FG->end())
+        if (FG && llvm::find(*FG, curFunction) == FG->end())
           continue;
         CallSitesPerFunction[curFunction].insert(Call);
         collectRelatedCallSites(curFunction, FuncsSeen,
