@@ -67,6 +67,7 @@ inline int log2(T Val)
 
 // Common functionality for media ld/st lowering and CISA builder
 template <typename T> inline T roundedVal(T Val, T RoundUp) {
+  IGC_ASSERT_EXIT(Val >= 1);
   T RoundedVal = static_cast<T>(1) << genx::log2(Val);
   if (RoundedVal < Val)
     RoundedVal *= 2;
@@ -564,12 +565,9 @@ CastInst *scalarizeOrVectorizeIfNeeded(Instruction *Inst,
   return scalarizeOrVectorizeIfNeeded(Inst, Types.begin(), Types.end());
 }
 
-CastInst *scalarizeOrVectorizeIfNeeded(Instruction *Inst, Type *RefType);
-
 CastInst *scalarizeOrVectorizeIfNeeded(Instruction *Inst,
                                        Instruction *InstToReplace);
 
-
 // Returns log alignment for align type and target grf width, because ALIGN_GRF
 // must be target-dependent.
 unsigned getLogAlignment(VISA_Align Align, unsigned GRFWidth);
@@ -587,7 +585,7 @@ bool isWrPredRegionLegalSetP(const CallInst &WrPredRegion);
 // Many of our analyzes do not check whether a function F's user
 // which is a CallInst calls exactly F. This may not be true
 // when a function pointer is passed as an argument of a call to
-// another function, e.g. genx.faddr intrinsic.
+// another function.
 // Returns V casted to CallInst if the check is true,
 // nullptr otherwise.
 const CallInst *checkFunctionCall(const Value *V, const Function *F);
@@ -783,7 +781,7 @@ bool checkGVClobberingByInterveningStore(
 
 void collectRelatedCallSitesPerFunction(
     Instruction *SI, class FunctionGroup *FG,
-    std::map<Function *, llvm::SetVector<Instruction *>>
+    std::unordered_map<Function *, llvm::SetVector<Instruction *>>
         &InstAndRelatedCallSitesPerFunction);
 
 llvm::SetVector<Instruction *> getAncestorGVLoads(Instruction *I,
