@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2018-2022 Intel Corporation
+Copyright (C) 2018-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -93,6 +93,9 @@ public:
   static char ID;
   explicit GenXPacketize() : ModulePass(ID) {}
   ~GenXPacketize() { releaseMemory(); }
+  GenXPacketize(const GenXPacketize &) = delete;
+  GenXPacketize &operator=(const GenXPacketize &) = delete;
+
   StringRef getPassName() const override { return "GenX Packetize"; }
   void getAnalysisUsage(AnalysisUsage &AU) const override {
     AU.addRequiredID(BreakCriticalEdgesID);
@@ -112,7 +115,7 @@ private:
 
   Value *getPacketizeValue(Value *OrigValue);
   Value *getUniformValue(Value *OrigValue);
-  Function *getVectorIntrinsic(Module *M, unsigned id, std::vector<Type *> &ArgTy);
+  Function *getVectorIntrinsic(Module *M, unsigned id, ArrayRef<Type *> ArgTy);
   Value *packetizeConstant(Constant *pConstant);
   Value *packetizeGenXIntrinsic(Instruction *pInst);
   Value *packetizeLLVMIntrinsic(Instruction *pInst);
@@ -289,7 +292,7 @@ Function *GenXPacketize::vectorizeSIMTFunction(Function *F, unsigned Width) {
       // FIXME: check the pointer defined by an argument or an alloca
       // [N x float]* should packetize to [N x <8 x float>]*
       auto VTy = PointerType::get(
-          B->GetVectorType(I.getType()->getPointerElementType()),
+          B->GetVectorType(IGCLLVM::getNonOpaquePtrEltTy(I.getType())),
           I.getType()->getPointerAddressSpace());
       ArgTypes.push_back(VTy);
     }
@@ -649,14 +652,12 @@ void GenXPacketize::findUniformInsts(Function &F) {
       const Value* use = (*UI);
       if (auto LI = dyn_cast<LoadInst>(use)) {
         UniformInsts.insert(LI);
-      }
-      else if (auto GEP = dyn_cast<GetElementPtrInst>(use)) {
+      } else if (auto GEP = dyn_cast<GetElementPtrInst>(use)) {
         if (GEP->getPointerOperand() == def) {
           UniformInsts.insert(GEP);
           uvset.push((Value *)GEP);
         }
-      }
-      else if (auto SI = dyn_cast<StoreInst>(use)) {
+      } else if (auto SI = dyn_cast<StoreInst>(use)) {
         if (SI->getPointerOperand() == def)
           UniformInsts.insert(SI);
         else {
@@ -666,8 +667,10 @@ void GenXPacketize::findUniformInsts(Function &F) {
             uvset.push((Value *)AI);
           }
         }
-      }
-      else if (auto CI = dyn_cast<CallInst>(use)) {
+      } else if (auto *Cast = dyn_cast<BitCastInst>(use)) {
+        UniformInsts.insert(Cast);
+        uvset.push((Value *)Cast);
+      } else if (auto CI = dyn_cast<CallInst>(use)) {
         if (Function *Callee = CI->getCalledFunction()) {
           if (GenXIntrinsic::isVLoadStore(Callee)) {
             UniformInsts.insert(CI);
@@ -686,17 +689,18 @@ void GenXPacketize::findUniformInsts(Function &F) {
       if (auto UseI = dyn_cast<Instruction>(use)) {
         if (isa<StoreInst>(UseI)) {
           UniformInsts.insert(UseI);
-        }
-        else if (auto LI = dyn_cast<LoadInst>(UseI)) {
+        } else if (auto LI = dyn_cast<LoadInst>(UseI)) {
           UniformInsts.insert(UseI);
           if (LI->getType()->isPointerTy())
             uvset.push(UseI);
-        }
-        else if (auto GEP = dyn_cast<GetElementPtrInst>(UseI)) {
+        } else if (auto GEP = dyn_cast<GetElementPtrInst>(UseI)) {
           if (GEP->hasAllConstantIndices()) {
             uvset.push(UseI);
             UniformInsts.insert(UseI);
           }
+        } else if (auto *Cast = dyn_cast<BitCastInst>(UseI)) {
+          uvset.push(UseI);
+          UniformInsts.insert(UseI);
         }
       }
     }
@@ -762,21 +766,13 @@ Value *GenXPacketize::getUniformValue(Value *OrigValue) {
 //////////////////////////////////////////////////////////////////////////
 /// @brief Returns the equivalent vector intrinsic for the input scalar
 /// intrinsic
-Function *GenXPacketize::getVectorIntrinsic(Module *M, unsigned id,
-  std::vector<Type *> &ArgTy)
-{
-  if (id == Intrinsic::fma) {
-    return Intrinsic::getDeclaration(M, (Intrinsic::ID)id, ArgTy[0]);
-  } else if (id == Intrinsic::pow) {
-    // for some reason, passing the 2 vector input args to the pow declaration
-    // results in a malformed vectored pow intrinsic. Forcing the expected
-    // vector input here.
-    return Intrinsic::getDeclaration(M, (Intrinsic::ID)id, B->mSimdFP32Ty);
-  } else if ((id == Intrinsic::maxnum) || (id == Intrinsic::minnum)) {
-    return Intrinsic::getDeclaration(M, (Intrinsic::ID)id, ArgTy[0]);
-  } else {
-    return GenXIntrinsic::getAnyDeclaration(M, id, ArgTy);
-  }
+Function *GenXPacketize::getVectorIntrinsic(Module *M, unsigned ID,
+                                            ArrayRef<Type *> ArgTy) {
+  if (GenXIntrinsic::isGenXIntrinsic(ID))
+    return GenXIntrinsic::getGenXDeclaration(
+        M, static_cast<GenXIntrinsic::ID>(ID), ArgTy);
+  return Intrinsic::getDeclaration(M, static_cast<Intrinsic::ID>(ID),
+                                   {ArgTy[0]});
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -822,7 +818,7 @@ Value *GenXPacketize::packetizeLLVMIntrinsic(Instruction *pInst) {
   // packetize intrinsic operands
   std::vector<Type *> vectorArgTys;
   std::vector<Value *> packetizedArgs;
-  for (auto &operand : IGCLLVM::args(pCall)) {
+  for (auto &operand : pCall->args()) {
     auto VV = getPacketizeValue(operand.get());
     packetizedArgs.push_back(VV);
     vectorArgTys.push_back(VV->getType());
@@ -880,7 +876,7 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *pInst) {
     Type *pReturnTy;
     if (pInst->getType()->isPointerTy()) {
       // two types of pointers, <N x Ty>* or <N x Ty*>
-      Type *pDstScalarTy = pInst->getType()->getPointerElementType();
+      Type *pDstScalarTy = IGCLLVM::getNonOpaquePtrEltTy(pInst->getType());
 
       if (pPacketizedSrc->getType()->isVectorTy()) {
         // <N x Ty*>
@@ -974,7 +970,7 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *pInst) {
       IGC_ASSERT(cast<VectorType>(pVecSrc->getType())
                      ->getElementType()
                      ->isPointerTy());
-      auto Align = LI->getAlignment();
+      auto Align = IGCLLVM::getAlignmentValue(LI);
       pReplacedInst = B->MASKED_GATHER(pVecSrc, Align);
     } else
       pReplacedInst = B->ALIGNED_LOAD(pVecSrc, IGCLLVM::getAlign(*LI));
@@ -989,7 +985,7 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *pInst) {
       IGC_ASSERT(cast<VectorType>(pVecDstPtrs->getType())
                      ->getElementType()
                      ->isPointerTy());
-      auto Align = cast<StoreInst>(pInst)->getAlignment();
+      auto Align = IGCLLVM::getAlignmentValue(cast<StoreInst>(pInst));
       pReplacedInst = B->MASKED_SCATTER(pVecSrc, pVecDstPtrs, Align);
     } else {
       pReplacedInst = B->STORE(pVecSrc, pVecDstPtrs);
@@ -1138,9 +1134,9 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *pInst) {
       // vector struct input, need to loop over components and build up new
       // struct allocation
       Value *pAlloca = B->ALLOCA(
-          B->GetVectorType(pInst->getType()->getPointerElementType()));
+          B->GetVectorType(IGCLLVM::getNonOpaquePtrEltTy(pInst->getType())));
       uint32_t numElems =
-          pInst->getType()->getPointerElementType()->getArrayNumElements();
+          IGCLLVM::getNonOpaquePtrEltTy(pInst->getType())->getArrayNumElements();
 
       for (uint32_t i = 0; i < numElems; ++i) {
         Value *pTrueSrcElem = B->LOAD(pTrueSrc, {0, i});
@@ -1565,6 +1561,16 @@ Value *GenXPacketize::packetizeGenXIntrinsic(Instruction *inst) {
         if (R.Indirect) {
           R.Indirect = getPacketizeValue(R.Indirect);
         }
+        auto *Src0Pack = getPacketizeValue(CI->getOperand(0));
+        auto *Src0PackVTy = cast<IGCLLVM::FixedVectorType>(Src0Pack->getType());
+        auto *ResVTy = cast<IGCLLVM::FixedVectorType>(CI->getType());
+        if (!R.Indirect && ResVTy->getNumElements() == 1 &&
+            Src0PackVTy->getNumElements() == B->mVWidth) {
+          R.Width = B->mVWidth;
+          replacement =
+              R.createWrRegion(Src0Pack, NewV1, CI->getName(), CI, DL);
+          return replacement;
+        }
         replacement =
             R.createWrRegion(CI->getOperand(0), NewV1, CI->getName(), CI, DL);
         return replacement;
@@ -1760,7 +1766,7 @@ void GenXPacketize::replaceAllUsesNoTypeCheck(Value *pInst, Value *pNewInst) {
 ///        So we have to remove these manually.
 void GenXPacketize::removeDeadInstructions(Function &F) {
   SmallVector<Instruction *, 8> unused;
-  for (auto RMI : ReplaceMap) {
+  for (const auto &RMI : ReplaceMap) {
     if (RMI.first != RMI.second) {
       if (Instruction *UnusedInst =
               (Instruction *)dyn_cast<Instruction>(RMI.first)) {
@@ -1833,7 +1839,7 @@ void GenXPacketize::fixupLLVMIntrinsics(Function &F) {
 GlobalVariable *GenXPacketize::findGlobalExecMask() {
   // look for the global EMask variable if exists
   for (auto &Global : M->getGlobalList()) {
-    auto Ty = Global.getType()->getPointerElementType();
+    auto Ty = IGCLLVM::getNonOpaquePtrEltTy(Global.getType());
     if (Ty->isVectorTy() &&
         cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements() ==
             CMSimdCFLower::MAX_SIMD_CF_WIDTH) {
