@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021 Intel Corporation
+Copyright (C) 2021-2023 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -118,6 +118,7 @@ bool GenXBTIAssignment::runOnModule(Module &M) {
 
 // Surfaces starting from 240 are reserved.
 static constexpr int MaxAvailableSurfaceIndex = 239;
+static constexpr int StatelessBti = 255;
 static constexpr int MaxAvailableSamplerIndex = 14;
 
 static bool isDescImageType(StringRef TypeDesc) {
@@ -170,7 +171,7 @@ int BTIAssignment::assignUAV(int SurfaceID, ZipTy &&Zippy) {
       continue;
     }
     if (Kind == vc::KernelMetadata::AK_NORMAL && isDescSvmPtr(Desc)) {
-      Idx = SurfaceID++;
+      Idx = StatelessBti;
       continue;
     }
     // These 'ands' are definitely super buggy. Kinds should be
@@ -180,11 +181,15 @@ int BTIAssignment::assignUAV(int SurfaceID, ZipTy &&Zippy) {
     // on L0 runtime.
     // FIXME(aus): investigate the reason and rewrite with KAI.
     if (Kind & vc::KernelMetadata::IMP_OCL_PRINTF_BUFFER) {
-      Idx = SurfaceID++;
+      Idx = StatelessBti;
       continue;
     }
     if (Kind & vc::KernelMetadata::IMP_OCL_PRIVATE_BASE) {
-      Idx = SurfaceID++;
+      Idx = StatelessBti;
+      continue;
+    }
+    if (Kind & vc::KernelMetadata::IMP_OCL_ASSERT_BUFFER) {
+      Idx = StatelessBti;
       continue;
     }
   }
