@@ -292,7 +292,7 @@ DecodeIITType(unsigned &NextElt, ArrayRef<unsigned char> Infos,
     return;
   }
   }
-  IGC_ASSERT_MESSAGE(0, "unhandled");
+  IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
 }
 
 static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
@@ -364,7 +364,7 @@ static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
             dyn_cast<IGCLLVM::FixedVectorType>(Ty)) {
       return IGCLLVM::FixedVectorType::get(EltTy, VTy->getNumElements());
     }
-    IGC_ASSERT_MESSAGE(0, "unhandled");
+    IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
   }
   case IITDescriptor::PtrToArgument: {
     Type *Ty = Tys[D.getArgumentNumber()];
@@ -373,9 +373,8 @@ static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
   case IITDescriptor::PtrToElt: {
     Type *Ty = Tys[D.getArgumentNumber()];
     VectorType *VTy = dyn_cast<VectorType>(Ty);
-    if (!VTy)
-      IGC_ASSERT_MESSAGE(0, "Expected an argument of Vector Type");
-    Type *EltTy = cast<VectorType>(VTy)->getElementType();
+    IGC_ASSERT_EXIT_MESSAGE(VTy, "Expected an argument of Vector Type");
+    Type *EltTy = VTy->getElementType();
     return PointerType::getUnqual(EltTy);
   }
   case IITDescriptor::VecOfAnyPtrsToElt:
@@ -384,7 +383,7 @@ static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
   default:
     break;
   }
-  IGC_ASSERT_MESSAGE(0, "unhandled");
+  IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
   return nullptr;
 }
 
@@ -394,7 +393,7 @@ static Type *DecodeFixedType(ArrayRef<Intrinsic::IITDescriptor> &Infos,
 static void
 getIntrinsicInfoTableEntries(InternalIntrinsic::ID id,
                              SmallVectorImpl<Intrinsic::IITDescriptor> &T) {
-  IGC_ASSERT(isInternalIntrinsic(id));
+  IGC_ASSERT_EXIT(isInternalIntrinsic(id));
 
   // transform id
   // from [not_internal_intrinsic; num_internal_intrinsic] to [0, ...]
@@ -444,6 +443,9 @@ getIntrinsicInfoTableEntries(InternalIntrinsic::ID id,
 /// parse ffXX as f(fXX) or f(fX)X.  (X is a placeholder for any other type.)
 static std::string getMangledTypeStr(Type *Ty) {
   std::string Result;
+  if (!Ty)
+    return Result;
+
   if (PointerType *PTyp = dyn_cast<PointerType>(Ty)) {
     Result += "p" + utostr(PTyp->getAddressSpace()) +
               getMangledTypeStr(IGCLLVM::getNonOpaquePtrEltTy(PTyp));
@@ -470,7 +472,7 @@ static std::string getMangledTypeStr(Type *Ty) {
     Result += "v" +
               utostr(cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements()) +
               getMangledTypeStr(cast<VectorType>(Ty)->getElementType());
-  else if (Ty)
+  else
     Result += EVT::getEVT(Ty).getEVTString();
   return Result;
 }
@@ -485,7 +487,7 @@ static const char *const InternalIntrinsicNameTable[] = {
 /// Intrinsic::isOverloaded(ID) - Returns true if the intrinsic can be
 /// overloaded.
 static bool isOverloaded(InternalIntrinsic::ID id) {
-  IGC_ASSERT(isInternalIntrinsic(id) && "Invalid intrinsic ID!");
+  IGC_ASSERT_EXIT(isInternalIntrinsic(id) && "Invalid intrinsic ID!");
   id = static_cast<InternalIntrinsic::ID>(
       id - InternalIntrinsic::not_internal_intrinsic);
 #define GET_INTRINSIC_OVERLOAD_TABLE
@@ -540,7 +542,7 @@ static InternalIntrinsic::ID lookupInternalIntrinsicID(StringRef Name) {
   if (Idx == -1) {
     return InternalIntrinsic::not_internal_intrinsic;
   }
-  IGC_ASSERT(Idx >= 0);
+  IGC_ASSERT_EXIT(Idx >= 0);
 
   // Intrinsic IDs correspond to the location in IntrinsicNameTable, but we have
   // an index into a sub-table.
@@ -563,10 +565,11 @@ static InternalIntrinsic::ID lookupInternalIntrinsicID(StringRef Name) {
 /// getInternalName(ID) - Return the LLVM name for a Internal intrinsic
 std::string InternalIntrinsic::getInternalName(InternalIntrinsic::ID id,
                                                ArrayRef<Type *> Tys) {
-  IGC_ASSERT(InternalIntrinsic::isInternalIntrinsic(id) &&
-             "Invalid intrinsic ID!");
-  IGC_ASSERT(Tys.empty() || (isOverloaded(id) &&
-                             "Non-overloadable intrinsic was overloaded!"));
+  IGC_ASSERT_EXIT(InternalIntrinsic::isInternalIntrinsic(id) &&
+                  "Invalid intrinsic ID!");
+  IGC_ASSERT_EXIT(
+      Tys.empty() ||
+      (isOverloaded(id) && "Non-overloadable intrinsic was overloaded!"));
   id = static_cast<InternalIntrinsic::ID>(
       id - InternalIntrinsic::not_internal_intrinsic);
   std::string Result(InternalIntrinsicNameTable[id]);
@@ -609,11 +612,11 @@ static FunctionType *getInternalType(LLVMContext &Context,
 /// F is required to be a Internal intrinsic function
 static void resetInternalAttributes(Function *F) {
 
-  IGC_ASSERT(F);
+  IGC_ASSERT_EXIT(F);
 
   InternalIntrinsic::ID GXID = InternalIntrinsic::getInternalIntrinsicID(F);
 
-  IGC_ASSERT(GXID != InternalIntrinsic::not_internal_intrinsic);
+  IGC_ASSERT_EXIT(GXID != InternalIntrinsic::not_internal_intrinsic);
 
   // Since Function::isIntrinsic() will return true due to llvm. prefix,
   // Module::getOrInsertFunction fails to add the attributes. explicitly adding
@@ -635,7 +638,7 @@ static void resetInternalAttributes(Function *F) {
 
 InternalIntrinsic::ID
 InternalIntrinsic::getInternalIntrinsicID(const Function *F) {
-  IGC_ASSERT(F);
+  IGC_ASSERT_EXIT(F);
   llvm::StringRef Name = F->getName();
   if (!Name.startswith(getInternalIntrinsicPrefix())) {
     return InternalIntrinsic::not_internal_intrinsic;
@@ -660,17 +663,18 @@ InternalIntrinsic::getInternalIntrinsicID(const Function *F) {
 
   // Fallback to string lookup.
   auto ID = lookupInternalIntrinsicID(Name);
-  IGC_ASSERT(ID != InternalIntrinsic::not_internal_intrinsic &&
-             "Intrinsic not found!");
+  IGC_ASSERT_EXIT(ID != InternalIntrinsic::not_internal_intrinsic &&
+                  "Intrinsic not found!");
   return ID;
 }
 
 Function *InternalIntrinsic::getInternalDeclaration(Module *M,
                                                     InternalIntrinsic::ID id,
                                                     ArrayRef<Type *> Tys) {
-  IGC_ASSERT(isInternalNonTrivialIntrinsic(id));
-  IGC_ASSERT(Tys.empty() || (isOverloaded(id) &&
-                             "Non-overloadable intrinsic was overloaded!"));
+  IGC_ASSERT_EXIT(isInternalNonTrivialIntrinsic(id));
+  IGC_ASSERT_EXIT(
+      Tys.empty() ||
+      (isOverloaded(id) && "Non-overloadable intrinsic was overloaded!"));
 
   auto InternalName = getInternalName(id, Tys);
   FunctionType *FTy = getInternalType(M->getContext(), id, Tys);
@@ -687,22 +691,29 @@ bool InternalIntrinsic::isInternalMemoryIntrinsic(InternalIntrinsic::ID id) {
   default:
     return false;
   case InternalIntrinsic::lsc_atomic_bti:
+  case InternalIntrinsic::lsc_atomic_bss:
   case InternalIntrinsic::lsc_atomic_slm:
   case InternalIntrinsic::lsc_atomic_ugm:
   case InternalIntrinsic::lsc_load_bti:
+  case InternalIntrinsic::lsc_load_bss:
   case InternalIntrinsic::lsc_load_slm:
   case InternalIntrinsic::lsc_load_ugm:
   case InternalIntrinsic::lsc_prefetch_bti:
+  case InternalIntrinsic::lsc_prefetch_bss:
   case InternalIntrinsic::lsc_prefetch_ugm:
   case InternalIntrinsic::lsc_store_bti:
+  case InternalIntrinsic::lsc_store_bss:
   case InternalIntrinsic::lsc_store_slm:
   case InternalIntrinsic::lsc_store_ugm:
   case InternalIntrinsic::lsc_load_quad_bti:
+  case InternalIntrinsic::lsc_load_quad_bss:
   case InternalIntrinsic::lsc_load_quad_slm:
   case InternalIntrinsic::lsc_load_quad_ugm:
   case InternalIntrinsic::lsc_prefetch_quad_bti:
+  case InternalIntrinsic::lsc_prefetch_quad_bss:
   case InternalIntrinsic::lsc_prefetch_quad_ugm:
   case InternalIntrinsic::lsc_store_quad_bti:
+  case InternalIntrinsic::lsc_store_quad_bss:
   case InternalIntrinsic::lsc_store_quad_slm:
   case InternalIntrinsic::lsc_store_quad_ugm:
     return true;
@@ -722,11 +733,14 @@ bool InternalIntrinsic::isMemoryBlockIntrinsic(const llvm::Instruction *I) {
   default:
     break;
   case InternalIntrinsic::lsc_load_quad_bti:
+  case InternalIntrinsic::lsc_load_quad_bss:
   case InternalIntrinsic::lsc_load_quad_slm:
   case InternalIntrinsic::lsc_load_quad_ugm:
   case InternalIntrinsic::lsc_prefetch_quad_bti:
+  case InternalIntrinsic::lsc_prefetch_quad_bss:
   case InternalIntrinsic::lsc_prefetch_quad_ugm:
   case InternalIntrinsic::lsc_store_quad_bti:
+  case InternalIntrinsic::lsc_store_quad_bss:
   case InternalIntrinsic::lsc_store_quad_slm:
   case InternalIntrinsic::lsc_store_quad_ugm:
     return false;
@@ -738,17 +752,20 @@ bool InternalIntrinsic::isMemoryBlockIntrinsic(const llvm::Instruction *I) {
 unsigned
 InternalIntrinsic::getMemoryVectorSizePerLane(const llvm::Instruction *I) {
   auto IID = getInternalIntrinsicID(I);
-  IGC_ASSERT(isInternalMemoryIntrinsic(IID));
+  IGC_ASSERT_EXIT(isInternalMemoryIntrinsic(IID));
 
   switch (IID) {
   default:
     break;
   case InternalIntrinsic::lsc_load_bti:
+  case InternalIntrinsic::lsc_load_bss:
   case InternalIntrinsic::lsc_load_slm:
   case InternalIntrinsic::lsc_load_ugm:
   case InternalIntrinsic::lsc_prefetch_bti:
+  case InternalIntrinsic::lsc_prefetch_bss:
   case InternalIntrinsic::lsc_prefetch_ugm:
   case InternalIntrinsic::lsc_store_bti:
+  case InternalIntrinsic::lsc_store_bss:
   case InternalIntrinsic::lsc_store_slm:
   case InternalIntrinsic::lsc_store_ugm: {
     auto *VectorSize = cast<ConstantInt>(I->getOperand(3));
@@ -778,11 +795,14 @@ InternalIntrinsic::getMemoryVectorSizePerLane(const llvm::Instruction *I) {
     IGC_ASSERT_UNREACHABLE();
   }
   case InternalIntrinsic::lsc_load_quad_bti:
+  case InternalIntrinsic::lsc_load_quad_bss:
   case InternalIntrinsic::lsc_load_quad_slm:
   case InternalIntrinsic::lsc_load_quad_ugm:
   case InternalIntrinsic::lsc_prefetch_quad_bti:
+  case InternalIntrinsic::lsc_prefetch_quad_bss:
   case InternalIntrinsic::lsc_prefetch_quad_ugm:
   case InternalIntrinsic::lsc_store_quad_bti:
+  case InternalIntrinsic::lsc_store_quad_bss:
   case InternalIntrinsic::lsc_store_quad_slm:
   case InternalIntrinsic::lsc_store_quad_ugm: {
     auto *ChannelMask = cast<ConstantInt>(I->getOperand(3));
@@ -797,7 +817,7 @@ InternalIntrinsic::getMemoryVectorSizePerLane(const llvm::Instruction *I) {
 }
 
 unsigned InternalIntrinsic::getMemorySimdWidth(const Instruction *I) {
-  IGC_ASSERT(isInternalMemoryIntrinsic(I));
+  IGC_ASSERT_EXIT(isInternalMemoryIntrinsic(I));
 
   auto *Pred = I->getOperand(0);
   auto *PredTy = Pred->getType();
@@ -810,7 +830,7 @@ unsigned InternalIntrinsic::getMemorySimdWidth(const Instruction *I) {
 
 unsigned
 InternalIntrinsic::getMemoryRegisterElementSize(const llvm::Instruction *I) {
-  IGC_ASSERT(isInternalMemoryIntrinsic(I));
+  IGC_ASSERT_EXIT(isInternalMemoryIntrinsic(I));
   unsigned ElementSizeIndex = 2;
 
   auto IID = getInternalIntrinsicID(I);
