@@ -19,6 +19,13 @@ namespace IGCLLVM
 #else
   using AliasResultEnum = llvm::AliasResult::Kind;
 #endif
+inline llvm::AAQueryInfo makeAAQueryInfo() {
+#if LLVM_VERSION_MAJOR >= 14
+  return llvm::AAQueryInfo(new llvm::SimpleCaptureInfo());
+#else
+  return llvm::AAQueryInfo();
+#endif
+}
 }
 
 #endif
