@@ -17,7 +17,6 @@ SPDX-License-Identifier: MIT
 #include "llvm/MC/StringTableBuilder.h"
 #include "llvm/Support/EndianStream.h"
 #include "llvm/Support/MathExtras.h"
-#include "llvm/Support/raw_ostream.h"
 
 #ifndef ZEBinStandAloneBuild
 #include "common/LLVMWarningsPop.hpp"
@@ -630,6 +629,23 @@ std::pair<uint64_t, uint64_t> ELFWriter::writeCompatibilityNote() {
     writeOneStrNote("IntelGT",
                     PreDefinedAttrGetter::getVersionNumber(),
                     NT_INTELGT_ZEBIN_VERSION);
+
+    if (m_ObjBuilder.m_zeInfoSection &&
+        std::any_of(m_ObjBuilder.m_zeInfoSection->getZeInfo().kernels.begin(),
+                    m_ObjBuilder.m_zeInfoSection->getZeInfo().kernels.end(),
+                    [](const zeInfoKernel &kernel) {
+                      return kernel.execution_env.has_stack_calls;
+                    })) {
+        // writeNT_INTELGT_VISA_ABI_VERSION
+        writeOneNote("IntelGT", m_ObjBuilder.getVISAABIVersion(),
+                     NT_INTELGT_VISA_ABI_VERSION);
+    }
+
+    // write NT_INTELGT_PRODUCT_CONFIG
+    writeOneNote("IntelGT",
+                 m_ObjBuilder.m_gmdID.Value,
+                 NT_INTELGT_PRODUCT_CONFIG);
+
     return std::make_pair(start_off, m_W.OS.tell() - start_off);
 }
 
@@ -1272,3 +1288,10 @@ void ZEInfoBuilder::addGlobalHostAccessSymbol(const std::string& device_name, co
 {
     mContainer.global_host_access_table.push_back(zeInfoHostAccess{ device_name, host_name });
 }
+
+void ZEInfoBuilder::printZEInfoInYaml(raw_ostream &os)
+{
+    // serialize mContainer
+    llvm::yaml::Output yout(os);
+    yout << mContainer;
+}
