@@ -25,6 +25,7 @@ SPDX-License-Identifier: MIT
 #endif
 
 #include "llvm/BinaryFormat/ELF.h"
+#include "llvm/Support/raw_ostream.h"
 
 #ifndef ZEBinStandAloneBuild
 #include "common/LLVMWarningsPop.hpp"
@@ -65,6 +66,12 @@ public:
     void setTargetMetadata(TargetMetadata metadata) { m_metadata = metadata; }
     TargetMetadata getTargetMetadata() const { return m_metadata; }
 
+    void setVISAABIVersion(unsigned int version) { m_VISAABIVersion = version; }
+    unsigned int getVISAABIVersion() const { return m_VISAABIVersion; }
+
+    void setGmdID(GFX_GMD_ID gmdID) { m_gmdID = gmdID; }
+    GFX_GMD_ID setGmdID() const { return m_gmdID; }
+
     // add a text section contains gen binary
     // - name: section name. This is usually the kernel or function name of
     //         this text section. Do not includes leading .text in given
@@ -89,9 +96,11 @@ public:
     // - size: input buffer size in byte
     // - padding: required padding at the end
     // - align: alignment requirement in byte
+    // - rodata: mark section as read-only
+    // - alloc: mark section as allocatable
     // - return a unique id for referencing in addSymbol
     SectionID addSectionData(
-        std::string name, const uint8_t* data, uint64_t size, uint32_t padding = 0, uint32_t align = 0, bool rodata = false);
+        std::string name, const uint8_t* data, uint64_t size, uint32_t padding = 0, uint32_t align = 0, bool rodata = false, bool alloc = true);
 
     // add a bss section which occupies no space in the ELF, but with size and other section information
     // The bss sections could be used for zero-initialized variables.
@@ -373,6 +382,8 @@ private:
     PRODUCT_FAMILY m_productFamily = IGFX_UNKNOWN;
     GFXCORE_FAMILY m_gfxCoreFamily = IGFX_UNKNOWN_CORE;
     TargetMetadata m_metadata;
+    unsigned int m_VISAABIVersion = 0;
+    GFX_GMD_ID m_gmdID;
 
     StandardSectionListTy m_textSections;
     StandardSectionListTy m_dataAndbssSections; // data and bss sections
@@ -436,7 +447,8 @@ public:
         int32_t offset,
         int32_t size,
         int32_t arg_index,
-        int32_t source_offset);
+        int32_t source_offset,
+        bool is_ptr);
 
     // addPayloadArgumentImage - add explicit kernel argument for image
     // into given arg_list
@@ -532,6 +544,9 @@ public:
     static void addExpPropertiesHasNonKernelArgLdSt(zeInfoKernel& zekernel,
         bool hasNonKernelArgLoad, bool hasNonKernelArgStore, bool hasNonKernelArgAtomic);
 
+    // dump mContainer in yaml format into given raw_ostream
+    void printZEInfoInYaml(llvm::raw_ostream &OS);
+
 private:
     zeInfoContainer mContainer;
 };
