@@ -43,9 +43,6 @@ namespace IGC
 {
     const unsigned int INVALID_CONSTANT_BUFFER_INVALID_ADDR = 0xFFFFFFFF;
 
-    static const char* NAMED_METADATA_COARSE_PHASE = "coarse_phase";
-    static const char* NAMED_METADATA_PIXEL_PHASE  = "pixel_phase";
-
     enum FunctionTypeMD
     {
         KernelFunction,
@@ -397,7 +394,6 @@ namespace IGC
         bool HasPositivePointerOffset                   = false;
         bool HasBufferOffsetArg                         = false;
         bool BufferOffsetArgOptional                    = true;
-        bool HasSubDWAlignedPtrArg                      = false;
         bool replaceGlobalOffsetsByZero                 = false;
         unsigned forcePixelShaderSIMDMode               = 0;
         bool pixelShaderDoNotAbortOnSpill               = false;
@@ -443,7 +439,7 @@ namespace IGC
         bool DisableFDivToFMulInvOpt                    = false;
         bool initializePhiSampleSourceWA                = false;
         bool WaDisableSubspanUseNoMaskForCB             = false;
-        
+
         unsigned FastestS1Options                       = 0;  // FCEXP_NO_EXPRIMENT. Can't access the enum here for some reason.
     };
 
@@ -469,7 +465,10 @@ namespace IGC
         bool allowLowerSimd = false;
         bool disableSimd32Slicing = false;
         bool disableSplitOnSpill = false;
+        bool enableNewSpillCostFunction = false;
         bool forcedVISAPreRAScheduler = false;
+        bool forceUniformBuffer = false;
+        bool forceUniformSurfaceSampler = false;
         // disables dispatch along y and tiled order optimizations
         bool disableLocalIdOrderOptimizations = false;
         // force disables dispatch along y optimization
@@ -568,6 +567,10 @@ namespace IGC
         // of runtime values with indices in [firstIndex, firstIndex + numOffsets).
         unsigned int firstIndex = 0;
         unsigned int numOffsets = 0;
+
+        // Some conditions, like robust buffer access, requires dynamic buffer support to be disabled regardless
+        // if the client support them and simple push
+        bool forceDisabled = false;
     };
 
     // simplePushInfoArr needs to be initialized to a vector of size g_c_maxNumberOfBufferPushed, which we are doing in module MD initialization done in code gen context
@@ -688,7 +691,7 @@ namespace IGC
         std::vector<uint32_t> RasterizerOrderedByteAddressBuffer;
         std::set<uint32_t> RasterizerOrderedViews;
         unsigned int MinNOSPushConstantSize = 0;
-        llvm::MapVector<llvm::GlobalVariable*, int> inlineProgramScopeOffsets;
+        llvm::MapVector<llvm::GlobalVariable*, uint64_t> inlineProgramScopeOffsets;
         ShaderData shaderData;
         URBLayoutInfo URBInfo;
         bool UseBindlessImage = false;
@@ -713,6 +716,12 @@ namespace IGC
         // some memory accesses.
         bool genericAccessesResolved = false;
 
+        //when true, compiler disables separate spill pvt scratch space
+        bool disableSeparateSpillPvtScratchSpace = false;
+
+        //when true, compiler disables scratch space slot0/slot1 sizes workaround
+        bool disableSeparateScratchWA = false;
+
         unsigned int privateMemoryPerWI = 0;
 
         llvm::MapVector<llvm::Function*, unsigned int> PrivateMemoryPerFG;
