@@ -9,13 +9,16 @@ SPDX-License-Identifier: MIT
 #pragma once
 
 // turn on/off GET_TIME_STATS and GET_SHADER_STATS for the reports.
-
 #define GET_TIME_STATS 1
 
-#if defined( _INTERNAL ) || defined( _DEBUG )
+#if defined( _DEBUG )
     #define GET_SHADER_STATS 1
     #define PRINT_PER_SHADER_STATS 1
     #define GET_MEM_STATS 1
+#elif defined( _INTERNAL )
+    #define GET_SHADER_STATS 1
+    #define PRINT_PER_SHADER_STATS 1
+    #define GET_MEM_STATS 0
 #endif
 
 #include "common/Types.hpp"
@@ -242,7 +245,7 @@ public:
 
 private:
     /// \deprecated Print aggregate times for multiple shaders in csv format
-    void printSumTimeCSV( const char* fileName ) const;
+    void printSumTimeCSV( std::string const& fileName ) const;
     /// Print aggregate times for multiple shaders in human readable format
     void printSumTimeTable( llvm::raw_ostream & OS ) const;
 
@@ -251,7 +254,7 @@ private:
     /// \deprecated Print the currently accumulated times in csv format
     void printTimeCSV( std::string const& corpusName, UINT64 psoDDIHash ) const;
     void printPerPassTimeCSV( std::string const& corpusName ) const;
-    void printPerPassSumTimeCSV(const char* fileName) const;
+    void printPerPassSumTimeCSV(std::string const& fileName) const;
 
     // Return a copy of *this, with Unaccounted timer values filled in
     TimeStats postProcess() const;
@@ -366,7 +369,8 @@ private:
 #define COMPILER_TIME_INIT( pointer, statName )     \
     do                                              \
     {                                               \
-        if (pointer)                                \
+        /* Check for pointer to avoid mem leak */   \
+        if ((pointer) && !((pointer)->statName))    \
         {                                           \
             (pointer)->statName = new TimeStats();  \
         }                                           \
@@ -375,7 +379,8 @@ private:
 #define COMPILER_TIME_INIT2( pointer, statName, size)     \
     do                                              \
     {                                               \
-        if (pointer)                                \
+        /* Check for pointer to avoid mem leak */   \
+        if ((pointer) && !((pointer)->statName))    \
         {                                           \
             (pointer)->statName = new TimeStats[size];  \
         }                                           \
