@@ -37,6 +37,7 @@ typedef enum {
     IGFX_DG2 = 1270, // aka - ACM/Alchemist
     IGFX_PVC = 1271,
     IGFX_METEORLAKE = 1272,
+    IGFX_ARROWLAKE = 1273,
     IGFX_MAX_PRODUCT,
 
 
@@ -88,6 +89,7 @@ typedef enum {
     IGFX_XE_HP_CORE   = 0x0c05, // XeHP Family
     IGFX_XE_HPG_CORE = 0x0c07,  // XE_HPG Family
     IGFX_XE_HPC_CORE = 0x0c08,  // XE_HPC Family
+    IGFX_XE2_LPG_CORE = 0x0c09, // XE2_LPG Family
     IGFX_MAX_CORE,              // Max Family, for lookup table
 
     IGFX_GENNEXT_CORE          = 0x7ffffffe,  //GenNext
@@ -185,8 +187,14 @@ typedef struct PLATFORM_STR {
 
     // Supported from MTL onwards that will hold IP's major/minor version and their individual RevID
     GFX_GMD_ID          sDisplayBlockID;
+    GFX_GMD_ID          sDisplayPicaBlockID;
     GFX_GMD_ID          sRenderBlockID;
     GFX_GMD_ID          sMediaBlockID;
+
+    // Since we override usRevId field for certain platforms ex: RPLS B0, RPLP J0,
+    // This field is required to hold original revid derived from pci config space which is further used to share outside KMD
+    unsigned short      usOriginalRevIdFromPciConfig;
+    unsigned short      padding;
 } PLATFORM;
 
 // add enums at the end
@@ -423,11 +431,6 @@ typedef enum __NATIVEGTTYPE
 #define ICHV_PLUS_DESK_DEVICE_F0_ID      0x22B3   // Reserved
 
 //BDW device ids
-#define IBDW_GT0_DESK_DEVICE_F0_ID              0x0BD0
-#define IBDW_GT1_DESK_DEVICE_F0_ID              0x0BD1
-#define IBDW_GT2_DESK_DEVICE_F0_ID              0x0BD2
-#define IBDW_GT3_DESK_DEVICE_F0_ID              0x0BD3
-#define IBDW_GT4_DESK_DEVICE_F0_ID              0x0BD4
 
 #define IBDW_GT1_HALO_MOBL_DEVICE_F0_ID         0x1602
 #define IBDW_GT1_ULT_MOBL_DEVICE_F0_ID          0x1606
@@ -709,6 +712,8 @@ typedef enum __NATIVEGTTYPE
 #define DEV_ID_56A9                             0x56A9
 #define DEV_ID_56B0                             0x56B0
 #define DEV_ID_56B1                             0x56B1
+#define DEV_ID_56B2                             0x56B2
+#define DEV_ID_56B3                             0x56B3
 #define DEV_ID_56C0                             0x56C0
 #define DEV_ID_56C1                             0x56C1
 #define DEV_ID_56CF                             0x56CF
@@ -717,15 +722,33 @@ typedef enum __NATIVEGTTYPE
                                  ( d == DEV_ID_56A6 )             ||   \
                                  ( d == DEV_ID_5693 )             ||   \
                                  ( d == DEV_ID_5694 )             ||   \
+                                 ( d == DEV_ID_5695 )             ||   \
                                  ( d == DEV_ID_56B0 )             ||   \
                                  ( d == DEV_ID_56B1 )             ||   \
                                  ( d == DEV_ID_56C1 )             ||   \
                                  ( d == DEV_ID_4F87 )             ||   \
                                  ( d == DEV_ID_4F88 ))
 
+#define GFX_IS_DG2_G10_CONFIG(d) ( ( d == DEV_ID_56A0 )                              ||   \
+                                      ( d == DEV_ID_56A1 )                              ||   \
+                                      ( d == DEV_ID_56A2 )                              ||   \
+                                      ( d == DEV_ID_5690 )                              ||   \
+                                      ( d == DEV_ID_5691 )                              ||   \
+                                      ( d == DEV_ID_5692 )                              ||   \
+                                      ( d == DEV_ID_56C0 )                              ||   \
+                                      ( d == DEV_ID_4F80 )                              ||   \
+                                      ( d == DEV_ID_4F81 )                              ||   \
+                                      ( d == DEV_ID_4F82 )                              ||   \
+                                      ( d == DEV_ID_4F83 )                              ||   \
+                                      ( d == DEV_ID_4F84 ))
+
 #define GFX_IS_DG2_G12_CONFIG(d)   ( ( d == DEV_ID_4F85 )                              ||   \
                                       ( d == DEV_ID_4F86 )                              ||   \
                                       ( d == DEV_ID_56A3 )                              ||   \
-                                      ( d == DEV_ID_56A4 ))
+                                      ( d == DEV_ID_56A4 )                              ||   \
+                                      ( d == DEV_ID_5696 )                              ||   \
+                                      ( d == DEV_ID_5697 )                              ||   \
+                                      ( d == DEV_ID_56B2 )                              ||   \
+                                      ( d == DEV_ID_56B3 ))
 
 #endif
