@@ -1037,6 +1037,7 @@ template <class T> void emitDataSubroutines(VISAKernelImpl *visaKernel, T &t) {
                          ->getRootDeclare();
           }
         }
+        vISA_ASSERT(retval && subLabel, "All of the basic blocks are empty.");
         emitDataName(subLabel->getLabelName(), t);
         emitDataUInt32(start, t);
         emitDataUInt32(end, t);
@@ -1578,7 +1579,7 @@ void resetGenOffsets(G4_Kernel &kernel) {
 void updateDebugInfo(G4_Kernel &kernel, G4_INST *inst,
                      const LivenessAnalysis &liveAnalysis,
                      const LiveRangeVec& lrs,
-                     llvm_SBitVector &live, DebugInfoState *state,
+                     SparseBitVector &live, DebugInfoState *state,
                      bool closeAllOpenIntervals) {
   if (closeAllOpenIntervals && !state->getPrevInst())
     return;
@@ -2237,6 +2238,7 @@ void emitSubRoutineInfo(VISAKernelImpl *visaKernel) {
 
         calleeBB = calleeBB->Preds.front();
       }
+      vISA_ASSERT(retval && subLabel, "All of the basic blocks are empty.");
       std::cerr << "Func info id " << subLabel->getLabelName() << "\n";
       std::cerr << "First inst " << start << ", last inst " << end << "\n";
       std::cerr << "Return value in dcl " << retval->getName() << "\n";
