@@ -159,7 +159,6 @@ bool G4_BB::isAllLaneActive() const {
   return false;
 }
 
-
 void G4_BB::emit(std::ostream &output) {
   for (INST_LIST_ITER it = instList.begin(); it != instList.end(); ++it) {
     emitInstruction(output, it);
@@ -196,7 +195,6 @@ void G4_BB::emitBasicInstruction(std::ostream &output, INST_LIST_ITER &it) {
       emitBankConflict(output, inst);
     }
   }
-
 }
 void G4_BB::emitBasicInstructionComment(std::ostream &output,
                                         INST_LIST_ITER &it, int *suppressRegs,
@@ -249,10 +247,9 @@ void G4_BB::emitBasicInstructionComment(std::ostream &output,
       }
       auto jitInfo = getParent().getKernel()->fg.builder->getJitInfo();
       jitInfo->statsVerbose.BCNum += BCNum;
-      jitInfo->statsVerbose.numRMWs += countReadModifyWrite(output, inst);
+      jitInfo->statsVerbose.numByteRMWs += countReadModifyWrite(output, inst);
     }
   }
-
 }
 
 void G4_BB::emitInstructionSourceLineMapping(std::ostream &output,
@@ -1213,7 +1210,7 @@ INST_LIST_ITER G4_BB::getFirstInsertPos() {
 void G4_BB::addEOTSend(G4_INST *lastInst) {
 
   // mov (8) r1.0<1>:ud r0.0<8;8,1>:ud {NoMask}
-  // send (8) null r1 0x27 desc
+  // send (1) null r1 0x27 desc
   IR_Builder *builder = parent->builder;
   G4_Declare *dcl =
       builder->createSendPayloadDcl(builder->numEltPerGRF<Type_UD>(), Type_UD);
@@ -1241,9 +1238,10 @@ void G4_BB::addEOTSend(G4_INST *lastInst) {
 
   auto msgDesc =
       builder->createGeneralMsgDesc(desc, exdesc, SendAccess::WRITE_ONLY);
-  G4_INST *sendInst = builder->createSendInst(
-      NULL, G4_send, g4::SIMD8, sendDst, sendSrc,
+  G4_InstSend *sendInst = builder->createSendInst(
+      NULL, G4_send, g4::SIMD1, sendDst, sendSrc,
       builder->createImm(desc, Type_UD), InstOpt_WriteEnable, msgDesc, false);
+  sendInst->setEOT();
   sendInst->inheritDIFrom(movInst);
   instList.push_back(sendInst);
 
@@ -1361,6 +1359,14 @@ void G4_BB::removeIntrinsics(Intrinsic intrinId) {
   });
 }
 
+void G4_BB::removeIntrinsics(std::vector<Intrinsic>& intrinIdVec) {
+  instList.remove_if([=](G4_INST *inst) {
+    return inst->isIntrinsic() &&
+           std::find(intrinIdVec.begin(), intrinIdVec.end(),
+               inst->asIntrinsicInst()->getIntrinsicId()) != intrinIdVec.end();
+  });
+}
+
 // Add two sampler cache flushes before the EOT send.
 // sampler cache flush 1 must have null return
 // sampler cache flush 2 must have valid return
@@ -1406,3 +1412,12 @@ void G4_BB::addSamplerFlushBeforeEOT() {
 bool G4_BB::dominates(G4_BB *other) {
   return getParent().getImmDominator().dominates(this, other);
 }
+
+void G4_BB::markEmpty(IR_Builder *IRB, const std::string str) {
+  vISA_ASSERT(empty(), "BB to be marked empty is not empty!");
+  G4_Label *label =
+      IRB->createLocalBlockLabel(str.empty() ? "LABEL__EMPTYBB" : str);
+  G4_INST *inst = IRB->createLabelInst(label, false);
+  push_back(inst);
+  specialEmptyBB = true;
+}
