@@ -220,19 +220,27 @@ public:
   }
   SWSBTokenType getTokenType() const { return (SWSBTokenType)swsb.tokenType; }
 
-  void setSetToken(unsigned short token) {
+  void setSBIDSetToken(unsigned short token) {
     swsb.SBToken = token;
     swsb.tokenType = SB_SET;
   }
-  unsigned short getSetToken() const {
+  // TODO: return the swsb struct and strip out the necessary info at the
+  // calling site
+  unsigned short getSBIDSetToken() const {
     if (swsb.tokenType == SB_SET)
       return swsb.SBToken;
     else
       return -1;
   }
 
+  unsigned short getSBIDToken(SWSBTokenType t) const {
+    if (swsb.tokenType == t)
+      return swsb.SBToken;
+    return -1;
+  }
+
   void setNoACCSBSet() { swsb.tokenType = NoACCSBSet; }
-  bool hasNoACCSBSet() { return swsb.tokenType == NoACCSBSet; }
+  bool hasNoACCSBSet() const { return swsb.tokenType == NoACCSBSet; }
 
   void setOperandTypeIndicated(bool indicated) {
     operandTypeIndicated = indicated;
@@ -402,7 +410,7 @@ public:
 
   bool isPartialWrite() const;
   bool isPartialWriteForSpill(bool inSIMDCF, bool useNonScratchForSpill) const;
-  bool isArithAddr() const;
+  bool isAddrAdd() const;
   bool isMovAddr() const;
   bool isAccSrcInst() const;
   bool isAccDstInst() const;
@@ -610,7 +618,7 @@ public:
   int32_t getLocalId() const { return localId; }
 
   void setEvenlySplitInst(bool val) { evenlySplitInst = val; }
-  bool getEvenlySplitInst() { return evenlySplitInst; }
+  bool getEvenlySplitInst() const { return evenlySplitInst; }
 
   void setVISAId(int offset) { vISAInstId = offset; }
   int getVISAId() const { return vISAInstId; }
@@ -684,7 +692,7 @@ public:
   bool isMathPipeInst() const;
   bool distanceHonourInstruction() const;
   bool tokenHonourInstruction() const;
-  bool hasNoPipe();
+  bool hasNoPipe() const;
   bool isLongPipeType(G4_Type type) const;
   bool isIntegerPipeType(G4_Type type) const;
   bool isJEUPipeInstructionXe() const;
@@ -743,7 +751,7 @@ public:
   bool isRawMov() const;
   bool hasACCSrc() const;
   bool hasACCOpnd() const;
-  G4_Type getOpExecType(int &extypesize);
+  G4_Type getOpExecType(int &extypesize) const;
   bool canHoistTo(const G4_INST *defInst, bool simdBB) const;
   enum MovType {
     Copy = 0,           // MOV is a copy.
@@ -779,7 +787,7 @@ public:
   bool isCommutative() const;
 
   bool hasNULLDst() const;
-  bool goodTwoGRFDst(bool &evenSplitDst);
+  bool goodTwoGRFDst(bool &evenSplitDst) const;
   void setGenOffset(int64_t off) { genOffset = off; }
   int64_t getGenOffset() const { return genOffset; }
 
@@ -1260,7 +1268,7 @@ public:
 
   bool isFence() const { return getMsgDesc()->isFence(); }
 
-  bool isDirectSplittableSend();
+  bool isDirectSplittableSend() const;
 
   void computeRightBound(G4_Operand *opnd) override;
 
@@ -1655,7 +1663,7 @@ public:
   void setOffset(uint32_t o) { offset = o; }
   void setFP(G4_Declare *f) { fp = f; }
 
-  bool isOffsetValid() { return offset != InvalidOffset; }
+  bool isOffsetValid() const { return offset != InvalidOffset; }
 
   void computeRightBound(G4_Operand *opnd) override;
 
