@@ -475,7 +475,7 @@ G4_Kernel::G4_Kernel(const PlatformInfo &pInfo, INST_LIST_NODE_ALLOCATOR &alloc,
   } else {
     gtPinInfo = nullptr;
   }
-  regSharingHeuristics = m_options->getOption(vISA_RegSharingHeuristics);
+  autoGRFSelection = m_options->getOption(vISA_AutoGRFSelection);
   // NoMask WA
   m_EUFusionNoMaskWAInfo = nullptr;
 
@@ -624,7 +624,7 @@ bool G4_Kernel::updateKernelToLargerGRF() {
     return false;
 
   // Scale number of GRFs, Acc, SWSB tokens.
-  setKernelParameters(grfMode.getLargerGRF());
+  setKernelParameters(grfMode.moveToLargerGRF());
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
   return true;
 }
@@ -637,7 +637,7 @@ bool G4_Kernel::updateKernelToSmallerGRF() {
     return false;
 
   // Scale number of GRFs, Acc, SWSB tokens.
-  setKernelParameters(grfMode.getSmallerGRF());
+  setKernelParameters(grfMode.moveToSmallerGRF());
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
   return true;
 }
@@ -646,8 +646,14 @@ bool G4_Kernel::updateKernelToSmallerGRF() {
 // Updates kernel's related structures based on register pressure
 //
 void G4_Kernel::updateKernelByRegPressure(unsigned regPressure) {
-  unsigned newGRF =
-      grfMode.findModeByRegPressure(regPressure, getLargestInputRegister());
+  unsigned largestInputReg = getLargestInputRegister();
+  if (m_kernelAttrs->isKernelAttrSet(Attributes::ATTR_MaxRegThreadDispatch)) {
+    unsigned maxRegPayloadDispatch = m_kernelAttrs->getInt32KernelAttr(
+        Attributes::ATTR_MaxRegThreadDispatch);
+    largestInputReg = std::max(largestInputReg, maxRegPayloadDispatch);
+  }
+
+  unsigned newGRF = grfMode.setModeByRegPressure(regPressure, largestInputReg);
 
   if (newGRF == numRegTotal)
     return;
@@ -670,7 +676,7 @@ bool G4_Kernel::updateKernelFromNumGRFAttr() {
   if (numRegTotal == attrNumGRF)
     return true;
 
-  regSharingHeuristics = false;
+  autoGRFSelection = false;
   // Scale number of GRFs, Acc, SWSB tokens.
   setKernelParameters(attrNumGRF);
   fg.builder->rebuildPhyRegPool(getNumRegTotal());
@@ -855,12 +861,17 @@ uint32_t StackCallABI::numReservedABIGRF() const {
 }
 
 uint32_t StackCallABI::getFPSPGRF() const {
-  // For ABI V1 return r125.
-  // For ABI V2, V3 return r127.
-  if (version == StackCallABIVersion::VER_1)
+  // For ABI V1, return (numRegTotal - 3), i.e. 125.
+  // For ABI V2, return (numRegTotal - 1), i.e. 127, 255.
+  // For ABI V3, return (numRegTotal - 1), i.e. 127, 255.
+
+  if (version == StackCallABIVersion::VER_1) {
     return getStackCallStartReg() + FPSPGRF;
-  else
+  } else if (version == StackCallABIVersion::VER_2) {
     return (kernel->getNumRegTotal() - 1) - FPSPGRF;
+  } else {
+    return (kernel->getNumRegTotal() - 1) - FPSPGRF;
+  }
 }
 
 uint32_t StackCallABI::getSpillHeaderGRF() const {
@@ -979,11 +990,11 @@ void G4_Kernel::setKernelParameters(unsigned newGRF) {
     // Forcing a specific number of threads
     grfMode.setModeByNumThreads(overrideNumThreads);
     overrideGRFNum = 0;
-    regSharingHeuristics = false;
+    autoGRFSelection = false;
   } else if (overrideGRFNum != grfMode.getDefaultGRF()) {
     // Forcing a specific number of GRFs
     grfMode.setModeByNumGRFs(overrideGRFNum);
-    regSharingHeuristics = false;
+    autoGRFSelection = false;
   } else {
     // Use default value
     grfMode.setDefaultGRF();
@@ -1733,10 +1744,12 @@ void G4_Kernel::emitDeviceAsmInstructionsIga(std::ostream &os,
     return;
 
   iga_gen_t igaPlatform = getIGAPlatform(getPlatform());
+
   const iga::Model *igaModel =
       iga::Model::LookupModel(iga::ToPlatform(igaPlatform));
-  KernelView kv(igaPlatform, binary, binarySize, igaModel->getSWSBEncodeMode(),
-                errBuf, ERROR_STRING_MAX_LENGTH);
+  iga::SWSB_ENCODE_MODE swsbEncodeMode = igaModel->getSWSBEncodeMode();
+  KernelView kv(igaPlatform, binary, binarySize, swsbEncodeMode, errBuf,
+                ERROR_STRING_MAX_LENGTH);
 
   if (!kv.decodeSucceeded()) {
     const char *MSG =
@@ -2041,6 +2054,7 @@ unsigned G4_Kernel::getComputeFFIDGP1NextOff() const {
 
 
 // GRF modes supported by HW
+// There must be at least one Config that is VRTEnable for each platform
 GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
   switch (platform) {
   case Xe_XeHPSDV:
@@ -2048,16 +2062,16 @@ GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
   case Xe_MTL:
     configs.resize(2);
     // Configurations with <numGRF, numThreads, SWSBTokens, numAcc>
-    configs[0] = {128, 8, 16, 4};
-    configs[1] = {256, 4, 16, 8};
+    configs[0] = Config(128, 8, 16, 4);
+    configs[1] = Config(256, 4, 16, 8);
     defaultMode = 0;
     break;
   case Xe_PVC:
   case Xe_PVCXT:
     configs.resize(2);
     // Configurations with <numGRF, numThreads, SWSBTokens, numAcc>
-    configs[0] = {128, 8, 16, 4};
-    configs[1] = {256, 4, 32, 8};
+    configs[0] = Config(128, 8, 16, 4);
+    configs[1] = Config(256, 4, 32, 8);
     defaultMode = 0;
     break;
   default:
@@ -2070,21 +2084,27 @@ GRFMode::GRFMode(const TARGET_PLATFORM platform, Options *op) : options(op) {
   currentMode = defaultMode;
 }
 
-unsigned GRFMode::findModeByRegPressure(unsigned maxRP, unsigned largestInputReg) {
-  unsigned size = configs.size();
-  unsigned i = 0, newGRF = 0;
+unsigned GRFMode::setModeByRegPressure(unsigned maxRP, unsigned largestInputReg) {
+  unsigned size = configs.size(), i = 0;
   // find appropiate GRF based on reg pressure
   for (; i < size; i++) {
-    if (maxRP <= configs[i].numGRF && largestInputReg <= configs[i].numGRF) {
-      newGRF = configs[i].numGRF;
+    if (configs[i].VRTEnable && maxRP <= configs[i].numGRF &&
+        // Check that we've at least 8 GRFs over and above
+        // those blocked for kernel input. This helps cases
+        // where an 8 GRF variable shows up in entry BB.
+        (largestInputReg + 8) <= configs[i].numGRF) {
+      currentMode = i;
       break;
     }
   }
+  return configs[currentMode].numGRF;
+}
 
-  // if not found, pressure is too high
-  // set largest grf mode
-  if (i == size)
-    newGRF = getMaxGRF();
+// Check if next larger GRF has the same number of threads per EU
+bool GRFMode::hasLargerGRFSameThreads() const {
+  unsigned largerGrfIdx = currentMode + 1;
+  if (largerGrfIdx == configs.size() || !configs[largerGrfIdx].VRTEnable)
+    return false;
 
-  return newGRF;
+  return configs[currentMode].numThreads == configs[largerGrfIdx].numThreads;
 }
