@@ -16,6 +16,7 @@ SPDX-License-Identifier: MIT
 #include "include/gtpin_IGC_interface.h"
 #include "Assertions.h"
 
+#include <algorithm>
 #include <cstdint>
 #include <iostream>
 #include <map>
@@ -147,23 +148,12 @@ public:
     return iter != configs.end();
   }
 
-  unsigned findModeByRegPressure(unsigned maxRP, unsigned largestInputReg);
+  unsigned setModeByRegPressure(unsigned maxRP, unsigned largestInputReg);
+  bool hasLargerGRFSameThreads() const;
 
   unsigned getNumGRF() const { return configs[currentMode].numGRF; }
-  unsigned getMinGRF() const { return configs[0].numGRF; }
-  unsigned getMaxGRF() const {
-    return configs[configs.size() - 1].numGRF;
-  }
   unsigned getDefaultGRF() const { return configs[defaultMode].numGRF; }
   void setDefaultGRF() { currentMode = defaultMode; }
-  unsigned getLargerGRF() const {
-    return currentMode + 1 < configs.size() ? configs[currentMode + 1].numGRF
-                                            : configs[currentMode].numGRF;
-  }
-  unsigned getSmallerGRF() const {
-    return (signed)currentMode - 1 >= 0 ? configs[currentMode - 1].numGRF
-                                        : configs[currentMode].numGRF;
-  }
 
   unsigned getNumThreads() const { return configs[currentMode].numThreads; }
   unsigned getMinNumThreads() const {
@@ -180,13 +170,75 @@ public:
 
   unsigned getNumAcc() const { return configs[currentMode].numAcc; }
 
+  // ----- helper functions for autoGRFSelection (VRT) ----- //
+  unsigned getMinGRF() const {
+    auto found = std::find_if(configs.begin(), configs.end(),
+                              [](const Config &c) { return c.VRTEnable; });
+    return found->numGRF;
+  }
+
+  unsigned getMaxGRF() const {
+    auto found = std::find_if(configs.rbegin(), configs.rend(),
+                              [](const Config &c) { return c.VRTEnable; });
+    return found->numGRF;
+  }
+
+  // Get the next larger GRF available
+  unsigned getLargerGRF() const {
+    // find the first larger mode that's available for VRT
+    for (auto i = currentMode + 1; i < configs.size(); ++i) {
+      if (configs[i].VRTEnable)
+        return configs[i].numGRF;
+    }
+    return configs[currentMode].numGRF;
+  }
+
+  // Get the next smaller GRF available
+  unsigned getSmallerGRF() const {
+    for (auto i = static_cast<int>(currentMode) - 1; i >= 0 ; --i) {
+      if (configs[i].VRTEnable)
+        return configs[i].numGRF;
+    }
+    return configs[currentMode].numGRF;
+  }
+
+  // Move GRF mode to the larger GRF available and return the number
+  unsigned moveToLargerGRF() {
+    for (auto i = currentMode + 1; i < configs.size(); ++i) {
+      if (configs[i].VRTEnable) {
+        currentMode = i;
+        break;
+      }
+    }
+    return configs[currentMode].numGRF;
+  }
+
+  // Move GRF mode to the smaller GRF available and return the number
+  unsigned moveToSmallerGRF() {
+    for (auto i = currentMode - 1; i >= 0; --i) {
+      if (configs[i].VRTEnable) {
+        currentMode = i;
+        break;
+      }
+    }
+    return configs[currentMode].numGRF;
+  }
+
 private:
   // Parameters associated to a GRF mode
   struct Config {
+    constexpr Config()
+        : numGRF(0), numThreads(0), numSWSB(0), numAcc(0), VRTEnable(false) {}
+    constexpr Config(unsigned NumGRF, unsigned NumThreads, unsigned NumSWSB,
+                     unsigned NumAcc, bool enableForVRT = true)
+        : numGRF(NumGRF), numThreads(NumThreads), numSWSB(NumSWSB),
+          numAcc(NumAcc), VRTEnable(enableForVRT) {}
     unsigned numGRF;
     unsigned numThreads;
     unsigned numSWSB;
     unsigned numAcc;
+    // if the config can be used by autoGRFSelection
+    bool     VRTEnable;
   };
   // Vector configs maintains all the GRF modes available for the platform
   // being compiled.
@@ -448,7 +500,7 @@ private:
   G4_ExecSize simdSize{0u}; // must start as 0
   bool channelSliced = true;
   bool hasAddrTaken;
-  bool regSharingHeuristics = false;
+  bool autoGRFSelection = false;
   bool needDPASWA = false;
   Options *m_options;
   const Attributes *m_kernelAttrs;
@@ -569,9 +621,9 @@ public:
 
   void setBuilder(IR_Builder *pBuilder) { fg.setBuilder(pBuilder); }
 
-  bool useRegSharingHeuristics() const {
+  bool useAutoGRFSelection() const {
     // Register sharing not enabled in presence of stack calls
-    return regSharingHeuristics && !m_hasIndirectCall &&
+    return autoGRFSelection && !m_hasIndirectCall &&
            !fg.getIsStackCallFunc() && !fg.getHasStackCalls();
   }
 
