@@ -51,6 +51,12 @@ void G4Verifier::verify() {
   for (auto BBI = kernel.fg.cbegin(), BBE = kernel.fg.cend(); BBI != BBE;
        ++BBI) {
     auto bb = *BBI;
+    if (bb->isSpecialEmptyBB()) {
+      // Special empty bb should have a single label instruction.
+      vISA_ASSERT(bb->size() == 1 && bb->front()->isLabel(),
+                  "illegal special basic block");
+    }
+
     for (auto I = bb->begin(), E = bb->end(); I != E; ++I) {
       G4_INST *inst = *I;
       verifyInst(inst);
@@ -87,7 +93,8 @@ bool G4Verifier::verifyInst(G4_INST *inst) {
         passIndex == Optimizer::PI_localCopyPropagation ||
         passIndex == Optimizer::PI_localInstCombine ||
         passIndex == Optimizer::PI_reassociateConst ||
-        passIndex == Optimizer::PI_cselPeepHoleOpt) {
+        passIndex == Optimizer::PI_cselPeepHoleOpt ||
+        passIndex == Optimizer::PI_dce) {
       // def-use chain should be valid after these passes
       return verifyDefUseChain(inst);
     }
@@ -277,7 +284,8 @@ void G4Verifier::verifySend(G4_INST *inst) {
     G4_SrcRegRegion *src1 =
         inst->isSplitSend() ? inst->getSrc(1)->asSrcRegRegion() : nullptr;
 
-    if (inst->isEOT() && kernel.fg.builder->hasEOTGRFBinding()) {
+    bool checkEoT = inst->isEOT() && kernel.fg.builder->hasEOTGRFBinding();
+    if (checkEoT) {
       auto checkEOTSrc = [this](G4_SrcRegRegion *src) {
         const unsigned int EOTStart = 112 * kernel.numEltPerGRF<Type_UB>();
         if (src->isNullReg()) {
@@ -296,7 +304,7 @@ void G4Verifier::verifySend(G4_INST *inst) {
       }
     }
 
-    if (inst->isSplitSend()) {
+    if (inst->isSplitSend() && kernel.fg.builder->noSrc0Src1OverlapSend()) {
       // simd1 split send is allowed to have srcs overlap. When it's simd1,
       // overlap for the rest of the payload shouldn't matter
       bool allowSrcOverlap = inst->getExecSize() == g4::SIMD1;
@@ -349,6 +357,11 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
     return;
   }
 
+  // Skip verifying operands of an intrinsic as it usually does not follow
+  // region rules.
+  if (inst->isIntrinsic())
+    return;
+
   // FIXME: If isImm() condition is removed then some assertions are hit.
   // This means somewhere in Jitter operand sharing is happening for
   // immediate type operands. This should be fixed.
@@ -465,20 +478,8 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
       newRgn.computeLeftBound(*kernel.fg.builder);
       newRgn.computeRightBound(execSize);
 
-      if (inst->isPseudoUse()) {
-        G4_Declare *topdcl = newRgn.getBase()->asRegVar()->getDeclare();
-
-        while (topdcl->getAliasDeclare() != NULL) {
-          topdcl = topdcl->getAliasDeclare();
-        }
-
-        newRgn.setLeftBound(0);
-        newRgn.setRightBound(topdcl->getByteSize() - 1);
-      }
-
       if ((opnd->getRightBound() - opnd->getLeftBound()) >
-              (2u * kernel.numEltPerGRF<Type_UB>()) &&
-          (inst->isPseudoUse() == false)) {
+              (2u * kernel.numEltPerGRF<Type_UB>())) {
         if (!(inst->opcode() == G4_pln && inst->getSrc(1) == opnd)) {
           DEBUG_VERBOSE(
               "Difference between left/right bound is greater than 2 GRF for "
@@ -558,20 +559,9 @@ void G4Verifier::verifyOpnd(G4_Operand *opnd, G4_INST *inst) {
       newRgn.computeLeftBound(*kernel.fg.builder);
       newRgn.computeRightBound(execSize);
 
-      if (inst->isPseudoKill()) {
-        G4_Declare *topdcl = newRgn.getBase()->asRegVar()->getDeclare();
-
-        while (topdcl->getAliasDeclare() != NULL) {
-          topdcl = topdcl->getAliasDeclare();
-        }
-
-        newRgn.setLeftBound(0);
-        newRgn.setRightBound(topdcl->getByteSize() - 1);
-      }
-
       if ((opnd->getRightBound() - opnd->getLeftBound()) >
               (2u * kernel.numEltPerGRF<Type_UB>()) &&
-          (inst->isPseudoKill() == false) && (inst->opcode() != G4_madw)) {
+          (inst->opcode() != G4_madw)) {
         DEBUG_VERBOSE(
             "Difference between left/right bound is greater than 2 GRF for dst "
             "region. Single non-send opnd cannot span 2 GRFs. lb = "
