@@ -46,6 +46,8 @@ enum BankConflict {
   BANK_CONFLICT_SECOND_HALF_ODD
 };
 
+class VarSplit;
+
 class BankConflictPass {
 private:
   GlobalRA &gra;
@@ -423,7 +425,7 @@ struct InterferenceMatrixStorage {
   // This member is a half triangle representation of interference graph implemented
   // as a sparse bitvector. Interference construction uses this member. When
   // incremental RA is enabled, this member is updated incrementally.
-  std::vector<llvm_SBitVector> sparseMatrix;
+  std::vector<SparseBitVector> sparseMatrix;
   // This member is constructed after SIMT and SIMD interference are computed.
   // It's a full triangle representation of interference matrix with trivial
   // traversal. This member is reconstructed in each graph color iteration.
@@ -438,7 +440,7 @@ struct InterferenceMatrixStorage {
 class IncrementalRA {
   friend Interference;
 
-  const llvm_SBitVector &getSparseMatrix(unsigned int id) {
+  const SparseBitVector &getSparseMatrix(unsigned int id) {
     return sparseMatrix[id];
   }
 
@@ -446,7 +448,7 @@ private:
   GlobalRA &gra;
   G4_Kernel &kernel;
   LiveRangeVec lrs;
-  std::vector<llvm_SBitVector>& sparseMatrix;
+  std::vector<SparseBitVector>& sparseMatrix;
   std::vector<std::vector<unsigned>>& sparseIntf;
   G4_RegFileKind selectedRF = G4_RegFileKind::G4_UndefinedRF;
   unsigned int level = 0;
@@ -597,12 +599,12 @@ public:
 
 private:
   // For verification only
-  std::vector<llvm_SBitVector> def_in;
-  std::vector<llvm_SBitVector> def_out;
-  std::vector<llvm_SBitVector> use_in;
-  std::vector<llvm_SBitVector> use_out;
-  std::vector<llvm_SBitVector> use_gen;
-  std::vector<llvm_SBitVector> use_kill;
+  std::vector<SparseBitVector> def_in;
+  std::vector<SparseBitVector> def_out;
+  std::vector<SparseBitVector> use_in;
+  std::vector<SparseBitVector> use_out;
+  std::vector<SparseBitVector> use_gen;
+  std::vector<SparseBitVector> use_kill;
 
   std::unique_ptr<VarReferences> prevIterRefs;
 
@@ -650,11 +652,11 @@ class Interference {
   // like dense matrix, interference is not symmetric (that is, if v1 and v2
   // interfere and v1 < v2, we insert (v1, v2) but not (v2, v1)) for better
   // cache behavior
-  std::vector<llvm_SBitVector>& sparseMatrix;
+  std::vector<SparseBitVector>& sparseMatrix;
 
   unsigned int denseMatrixLimit = 0;
 
-  static void updateLiveness(llvm_SBitVector &live, uint32_t id, bool val) {
+  static void updateLiveness(SparseBitVector &live, uint32_t id, bool val) {
     if (val) {
       live.set(id);
     } else {
@@ -711,23 +713,23 @@ class Interference {
     return matrix[idx];
   }
 
-  void addCalleeSaveBias(const llvm_SBitVector &live);
+  void addCalleeSaveBias(const SparseBitVector &live);
 
-  void buildInterferenceAtBBExit(const G4_BB *bb, llvm_SBitVector &live);
-  void buildInterferenceWithinBB(G4_BB *bb, llvm_SBitVector &live);
-  void buildInterferenceForDst(G4_BB *bb, llvm_SBitVector &live, G4_INST *inst,
+  void buildInterferenceAtBBExit(const G4_BB *bb, SparseBitVector &live);
+  void buildInterferenceWithinBB(G4_BB *bb, SparseBitVector &live);
+  void buildInterferenceForDst(G4_BB *bb, SparseBitVector &live, G4_INST *inst,
                                std::list<G4_INST *>::reverse_iterator i,
                                G4_DstRegRegion *dst);
-  void buildInterferenceForFcall(G4_BB *bb, llvm_SBitVector &live,
+  void buildInterferenceForFcall(G4_BB *bb, SparseBitVector &live,
                                  G4_INST *inst,
                                  std::list<G4_INST *>::reverse_iterator i,
                                  const G4_VarBase *regVar);
 
   inline void filterSplitDclares(unsigned startIdx, unsigned endIdx, unsigned n,
                                  unsigned col, unsigned &elt, bool is_split);
-  void buildInterferenceWithLive(const llvm_SBitVector &live, unsigned i);
+  void buildInterferenceWithLive(const SparseBitVector &live, unsigned i);
   void buildInterferenceWithSubDcl(unsigned lr_id, G4_Operand *opnd,
-                                   llvm_SBitVector &live, bool setLive,
+                                   SparseBitVector &live, bool setLive,
                                    bool setIntf);
   void buildInterferenceWithAllSubDcl(unsigned v1, unsigned v2);
 
@@ -887,6 +889,22 @@ class GraphColor {
 
   unsigned edgeWeightGRF(const LiveRange *lr1, const LiveRange *lr2);
   unsigned edgeWeightARF(const LiveRange *lr1, const LiveRange *lr2);
+  static unsigned edgeWeightGRF(bool lr1EvenAlign, bool lr2EvenAlign,
+                                unsigned lr1_nreg, unsigned lr2_nreg) {
+    if (!lr1EvenAlign) {
+      return lr1_nreg + lr2_nreg - 1;
+    }
+
+    if (!lr2EvenAlign) {
+      unsigned sum = lr1_nreg + lr2_nreg;
+      return sum + 1 - ((sum) % 2);
+    } else if (lr2EvenAlign) {
+      return lr1_nreg + lr2_nreg - 1 + (lr1_nreg % 2) + (lr2_nreg % 2);
+    } else {
+      vISA_ASSERT_UNREACHABLE("should be unreachable");
+      return 0;
+    }
+  }
 
   void computeDegreeForGRF();
   void computeDegreeForARF();
@@ -1721,12 +1739,81 @@ public:
     return iter != bbLocalRAMap.end() ? iter->second : nullptr;
   }
 
+  unsigned computeSpillSize(const LIVERANGE_LIST &spilledLRs);
+  unsigned computeSpillSize(std::list<LSLiveRange *> &spilledLRs);
+  bool spillSpaceCompression(int spillSize,
+                             const int globalScratchOffset);
+
 public:
   // Store new variables created when inserting scalar imm
   // spill/fill code. Such variables are not infinite spill
   // cost. So if the variable spills again, we shouldn't
   // get in an infinite loop by retrying same spill/fill.
   std::unordered_set<G4_Declare *> scalarSpills;
+
+private:
+  // Following methods are invoked from coloringRegAlloc() method. Any
+  // operation needed in global graph coloring loop should be extracted to a
+  // method instead of inlining logic in code directly in interest of keeping
+  // the loop maintainable.
+  void stackCallSaveRestore(bool hasStackCall);
+  int doGlobalLinearScanRA();
+  void incRABookKeeping();
+  // return pair <whether remat modified program, rematDone>
+  std::pair<bool, bool> remat(bool fastCompile, bool rematDone,
+                              LivenessAnalysis &liveAnalysis,
+                              GraphColor &coloring, RPE &rpe);
+  // rerun GRA, alignedScalarSplitDone, isEarlyExit
+  std::tuple<bool, bool, bool> alignedScalarSplit(bool fastCompile,
+                                                  bool alignedScalarSplitDone,
+                                                  GraphColor &coloring);
+  bool globalSplit(VarSplit &splitPass, GraphColor &coloring);
+  void localSplit(bool fastCompile, VarSplit &splitPass);
+  // return <doBCReduction, highInternalConflict>
+  std::pair<bool, bool> bankConflict();
+  // return reserveSpillReg
+  bool setupFailSafeIfNeeded(bool fastCompile, bool hasStackCall,
+                             unsigned int maxRAIterations,
+                             unsigned int failSafeRAIteration);
+  void undefinedUses(bool rematDone, LivenessAnalysis &liveAnalysis);
+  void writeVerboseStatsNumVars(LivenessAnalysis &liveAnalysis,
+                                FINALIZER_INFO *jitInfo);
+  void writeVerboseRPEStats(RPE &rpe);
+  void splitOnSpill(bool fastCompile, GraphColor &coloring,
+                    LivenessAnalysis &livenessAnalysis);
+  bool convertToFailSafe(bool reserveSpillReg, GraphColor &coloring,
+                         LivenessAnalysis &liveAnalysis,
+                         unsigned int nextSpillOffset);
+  unsigned int instCount() const {
+    unsigned int instNum = 0;
+    for (auto bb : kernel.fg) {
+      instNum += (int)bb->size();
+    }
+    return instNum;
+  }
+  // pair of abort, updated GRFSpillFillCount
+  std::pair<bool, unsigned int> abortOnSpill(unsigned int GRFSpillFillCount,
+                                             GraphColor &coloring);
+  void verifyNoInfCostSpill(GraphColor &coloring, bool reserveSpillReg);
+  void setupA0Dot2OnSpill(bool hasStackCall, unsigned int nextSpillOffset,
+                          int globalScratchOffset);
+  // isEarlyExit
+  bool spillCleanup(bool fastCompile, bool useScratchMsgForSpill,
+                    bool hasStackCall, bool reserveSpillReg, RPE &rpe,
+                    GraphColor &coloring, LivenessAnalysis &liveAnalysis,
+                    SpillManagerGRF &spillGRF);
+
+  // success, enableSpillSpaceCompression, isEarlyExit, scratchOffset,
+  // nextSpillOffset
+  std::tuple<bool, bool, bool, unsigned int, unsigned int>
+  insertSpillCode(bool enableSpillSpaceCompression, GraphColor &coloring,
+                  LivenessAnalysis &liveAnalysis, RPE &rpe,
+                  unsigned int scratchOffset, bool fastCompile,
+                  bool hasStackCall, int globalScratchOffset,
+                  unsigned int nextSpillOffset, bool reserveSpillReg,
+                  unsigned int spillRegSize, unsigned int indrSpillRegSize,
+                  bool useScratchMsgForSpill);
+  bool rerunGRAIter(bool rerunGRA);
 };
 
 inline G4_Declare *Interference::getGRFDclForHRA(int GRFNum) const {
