@@ -72,9 +72,11 @@ struct LiveNode {
   static unsigned getMaskSize(G4_INST *Inst, Gen4_Operand_Number OpNum) {
     G4_Operand *Opnd = Inst->getOperand(OpNum);
     vISA_ASSERT(Opnd, "null opnd");
-
     if (Opnd) {
       G4_Declare *Dcl = Opnd->getTopDcl();
+      if (Opnd->isAddrExp()) {
+        Dcl = Opnd->asAddrExp()->getRegVar()->getDeclare();
+      }
       if (Dcl == nullptr) {
         // There is no top declaration for this operand, so this is ARF.
         return 32;
@@ -259,7 +261,14 @@ bool LiveNode::addDefinition(G4_INST *DefInst, Gen4_Operand_Number DefOpNum,
 
 bool LiveNode::dependsOnChannelMask(bool IsInSimdFlow, G4_INST *Inst,
                                     Gen4_Operand_Number OpNum) {
-  if (!IsInSimdFlow || Inst->isWriteEnableInst())
+  // Treat non-flag scalar and indirect 1x1/Vx1 uses as noMask. Any non-noMask
+  // inst can't kill it.
+  auto opnd = Inst->getOperand(OpNum);
+  bool isScalarSrc = opnd->isScalarSrc() && !opnd->isFlag();
+  bool isIndirect1x1Vx1 = opnd->isIndirect() && !opnd->isVxHIndirect();
+
+  if (!IsInSimdFlow || Inst->isWriteEnableInst() || isScalarSrc ||
+      isIndirect1x1Vx1)
     return false;
 
   // Otherwise.
@@ -302,6 +311,13 @@ bool LiveNode::alignedWithChannelMask(G4_INST *DefInst,
     return DefOffset == UseOffset;
   }
 
+  // UseOpnd is indirect VxH
+  if (UseOpnd->isVxHIndirect()) {
+    int DefOffset = int(DefLB - DefMaskOffset * TypeSize(ADDR_REG_TYPE));
+    int UseOffset = int(UseLB - UseMaskOffset * TypeSize(ADDR_REG_TYPE));
+    return DefOffset == UseOffset;
+  }
+
   // Do not analyze instructions that may exceed two GRF boundary
   if (DefInst->mayExceedTwoGRF() || UseInst->mayExceedTwoGRF())
     return true;
