@@ -346,6 +346,7 @@ public:
   preRA_Scheduler(G4_Kernel &k);
   ~preRA_Scheduler();
   bool run(unsigned &KernelPressure);
+  bool runWithGRFSelection(unsigned &KernelPressure);
 
 private:
   G4_Kernel &kernel;
@@ -363,15 +364,6 @@ private:
   Options *m_options;
 };
 
-class preRA_RegSharing {
-public:
-  preRA_RegSharing(G4_Kernel &k);
-  ~preRA_RegSharing();
-  bool run(unsigned &KernelPressure);
-
-private:
-  G4_Kernel &kernel;
-};
 // Restrictions of candidate for 2xDP:
 //    1, Only support SIMD16 DF mad with M0
 //    2, Inst must not have predicate/cond mod
@@ -687,8 +679,9 @@ public:
     }
 
     auto dst = inst->getDst();
-    // 1, dst cannot be null and cannot cross GRF boundary
-    if (!dst || dst->asDstRegRegion()->isCrossGRFDst(inst->getBuilder())) {
+    // 1, dst cannot cross GRF boundary
+    if (!dst->isGreg() ||
+        dst->asDstRegRegion()->isCrossGRFDst(inst->getBuilder())) {
       return false;
     }
 
@@ -747,6 +740,15 @@ public:
     if (inst->getCondMod())
       return false;
 
+    // 9, Source GRF register are different and do not overlap with destination
+    //    GRF register.
+    if (src->isGreg()) {
+      int srcRegStart = src->getLinearizedStart() / grfSize;
+      int srcRegEnd = src->getLinearizedEnd() / grfSize;
+      if (srcRegStart <= dstReg && dstReg <= srcRegEnd)
+        return false;
+    }
+
     return true;
   }
 
