@@ -21,20 +21,20 @@ G4_BB *ImmDominator::InterSect(G4_BB *bb, int i, int k) {
   G4_BB *finger2 = immDoms[bb->getId()][k];
 
   while ((finger1 != finger2) && (finger1 != nullptr) && (finger2 != nullptr)) {
-    if (finger1->getPreId() == finger2->getPreId()) {
+    if (preIDMap.at(finger1) == preIDMap.at(finger2)) {
       vASSERT(finger1 == kernel.fg.getEntryBB() ||
              finger2 == kernel.fg.getEntryBB());
       return kernel.fg.getEntryBB();
     }
 
     while ((iDoms[finger1->getId()] != nullptr) &&
-           (finger1->getPreId() > finger2->getPreId())) {
+           (preIDMap.at(finger1) > preIDMap.at(finger2))) {
       finger1 = iDoms[finger1->getId()];
       immDoms[bb->getId()][i] = finger1;
     }
 
     while ((iDoms[finger2->getId()] != nullptr) &&
-           (finger2->getPreId() > finger1->getPreId())) {
+           (preIDMap.at(finger2) > preIDMap.at(finger1))) {
       finger2 = iDoms[finger2->getId()];
       immDoms[bb->getId()][k] = finger2;
     }
@@ -47,7 +47,7 @@ G4_BB *ImmDominator::InterSect(G4_BB *bb, int i, int k) {
 
   if (finger1 == finger2) {
     return finger1;
-  } else if (finger1->getPreId() > finger2->getPreId()) {
+  } else if (preIDMap.at(finger1) > preIDMap.at(finger2)) {
     return finger2;
   } else {
     return finger1;
@@ -64,7 +64,7 @@ void ImmDominator::runIDOM() {
   iDoms.resize(kernel.fg.size());
   immDoms.resize(kernel.fg.size());
 
-  kernel.fg.recomputePreId();
+  kernel.fg.recomputePreId(preIDMap);
 
   for (auto I = kernel.fg.cbegin(), E = kernel.fg.cend(); I != E; ++I) {
     auto bb = *I;
@@ -216,7 +216,7 @@ void ImmDominator::dumpImmDom(std::ostream &os) {
     for (auto domBB : domBBs) {
       os << "BB" << domBB->getId();
       if (domBB->getLabel()) {
-        os << " (" << domBB->getLabel()->getLabel() << ")";
+        os << " (" << domBB->getLabel()->getLabelName() << ")";
       }
       os << ", ";
     }
@@ -340,7 +340,7 @@ void PostDom::dumpImmDom(std::ostream &os) {
     for (auto pdomBB : pdomBBs) {
       os << "BB" << pdomBB->getId();
       if (pdomBB->getLabel()) {
-        os << " (" << pdomBB->getLabel()->getLabel() << ")";
+        os << " (" << pdomBB->getLabel()->getLabelName() << ")";
       }
       os << ", ";
     }
@@ -502,7 +502,7 @@ G4_BB *LoopDetection::getPreheader(Loop *loop) {
   // a valid pre-header wasnt found, so create one and return it
   // unless any pred uses SIMD CF to loop header
   if (header->getLabel()) {
-    auto headerLblStr = std::string(header->getLabel()->getLabel());
+    auto headerLbl = header->getLabel();
     for (auto pred : headerPreds) {
       if (pred->size() == 0 || loop->contains(pred))
         continue;
@@ -527,9 +527,9 @@ G4_BB *LoopDetection::getPreheader(Loop *loop) {
       if (pred->back()->isCFInst()) {
         auto cfInst = pred->back()->asCFInst();
         // Punt if pred refers to header's label in JIP/UIP field.
-        if (cfInst->getJip() && headerLblStr == cfInst->getJipLabelStr())
+        if (cfInst->getJip() && headerLbl == cfInst->getJip())
           return nullptr;
-        if (cfInst->getUip() && headerLblStr == cfInst->getUipLabelStr())
+        if (cfInst->getUip() && headerLbl == cfInst->getUip())
           return nullptr;
       }
     }
@@ -921,7 +921,7 @@ void Loop::dump(std::ostream &os) {
       (preHeader ? std::to_string(preHeader->getId()) : std::string("--"));
 
   if (preHeader && preHeader->getLabel())
-    labelStr += "(" + std::string(preHeader->getLabel()->getLabel()) + ")";
+    labelStr += "(" + std::string(preHeader->getLabel()->getLabelName()) + ")";
 
   std::string exitBBs = "{ ";
   for (auto bb : loopExits) {
