@@ -154,11 +154,10 @@ class Optimizer {
 
   void preRA_Schedule() {
     unsigned KernelPressure = 0;
-    if (kernel.useRegSharingHeuristics()) {
-      preRA_RegSharing Sched(kernel);
-      Sched.run(KernelPressure);
+    preRA_Scheduler Sched(kernel);
+    if (kernel.useAutoGRFSelection()) {
+      Sched.runWithGRFSelection(KernelPressure);
     } else {
-      preRA_Scheduler Sched(kernel);
       Sched.run(KernelPressure);
     }
     // Update Jit info for max register pressure
@@ -221,8 +220,7 @@ private:
 
   void staticProfiling();
 
-  void removeInstrinsics();
-  void markBreakpoint();
+  void removeIntrinsics();
 
   unsigned int numBankConflicts;
 
@@ -391,7 +389,7 @@ public:
     PI_mapOrphans,
     PI_legalizeType,
     PI_analyzeMove,
-    PI_removeInstrinsics,
+    PI_removeIntrinsics,
     PI_expandMulPostSchedule,
     PI_zeroSomeARF,
     PI_addSWSBInfo,
