@@ -106,14 +106,27 @@ bool Options::parseOptions(int argc, const char *argv[]) {
       return true;
     };
 
-    // If the current argv has a value, ie, the next argv isn't a flag
-    //   (flag starts with '-') treat it as a unsigned 32bit integer
-    //   and return true if the value is of U32;
+    auto parseBool = [&](bool &val) -> bool {
+      std::string_view arg(argv[i]);
+      if (arg == "true" || arg == "TRUE") {
+        val = true;
+        return true;
+      }
+      if (arg == "false" || arg == "FALSE") {
+        val = false;
+        return true;
+      }
+      return false;
+    };
+
+    // If the current argv has a value, i.e., the next argv isn't a flag
+    //   (flag starts with '-') treat it as either bool ("true" or "false") or a
+    //   unsigned 32bit integer and return true if the value is non-zero;
     // otherwise, return false.
     //
     // Note that this is to make sure a boolean flag can be set true or
     // false explicitly. If no value, it will be inversed.
-    auto parseOptionalU32 = [&]() -> std::optional<uint32_t> {
+    auto parseOptionalBool = [&]() -> std::optional<bool> {
       const int next_i = i + 1;
       if (next_i >= argc)
         return std::nullopt;
@@ -124,29 +137,35 @@ bool Options::parseOptions(int argc, const char *argv[]) {
 
       // advance i to the next argv : the value for this flag
       ++i;
-      uint32_t val;
-      if (!parseU32(val)) {
+      bool boolVal;
+      if (parseBool(boolVal))
+        return boolVal;
+      uint32_t intVal;
+      if (!parseU32(intVal)) {
         // Assume this flag does not have a value
         --i;
         return std::nullopt;
       }
-      return val;
+      return intVal != 0;
     };
 
-    // Arg corrsponds to vISAOpt.
-    // If bool, set it with the inverse of the default value if no explicit
-    //          value is given; set it to the given value otherwise.
-    // Else if int32,int64, or cstr, set parse argv[i+1] and set the value.
+    // Arg corresponds to vISAOpt.
+    // If bool:
+    //  If argv[i+1] is either bool or int value, set the value to it.
+    //  Otherwise, either set it to true or flip the default value depending on
+    //  the option.
+    // If int32,int64, or cstr, parse argv[i+1] and set the value.
     vISAOptions vISAOpt = it->second;
     EntryType type = m_vISAOptions.getType(vISAOpt);
     switch (type) {
     case ET_BOOL: {
       bool val;
-      std::optional<uint32_t> o = parseOptionalU32();
+      std::optional<bool> o = parseOptionalBool();
       if (o.has_value())
-        val = (o.value() != 0 ? true : false);
+        val = o.value();
       else
-        val = !m_vISAOptions.getDefaultBool(vISAOpt);
+        val = m_vISAOptions.getFlipBoolDefaultVal(vISAOpt)
+            ? !m_vISAOptions.getDefaultBool(vISAOpt) : true;
       m_vISAOptions.setBool(vISAOpt, val);
       m_vISAOptions.setArgSetByUser(vISAOpt);
       break;
@@ -243,6 +262,7 @@ bool Options::parseOptions(int argc, const char *argv[]) {
     m_vISAOptions.setBool(vISA_preRA_Schedule, false);
     m_vISAOptions.setBool(vISA_SplitGRFAlignedScalar, false);
     m_vISAOptions.setBool(vISA_SkipRedundantFillInRMW, false);
+    m_vISAOptions.setBool(vISA_EnableDCE, false);
     m_vISAOptions.setBool(vISA_Debug, true);
   }
   if (m_vISAOptions.isArgSetByUser(vISA_Stepping)) {
@@ -333,6 +353,10 @@ bool Options::parseOptions(int argc, const char *argv[]) {
     m_vISAOptions.setUint32(vISA_SWSBTokenBarrier, 0);
   }
 
+  if (m_vISAOptions.isArgSetByUser(vISA_ForceSpills)) {
+    m_vISAOptions.setBool(vISA_LinearScan, false);
+  }
+
   if (m_vISAOptions.getBool(vISA_dumpVISAOptionsAll)) {
     dump();
   }
@@ -341,9 +365,8 @@ bool Options::parseOptions(int argc, const char *argv[]) {
 }
 
 void Options::showUsage(std::ostream &output) {
-  output << "USAGE: GenX_IR <InputFilename.isa> {Option List}\n";
-  output << "Converts a vISA assembly or binary file into Gen binary or "
-            "assembly\n";
+  output << "USAGE: GenX_IR <InputFilename.visaasm> {Option List}\n";
+  output << "Converts a vISA assembly into Gen binary or assembly\n";
   output << "Options:\n";
 #ifndef DLL_MODE
   output << "    -platform PLT                   - Gen platform to use "
@@ -351,8 +374,8 @@ void Options::showUsage(std::ostream &output) {
             "        supported platforms are: "
          << makePlatformsString()
          << "\n"
-            "    -binary                         - Emit the binary code (CISA "
-            "binary .isa and GEN bits as .dat).\n";
+            "    -binary                         - Emit the binary code (Gen "
+            "bits as .dat).\n";
 #endif
   output
       << "    -output                         - Emit GEN assembly code to a "
@@ -361,7 +384,7 @@ void Options::showUsage(std::ostream &output) {
          "(.visaasm).\n"
          "    -noschedule                     - Turn off code scheduling.\n"
          "    -nocompaction                   - Turn off binary compaction.\n"
-         "    -outputCisaBinaryName <PATH>    - name for the CISA binary "
+         "    -outputIsaasmName <PATH>        - name for the combined .isaasm "
          "file.\n"
          "    -... many more; use -dumpVisaOptionsAll\n"
          "\n";
@@ -403,6 +426,12 @@ void Options::initialize_m_vISAOptions(void) {
     Opts->setDefaultBool(ENUM, (bool)(DEFAULT_VAL));                           \
     Opts->setBool(ENUM, (bool)(DEFAULT_VAL));                                  \
     break;                                                                     \
+  case ET_BOOL_TRUE:                                                           \
+    Opts->setType(ENUM, ET_BOOL);                                              \
+    Opts->setDefaultBool(ENUM, (bool)(DEFAULT_VAL));                           \
+    Opts->setBool(ENUM, (bool)(DEFAULT_VAL));                                  \
+    Opts->setFlipBoolDefaultVal(ENUM, false);                                  \
+    break;                                                                     \
   case ET_INT32:                                                               \
     /* The casts are here to avoid type conversion warnings */                 \
     Opts->setDefaultUint32(ENUM, (uint32_t)(uint64_t)(DEFAULT_VAL));           \
@@ -694,9 +723,7 @@ void Options::getOptionsFromEV() {
 
 void Options::dump(void) const { m_vISAOptions.dump(); }
 
-Options::Options() {
-  m_vISAOptions = VISAOptionsDB(this);
-
+Options::Options() : m_vISAOptions(this) {
   target = VISA_CM;
 
   initialize_vISAOptionsToStr();
