@@ -357,16 +357,16 @@ public:
       r0(
         b.createHardwiredDeclare(
           k.numEltPerGRF<Type_UD>(), Type_UD, 0, 0)),
-      rtmp(
-        b.createHardwiredDeclare(
-          k.numEltPerGRF<Type_UD>(), Type_UD,
-          k.getNumRegTotal() - 1, 0)),
       perThreadLoadStartGRF(
         k.getOptions()->getuInt32Option(vISA_loadThreadPayloadStartReg)),
       numPerThreadGRF(
         AlignUp(k.getInt32KernelAttr(Attributes::ATTR_PerThreadInputSize),
                 k.numEltPerGRF<Type_UB>()) / k.numEltPerGRF<Type_UB>())
   {
+
+    auto rtmpRegNum = k.getNumRegTotal() - 1;
+    rtmp = b.createHardwiredDeclare(k.numEltPerGRF<Type_UD>(), Type_UD, rtmpRegNum, 0);
+
     r0->setName("r0");
     rtmp->setName("rtmp");
 
@@ -901,9 +901,11 @@ public:
       if (builder.getOption(vISA_useInlineData)) {
         // copy inline data to the first GRF of cross-thread-data
         // e.g. (W) mov (8) inlineDataReg.0:ud r1.0:ud
-        // Inline GRF is only 8 DWords
+        // Inline data size is 8 DWords.
+
         emitMovInlineData(perThreadLoadStartGRF + numPerThreadGRF,
-                          perThreadLoadStartGRF, 8);
+                          perThreadLoadStartGRF,
+                          builder.getInlineDataSize()/TypeSize(Type_UD));
       }
 
       loadFromMemory(rtmp, perThreadLoadStartGRF,
