@@ -606,25 +606,25 @@ bool VerifyAugmentation::interfereBetween(G4_Declare *dcl1, G4_Declare *dcl2) {
 }
 
 void VerifyAugmentation::verify() {
-  std::cerr << "Start verification for kernel: "
+  std::cout << "Start verification for kernel: "
             << kernel->getOptions()->getOptionCstr(VISA_AsmFileName) << "\n";
 
   for (auto dcl : kernel->Declares) {
     if (dcl->getIsSplittedDcl()) {
       auto &tup = masks[dcl];
-      std::cerr << dcl->getName() << "(" << getStr(std::get<1>(tup))
+      std::cout << dcl->getName() << "(" << getStr(std::get<1>(tup))
                 << ") is split"
                 << "\n";
       for (const G4_Declare *subDcl : gra->getSubDclList(dcl)) {
         auto &tupSub = masks[subDcl];
-        std::cerr << "\t" << subDcl->getName() << " ("
+        std::cout << "\t" << subDcl->getName() << " ("
                   << getStr(std::get<1>(tupSub)) << ")"
                   << "\n";
       }
     }
   }
 
-  std::cerr << "\n"
+  std::cout << "\n"
             << "\n"
             << "\n";
 
@@ -668,7 +668,7 @@ void VerifyAugmentation::verify() {
       return str;
     };
 
-    std::cerr << dcl->getName() << " - " << getMaskStr(dclMask);
+    std::cout << dcl->getName() << " - " << getMaskStr(dclMask);
 
     verifyAlign(dcl);
 
@@ -693,7 +693,7 @@ void VerifyAugmentation::verify() {
           continue;
 
         if (!interfere) {
-          std::cerr << dcl->getRegVar()->getName() << "(" << getStr(dclMask)
+          std::cout << dcl->getRegVar()->getName() << "(" << getStr(dclMask)
                     << ") and " << activeDcl->getRegVar()->getName() << "("
                     << getStr(aDclMask)
                     << ") are overlapping with incompatible emask but not "
@@ -703,7 +703,7 @@ void VerifyAugmentation::verify() {
 
         if (overlapDcl(activeDcl, dcl)) {
           if (!interfere) {
-            std::cerr << dcl->getRegVar()->getName() << "(" << getStr(dclMask)
+            std::cout << dcl->getRegVar()->getName() << "(" << getStr(dclMask)
                       << ") and " << activeDcl->getName() << "("
                       << getStr(aDclMask)
                       << ") use overlapping physical assignments but not "
@@ -717,7 +717,7 @@ void VerifyAugmentation::verify() {
     active.push_back(dcl);
   }
 
-  std::cerr << "End verification for kenel: "
+  std::cout << "End verification for kenel: "
             << kernel->getOptions()->getOptionCstr(VISA_AsmFileName) << "\n"
             << "\n"
             << "\n";
@@ -809,7 +809,7 @@ bool VerifyAugmentation::isClobbered(LiveRange *lr, std::string &msg) {
           if (oiDst && oiDst->isDstRegRegion() && oiDst->getTopDcl()) {
             unsigned oilb = 0, oirb = 0;
             auto oiLR = DclLRMap[oiDst->getTopDcl()];
-            if (oiLR && !oiLR->getPhyReg())
+            if (!oiLR->getPhyReg())
               continue;
 
             oilb = oiLR->getPhyReg()->asGreg()->getRegNum() *
@@ -851,7 +851,7 @@ bool VerifyAugmentation::isClobbered(LiveRange *lr, std::string &msg) {
 
     if (rd.size() > 0) {
       printf("Current use str = %s for inst:\t", useStr.data());
-      inst->emit(std::cerr);
+      inst->emit(std::cout);
       printf("\t$%d\n", inst->getVISAId());
     }
     // process all reaching defs
@@ -907,7 +907,7 @@ bool VerifyAugmentation::isClobbered(LiveRange *lr, std::string &msg) {
           printed = true;
         }
         printf("\t");
-        std::get<0>(reachingDef)->emit(std::cerr);
+        std::get<0>(reachingDef)->emit(std::cout);
         printf("\t$%d\n", std::get<0>(reachingDef)->getVISAId());
       }
     }
@@ -1235,5 +1235,247 @@ void DynPerfModel::dump() {
     Buffer += std::string(LocalBuffer);
   }
 
-  std::cerr << Buffer << "\n";
+  std::string FN = AsmName;
+  FN += std::string("-dyn-perf.txt");
+  std::ofstream OF;
+  OF.open(FN, std::ofstream::out);
+  OF << Buffer << "\n";
+  OF.close();
+
+  std::cout << Buffer << "\n";
+}
+
+SpillAnalysis::~SpillAnalysis() {
+  if (Refs) {
+    delete Refs;
+    Refs = nullptr;
+  }
+}
+
+void SpillAnalysis::Dump(std::ostream &OS) {
+  auto &GRA = GC->getGRA();
+  auto &Kernel = GRA.kernel;
+  auto &Loops = Kernel.fg.getLoops();
+  const auto &Spills = GC->getSpilledLiveRanges();
+  std::unordered_map<G4_INST *, G4_BB *> InstBBMap;
+
+  for (auto *BB : Kernel.fg.getBBList())
+    for (auto *Inst : BB->getInstList())
+      InstBBMap[Inst] = BB;
+
+  OS << "Name, Dcl Byte Size, Spill Cost, Degree, #Defs, #Uses, Distance, "
+        "#BBs, All BBs Where Live"
+     << "\n";
+
+  for (auto *Spill : Spills) {
+    // dump - {Dcl size, Spill cost, Live BBs (loop annotation)}
+    auto Dcl = Spill->getDcl();
+    auto DclSizeBytes = Dcl->getByteSize();
+    auto SpillCost = Spill->getSpillCost();
+    auto Degree = DclDegree[Dcl];
+    auto LiveBBs = GetLiveBBs(Dcl, InstBBMap);
+    auto Distance = GetDistance(Dcl);
+    auto NumDefs = Refs->getDefCount(Dcl);
+    auto NumUses = Refs->getUseCount(Dcl);
+
+    OS << Dcl->getName() << "," << DclSizeBytes << ", " << SpillCost << ", "
+       << Degree << ", " << NumDefs << ", " << NumUses << ", " << Distance
+       << ", " << LiveBBs.size() << ", ";
+
+    for (auto *LiveBB : LiveBBs) {
+      OS << "BB" << LiveBB->getId();
+      auto *ClosestLoop = Loops.getInnerMostLoop(LiveBB);
+      if (ClosestLoop) {
+        OS << " [L" << ClosestLoop->id << "]";
+      }
+      OS << " ";
+    }
+
+    OS << "\n";
+  }
+}
+
+unsigned int SpillAnalysis::GetDistance(G4_Declare *Dcl) {
+  if (AugIntervals.count(Dcl) == 0) {
+    // Construct distance in conventional way
+    auto &Kernel = GC->getGRA().kernel;
+    unsigned int Start = 0xffffffff, End = 0x0;
+
+    auto *Defs = Refs->getDefs(Dcl);
+    auto *Uses = Refs->getUses(Dcl);
+
+    for (auto &Def : *Defs) {
+      auto *DefInst = std::get<0>(Def);
+      Start = std::min(Start, DefInst->getLexicalId());
+      End = std::max(End, DefInst->getLexicalId());
+    }
+
+    for (auto &Use : *Uses) {
+      auto *UseInst = std::get<0>(Use);
+      Start = std::min(Start, UseInst->getLexicalId());
+      End = std::max(End, UseInst->getLexicalId());
+    }
+
+    for (auto *BB : Kernel.fg.getBBList()) {
+      if (LA->isLiveAtEntry(BB, Dcl->getRegVar()->getId()))
+        Start = std::min(Start, BB->front()->getLexicalId());
+      if (LA->isLiveAtExit(BB, Dcl->getRegVar()->getId()))
+        End = std::max(End, BB->back()->getLexicalId());
+    }
+
+    return End - Start;
+  }
+
+  // Return augmentation distance when available
+  auto Distance = AugIntervals[Dcl];
+  return Distance.second->getLexicalId() - Distance.first->getLexicalId();
+}
+
+void SpillAnalysis::LoadAugIntervals(DECLARE_LIST &SortedIntervals,
+                                     GlobalRA &GRA) {
+  for (auto &LR : SortedIntervals) {
+    auto *Start = GRA.getStartInterval(LR);
+    auto *End = GRA.getEndInterval(LR);
+    AugIntervals[LR] = std::make_pair(Start, End);
+  }
+}
+
+void SpillAnalysis::LoadDegree(G4_Declare *Dcl, unsigned int degree) {
+  // This should be called after degree computation and before simplification.
+  DclDegree[Dcl] = degree;
+}
+
+void SpillAnalysis::Clear() {
+  if (Refs)
+    delete Refs;
+
+  Refs = nullptr;
+  LA = nullptr;
+  GC = nullptr;
+  SM = nullptr;
+  AugIntervals.clear();
+  DclDegree.clear();
+}
+
+void SpillAnalysis::DumpHistogram(std::ostream &OS) {
+  // Compute and dump histogram
+  std::map<unsigned int, unsigned int> SpillSizeHistogram;
+  for (auto Spill : GC->getSpilledLiveRanges()) {
+    auto ByteSize = Spill->getDcl()->getByteSize();
+    SpillSizeHistogram[ByteSize] += 1;
+  }
+
+  OS << "Spill Size Histogram For Iter#" << GC->getGRA().getIterNo() << " : "
+     << "\n";
+  for (auto &Item : SpillSizeHistogram) {
+    OS << "# vars of " << Item.first << " bytes spilled: " << Item.second
+       << "\n";
+  }
+
+  OS << "\n";
+}
+
+void SpillAnalysis::Do(LivenessAnalysis *L, GraphColor *C, SpillManagerGRF *S) {
+  SetLivenessAnalysis(L);
+  SetGraphColor(C);
+  SetSpillManager(S);
+
+  unsigned int LexId = 0;
+  for (auto *BB : C->getGRA().kernel.fg.getBBList())
+    for (auto *Inst : BB->getInstList())
+      Inst->setLexicalId(LexId++);
+
+  Refs = new VarReferences(C->getGRA().kernel);
+
+  auto IterNo = C->getGRA().getIterNo();
+
+  std::string FN =
+      C->getGRA().kernel.getOptions()->getOptionCstr(VISA_AsmFileName);
+  FN += std::string("-spill-iter-");
+  FN += std::to_string(IterNo);
+  FN += std::string(".csv");
+  std::ofstream OF;
+  OF.open(FN, std::ofstream::out);
+  Dump(OF);
+  OF.close();
+
+  FN = C->getGRA().kernel.getOptions()->getOptionCstr(VISA_AsmFileName);
+  FN += std::string("-misc-spill-data.txt");
+  OF.open(FN, IterNo == 0 ? std::ofstream::out : std::ofstream::app);
+  if (IterNo == 0) {
+    ((vISA::Analysis *)&C->getGRA().kernel.fg.getLoops())->dump(OF);
+  }
+  DumpHistogram(OF);
+  OF.close();
+}
+
+std::vector<G4_BB *>
+SpillAnalysis::GetLiveBBs(G4_Declare *Dcl,
+                          std::unordered_map<G4_INST *, G4_BB *> &InstBBMap) {
+  // Return all BBs over which Dcl is live. This includes augmentation data.
+  auto Order = [](const G4_BB *First, const G4_BB *Second) {
+    return First->getId() < Second->getId();
+  };
+  std::set<G4_BB *, decltype(Order)> BBs(Order);
+  auto &Kernel = GC->getGRA().kernel;
+
+  VarReferences VarRefs(Kernel);
+  auto *Defs = VarRefs.getDefs(Dcl);
+  auto *Uses = VarRefs.getUses(Dcl);
+
+  for (auto Def : *Defs) {
+    auto *BB = std::get<1>(Def);
+    BBs.insert(BB);
+  }
+
+  for (auto Use : *Uses) {
+    auto *BB = std::get<1>(Use);
+    BBs.insert(BB);
+  }
+
+  for (auto BB : Kernel.fg.getBBList()) {
+    if (LA->isLiveAtEntry(BB, Dcl->getRegVar()->getId()) ||
+        LA->isLiveAtExit(BB, Dcl->getRegVar()->getId())) {
+      BBs.insert(BB);
+    }
+  }
+
+  if (AugIntervals.count(Dcl)) {
+    auto &Interval = AugIntervals[Dcl];
+    auto AugBBs = GetIntervalBBs(Interval.first, Interval.second, InstBBMap);
+    std::for_each(AugBBs.begin(), AugBBs.end(),
+                  [&](G4_BB *BB) { BBs.insert(BB); });
+  }
+
+  std::vector<G4_BB *> RetBBs;
+  std::for_each(BBs.begin(), BBs.end(),
+                [&](G4_BB *BB) { RetBBs.push_back(BB); });
+
+  return RetBBs;
+}
+
+std::vector<G4_BB *> vISA::SpillAnalysis::GetIntervalBBs(
+    G4_INST *Start, G4_INST *End,
+    std::unordered_map<G4_INST *, G4_BB *> &InstBBMap) {
+  // Return vector of BBs given Start/End G4_INST*s
+  std::vector<G4_BB *> BBs;
+  auto &Kernel = GC->getGRA().kernel;
+  bool Started = false;
+  for (auto *BB : Kernel.fg.getBBList()) {
+    bool BBAdded = false;
+    for (auto *Inst : BB->getInstList()) {
+      if (Inst == Start)
+        Started = true;
+
+      if (Started && !BBAdded) {
+        BBs.push_back(BB);
+        BBAdded = true;
+      }
+
+      if (Inst == End)
+        return BBs;
+    }
+  }
+
+  return BBs;
 }
