@@ -458,6 +458,8 @@ private:
                                                   unsigned regOff = 0,
                                                   unsigned subregOff = 0);
 
+  std::optional<G4_Declare *> getPreDefinedMRangeDeclare() const;
+
   G4_Declare *createMRangeDeclare(G4_RegVar *regVar);
 
   G4_Declare *createMRangeDeclare(G4_DstRegRegion *region,
@@ -557,21 +559,20 @@ private:
                                G4_SrcRegRegion *filledRangeRegion,
                                G4_ExecSize execSize);
 
-  G4_SrcRegRegion *getLSCSpillFillHeader(G4_Declare *mRangeDcl,
-                                         const G4_Declare *fp, int offset);
+  G4_SrcRegRegion *getLSCSpillFillHeader(const G4_Declare *fp, int offset);
 
-  G4_INST *createLSCSpill(G4_Declare *spillRangeDcl, G4_Declare *mRangeDcl,
-                          unsigned regOff, unsigned height, unsigned spillOff);
+  G4_INST *createLSCSpill(G4_Declare *spillRangeDcl, unsigned regOff,
+                          unsigned height, unsigned spillOff);
 
-  G4_INST *createLSCSpill(G4_Declare *spillRangeDcl, G4_Declare *mRangeDcl,
+  G4_INST *createLSCSpill(G4_Declare *spillRangeDcl,
                           G4_DstRegRegion *spilledRangeRegion,
                           G4_ExecSize execSize, unsigned option,
                           bool isScatter = false);
 
-  G4_INST *createLSCFill(G4_Declare *fillRangeDcl, G4_Declare *mRangeDcl,
-                         unsigned regOff, unsigned height, unsigned spillOff);
+  G4_INST *createLSCFill(G4_Declare *fillRangeDcl, unsigned regOff,
+                         unsigned height, unsigned spillOff);
 
-  G4_INST *createLSCFill(G4_Declare *fillRangeDcl, G4_Declare *mRangeDcl,
+  G4_INST *createLSCFill(G4_Declare *fillRangeDcl,
                          G4_SrcRegRegion *filledRangeRegion,
                          G4_ExecSize execSize);
 
@@ -635,12 +636,12 @@ private:
 
   int globalScratchOffset;
 
-  bool useScratchMsg_;
+  const bool useScratchMsg_;
   bool avoidDstSrcOverlap_;
   // spilled declares that represent a scalar immediate (created due to encoding
   // restrictions) We rematerialize the immediate value instead of spill/fill
-  // them
-  std::unordered_map<G4_Declare *, G4_Imm *> scalarImmSpill;
+  // them. Map stores type used on dst that defined the immediate.
+  std::unordered_map<G4_Declare *, std::pair<G4_Type, G4_Imm *>> scalarImmSpill;
   // distance to reuse filled scalar imm
   const unsigned int scalarImmReuseDistance = 10;
   // use cache to reuse filled scalar immediates for nearby uses
@@ -722,11 +723,10 @@ static inline bool isPartialRegion(REGION_TYPE *region, unsigned execSize) {
 // Return true if inst is a simple mov with exec size == 1 and imm as src0.
 // Def of such instructions are trivially fillable.
 static bool immFillCandidate(G4_INST *inst) {
+  auto src0 = inst->getSrc(0);
   return inst->opcode() == G4_mov && inst->getExecSize() == g4::SIMD1 &&
-         inst->getSrc(0)->isImm() && inst->isWriteEnableInst() &&
-         inst->getDst()->getType() == inst->getSrc(0)->getType() &&
-         !inst->getPredicate() && !inst->getCondMod() && !inst->getSaturate() &&
-         !inst->getSrc(0)->isRelocImm();
+         src0->isImm() && inst->isWriteEnableInst() && !inst->getPredicate() &&
+         !inst->getCondMod() && !inst->getSaturate() && !src0->isRelocImm();
 }
 
 G4_SrcRegRegion *getSpillFillHeader(IR_Builder &builder, G4_Declare *decl);
