@@ -14,6 +14,10 @@ SPDX-License-Identifier: MIT
 
 #include "visa_igc_common_header.h"
 
+#ifndef GIT_COMMIT_HASH
+#define GIT_COMMIT_HASH "?"
+#endif
+
 #ifndef _WIN32
 #include "common/secure_string.h"
 #include <errno.h>
@@ -174,7 +178,6 @@ enum class PlatformGen {
 #define ERROR_DATA_RANGE(x) "ERROR: Out of boundary or invalid data value in #x"
 // end of Error Message
 
-
 template <typename Type>
 constexpr Type AlignUp(const Type value, const size_t alignment) {
   Type common = value + alignment - 1;
