@@ -1,23 +1,25 @@
+/*========================== begin_copyright_notice ============================
+
+Copyright (C) 2017-2021 Intel Corporation
+
+SPDX-License-Identifier: MIT
+
+============================= end_copyright_notice ===========================*/
+
 #ifndef GED_INS_H
 #define GED_INS_H
 
 #include <set>
-# if GED_VALIDATION_API
 #include <vector>
-# endif // GED_VALIDATION_API
 #include "common/ged_base.h"
 #include "common/ged_ins_decoding_table.h"
 #include "common/ged_compact_mapping_table.h"
-# if GED_VALIDATION_API
 #include "common/ged_validation_utils.h"
-# endif // GED_VALIDATION_API
 #include "xcoder/ged_internal_api.h"
 #include "xcoder/ged_restrictions_handler.h"
 
 using std::set;
-# if GED_VALIDATION_API
 using std::vector;
-# endif // GED_VALIDATION_API
 
 
 enum GED_INS_STATUS
@@ -45,7 +47,7 @@ public:
     /*!
      * Initialize an empty instruction with a new opcode.
      *
-     * @param[in]   modelId     GEN version as specified by @ref GED_MODEL.
+     * @param[in]   modelId     GED model version as specified by @ref GED_MODEL.
      * @param[in]   opcode      The enumerator representing the new opcode.
      *
      * @return      GED_RETURN_VALUE indicating success or invalid opcode.
@@ -56,7 +58,7 @@ public:
     /*!
      * Assign raw (undecoded) bytes to the GEDIns object. If the instruction is compact, it is mapped to the native format.
      *
-     * @param[in]   modelId     GEN version as specified by @ref GED_MODEL.
+     * @param[in]   modelId     GED model version as specified by @ref GED_MODEL.
      * @param[in]   rawBytes    An array of size "size" containing the raw bytes.
      * @param[in]   size        The size of the array.
      *
@@ -125,9 +127,22 @@ public:
 
 # endif // GED_VALIDATION_API
 
+    /*!
+    * Return description of how a field is encoded within instructions. Top 16bits of
+    * each fragment specifies the length, whereas bottom 16bits specifies LSB of field.
+    * Pass NULL as fragments to have a function return the number of elements in 'length'.
+    *
+    * @param[in]   field   Id of the requested field.
+    * @param[in]   fragments Array of fragments (*length in size)
+    * @param[in]   length  Number of entries in fragments
+    *
+    * @return      GED_RETURN_VALUE indicating success or encoding error.
+    *
+    */
+    GED_RETURN_VALUE QueryFieldBitLocation(const /* GED_INS_FIELD */ uint32_t field, uint32_t *fragments, uint32_t *length) const;
 
     /*!
-     * Get the GED_MODEL Id for the GEN version by which this instruction is decoded.
+     * Get the GED_MODEL Id for the GED model version by which this instruction is decoded.
      *
      * @return      The requested GED_MODEL Id.
      */
@@ -136,7 +151,7 @@ public:
 
     /*!
      * Get the instruction's opcode in the form of a GED_OPCODE enumerator. Use @ref GetRawOpcode for obtaining the raw encoding of
-     * the opcode in the GEN instruction.
+     * the opcode in the GED model instruction.
      *
      * @return      The enumerator representing the instruction's opcode.
      *
@@ -146,7 +161,7 @@ public:
 
 
     /*!
-     * Get the instruction raw opcode as it is encoded in the GEN instruction. Use @ref GetOpcode for obtaining the GED_OPCODE for
+     * Get the instruction raw opcode as it is encoded in the GED model instruction. Use @ref GetOpcode for obtaining the GED_OPCODE for
      * this opcode.
      *
      * @return      The opcode encoding.
@@ -589,11 +604,11 @@ private:
     string GetInstructionBytes(const unsigned char* instructionBytes, int dwords) const;
 # if GED_VALIDATION_API
     bool RecordPadding(vector<ged_ins_field_mapping_fragment_t> &mappingFragments, const ged_ins_field_entry_t* dataEntry) const;
+# endif // GED_VALIDATION_API
     bool RecordPosition(vector<ged_ins_field_mapping_fragment_t> &mappingFragments, const ged_ins_field_entry_t* dataEntry) const;
     void RecordSingleFragment(vector<ged_ins_field_mapping_fragment_t> &mappingFragments,
                               const ged_ins_field_position_fragment_t &position) const;
     void MergeFragments(vector<ged_ins_field_mapping_fragment_t> &mappingFragments) const;
-# endif // GED_VALIDATION_API
 
 private:
     // PRIVATE STATIC DATA MEMBERS
