@@ -176,6 +176,7 @@ public:
 private:
   void setInputsFlagDep();
   void setInputsSrcDep();
+  void setInputsSendDescDep();
 
   void setOutputsFlagDep();
   void setOutputsDstDep();
@@ -198,6 +199,7 @@ private:
   std::pair<uint32_t, uint32_t> setSrcRegion(
     RegName rn, RegRef rr, Region r, uint32_t execSize, uint32_t typeSizeBits);
 
+
   // Set the bits to this DepSet with the given reg_range
   void addDependency(const RegRangeType &reg_range);
   void addDependency(const RegRangeListType &reg_range);
@@ -450,7 +452,7 @@ private:
     bool nextIsNotMacroCandidate(const Instruction &dpas,
                                  const Instruction &next_inst) const;
 
-    // set register fange from start_reg to upper_reg into bit_set
+    // set register range from start_reg to upper_reg into bit_set
     void setBits(BitSet<> &bit_set, uint32_t start_reg,
                  uint32_t upper_reg) const;
 
