@@ -239,13 +239,8 @@ struct Region {
   }
 
   bool isScalar() const {
-    if (bits == SRC010.bits || bits == SRC0X0.bits || bits == SRCXX0.bits)
-      return true;
-    return false;
-  }
-
-  bool isFlat() const {
-    if (bits == SRC110.bits || bits == SRC1X0.bits)
+    if ((v == 0 && h == 0) || (v == 0 && w == 1) ||
+        bits == SRCXX0.bits)
       return true;
     return false;
   }
@@ -266,10 +261,12 @@ struct Region {
   static const Region DST4; // <4>
   //
   // generalized src regions
-  static const Region SRC010; // <0;1,0> (broadcast scalar)
-  static const Region SRC110; // <1;1,0> (packed access)
-  static const Region SRC210; // <2;1,0> (even strided access)
-  static const Region SRC410; // <4;1,0> (quarter stided access)
+  static const Region SRC010;  // <0;1,0> (broadcast scalar)
+  static const Region SRC110;  // <1;1,0> (packed access)
+  static const Region SRC210;  // <2;1,0> (even strided access)
+  static const Region SRC410;  // <4;1,0> (quarter stided access)
+  static const Region SRC810;  // <8;1,0>
+  static const Region SRC1610; // <16;1,0>
   //
   // older src regions
   static const Region SRC221; // <2;2,1>
@@ -283,6 +280,8 @@ struct Region {
   static const Region SRC2X1; // <2;1> (ternary align1 src0 and src1)
   static const Region SRC1X0; // <1;0> XE_LP changes 2 to 1 in encoding
   static const Region SRC4X1; // <4;1> (ternary align1 src0 and src1)
+  static const Region SRC4X2; // <4;2> single-stride 2
+  static const Region SRC8X4; // <8;4> single-stride 4
   // ternary src2
   static const Region SRC8X1; // <8;1> (ternary align1 src0 and src1)
   static const Region SRCXX0; // <0>   (ternary align1 src2)
